/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"
#include "ArP2Arm.h"


/*
  P2ArmSimple demonstrates how to connect with the Pioneer 2 controller
  and set up the P2Arm class, including the ARMpac handler.
  It simply queries and prints the status of the arm, then exits.
*/

int
main(int argc, const char *argv[])
{
  int i, iret, moving;
  signed char deploy_offset[] = {0, -100, 10, 40, 80, -55, 20};
  std::string str;
  ArSerialConnection con;
  ArRobot robot;
  ArP2Arm arm;

  // Initialize Aria
  Aria::init();

  // Setup the connection to the robot
  if ((iret = con.open()) != 0)
  {
    str = con.getOpenMessage(iret);
    printf("Open failed: %s\n", str.c_str());
    Aria::shutdown();
    return 1;
  }
  robot.setDeviceConnection(&con);

  // try to connect, if we fail exit
  if (!robot.blockingConnect())
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    return 1;
  }

  // Start the robot loop so we can process arm packets
  robot.runAsync(true);
  ArUtil::sleep(100);
  // turn off sonar
  robot.comInt(28, 0);
  ArUtil::sleep(100);

  // Set up and initialize the arm
  arm.setRobot(&robot);
  if (arm.init() != ArP2Arm::SUCCESS)
  {
    printf("Arm initialization failed.\n");
    return(1);
  }

  // Initialization gets ARMINFOpac
  // Print out some of the settings
  P2ArmJoint *joint;
  printf("Joint   Vel  Home  Center\n");
  for (i=1; i<=ArP2Arm::NumJoints; i++)
    {
      joint = arm.getJoint(i);
      printf("  %2i:  %5i %5i   %5i\n", i, joint->myVel, joint->myHome, joint->myCenter);
    }

  // Put the arm to work
  printf("Power on takes a couple seconds to stabilize.\n");
  arm.powerOn();

  // Request one status packet and print out the arm's status
  arm.requestStatus(ArP2Arm::StatusSingle);
  ArUtil::sleep(100);  // Give time to get the packet
  printf("Arm Status: ");
  if (arm.getStatus() & ArP2Arm::ArmGood)
    printf("Good=1 ");
  else
    printf("Good=0 ");
  if (arm.getStatus() & ArP2Arm::ArmInited)
    printf("Inited=1 ");
  else
    printf("Inited=0 ");
  if (arm.getStatus() & ArP2Arm::ArmPower)
    printf("Power=1 ");
  else
    printf("Power=0 ");
  if (arm.getStatus() & ArP2Arm::ArmHoming)
    printf("Homing=1 ");
  else
    printf("Homing=0 ");
  printf("\n");

  // Move the arm joints
  for (i=1; i<=ArP2Arm::NumJoints; i++)
    {
      joint = arm.getJoint(i);
      arm.moveToTicks(i, joint->myCenter + deploy_offset[i]);
    }

  // Wait for arm to achieve new position
  arm.requestStatus(ArP2Arm::StatusContinuous);
  ArUtil::sleep(100); // wait a moment for arm status packet update
  moving = 1;
  while (moving)
    {
      moving = 0;
      printf("Moving joints= ");
      for (i=1; i<=ArP2Arm::NumJoints; i++)
	{
	  if (arm.getMoving(i))
	    {
	      printf("1 ");
	      moving = 1;
	    }
	  else
	    printf("0 ");
	}
      printf("\r");
    }
  printf("\n");

  // Return arm to home and disconnect.
  printf("Going home. Bye!\n");
  arm.park();
  printf("Arm automatically parks on client disconnect, too.\n");
  i = 5;
  while (arm.getStatus() & ArP2Arm::ArmPower)
    {
      // Give it 5 seconds
      ArUtil::sleep(1000);
      if (i-- == 0)
	break;
    }
  Aria::shutdown();

  return(0);
}

