/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"


int main(int argc, char** argv)
{
  // the serial connection (robot)
  ArSerialConnection serConn;
  // tcp connection (sim)
  ArTcpConnection tcpConn;
  // robot
  ArRobot robot;
  // a key handler so we can do our key handling
  ArKeyHandler keyHandler;
  // sonar, must be added to the robot, for teleop and wander
  ArSonarDevice sonarDev;
  // if we're using the sim or not
  bool usingSim;
  // if we're connecting via tcp (not to the sim), what remote host
  char *remoteHost;
  // robot port, if there isn't one this'll be NULL, which will just
  // be the default of ArUtil::COM1
  const char *robotPort;
  // laser port, if there isn't one this'll be NULL, which will just
  // be the default of ArUtil::COM3
  const char *laserPort;
  // laser tcp port if we're doing a remote host
  char *remoteLaserTcpPortStr;
  int remoteLaserTcpPort;
  // robot tcp port if we're doing a remote host (defaults to 8101)
  char *remoteRobotTcpPortStr;
  int remoteRobotTcpPort;
  
  // Lets parse some arguments
  ArArgumentParser parser(&argc, argv);

  robotPort = parser.checkParameterArgument("--robotPort");
  if (robotPort == NULL)
    robotPort = ArUtil::COM1;
  laserPort = parser.checkParameterArgument("--laserPort");
  if (laserPort == NULL)
    laserPort = NULL;
  remoteHost = parser.checkParameterArgument("--remoteHost");
  remoteRobotTcpPortStr =parser.checkParameterArgument("--remoteRobotTcpPort");
  if (remoteRobotTcpPortStr == NULL)
    remoteRobotTcpPort = 8101;
  else
    remoteRobotTcpPort = atoi(remoteRobotTcpPortStr);
  remoteLaserTcpPortStr =parser.checkParameterArgument("--remoteLaserTcpPort");
  if (remoteLaserTcpPortStr == NULL)
    remoteLaserTcpPort = 8102;
  else
    remoteLaserTcpPort = atoi(remoteLaserTcpPortStr);
  
  if (argc > 1)
  {    
    printf("Usage: %s [--robotPort <robotSerialPort>] [--laserPort <laserSerialPort>] [--remoteHost <remoteHostNameOrIP>] [--remoteRobotTcpPort <remoteRobotTcpPort>] [--remoteLaserTcpPort <remoteLaserTcpPort>]\n", argv[0]);
    keyHandler.restore();
    exit(1);
  }

  // mandatory init
  Aria::init();
  //ArLog::init(ArLog::StdOut, ArLog::Verbose);

  // let the global aria stuff know about it
  Aria::setKeyHandler(&keyHandler);
  // toss it on the robot
  robot.attachKeyHandler(&keyHandler);

  // First we see if we can open the tcp connection, if we can we'll
  // assume we're connecting to the sim, and just go on...  if we
  // can't open the tcp it means the sim isn't there, so just try the
  // robot

  // see if we're doing remote host or not
  if (remoteHost != NULL)
    tcpConn.setPort(remoteHost, remoteRobotTcpPort);
  else
    tcpConn.setPort();

  // see if we can get to the simulator  (true is success)
  if (tcpConn.openSimple())
  {
    robot.setDeviceConnection(&tcpConn);
    // we could get to the sim, so set the robots device connection to the sim
    if (remoteHost != NULL)
    {
      printf("Connected to remote host %s through tcp.\n", remoteHost);
      usingSim = false;
    }
    else
    {
      printf("Connecting to simulator through tcp.\n");
      usingSim = true;
    }
  }
  else
  {
    // if we were trying for a remote host and it failed, just exit
    if (remoteHost != NULL)
    {
      printf("Could not connect to remote host %s, exiting.\n", remoteHost);
      keyHandler.restore();
      exit(1);
    }
    // we couldn't get to the sim, so set the port on the serial
    // connection and then set the serial connection as the robots
    // device

    serConn.setPort(robotPort);
    printf(
      "Could not connect to simulator, connecting to robot through serial port %s.\n", robotPort);
    robot.setDeviceConnection(&serConn);
    usingSim = false;
  }

  // add the sonar to the robot
  robot.addRangeDevice(&sonarDev);
  
  // try to connect, if we fail exit
  if (!robot.blockingConnect())
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    keyHandler.restore();
    return 1;
  }

  ArModeTeleop teleop(&robot, "teleop", 't', 'T');
  ArModeWander wander(&robot, "wander", 'w', 'W');
  ArModeGripper gripper(&robot, "gripper", 'g', 'G');
  ArModeCamera camera(&robot, "camera", 'c', 'C');
  ArModeSonar sonar(&robot, "sonar", 's', 'S');
  ArModeBumps bumps(&robot, "bumps", 'b', 'B');
  ArModePosition position(&robot, "position", 'p', 'P');
  ArModeIO io(&robot, "io", 'i', 'I');
  ArModeLaser laser(&robot, "laser", 'l', 'L', usingSim, remoteHost, 
		    remoteLaserTcpPort, laserPort);

  teleop.activate();
  // turn on the motors, turn off amigobot sounds
  robot.comInt(ArCommands::ENABLE, 1);
  //robot.comInt(ArCommands::SOUNDTOG, 0);

  // start the robot running, true so that if we lose connection the run stops
  robot.run(true);
  
  // now exit
  Aria::shutdown();
  return 0;


}

