/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

/*
  This is a simple example of how to use Aria functors. For a full example
  of how to use the functors refer to functortest.cpp in the tests directory.
*/


#include <string>
#include "Aria.h"


/*
  This is the class that has some callbacks. Functors which refer to these
  callbacks will be passed to the DriverClass.
*/
class CallbackClass
{
public:

  void callback1();
  void callback2(int i);
  bool callback3(std::string str);
};

void CallbackClass::callback1()
{
  printf("Invoking callback1\n");
}

void CallbackClass::callback2(int i)
{
  printf("Invoking callback2 with argument of '%d'\n", i);
}

bool CallbackClass::callback3(std::string str)
{
  printf("Invoking callback3 with argument of '%s'\n", str.c_str());
  return(true);
}


/*
  This is the driver class. It takes three functors of different types and
  will invoke the three functors. This is an example of the typical use of
  functors without all the extraneous code that actualy uses the functors.
*/
class DriverClass
{
public:

  void invoke();

  void setCallback1(ArFunctor *func) {myFunc1=func;}
  void setCallback2(ArFunctor1<int> *func) {myFunc2=func;}
  void setCallback3(ArRetFunctor1<bool, std::string> *func) {myFunc3=func;}

protected:

  ArFunctor *myFunc1;
  ArFunctor1<int> *myFunc2;
  ArRetFunctor1<bool, std::string> *myFunc3;
};

void DriverClass::invoke()
{
  bool ret;

  printf("Invoking functor1...\n");
  myFunc1->invoke();

  printf("Invoking functor2...\n");
  myFunc2->invoke(213);

  /*
     For functors with return values, because of the limitations of C++, the
     invoke() function cannot be used to get the return value. The invokeR()
     function must be used instead. This function is first declared in the
     ArRetFunctor base class. The invoke() function can be used to invoke
     the functor, but the return value is lost. This can be very bad if the
     the return value is a pointer to an allocated piece of memory.
  */
  printf("Invoking functor3...\n");
  ret=myFunc3->invokeR("This is a string argument");
  if (ret)
    printf("Functor3 returned 'true'\n");
  else
    printf("Functor3 returned 'false'\n");
}

/*
  Here is the main() which will instantiate a CallbackClass, a DriverClass,
  and functors for the three callback functions in CallbackClass.
*/
int main()
{
  CallbackClass cb;
  ArFunctorC<CallbackClass> functor1(cb, &CallbackClass::callback1);
  ArFunctor1C<CallbackClass, int> functor2(cb, &CallbackClass::callback2);
  ArRetFunctor1C<bool, CallbackClass, std::string>
    functor3(cb, &CallbackClass::callback3);
  DriverClass driver;

  driver.setCallback1(&functor1);
  driver.setCallback2(&functor2);
  driver.setCallback3(&functor3);

  driver.invoke();

  return(0);
}
