/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"

/*
  This is basically just a demo of how to use the AMPTU, but made fancy so
  you can drive around the robot and look at stuff with the camera.  Press
  button 1 to drive the robot and button two to move the gripper.
*/

/*
  This class is the core of this demo, it adds itself to the robot given
  as a user task, then drives the robot and camera from the joystick
*/
class Joydrive
{
public:
  // constructor
  Joydrive(ArRobot *robot);
  // destructor, its just empty
  ~Joydrive(void) {}

  // the callback function
  void drive(void);

protected:
  // the rotational max for the robot
  int myRotVal;
  // the translational max for the robot
  int myTransVal;
  // joystick handler
  ArJoyHandler myJoyHandler;
  // robot pointer
  ArRobot *myRobot;
  // gripper
  ArGripper myGripper;
  // functor callback for the drive function
  ArFunctorC<Joydrive> myDriveCB;

};

/*
  Constructor, sets the robot pointer, and some initial values, also note the
  use of constructor chaining on myGripper and myDriveCB.
*/
Joydrive::Joydrive(ArRobot *robot) :
  myGripper(robot),
  myDriveCB(this, &Joydrive::drive)
{
  // set the robot poitner and add the joydrive as a user task
  myRobot = robot;
  myRobot->addUserTask("joydrive", 50, &myDriveCB);

  // set some initial values
  myRotVal = 100;
  myTransVal = 700;
  myJoyHandler.setSpeeds(myRotVal, myTransVal);

  // initalize the joystick
  myJoyHandler.init();
  // see if we have a joystick, and let the user know the results
  if (myJoyHandler.haveJoystick())
  {
    printf("Have a joystick\n\n");
  }
  // we don't have a joystick so get out
  else
  {
    printf("Do not have a joystick, set up the joystick then rerun the program\n\n");
    Aria::shutdown();
    exit(0);
  }
}

// the important function
void Joydrive::drive(void)
{
  int trans, rot;
  int x, y;

  // get the values of the joystick
  myJoyHandler.getAdjusted(&rot, &trans);
  
  // if button one is pushed, drive the robot
  if (myJoyHandler.haveJoystick() && myJoyHandler.getButton(1))
  {
    myRobot->setVel(trans);
    myRobot->setRotVel(-rot);
  }
  // if button one isn't pushed, stop the robot
  else
  {
    myRobot->setVel(0);
    myRobot->setRotVel(0);
  }
  
  // if button two is pushed, drive the gripper
  if (myJoyHandler.haveJoystick() && myJoyHandler.getButton(2) && 
      myGripper.getType() != ArGripper::NOGRIPPER)
  {
    // just get the values again, calling x and y, so its more clear
    myJoyHandler.getAdjusted(&x, &y);
    // if y is above two third of its poseitive range, move the lift up
    if (y > myTransVal * .66)
      myGripper.liftUp();
    // if y is in the lower third of its negative range, move the lift down
    else if (y < -myTransVal * .66)
      myGripper.liftDown();
    // otherwise just stop the lift
    else
      myGripper.liftStop();
    // if x is in the greatest third of its poseitive range, open
    if (x > myRotVal * .66)
      myGripper.gripOpen();
    // if x is in the lowest third of its negative range, cloes
    else if (x < -myRotVal * .66)
      myGripper.gripClose();
    // otherwise stop the gripper
    else
      myGripper.gripStop();
  } 
  // if button two isn't pushed and there is a gripper, stop the gripper
  else if (myGripper.getType() != ArGripper::NOGRIPPER)
  {
    myGripper.gripperHalt();
  }
  // if we have a gripper, print its state out
  if (myGripper.getType() != ArGripper::NOGRIPPER)
    myGripper.printState();
}

int main(int argc, char **argv) 
{
  // just some stuff for returns
  std::string str;
  int ret;
  
  // robots connection
  ArSerialConnection con;
  // the robot, this turns state reflection off
  ArRobot robot(NULL, false);
  // the joydrive as defined above, this also adds itself as a user task
  Joydrive joyd(&robot);

  // mandatory init
  Aria::init();

  // open the connection, if it fails, exit
  if ((ret = con.open()) != 0)
  {
    str = con.getOpenMessage(ret);
    printf("Open failed: %s\n", str.c_str());
    Aria::shutdown();
    return 1;
  }

  // set the connection o nthe robot
  robot.setDeviceConnection(&con);
  // connect, if we fail, exit
  if (!robot.blockingConnect())
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    return 1;
  }

  // turn off the sonar, enable the motors, turn off amigobot sounds
  robot.comInt(ArCommands::SONAR, 0);
  robot.comInt(ArCommands::ENABLE, 1);
  robot.comInt(ArCommands::SOUNDTOG, 0);

  // run, if we lose connection to the robot, exit
  robot.run(true);
  
  // shutdown and go away
  Aria::shutdown();
  return 0;
}


