/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include <stdlib.h>
#include "Aria.h"


/*
  This example is to show to control access to something using a mutex.
  There are 4 threads running, each trying to print out their ID number.
  The critical thing that is being protected here is the right to do
  print something to the screen. This is just a simple example to show
  something that generaly shows up in complex code, if that code is not
  written correctly.

  Each thread will print half its line, sleep for 100ms, and then print
  the second half of its line. This is to simulate a work load since doing a
  single printf() takes virtualy no time at all.

  The Task class will run the correct way and the broken way depending on
  if its passed an ArMutex to do locking with. When the program is run, it
  first runs the correct code, then it runs the broken code. The differences
  are very obvious. The correct way each thread prints out 'task number N'
  and the threads do no step all over each other. This is because a mutex
  is used to protect the critical section.

  In the broken function, the crictical section is not protected by the
  mutex. Each thread will step over the other threads. The results are
  basicly random due to how the operating system schedules the threads. Also,
  the results can be very different between a single processor machine and
  a multi-processor machine.
*/


// The thread class
class Task : public ArASyncTask
{
public:

  Task(int num, ArMutex *mutex);
  virtual ~Task() {}

  // The run loop of the thread.
  void * runThread(void *arg);

  // The correct way to do it
  void runCorrect();

  // The broken way to do it
  void runBroken();

protected:

  int myNum;
  ArMutex *myMutex;
};


// Constructor
Task::Task(int num, ArMutex *mutex) :
  myNum(num),
  myMutex(mutex)
{
}

// This simply calls runCorrect() or runBroken() depending on if it has
// a mutex pointer.
void * Task::runThread(void *arg)
{
  if (myMutex)
    runCorrect();
  else
    runBroken();

  return(NULL);
}

// The correct way of printing out in a multi-threaded fashion. First lock
// the mutex that protects the screen, print, then unlock the mutex. Each
// of the four threads runs this same function.
void Task::runCorrect()
{
  int i;

  for (i=0; i<10; ++i)
  {
    myMutex->lock();
    printf("Task number");
    ArUtil::sleep(100);
    printf(" %d\n", myNum);
    myMutex->unlock();
  }
}

// The wrong way of printing out in a multi-threaded fashion. Just
// do the print without caring about the possiblity of other threads
// trying to do the same thing during the time that it needs to be printing.
// Each of the four threads runs this same function.
void Task::runBroken()
{
  int i;

  for (i=0; i<10; ++i)
  {
    printf("Broken task number");
    ArUtil::sleep(100);
    printf(" %d\n", myNum);
  }
}


int main()
{
  // The mutex which protects printing out to the screen
  ArMutex mutex;
  // The good tasks which have a mutex
  Task task1(1, &mutex);
  Task task2(2, &mutex);
  Task task3(3, &mutex);
  Task task4(4, &mutex);
  // The broken tasks, ones without a mutex
  Task brokenTask1(1, NULL);
  Task brokenTask2(2, NULL);
  Task brokenTask3(3, NULL);
  Task brokenTask4(4, NULL);

  // Initialize Aria, which in turn initializes the thread layer
  Aria::init();

  // Lets start all correct threads.
  task1.create();
  task2.create();
  task3.create();
  task4.create();

  // Lets wait for them to finish running. This will only return when all
  // currently running threads have exited.
  ArThread::joinAll();

  // Lets start all the broken threads.
  brokenTask1.create();
  brokenTask2.create();
  brokenTask3.create();
  brokenTask4.create();

  // Lets wait for them to finish running. This will only return when all
  // currently running threads have exited.
  ArThread::joinAll();

  // Uninit Aria
  Aria::uninit();

  return(0);
}
