/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"

/*
This program will let you joystick the robot around, and take logs for
the mapper while you drive, automatically.
 */
int main(int argc, char **argv)
{
  // whether to use the sim for the laser or not, if you use the sim
  // for hte laser, you have to use the sim for the robot too
  bool useSim = false;
  // robot
  ArRobot robot;
  // the laser
  ArSick sick;
  // connection
  ArDeviceConnection *con;
  // Laser connection
  ArSerialConnection laserCon;

  std::string filename = "1scans.2d";
  if (argc > 1)
    filename = argv[1];
  
  printf("Logging to file %s\n", filename.c_str());
  // configure the laser before we make the logger
  sick.configureShort(useSim, ArSick::BAUD38400, ArSick::DEGREES180, 
		      ArSick::INCREMENT_HALF);

  
  // mandatory init
  Aria::init();

  printf("Pausing 5 seconds so you can disconnect VNC if you are using it.\n");
  ArUtil::sleep(5000);
  // attach the laser to the robot
  robot.addRangeDevice(&sick);

  // if we're not using the sim, make a serial connection and set it up
  if (!useSim)
  {
    ArSerialConnection *serCon;
    serCon = new ArSerialConnection;
    serCon->setPort();
    con = serCon;
  }
  // if we are using the sim, set up a tcp connection
  else
  {
    ArTcpConnection *tcpCon;
    tcpCon = new ArTcpConnection;
    tcpCon->setPort();
    con = tcpCon;
  }

  if (ArModuleLoader::load("libArInertial", &robot, NULL, true) == 0)
  {
    printf("Loaded the base inertial library\n");
    if (ArModuleLoader::load("ISense_Mod", &robot, (void *)"2", true) == 0)
    {
      printf("The ISense inertial module has been loaded and should be correcting heading now.\n");
    }	
    else if (
    ArModuleLoader::load("ISIS_Mod", &robot,(void *)ArUtil::COM2, false) == 0)
    {
      printf("The ISIS inertial module has been loaded and should be correcting heading now.\n");
    }
  }
  // set the connection on the robot
  robot.setDeviceConnection(con);
  // try to connect, if we fail exit
  if (!robot.blockingConnect())
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    return 1;
  }
  
  // if we're not using the sim, set up the port for the laser
  if (!useSim)
  {
#ifdef WIN32
    laserCon.setPort("COM3");
#else
    laserCon.setPort("/dev/ttyS2");
#endif
    sick.setDeviceConnection(&laserCon);
  }

  // give p2os a command to make it respond to the on-microcontroller
  // joystick port
  robot.comInt(ArCommands::JOYDRIVE, 1);

  // This must be created after the robot is connected so that it'll
  // get the right laser pos
  ArSickLogger logger(&robot, &sick, 300, 25, filename.c_str());

  sick.setSensorPosition(0,0,0);


  // add a keydrive action so that the robot can be driven with a
  // keyboard as well, toss in a joystick one too, just in case
  // someone does get a joystick working on a robot somehow

  ArActionJoydrive joydrive;
  joydrive.setStopIfNoButtonPressed(false);
  
  ArActionKeydrive keydrive;

  robot.addAction(&joydrive, 100);
  robot.addAction(&keydrive, 99);

  // run the robot, true here so that the run will exit if connection lost
  robot.runAsync(true);


  // now that we're connected to the robot, connect to the laser
  sick.runAsync();

  // connect to the laser
  if (!sick.blockingConnect())
  {
    printf("Could not connect to SICK laser... exiting\n");
    robot.comInt(ArCommands::SOUND, 13);
    Aria::shutdown();
    return 1;
  }


  // enable the motors, disable amigobot sounds
  robot.comInt(ArCommands::SONAR, 0);
  robot.comInt(ArCommands::ENABLE, 1);
  robot.comInt(ArCommands::SOUND, 32);
  robot.comInt(ArCommands::SOUNDTOG, 0);
  robot.comInt(ArCommands::JOYDRIVE, 1);
  // just hang out and wait for the end
  robot.waitForRunExit();
  sick.lock();
  sick.disconnect();
  //sick.setRobot(NULL);
  sick.unlock();
  // now exit
  Aria::shutdown();
  return 0;
}
