/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "Aria.h"

/*
  This programs purpose is to demonstrate how to use ArSimpleUserTask.

  This program will just have the robot wander around, it uses some avoidance 
  routines, then just has a constant velocity.  The programs
*/

class PrintingTask
{
public:
  // the constructor, it must use constructor chaining to intialize its base
  // class ArSimpleUserTask
  PrintingTask(ArRobot *robot);
  // empty destructor
  ~PrintingTask(void) {}
  
  // the task we want to do
  void doTask(void);
protected:
  ArRobot *myRobot;
  ArFunctorC<PrintingTask> myTaskCB;
};


// the constructor, note how it uses chaining to initialize the myTaskCB
PrintingTask::PrintingTask(ArRobot *robot) :
  myTaskCB(this, &PrintingTask::doTask)
{
  myRobot = robot;
  // just add it to the robot
  myRobot->addUserTask("PrintingTask", 50, &myTaskCB);
}

void PrintingTask::doTask(void)
{
  // print out some info about hte robot
  printf("\rx %6.1f  y %6.1f  tth  %6.1f vel %7.1f mpacs %3d", myRobot->getX(),
	 myRobot->getY(), myRobot->getTh(), myRobot->getVel(), 
	 myRobot->getMotorPacCount());
  fflush(stdout);
}

int main(void)
{
  int ret;
  std::string str;

  // the connection
  ArTcpConnection con;

  // robot
  ArRobot robot;
  // sonar, msut be added to the robot
  ArSonarDevice sonar;

  // make the simple task, which'll add it to the robot
  PrintingTask pt(&robot);

  // the actions we'll use to wander
  ArActionStallRecover recover;
  ArActionAvoidFront avoidFront;
  ArActionConstantVelocity constantVelocity("Constant Velocity", 400);

  // mandatory init
  Aria::init();

  // open the connection, if this fails exit
  if ((ret = con.open()) != 0)
  {
    str = con.getOpenMessage(ret);
    printf("Open failed: %s\n", str.c_str());
    Aria::shutdown();
    return 1;
  }
  
  // add the sonar to the robot
  robot.addRangeDevice(&sonar);
  
  // set the device connection on the robot
  robot.setDeviceConnection(&con);
  
  // try to connect, if we fail exit
  if (!robot.blockingConnect())
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    return 1;
  }

  // turn on the motors, turn off amigobot sounds
  robot.comInt(ArCommands::ENABLE, 1);
  robot.comInt(ArCommands::SOUNDTOG, 0);

  // add the actions
  robot.addAction(&recover, 100);
  robot.addAction(&avoidFront, 50);
  robot.addAction(&constantVelocity, 25);
  
  // start the robot running, true so that if we lose connection the run stops
  robot.run(true);
  
  // now exit
  Aria::shutdown();
  return 0;
}
