/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARACTIONDESIRED_H
#define ARACTIONDESIRED_H

#include "ariaUtil.h"

/// Class used by ArActionDesired for each channel, internal
class ArActionDesiredChannel
{
public:
  AREXPORT const static double NO_STRENGTH;
  AREXPORT const static double MIN_STRENGTH;
  AREXPORT const static double MAX_STRENGTH;
  
  ArActionDesiredChannel(void) { reset(); }
  ~ArActionDesiredChannel(void) {}
  void setDesired(double desired, double desiredStrength) 
    {
      myDesired = desired;
      myStrength = desiredStrength; 
      if (myStrength > MAX_STRENGTH)
	myStrength = MAX_STRENGTH;
      if (myStrength < MIN_STRENGTH)
	myStrength = NO_STRENGTH;
    }
  double getDesired(void)  { return myDesired; }
  double getStrength(void)  { return myStrength; }
  void reset(void) { myDesired = 0; myStrength = NO_STRENGTH;}
  void merge(ArActionDesiredChannel *desiredChannel)
    {
      double otherStrength = desiredChannel->getStrength();
      double oldStrength = myStrength;
      if (myStrength + otherStrength > MAX_STRENGTH)
	otherStrength = MAX_STRENGTH - myStrength;
      myStrength = myStrength + otherStrength;
      if (myStrength > MIN_STRENGTH)
	myDesired = (((oldStrength * myDesired) + 
		      (desiredChannel->getDesired() * otherStrength)) 
		     / (myStrength));
    }
  void startAverage(void)
    {
      myDesiredTotal = myDesired * myStrength;
      myStrengthTotal = myStrength;
    }
  void addAverage(ArActionDesiredChannel *desiredChannel)
    {
      myDesiredTotal += (desiredChannel->getDesired() * 
			 desiredChannel->getStrength());
      myStrengthTotal += desiredChannel->getStrength();
    }
  void endAverage(void)
    {
      if (myStrengthTotal < MIN_STRENGTH)
      {
	myStrength = NO_STRENGTH;
	return;
      }
      myDesired = (myDesiredTotal / myStrengthTotal);
      myStrength = myStrengthTotal;
      if (myStrength > MAX_STRENGTH)
	myStrength = MAX_STRENGTH;
    }
protected:
  double myDesired;
  double myStrength;
  double myDesiredTotal;
  double myStrengthTotal;
};

/// Class used to say what movement is desired
/**
   This class is use by actions to report what they want to want to do (hence
   the name). 

   The way it works, is that translational (front/back) and rotational
   (right/left) are seperate.  Translational movement uses velocity, while 
   rotational movement uses change in heading from current heading. 
   Translational and rotational each have their own strength value.  
   Both translational and rotational movement have maximum velocities as well,
   that also have their own strengths.

   The strength value reflects how strongly an action wants to do the chosen 
   movement command, the resolver (ArResolver) will combine these strengths 
   and figure out what to do based on them.

   For all strength values there is a total of 1.0 strength to be had.
   The range for strength is from 0 to 1.  This is simply a convention that 
   ARIA uses by default, if you don't like it, you can override this
   class and make an ArResolver.
*/
class ArActionDesired
{
public:
  AREXPORT const static double NO_STRENGTH;
  AREXPORT const static double MIN_STRENGTH;
  AREXPORT const static double MAX_STRENGTH;
  /// Constructor
  ArActionDesired(void) { myHeadingSet = false; }
  /// Destructor
  virtual ~ArActionDesired(void) {}
  
  /// Sets the velocity (mm/sec) and strength
  /**
     @param vel desired vel (mm/sec)
     @param strength strength given to this, defaults to MAX_STRENGTH (1.0)
  */
  virtual void setVel(double vel, double strength = MAX_STRENGTH)
    { myVelDes.setDesired(vel, strength); }
  /// Sets the delta heading (deg) and strength
  /**
     @param deltaHeading desired change in heading (deg)
     @param strength strength given to this, defaults to MAX_STRENGTH (1.0)
  */
  virtual void setDeltaHeading(double deltaHeading, double strength = MAX_STRENGTH)
    { myDeltaHeadingDes.setDesired(deltaHeading, strength); }
  /// Sets the absolute heading (deg) 
  /**
     This is a way to set the heading instead of using a delta, there is no
     get for this, because accountForRobotHeading MUST be called (this should
     be called by all resolvers, but if you want to call it you can,
     thats fine).
     @param heading desired heading (deg)
     @param strength strength given to this, defaults to MAX_STRENGTH (1.0)
  */
  virtual void setHeading(double heading, double strength = MAX_STRENGTH)
    { myHeading = heading; myHeadingStrength = strength; myHeadingSet = true; }
  /// Sets the maximum velocity (+mm/sec) and strength
  /**
     @param maxVel desired maximum velocity (+mm/sec)
     @param strength strength given to this, defaults to MAX_STRENGTH (1.0)
  */
  virtual void setMaxVel(double maxVel, double strength = MAX_STRENGTH)
    { myMaxVelDes.setDesired(maxVel, strength); }
  /// Sets the maximum velocity for going backwards (-mm/sec) and strength
  /**
     @param maxVel desired maximum velocity for going backwards (-mm/sec)
     @param strength strength given to this, defaults to MAX_STRENGTH (1.0)
  */
  virtual void setMaxNegVel(double maxVel, double strength = MAX_STRENGTH)
    { myMaxNegVelDes.setDesired(maxVel, strength); }
  /// Sets the maximum rotational velocity (deg/sec) and strength
  /**
     @param maxVel desired maximum rotational velocity (deg/sec)
     @param strength strength given to this, defaults to MAX_STRENGTH (1.0)
  */
  virtual void setMaxRotVel(double maxVel, double strength = MAX_STRENGTH)
    { myMaxRotVelDes.setDesired(maxVel, strength); }


  /// Resets the strengths to 0
  virtual void reset(void) 
    {
      myVelDes.reset(); myDeltaHeadingDes.reset(); 
      myMaxVelDes.reset(); myMaxNegVelDes.reset(); myMaxRotVelDes.reset(); 
      myHeadingSet = false;
    }

  /// Gets the translational velocity desired (mm/sec)
  virtual double getVel(void)  
    { return myVelDes.getDesired(); }
  /// Gets the strength of the translational velocity desired
  virtual double getVelStrength(void) 
    { return myVelDes.getStrength(); }
  /// Gets the heading desired (deg)
  virtual double getHeading(void)
    { return myHeading; }
  /// Gets the strength of the heading desired
  virtual double getHeadingStrength(void)
    { return myHeadingStrength; }

  /// Gets the delta heading desired (deg)
  virtual double getDeltaHeading(void)
    { return myDeltaHeadingDes.getDesired(); }
  /// Gets the strength of the delta heading desired
  virtual double getDeltaHeadingStrength(void)
    { return myDeltaHeadingDes.getStrength(); }
  /// Gets the desired maximum velocity (mm/sec)
  virtual double getMaxVel(void)
    { return myMaxVelDes.getDesired(); }
  /// Gets the maximum velocity strength
  virtual double getMaxVelStrength(void)
    { return myMaxVelDes.getStrength(); }
  /// Gets the desired maximum negative velocity (-mm/sec)
  virtual double getMaxNegVel(void)
    { return myMaxNegVelDes.getDesired(); }
  /// Gets the desired maximum negative velocity strength
  virtual double getMaxNegVelStrength(void)
    { return myMaxNegVelDes.getStrength(); }
  /// Gets the maximum rotational velocity
  virtual double getMaxRotVel(void)
    { return myMaxRotVelDes.getDesired(); }
  /// Gets the maximum rotational velocity strength
  virtual double getMaxRotVelStrength(void)
    { return myMaxRotVelDes.getStrength(); }
  /// Merges the given ArActionDesired into this one (this one has precedence),
  /// internal
  /** 
      This merges in the two different action values, accountForRobotHeading
      MUST be done before this is called (on both actions), since this merges
      their delta headings, and the deltas can't be known unless the account
      for angle is done.
      @param actDesired the actionDesired to merge with this one
  */
  virtual void merge(ArActionDesired *actDesired)
    {
      if (actDesired == NULL)
	return;
      myVelDes.merge(actDesired->getVelDesiredChannel());
      myDeltaHeadingDes.merge(actDesired->getDeltaHeadingDesiredChannel());
      myMaxVelDes.merge(actDesired->getMaxVelDesiredChannel());
      myMaxNegVelDes.merge(actDesired->getMaxNegVelDesiredChannel());
      myMaxRotVelDes.merge(actDesired->getMaxRotVelDesiredChannel());
    }
  /// Starts the process of avereraging together different desireds
  /**
     There is a three step process for averaging actionDesireds together,
     first startAverage must be done to set up the process, then addAverage
     must be done with each average that is desired, then finally endAverage
     should be used, after that is done then the normal process of getting
     the results out should be done.
  */
  virtual void startAverage(void)
    {
      myVelDes.startAverage();
      myDeltaHeadingDes.startAverage();
      myMaxVelDes.startAverage();
      myMaxNegVelDes.startAverage();
      myMaxRotVelDes.startAverage();
    }
  /// Adds another actionDesired into the mix to average
  /**
     For a description of how to use this, see startAverage.
     @param actDesired the actionDesired to add into the average
  */
  virtual void addAverage(ArActionDesired *actDesired)
    {
      if (actDesired == NULL)
	return;
      myVelDes.addAverage(actDesired->getVelDesiredChannel());
      myDeltaHeadingDes.addAverage(
	      actDesired->getDeltaHeadingDesiredChannel());
      myMaxVelDes.addAverage(actDesired->getMaxVelDesiredChannel());
      myMaxNegVelDes.addAverage(actDesired->getMaxNegVelDesiredChannel());
      myMaxRotVelDes.addAverage(actDesired->getMaxRotVelDesiredChannel());
    }
  /// Ends the process of avereraging together different desireds
  /**
     For a description of how to use this, see startAverage.
  */
  virtual void endAverage(void)
    {
      myVelDes.endAverage();
      myDeltaHeadingDes.endAverage();
      myMaxVelDes.endAverage();
      myMaxNegVelDes.endAverage();
      myMaxRotVelDes.endAverage();
    }
  /// Accounts for robot heading, mostly internal
  /**
     This accounts for the robots heading, and transforms the set heading
     on this actionDesired into a delta heading so it can be merged and 
     averaged and the like
     @param robotHeading the heading the real actual robot is at now
   */
  virtual void accountForRobotHeading(double robotHeading)
    {
      if (myHeadingSet)
	setDeltaHeading(ArMath::subAngle(myHeading, robotHeading), 
			myHeadingStrength);
      myHeadingSet = false;
    }
  /// Accessor for the channel structor for merge, internal
  ArActionDesiredChannel *getVelDesiredChannel(void) 
    { return &myVelDes; }
  /// Accessor for the channel structor for merge, internal
  ArActionDesiredChannel *getDeltaHeadingDesiredChannel(void) 
    { return &myDeltaHeadingDes; }
  /// Accessor for the channel structor for merge, internal
  ArActionDesiredChannel *getMaxVelDesiredChannel(void) 
    { return &myMaxVelDes; }
  /// Accessor for the channel structor for merge, internal
  ArActionDesiredChannel *getMaxNegVelDesiredChannel(void) 
    { return &myMaxNegVelDes; }
  /// Accessor for the channel structor for merge, internal
  ArActionDesiredChannel *getMaxRotVelDesiredChannel(void) 
    { return &myMaxRotVelDes; }

protected:
  double myHeading;
  double myHeadingStrength;
  bool myHeadingSet;
  ArActionDesiredChannel myVelDes;
  ArActionDesiredChannel myDeltaHeadingDes;
  ArActionDesiredChannel myMaxVelDes;
  ArActionDesiredChannel myMaxNegVelDes;
  ArActionDesiredChannel myMaxRotVelDes;
};


#endif
