/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARACTIONGROUP_H
#define ARACTIONGROUP_H

#include "ariaTypedefs.h"

#include <list>

class ArRobot;
class ArAction;

/// Class for groups of actions to accomplish one thing
/**
   This class is used to have a group of ArActions and turn them on
   and off in aggregate... this is so that you can say have a group of
   like 5 behaviors for teleop or wander, and just turn 'em all on and
   off at once.  Note that the destructor by default will delete the
   actions added to the group, this is controlled with a flag to the
   constructor though, so you can have it how you want.... this is
   nice though so you can just do addAction(new ArActionWhatever(blah,
   blah, blah), 90); and not worry about the deletion (since the
   destructor will do it), just delete the group... if for some reason
   (I'd advise against it) you are using one action in multiple
   groups, don't use this feature, ie pass in false to the constructor
   for it or you'll wind up with a crash when the action is deleted by
   both groups (again, you should probably only have an action in one
   group).
**/
class ArActionGroup
{
public:
  /// Constructor
  AREXPORT ArActionGroup(ArRobot * robot);
  /// Destructor, it also deletes the actions in its group
  AREXPORT virtual ~ArActionGroup(void);
  /// Adds the action to the robot this group uses with the given priority
  AREXPORT virtual void addAction(ArAction *action, int priority);
  /// Removes the action from the robot this group uses
  AREXPORT virtual void remAction(ArAction *action);
  /// Activates all the actions in this group
  AREXPORT virtual void activate(void);
  /// Activates all the actions in this group and deactivates all others
  AREXPORT virtual void activateExclusive(void);
  /// Deactivates all the actions in this group
  AREXPORT virtual void deactivate(void);
  /// Removes all the actions in this group from the robot
  AREXPORT virtual void removeActions(void);
  /// Gets the action list (use this to delete actions after doing removeActions)
  AREXPORT virtual std::list<ArAction *> *getActionList(void);
protected:
  std::list<ArAction *> myActions;
  ArRobot *myRobot;
};

#endif // ARACTIONGROUP_H
