/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARARG_H
#define ARARG_H

#include "ariaTypedefs.h"
#include "ariaUtil.h"
#include <string>

/// Argument class, mostly for actions, could be used for other things
/** 
    This is designed to be easy to add another type to the arguments... 
    All you have to do to do so, is add an enum to the Type enum, add a 
    newType getNewType(void), add a void setNewType(newType nt), and add
    a case statement for the newType to ArArg::print.  You should probably also
    add an @see newType to the documentation for ArArg::getType.

*/
class ArArg
{
public:
  typedef enum 
  { 
    INVALID, ///< An invalid argument, the argument wasn't created correctly
    INT, ///< Integer argument
    DOUBLE, ///< Double argument
    STRING, ///< String argument
    BOOL, ///< Boolean argument
    POSE, ///< ArPose argument
  } Type;
  /// Default empty contructor
  AREXPORT ArArg(void);
  /// Constructor for making an integer argument
  AREXPORT ArArg(std::string name, int *pointer, 
		 std::string description = ""); 
  /// Constructor for making a double argument
  AREXPORT ArArg(std::string name, double *pointer,
		 std::string description = ""); 
  /// Constructor for making a string argument
  AREXPORT ArArg(std::string name, std::string *pointer,
		 std::string description = ""); 
  /// Constructor for making a boolean argument
  AREXPORT ArArg(std::string name, bool *pointer,
		 std::string description = ""); 
  /// Constructor for making a position argument
  AREXPORT ArArg(std::string name, ArPose *pointer,
		 std::string description = ""); 
  /// Copy constructor
  AREXPORT ArArg(const ArArg & arg);
  /// Destructor
  AREXPORT virtual ~ArArg(void);
  /// Gets the type of the argument
  AREXPORT Type getType(void);
  /// Gets the name of the argument
  AREXPORT std::string getName(void);
  /// Gets the long description of the argument
  AREXPORT std::string getDescription(void);
  /// Sets the argument value, for int arguments
  AREXPORT void setInt(int val);
  /// Sets the argument value, for double arguments
  AREXPORT void setDouble(double val);
  /// Sets the argument value, for string arguments
  AREXPORT void setString(std::string str);
  /// Sets the argument value, for bool arguments
  AREXPORT void setBool(bool val);
  /// Sets the argument value, for ArPose arguments
  AREXPORT void setPose(ArPose pose);
  /// Gets the argument value, for int arguments
  AREXPORT int getInt(void); 
  /// Gets the argument value, for double arguments
  AREXPORT double getDouble(void);
  /// Gets the argument value, for string arguments
  AREXPORT std::string getString(void);
  /// Gets the argument value, for bool arguments
  AREXPORT bool getBool(void);
  /// Gets the argument value, for pose arguments
  AREXPORT ArPose getPose(void);
  /// Logs the type, name, and value of this argument
  AREXPORT void print(void);
  /// Internal helper function
  AREXPORT void clearPointers(void);
protected:

  ArArg::Type myType;
  std::string myName;
  std::string myDescription;
  int *myIntPointer;
  double *myDoublePointer;
  std::string *myStringPointer;
  bool *myBoolPointer;
  ArPose *myPosePointer;
};

#endif // ARARGUMENT_H
