/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARJOYHANDLER_H
#define ARJOYHANDLER_H

#include "ariaTypedefs.h"

#ifdef WIN32
#include <mmsystem.h>
#endif // WIN32

#ifdef linux
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/joystick.h>
#endif


/// Interfaces to a joystick
/** 
    The joystick handler keeps track of the minimum and maximums for both
    axes, updating them to constantly be better calibrated.  The speeds set 
    influence what is returned by getAdjusted...
    
    The joystick is not opened until init is called.  What should basically 
    be done to use this class is to 'init' a joystick, do a 'setSpeed' so you 
    can use 'getAdusted', then at some point do a 'getButton' to see if a
    button is pressed, and then do a 'getAdjusted' to get the values to act
    on.
    
    Also note that x is usually rotational velocity (since it right/left),
    whereas Y is translational (since it is up/down).
    
    You can also use this to do multiple uses with the joystick, for example to
    have button 1 drive the robot while to have button 2 move the camera, 
    you can get the different values you want (don't want to move the camera
    as quickly or as far as the robot) by using setSpeed before doing 
    getAdjusted since setSpeed is fast and won't take any time.
*/
class ArJoyHandler
{
 public:
  /// Constructor
  AREXPORT ArJoyHandler(bool useOSCal = true);
  /// Destructor
  AREXPORT ~ArJoyHandler(void);
  /// Intializes the joystick, returns true if successful
  AREXPORT bool init(void);
  /// Sets whether to just use OS calibration or not
  AREXPORT void setUseOSCal(bool useOSCal);
  /// Gets whether to just use OS calibration or not
  AREXPORT bool getUseOSCal(void);
  /// Returns if the joystick was successfully initialized or not
  AREXPORT bool haveJoystick(void) { return myInitialized; }
  /// Sets the max that X or Y will return
  AREXPORT void setSpeeds(int x, int y) { myTopX = x; myTopY = y; }
  /// Gets the adjusted reading, as integers, based on the setSpeed
  AREXPORT void getAdjusted(int *x, int *y);
  /// Gets the adjusted reading, as floats, between -1.0 and 1.0
  AREXPORT void getDoubles(double *x, double *y);
  /// Checkes whether the given button is pressed or not
  AREXPORT bool getButton(int button);

  /// Starts the calibration process
  AREXPORT void startCal(void);
  /// Ends the calibration process
  AREXPORT void endCal(void);
  /// Gets the unfilitered reading, mostly for internal use, maybe
  /// useful for Calibration
  AREXPORT void getUnfiltered(int *x, int *y);
  /// Gets the stats for the joystick, useful after calibrating to save values
  AREXPORT void getStats(int *maxX, int *minX, int *maxY, int*minY, 
		 int *cenX, int *cenY);
  /// Sets the stats for the joystick, useful for restoring calibrated settings
  AREXPORT void setStats(int maxX, int minX, int maxY, int minY, 
		int cenX, int cenY);
  /// Gets the speeds that X and Y are set to
  AREXPORT void getSpeeds(int *x, int *y);

 protected:
  int myMaxX, myMinX, myMaxY, myMinY, myCenX, myCenY, myTopX, myTopY;
  int myPhysMax;
  bool myInitialized;
  bool myUseOSCal;
#ifdef WIN32
  JOYINFO myJoyInfo;
  JOYCAPS myJoyCaps;
#endif // WIN32
#ifdef linux
  struct JS_DATA_TYPE myJoyData; // structure for the buttons and x,y coords
  FILE * myJoyDesc;
#endif // linux
};


#endif // ARJOYHANDLER_H

