/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARKEYHANDLER_H
#define ARKEYHANDLER_H

#include "ariaTypedefs.h"
#include "ArFunctor.h"
#include <map>

#ifdef linux
#include <termios.h>
#include <unistd.h>
#endif

class ArRobot;

/// This class will read input from the keyboard
/**
   This class is for handling input from the keyboard, you just
   addKeyHandler the keys you want to deal with.

   You should also register the keyhandler with Aria::setKeyHandler,
   and before you create a key handler you should see if one is
   already there with Aria::getKeyHandler.
   
   You can attach a key handler to a robot with
   ArRobot::attachKeyHandler which will put a task into the robots
   list of tasks so that it'll get checked every cycle or you can just
   call checkKeys yourself (like in its own thread or in the main
   thread).  You should only attach a key handler to one robot, even
   if you're using multiple robots.
**/

class ArKeyHandler
{
public:
  /// Constructor
  AREXPORT ArKeyHandler(bool blocking = false);
  /// Destructor
  AREXPORT ~ArKeyHandler(void);

  /// These are enums for the non-ascii keys
  enum KEY {
    UP = 256, ///< Up arrow (keypad or 4 key dirs)
    DOWN, ///< Down arrow (keypad or 4 key dirs)
    LEFT, ///< Left arrow (keypad or 4 key dirs)
    RIGHT, ///< Right arrow (keypad or 4 key dirs)
    ESCAPE, ///< Escape key
    SPACE, ///< Space key
    TAB, ///< Tab key
    ENTER, ///< Enter key
    BACKSPACE, ///< Backspace key
    F1, ///< F1
    F2, ///< F2
    F3, ///< F3
    F4 ///< F4
  };

  /// This adds a keyhandler, when the keyToHandle is hit, functor will fire
  AREXPORT bool addKeyHandler(int keyToHandle, ArFunctor *functor);

  /// This removes a key handler, by key
  AREXPORT bool remKeyHandler(int keyToHandler);
  /// This removes a key handler, by key
  AREXPORT bool remKeyHandler(ArFunctor *functor);

  /// Sets stdin back to its original settings, if its been restored
  /// it won't read anymore
  AREXPORT void restore(void);

  /// intnernal, use addKeyHandler, Checks for keys and handles them
  AREXPORT void checkKeys(void);
  
  /// internal, use addKeyHandler instead... Gets a key from the stdin if ones
  /// available, -1 if there aren't any available
  AREXPORT int getKey(void);

protected:
  std::map<int, ArFunctor *> myMap;
  bool myBlocking;
  
  bool myRestored;
#ifdef linux
  struct termios myOriginalTermios;
#endif
};


#endif // ARKEYHANDLER_H
