/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARPREF_H
#define ARPREF_H


#include <string>
#include <list>
#include "ariaTypedefs.h"


class ArPreferences;


/// Preference instance. Used by ArPreferences
/**
   This represents an individual preference which is loaded from compiled in
   defaults of from a preferences file. A preference can be one of four
   different types: Integer, Double, Boolean, String. The preference itself
   is stored as a string. There are accessors which convert from string to
   the desired format and vice versa: getBool(), getInt(), getDouble(),
   getString(), setBool(), setInt(), setDouble(), setString(). A preference
   can also have a set of values of all the same type. In the file would look
   like:

   <key> <int> <int> <int> ...

   The 'set' accessors can deal with an abitrary amount of values:
   getBoolSet(), getIntSet(), getDoubleSet(), getStringSet(), setBoolSet(),
   setIntSet(), setDoubleSet().

   A preference can have an array of valid values. When the file is loaded,
   ArPreferences checks all the values from the file against the supplied
   valid values. The check is done with a string compare. It is most
   usefull for string values. So it will apply to numbers as long as they
   are formated in the correct way.
*/
class ArPref
{
public:

  typedef enum {
    Integer, ///< integer number value
    Double, ///< double number value
    Boolean, ///< boolean value, expressed as 'true' or 'false' in the file
    String ///< a string value
  } ValType;

  /// Constructor
  AREXPORT ArPref(int section, int pref, const char *name, const char *val,
		  const char **validVals, ValType valType,
		  const char *comment);
  /// Copy constructor
  AREXPORT ArPref(const ArPref &pref);
  /// Destructor
  AREXPORT virtual ~ArPref();

  ArPref * find(int id);

  // Functions to get/set a single value.

  /// Get the value as a boolean
  AREXPORT virtual bool getBool();
  /// Get the value as an integer
  AREXPORT virtual int getInt();
  /// Get the value as a double
  AREXPORT virtual double getDouble();
  /// Get the value as a std::string
  AREXPORT virtual std::string getString();

  /// Set the value to be the supplied boolean
  AREXPORT virtual bool setBool(bool val, bool append=false);
  /// Set the value to be the supplied integer
  AREXPORT virtual bool setInt(int val, bool append=false);
  /// Set the value to be the supplied doule
  AREXPORT virtual bool setDouble(double val, bool append=false);
  /// Set the value to be the supplied std::string
  AREXPORT virtual bool setString(const char *val, bool append=false);

  // Functions to get/set multiple values.

  /// Get the number of values that would be in the set regardless of type
  AREXPORT virtual int getSetCount();
  /// Get the value as multiple booleans
  AREXPORT virtual int getBoolSet(bool *boolArray, int size);
  /// Get the value as multiple integers
  AREXPORT virtual int getIntSet(int *intArray, int size);
  /// Get the value as multiple doubles
  AREXPORT virtual int getDoubleSet(double *doubleArray, int size);
  /// Get the value as multiple std::strings
  AREXPORT virtual int getStringSet(std::string *stringArray, int size);

  /// Set the value to be the supplied booleans
  AREXPORT virtual bool setBoolSet(bool append, int count, ...);
  /// Set the value to be the supplied integers
  AREXPORT virtual bool setIntSet(bool append, int count, ...);
  /// Set the value to be the supplied doubles
  AREXPORT virtual bool setDoubleSet(bool append, int count, ...);
  /* 
     Taking this out since it won't work anyways (string can't go through ...)
     If we need this someday use void *'s to chars or something
  /// Set the value to be the supplied std::strings
  AREXPORT virtual bool setStringSet(bool append, int count, ...);
  */
  // Get the index to the valid values array for this preference. Returns -1
  // for unfound.
  AREXPORT virtual int getValidIndex();

  // Set the preference by index into the valid value array.
  AREXPORT virtual bool setByValidIndex(int index, bool append=false);

  ArPreferences *myParent;
  int mySection;
  int myPref;
  std::string myName;
  std::string myVal;
  const char **myValidVals;
  ValType myValType;
  std::string myComment;
  ArPref *myNext;
  ArPref *myPrev;
};


#endif // ARPREF_H
