/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARRANGEDEVICE_H
#define ARRANGEDEVICE_H

#include "ariaTypedefs.h"
#include "ArRangeBuffer.h"
#include "ArSensorReading.h"
#include "ArMutex.h"

class ArRobot;

/// The class for all devices which return range info (laser, sonar)
/** 
    This class has two buffers, a current buffer for storing just recent 
    (relevant) readings, and a cumulative buffer for a longer history...
    the sizes of both can be set in the constructor.

    This class should be used for all sensors like lasers and sonar,
    also note that it has the locking functions for such a time when there
    is a device like a laser that runs in its own thread, so that every
    device can be locked and unlocked and the users don't have to worry
    about the detail, because of functions on the ArRobot structure which
    check all of the ArRangeDevice s attached to a robot.
*/

class ArRangeDevice
{
public:
  /// Constructor
  AREXPORT ArRangeDevice(size_t currentBufferSize, size_t cumulativeBufferSize,
			 const char *name, unsigned int maxRange);
  /// Destructor
  AREXPORT virtual ~ArRangeDevice(void);
  /// Gets the name of the device
  AREXPORT virtual std::string getName(void);
  /// Sets the robot this device is attached to
  AREXPORT virtual void setRobot(ArRobot *robot);
  /// Gets the robot this device is attached to
  AREXPORT virtual ArRobot *getRobot(void);
  /// Sets the size of the buffer for current readings
  AREXPORT virtual void setCurrentBufferSize(size_t size);
  /// Sets the size of the buffer for cumulative readings
  AREXPORT virtual void setCumulativeBufferSize(size_t size);
  /// Adds a reading to the buffer
  AREXPORT virtual void addReading(double x, double y);
  /// Gets the closest current reading in the given polar region
  AREXPORT virtual double currentReadingPolar(double startAngle, 
					      double endAngle,
					      double *angle = NULL);
  /// Gets the closest cumulative reading in the given polar region
  AREXPORT virtual double cumulativeReadingPolar(double startAngle, 
						 double endAngle,
						 double *angle = NULL);
  /// Gets the closest current reading from the given box region
  AREXPORT virtual double currentReadingBox(double x1, double y1, double x2,
					    double y2, 
					    ArPose *readingPos = NULL);
  /// Gets the closest current reading from the given box region
  AREXPORT virtual double cumulativeReadingBox(double x1, double y1, double x2,
					       double y2, 
					       ArPose *readingPos = NULL);
  /// Gets the current range buffer
  virtual ArRangeBuffer *getCurrentRangeBuffer(void)
    { return &myCurrentBuffer; }
  /// Gets the cumulative range buffer
  virtual ArRangeBuffer *getCumulativeRangeBuffer(void) 
    { return &myCumulativeBuffer; }
  /// Gets the current buffer of readings
  virtual std::list<ArPose *> *getCurrentBuffer(void)
    { return myCurrentBuffer.getBuffer(); }
  /// Gets the current buffer of readings
  virtual std::list<ArPose *> *getCumulativeBuffer(void)
    { return myCumulativeBuffer.getBuffer(); }
  /// Gets the raw unfiltered readings from the device
  /** The raw readings are the full set of unfiltered readings from the device,
      they are the latest reading, you should manipulate the list you get from
      this function, the only manipulation of this list should be done by
      the range device itself.  Its only pointers for speed.
  */
  virtual std::list<ArSensorReading *> *getRawReadings(void)
    { return myRawReadings; }
  /// Gets the maximum range for this device
  AREXPORT virtual unsigned int getMaxRange(void) { return myMaxRange; }
  /// Sets the maximum range for this device
  AREXPORT virtual void setMaxRange(unsigned int maxRange) 
    { myMaxRange = maxRange; }

  /// Applies a transform to the buffers
  AREXPORT virtual void applyTransform(ArTransform trans, 
				       bool doCumulative = true);

  /// Lock this device
  /**
     If you are also inheriting an ASyncTask you MUST override this
     to use the lock from the ArASyncTask
  */
  AREXPORT virtual int lockDevice() { return(myDeviceMutex.lock());}
  /// Try to lock this device
  /**
     If you are also inheriting an ASyncTask you MUST override this
     to use the lock from the ArASyncTask
  */
  AREXPORT virtual int tryLockDevice() {return(myDeviceMutex.tryLock());}
  /// Unlock this device
  /**
     If you are also inheriting an ASyncTask you MUST override this
     to use the lock from the ArASyncTask
  */
  AREXPORT virtual int unlockDevice() {return(myDeviceMutex.unlock());}
protected:
  std::string myName;
  ArRobot *myRobot;
  unsigned int myMaxRange; 
  ArRangeBuffer myCurrentBuffer;
  ArRangeBuffer myCumulativeBuffer;
  std::list<ArSensorReading *> *myRawReadings;
  ArMutex myDeviceMutex;
};

#endif // ARRANGEDEVICE_H
