/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARROBOTPARAMS_H
#define ARROBOTPARAMS_H

#include "ariaTypedefs.h"
#include "ArRobotParamFile.h"

/// Contains the robot parameters, according to the parameter file
class ArRobotParams
{
public:
  /// Constructor
  AREXPORT ArRobotParams(void);
  /// Destructor
  AREXPORT virtual ~ArRobotParams(void);
  /// Given the robot parameters in preference form, fills in this instance
  AREXPORT void init(ArRobotParamFile *param);
  /// Returns the class from the parameter file
  std::string getClass(void) { return myClass; }
  /// Returns the subclass from the parameter file
  std::string getSubClass(void) { return mySubClass; }
  /// Returns the robot's radius
  double getRobotRadius(void) { return myRobotRadius; }
  /// Returns the robot diagonal (half-height to diagonal of octagon)
  double getRobotDiagonal(void) { return myRobotDiagonal; }
  /// Returns whether the robot is holonomic or not
  bool isHolonomic(void) { return myHolonomic; }
  /// Returns if the robot has a built in move command
  bool hasMoveCommand(void) { return myHaveMoveCommand; }
  /// Returns the max velocity of the robot
  int getMaxVelocity(void) { return myMaxVelocity; }
  /// Returns the max rotational velocity of the robot
  int getMaxRotVelocity(void) { return myMaxRVelocity; }
  /// Returns true if IO packets are automatically requested upon connection to the robot.
  bool getRequestIOPackets(void) { return myRequestIOPackets; }
  /// Returns the angle conversion factor 
  double getAngleConvFactor(void) { return myAngleConvFactor; }
  /// Returns the distance conversion factor
  double getDistConvFactor(void) { return myDistConvFactor; }
  /// Returns the velocity conversion factor
  double getVelConvFactor(void) { return myVelConvFactor; }
  /// Returns the sonar range conversion factor
  double getRangeConvFactor(void) { return myRangeConvFactor; }
  /// Returns the wheel velocity difference to angular velocity conv factor
  double getDiffConvFactor(void) { return myDiffConvFactor; }
  /// Returns the multiplier for VEL2 commands
  double getVel2Divisor(void) { return myVel2Divisor; }
  /// Returns true if the robot has table sensing IR
  bool haveTableSensingIR(void) { return myTableSensingIR; }
  /// Returns true if the robot's table sensing IR bits are sent in the 4th-byte of the IO packet
  bool haveNewTableSensingIR(void) { return myNewTableSensingIR; }
  /// Returns true if the robot has front bumpers
  bool haveFrontBumpers(void) { return myFrontBumpers; }
  /// Returns the number of front bumpers
  int numFrontBumpers(void) { return myNumFrontBumpers; }
  /// Returns true if the robot has rear bumpers
  bool haveRearBumpers(void) { return myRearBumpers; }
  /// Returns the number of rear bumpers
  int numRearBumpers(void) { return myNumRearBumpers; }
  /// Returns the number of sonar
  int getNumSonar(void) { return myNumSonar; }
  /// Returns if the sonar of the given number is valid
  bool haveSonar(int number)
    {
      if (mySonarMap.find(number) != mySonarMap.end())
	return true;
      else
	return false;
    }
  /// Returns the X location of the given numbered sonar disc
  int getSonarX(int number)
    {
      std::map<int, std::map<int, int> >::iterator it;
      std::map<int, int>::iterator it2;
      if ((it = mySonarMap.find(number)) == mySonarMap.end())
	return 0;
      if ((it2 = (*it).second.find(SONAR_X)) == (*it).second.end())
	return 0;
      return (*it2).second;
    }
  /// Returns the Y location of the given numbered sonar disc
  int getSonarY(int number)
    {
      std::map<int, std::map<int, int> >::iterator it;
      std::map<int, int>::iterator it2;
      if ((it = mySonarMap.find(number)) == mySonarMap.end())
	return 0;
      if ((it2 = (*it).second.find(SONAR_Y)) == (*it).second.end())
	return 0;
      return (*it2).second;
    }
  /// Returns the heading of the given numbered sonar disc
  int getSonarTh(int number)
    {
      std::map<int, std::map<int, int> >::iterator it;
      std::map<int, int>::iterator it2;
      if ((it = mySonarMap.find(number)) == mySonarMap.end())
	return 0;
      if ((it2 = (*it).second.find(SONAR_TH)) == (*it).second.end())
	return 0;
      return (*it2).second;
    }
  /// Returns if the robot has a laser (according to param file)
  bool getLaserPossessed(void) { return myLaserPossessed; }
  /// What port the laser is on
  std::string getLaserPort(void) { return myLaserPort; }
  /// If the laser power is controlled by the serial port lines
  bool getLaserPowerControlled(void) { return myLaserPowerControlled; }
  /// If the laser is flipped on the robot
  bool getLaserFlipped(void) { return myLaserFlipped; }
  /// The X location of the laser
  int getLaserX(void) { return myLaserX; }
  /// The Y location of the laser 
  int getLaserY(void) { return myLaserY; }
protected:
  std::string myClass;
  std::string mySubClass;
  double myRobotRadius;
  double myRobotDiagonal;
  bool myHolonomic;
  bool myHaveMoveCommand;
  bool myRequestIOPackets;
  int myMaxVelocity;
  int myMaxRVelocity;
  double myAngleConvFactor;
  double myDistConvFactor;
  double myVelConvFactor;
  double myRangeConvFactor;
  double myDiffConvFactor;
  double myVel2Divisor;
  int myNumSonar;
  std::map<int, std::map<int, int> > mySonarMap;
  enum SonarInfo 
  { 
    SONAR_X, 
    SONAR_Y, 
    SONAR_TH
  };
  bool myTableSensingIR;
  bool myNewTableSensingIR;
  bool myFrontBumpers;
  int myNumFrontBumpers;
  bool myRearBumpers;
  int myNumRearBumpers;
  bool myLaserPossessed;
  std::string myLaserPort;
  bool myLaserPowerControlled;
  bool myLaserFlipped;
  int myLaserX;
  int myLaserY;
};

#endif // ARROBOTPARAMS_H
