/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#ifndef ARVCC4_H
#define ARVCC4_H

#include "ariaTypedefs.h"
#include "ArBasePacket.h"
#include "ArPTZ.h"
#include "ariaUtil.h"

/** This class is for controlling the Canon VC-C4 camera.

Note that currently the pan/tilt commands send a stop to the camera, halting the previous command.  The stop command waits for 200ms for processing.  The same is true for the zoom commands.

Also note that the camera takes about 4 seconds to complete the power-on cycle, and 2 seconds for an init.  Commands sent during these periods will be ignored, and may confuse the camera.

The camera's pan and tilt commands work on a number of units equal to (degrees / 0.1125).  The panTilt function always rounds the conversion closer to zero, so that a magnitude greater than the allowable range of movement is not sent to the camera.
*/

/// A class with the commands for the VCC4
class ArVCC4Commands
{
public:
  enum {
    DELIM = 0x00, ///<Delimeter character
    DEVICEID = 0x30, ///<Default device ID
    PANSLEW = 0x50, ///<Sets the pan slew
    TILTSLEW = 0x51, ///<Sets the tilt slew
    STOP = 0x53, ///<Stops current pan/tilt motion
    INIT = 0x58, ///<Initializes the camera
    PANTILT = 0x62, ///<Pan/tilt command
    SETRANGE = 0x64, ///<Pan/tilt min/max range assignment
    CONTROL = 0x90, ///<Puts camera in Control mode
    POWER = 0xA0, ///<Turns on/off power
    ZOOMSTOP = 0xA2, ///<Stops zoom motion
    ZOOM = 0xA3, ///<Zooms camera lens
    FOOTER = 0xEF, ///<Packet Footer
    HEADER = 0xFF ///<Packet Header
  };

};

/// A class for for making commands to send to the VCC4
/** 
    There are only a few functioning ways to put things into this packet, you
    MUST use thse, if you use anything else your commands won't work.  You 
    must use byteToBuf and byte2ToBuf.  
*/
class ArVCC4Packet: public ArBasePacket
{
public:
  /// Constructor
  AREXPORT ArVCC4Packet(ArTypes::UByte2 bufferSize = 30);
  /// Destructor
  AREXPORT virtual ~ArVCC4Packet(void);

  AREXPORT virtual void byte2ToBuf(ArTypes::Byte4 val);

  AREXPORT virtual void finalize(void);

protected:
};

/// Driver for the VCC4
class ArVCC4 : public ArPTZ
{
public:
  AREXPORT ArVCC4(ArRobot *robot, bool myInverted = false);
  AREXPORT virtual ~ArVCC4(void);

  /// Turn on/off the camera
  AREXPORT bool power(int status);
  AREXPORT bool init(void);
  AREXPORT void connectHandler(void);

  AREXPORT bool pan(int deg) { return panTilt(deg, myTilt); }
  AREXPORT bool panRel(int deg) { return panTilt(myPan+deg, myTilt); }
  AREXPORT bool tilt(int deg) { return panTilt(myPan, deg); }
  AREXPORT bool tiltRel(int deg) { return panTilt(myPan, myTilt+deg); }
  AREXPORT bool panTiltRel(int pdeg, int tdeg) { return panTilt(myPan+pdeg, myTilt+tdeg); }

  AREXPORT int getMaxPosPan(void) 
    { if (myInverted) return invert(MIN_PAN); else return MAX_PAN; }
  AREXPORT int getMaxNegPan(void) 
    { if (myInverted) return invert(MAX_PAN); else return MIN_PAN; }
  AREXPORT int getMaxPosTilt(void) 
    { if (myInverted) return invert(MIN_TILT); else return MAX_TILT; }
  AREXPORT int getMaxNegTilt(void)
    { if (myInverted) return invert(MAX_TILT); else return MIN_TILT; }

  AREXPORT bool canZoom(void) { return true; }

  AREXPORT bool panTilt(int pdeg, int tdeg);
  AREXPORT bool zoom(int deg);


  /// Halts all pan-tilt movement
  AREXPORT bool haltPanTilt(void);
  /// Halts zoom movement
  AREXPORT bool haltZoom(void);

  /// Sets the rate that the unit pans at
  AREXPORT bool panSlew(int deg);
  /// Sets the rate the unit tilts at 
  AREXPORT bool tiltSlew(int deg);

  /// Sends this packet to the unit
  AREXPORT void preparePacket(ArVCC4Packet *packet);

  int getPan(void) { return myPan; }
  int getTilt(void) { return myTilt; }
  int getZoom(void) { return myZoom; }

  /// Gets the current pan slew
  int getPanSlew(void) { return myPanSlew; }
  /// Gets the maximum pan slew
  int getMaxPanSlew(void) { return MAX_PAN_SLEW; }
  /// Gets the minimum pan slew
  int getMinPanSlew(void) { return MIN_PAN_SLEW; }

  /// Gets the current tilt slew
  int getTiltSlew(void) { return myTiltSlew; }
  /// Gets the maximum tilt slew
  int getMaxTiltSlew(void) { return MAX_TILT_SLEW; }
  /// Gets the minimum tilt slew
  int getMinTiltSlew(void) { return MIN_TILT_SLEW; }

  int getMaxZoom(void) { return MAX_ZOOM; }
  int getMinZoom(void) { return MIN_ZOOM; }


  enum {
    MAX_PAN = 98,		// 875 units is max pan assignment
    MIN_PAN = -98,		// -875 units is min pan assignment
    MAX_TILT = 88,		// 790 units is max tilt assignment
    MIN_TILT = -30,		// -267 units is min tilt assignment
    MAX_PAN_SLEW = 90,		// 800 positions per sec (PPS)
    MIN_PAN_SLEW = 1,		// 8 positions per sec (PPS)
    MAX_TILT_SLEW = 69,		// 662 positions per sec (PPS)
    MIN_TILT_SLEW = 1,		// 8 positions per sec (PPS)
    MAX_ZOOM = 80,
    MIN_ZOOM = 0
  };
protected:
  // flips the sign if needed
  int invert(int before) 
    { if (myInverted) return -before; else return before; }
  bool myInverted;
  ArRobot *myRobot;
  ArVCC4Packet myPacket;
  ArTime myTime;
  int myPan;
  int myTilt;
  int myPanSlew;
  int myTiltSlew;
  int myZoom;
};

#endif // ARVCC4_H

