/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArActionBumpers.h"
#include "ArRobot.h"

/**
   @param name name of the action
   @param backOffSpeed speed at which to back away (mm/sec)
   @param backOffTime number of msec to back up for (msec)
   @param turnTime number of msec to alow for turn (msec)
*/
AREXPORT ArActionBumpers::ArActionBumpers(const char *name, 
					  double backOffSpeed,
					  int backOffTime, int turnTime,
					  bool setMaximums) :
  ArAction(name, "Reacts to the bumpers triggering")
{
  setNextArgument(ArArg("back off speed", &myBackOffSpeed, 
			"Speed at which to back away (mm/sec)"));
  myBackOffSpeed = backOffSpeed;
  
  setNextArgument(ArArg("back off time", &myBackOffTime,
			"Number of msec to back up for (msec)"));
  myBackOffTime = backOffTime;

  setNextArgument(ArArg("turn time", &myTurnTime,
			"Number of msec to allow for turn (msec)"));
  myTurnTime = turnTime;

  setNextArgument(ArArg("set maximums", &mySetMaximums,
			"Whether to set maximum vels or not (bool)"));
  mySetMaximums = setMaximums;
  
  myFiring = false;
  myBumpMask = (ArUtil::BIT1 | ArUtil::BIT2 | ArUtil::BIT3 | ArUtil::BIT4 | 
		ArUtil::BIT5); 
}

AREXPORT ArActionBumpers::~ArActionBumpers(void)
{

}

double ArActionBumpers::findDegreesToTurn(int bumpValue)
{
  int i;
  double totalTurn = 0;
  int numTurn = 0;
  double middleTurn;
  int numMiddleTurn;
  double temp;

  for (i = 0; i < 5; i++)
  {
    if (i == 0 && (bumpValue & ArUtil::BIT1))
    {
      totalTurn += -25;
      ++numTurn;
    }
    if (i == 1 && (bumpValue & ArUtil::BIT2))
    {
      totalTurn += -35;
      ++numTurn;
    }
    if (i == 2 && (bumpValue & ArUtil::BIT3))
    {
      if ((bumpValue & ArUtil::BIT1) && 
	  !(bumpValue & ArUtil::BIT5))
      {
	middleTurn += -25;
	++numMiddleTurn;
      }
      if ((bumpValue & ArUtil::BIT2) && 
	  !(bumpValue & ArUtil::BIT4))
      {
	middleTurn += -35;
	++numMiddleTurn;
      }
      if (!(bumpValue & ArUtil::BIT1) && 
	  (bumpValue & ArUtil::BIT5))
      {
	middleTurn += 25;
	++numMiddleTurn;
      }
      if (!(bumpValue & ArUtil::BIT2) && 
	  (bumpValue & ArUtil::BIT4))
      {
	middleTurn += 35;
	++numMiddleTurn;
      }
      temp = middleTurn / numMiddleTurn;
      if (temp > 0 && temp < 50)
	temp = 50;
      else if (temp <= 0 && temp <= -50)
	temp = -50;
      
      totalTurn += temp;
      ++numTurn;
    }
    if (i == 3 && (bumpValue & ArUtil::BIT4))
    {
      totalTurn += 35;
      ++numTurn;
    }
    if (i == 4 && (bumpValue & ArUtil::BIT5))
    {
      totalTurn += 25;
      ++numTurn;
    }
  }
  return totalTurn / (double)numTurn;
}

AREXPORT ArActionDesired *ArActionBumpers::fire(ArActionDesired currentDesired)
{
  int frontBump;
  int rearBump;

  if (myRobot->hasFrontBumpers())
    frontBump = ((myRobot->getStallValue() & 0xff00) >> 8) & myBumpMask;
  else
    frontBump = 0;
  if (myRobot->hasRearBumpers())
    rearBump = (myRobot->getStallValue() & 0xff) & myBumpMask;
  else
    rearBump = 0;

  myDesired.reset();
  if (myFiring)
  {
    if (myStartBack.mSecSince() < myBackOffTime)
    {
      if ((mySpeed < 0 && rearBump != 0) ||
	  (mySpeed > 0 && frontBump != 0))
      {
	//printf("0 norot\n");
	myDesired.setVel(0);
      }
      else
      {
	//printf("%.0f norot\n", mySpeed);
	myDesired.setVel(mySpeed);
      }
      myDesired.setDeltaHeading(0);
      return &myDesired;
    }
    else if (myStartBack.mSecSince() < myBackOffTime + myTurnTime &&
	     ArMath::fabs(ArMath::subAngle(myRobot->getTh(), myHeading)) > 3)
    {
      //printf("0 %.0f\n", myHeading);
      myDesired.setVel(0);
      myDesired.setHeading(myHeading);
      return &myDesired;
    }
    
    myFiring = false;
  }

  if (myRobot->getVel() > 1)
  {
    if (frontBump == 0)
      return NULL;
    printf("Bumped a forward bumper while going forward, turning %.0f \n",
	   findDegreesToTurn(frontBump));
    myHeading = ArMath::addAngle(myRobot->getTh(), 
				 findDegreesToTurn(frontBump));
    mySpeed = -myBackOffSpeed;
  }
  else if (myRobot->getVel() < -1)
  {
    if (rearBump == 0)
      return NULL;
    printf("Bumped a rear bumper while going backwards, turning %.0f \n",
	   findDegreesToTurn(rearBump));
    myHeading = ArMath::subAngle(myRobot->getTh(), 
				 findDegreesToTurn(rearBump));
    mySpeed = myBackOffSpeed;
  } else
    return NULL;

  myStartBack.setToNow();
  myFiring = true;
  myDesired.setVel(mySpeed);
  myDesired.setHeading(myHeading);
  // Set the maximums if that feature is desired
  if (mySetMaximums)
  {
    if (mySpeed > 0)
      myDesired.setMaxVel(mySpeed);
    else
      myDesired.setMaxNegVel(mySpeed);
  }
  return &myDesired;
}  
