/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArActionGroup.h"
#include "ArAction.h"
#include "ArRobot.h"
#include "ArLog.h"

/**
   @@param robot The robot that this action group is attached to 

   @@param deleteActionsOnDestruction if this is true then when the
   destructor is called the actions that this group has will be
   deleted
**/

AREXPORT ArActionGroup::ArActionGroup(ArRobot *robot)
{
  myRobot = robot;
}

AREXPORT ArActionGroup::~ArActionGroup(void)
{
  removeActions();
}


/**
   @@param action the action to add to the robot
   @@param priority the priority to give the action
   @@see ArRobot::addAction
*/
AREXPORT void ArActionGroup::addAction(ArAction *action, int priority)
{
  if (myRobot == NULL)
  {
    ArLog::log(ArLog::Terse, 
            "ArActionGroup::addAction: NULL robot pointer... failed.");
    return;
  }
  myActions.push_front(action);
  myRobot->addAction(action, priority);
}

/**
   @@param action the action to remove from the robot
   @@see ArRobot::remAction
*/
AREXPORT void ArActionGroup::remAction(ArAction *action)
{
  if (myRobot == NULL)
  {
    ArLog::log(ArLog::Terse, 
            "ArActionGroup::remAction: NULL robot pointer... failed.");
    return;
  }
  myActions.remove(action);
  myRobot->remAction(action);
}

AREXPORT void ArActionGroup::activate(void)
{
  std::list<ArAction *>::iterator it;
  if (myRobot == NULL)
  {
    ArLog::log(ArLog::Terse, 
            "ArActionGroup::activate: NULL robot pointer... failed.");
    return;
  }
  for (it = myActions.begin(); it != myActions.end(); it++)
    (*it)->activate();
}

AREXPORT void ArActionGroup::activateExclusive(void)
{
  std::list<ArAction *>::iterator it;
  ArResolver::ActionMap *am;
  ArResolver::ActionMap::iterator amit;
  
  
  if (myRobot == NULL)
  {
    ArLog::log(ArLog::Terse, 
            "ArActionGroup::activateExclusive: NULL robot pointer... failed.");
    return;
  }

  am = myRobot->getActionMap();
  if (am == NULL)
  {
    ArLog::log(ArLog::Terse, 
            "ArActionGroup::activateExclusive: NULL action map... failed.");
    return;
  }
  for (amit = am->begin(); amit != am->end(); amit++)
    (*amit).second->deactivate();
  
  for (it = myActions.begin(); it != myActions.end(); it++)
    (*it)->activate();
}

AREXPORT void ArActionGroup::deactivate(void)
{
  std::list<ArAction *>::iterator it;
  if (myRobot == NULL)
  {
    ArLog::log(ArLog::Terse, 
            "ArActionGroup::deactivate: NULL robot pointer... failed.");
    return;
  }
  for (it = myActions.begin(); it != myActions.end(); it++)
    (*it)->deactivate();
}

AREXPORT std::list<ArAction *> *ArActionGroup::getActionList(void)
{
  return &myActions;
}

AREXPORT void ArActionGroup::removeActions(void)
{
  std::list<ArAction *>::iterator it;

  if (myRobot == NULL)
  {
    ArLog::log(ArLog::Terse, 
            "ArActionGroup::removeActions: NULL robot pointer... very bad.");
    return;
  }

  for (it = myActions.begin(); it != myActions.end(); ++it)
    myRobot->remAction((*it));
}

