/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArArg.h"
#include "ArLog.h"

AREXPORT ArArg::ArArg(void)
{
  myIntPointer = NULL; 
  myDoublePointer = NULL; 
  myStringPointer = NULL;
}

AREXPORT ArArg::ArArg(std::string name, int *pointer, std::string description) 
{ 
  myType = INT;
  myName = name; 
  myDescription = description;
  clearPointers();
  myIntPointer = pointer;
}

AREXPORT ArArg::ArArg(std::string name, double *pointer,
		      std::string description) 
{ 
  myType = DOUBLE;
  myName = name; 
  myDescription = description;
  clearPointers();
  myDoublePointer = pointer;
}

AREXPORT ArArg::ArArg(std::string name, std::string *pointer, 
		      std::string description) 
{ 
  myType = STRING;
  myName = name; 
  myDescription = description;
  clearPointers();
  myStringPointer = pointer;
}

AREXPORT ArArg::ArArg(std::string name, bool *pointer, 
		      std::string description) 
{ 
  myType = BOOL;
  myName = name; 
  myDescription = description;
  clearPointers();
  myBoolPointer = pointer;
}

AREXPORT ArArg::ArArg(std::string name, ArPose *pointer, 
		      std::string description) 
{ 
  myType = POSE;
  myName = name; 
  myDescription = description;
  clearPointers();
  myPosePointer = pointer;
}

AREXPORT ArArg::ArArg(const ArArg & arg) :
  myType(arg.myType),
  myName(arg.myName),
  myDescription(arg.myDescription),
  myIntPointer(arg.myIntPointer),
  myDoublePointer(arg.myDoublePointer),
  myStringPointer(arg.myStringPointer)
{
}

AREXPORT ArArg::~ArArg(void)
{
}

AREXPORT void ArArg::clearPointers(void)
{
  myIntPointer = NULL;
  myDoublePointer = NULL;
  myStringPointer = NULL;
  myBoolPointer = NULL;
  myPosePointer = NULL;
}

/**
   @see INVALID
   @see INT
   @see DOUBLE
   @see STRING
   @see BOOL
   @see POSE
*/
AREXPORT ArArg::Type ArArg::getType(void)
{
  return myType;
}

AREXPORT std::string ArArg::getName(void)
{
  return myName;
}

AREXPORT std::string ArArg::getDescription(void)
{
  return myDescription;
}

AREXPORT int ArArg::getInt(void)
{ 
  if (myIntPointer != NULL)
    return *myIntPointer;
  else
    return 0;
}

AREXPORT double ArArg::getDouble(void) 
{
  if (myDoublePointer != NULL)
    return *myDoublePointer; 
  else
    return 0;
}

AREXPORT std::string ArArg::getString(void) 
{ 
  std::string str = "";

  if (myStringPointer != NULL)
    return *myStringPointer;
  else
    return str;
}

AREXPORT bool ArArg::getBool(void)
{
  if (myBoolPointer != NULL)
    return *myBoolPointer;
  else
    return false;
}

AREXPORT ArPose ArArg::getPose(void)
{
  ArPose pose;
  if (myPosePointer != NULL)
    return *myPosePointer;
  else
    return pose;
}

AREXPORT void ArArg::setInt(int val)
{
  if (myIntPointer != NULL) 
    *myIntPointer = val;
}

AREXPORT void ArArg::setDouble(double val)
{ 
  if (myDoublePointer != NULL)
    *myDoublePointer = val; 
}

AREXPORT void ArArg::setString(std::string str) 
{ 
  if (myStringPointer != NULL)
    *myStringPointer = str;
}

AREXPORT void ArArg::setBool(bool val)
{
  if (myBoolPointer != NULL)
    *myBoolPointer = val;
}

AREXPORT void ArArg::setPose(ArPose pose)
{
  if (myPosePointer != NULL)
    *myPosePointer = pose;
}

AREXPORT void ArArg::print(void)
{
  switch (getType()) 
  {
  case ArArg::INVALID:
    ArLog::log(ArLog::Terse, 
	       "\tType: %10s.  This argument was not created properly.", 
	       "invalid");
  case ArArg::INT:
    ArLog::log(ArLog::Terse, "\tType: %10s name: %12s value: %d", "int", 
	       getName().c_str(), getInt());
    if (getDescription().size() != 0)
      ArLog::log(ArLog::Terse, "\t\tDescription: %s",
		 getDescription().c_str());
    break;
  case ArArg::DOUBLE:
    ArLog::log(ArLog::Terse, "\tType: %10s name: %12s value: %f", "double",
	       getName().c_str(), getDouble());
    if (getDescription().size() != 0)
      ArLog::log(ArLog::Terse, "\t\tDescription: %s",
		 getDescription().c_str());
    break;
  case ArArg::STRING:
    ArLog::log(ArLog::Terse, "\tType: %10s name: %12s value: %s", "string", 
	       getName().c_str(), getString().c_str());
    if (getDescription().size() != 0)
      ArLog::log(ArLog::Terse, "\t\tDescription: %s",
		 getDescription().c_str());
    break;
  case ArArg::BOOL:
    ArLog::log(ArLog::Terse, "\tType: %10s name: %12s value: %d", "double",
	       getName().c_str(), getBool());
    if (getDescription().size() != 0)
      ArLog::log(ArLog::Terse, "\t\tDescription: %s",
		 getDescription().c_str());
    break;
  case ArArg::POSE:
    ArLog::log(ArLog::Terse, "\tType: %10s name: %12s value: (%.1f %.1f %.1f)",
	       "double", getName().c_str(), getPose().getX(), getPose().getY(),
	       getPose().getTh());
    if (getDescription().size() != 0)
      ArLog::log(ArLog::Terse, "\t\tDescription: %s",
		 getDescription().c_str());
    break;

  default:
    ArLog::log(ArLog::Terse, 
	       "\tType: %10s.  This type doesn't have a case in ArArg::print.",
	       "unknown");
    break;
  }


}
