/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArKeyHandler.h"
#include "ArLog.h"

#ifdef linux
#include <stdio.h>
#endif

#ifdef WIN32
#include <conio.h>
#endif

#include "ArRobot.h"
#include "ariaInternal.h"

/**
   @param blocking whether or not to block waiting on keys, default is
   false, ie not to wait... you probably only want to block if you are
   using checkKeys yourself like after you start a robot run or in its
   own thread or something along those lines
**/
AREXPORT ArKeyHandler::ArKeyHandler(bool blocking)
{
  myBlocking = blocking;
#ifdef linux
  struct termios newTermios;

  tcgetattr(fileno(stdin), &myOriginalTermios);
  
  tcgetattr(fileno(stdin), &newTermios);
  
  newTermios.c_cc[VTIME] = 0;
  if (myBlocking)
    newTermios.c_cc[VMIN] = 1;
  else
    newTermios.c_cc[VMIN] = 0;
  newTermios.c_lflag &= (~ECHO & ~ICANON);
  tcsetattr(fileno(stdin), TCSANOW, &newTermios);
#endif
  myRestored = false;
}

AREXPORT ArKeyHandler::~ArKeyHandler(void)
{
  restore();
}

AREXPORT void ArKeyHandler::restore(void)
{
#ifdef linux
  tcsetattr(fileno(stdin), TCSANOW, &myOriginalTermios);
#endif
  myRestored = true;
}

/**
   @param keyToHandle this is an ascii character, such as 'a' or '1'
   or '[', or a member of the KEY enum.

   @param functor a functor to call when the given key is pressed

   @return true if the addKeyHandler succeeded, which means that the
   key added was unique and it will be handled... false means that the
   add failed, because there was already a keyHandler in place for
   that key
 **/
AREXPORT bool ArKeyHandler::addKeyHandler(int keyToHandle, ArFunctor *functor)
{
  std::map<int, ArFunctor *>::iterator it;
  if ((it = myMap.find(keyToHandle)) != myMap.end())
  {
    ArLog::log(ArLog::Normal, "There is already a key to handle '%c' which is number %d 0x%x", keyToHandle, keyToHandle, keyToHandle);
    return false;
  }
  //ArLog::log(ArLog::Verbose, "keyhandler %p added key '%c' number '%d'", 
  //this, keyToHandle, keyToHandle);
  myMap[keyToHandle] = functor;

  return true;
}

/**
   @param keyToHandle this is an ascii character, such as 'a' or '1'
   or '[', or a member of the KEY enum.

    @return true if the remKeyHandler succeeded, which means that the
   key wad found and rmeoved... false means that the remove failed
   because there was no key for that
 **/
AREXPORT bool ArKeyHandler::remKeyHandler(int keyToHandle)
{
  ArLog::log(ArLog::Verbose, "keyhandler %p removed key '%c' number '%d'",
	     this, keyToHandle, keyToHandle);
  if (myMap.find(keyToHandle) == myMap.end())
  {
    //ArLog::log(ArLog::Normal, "There is no key to handle '%c' which is number %d 0x%x", keyToHandle, keyToHandle, keyToHandle);
    return false;
  }
  myMap.erase(keyToHandle);
  return true;
}

/**
   @param keyToHandle the functor to remove

   @return true if the remKeyHandler succeeded, which means that the
   key wad found and rmeoved... false means that the remove failed
   because there was no key for that
 **/
AREXPORT bool ArKeyHandler::remKeyHandler(ArFunctor *functor)
{
  ArLog::log(ArLog::Verbose, "keyhandler %p removed functor %p", this, 
	     functor);
  std::map<int, ArFunctor *>::iterator it;
  std::list<std::map<int, ArFunctor *>::iterator> iters;
  std::list<std::map<int, ArFunctor *>::iterator>::iterator iterIter;

  for (it = myMap.begin(); it != myMap.end(); ++it)
  {
    if (it->second == functor)
    {
      iters.push_front(it);
    }
  }
  if (iters.size() > 0)
  {
    while((iterIter = iters.begin()) != iters.end())
    {
      myMap.erase((*iterIter));
      iters.pop_front();
    }
    return true;
  }
  return false;
}

AREXPORT void ArKeyHandler::checkKeys(void)
{
  int key;
  std::map<int, ArFunctor *>::iterator it;

  if (myRestored)
    return;

  // get keys until they're gone
  while (!myRestored && (key = getKey()) != -1)
  {
    // if there's something to handle it, handle it
    if ((it = myMap.find(key)) != myMap.end())
    {
      //ArLog::log(ArLog::Verbose, "key '%c' num %d pressed\n", key, key);
      it->second->invoke();
    }
  }
}

#ifdef linux

AREXPORT int ArKeyHandler::getKey(void)
{
  char key;

  key = getchar();
  if (key == -1 || key == 0)
    return -1;

  if (key == 27)
  {
    key = getchar();
    if (key == '[')
    {
      key = getchar();
      if (key == 'A')
	return UP;
      else if (key == 'B')
	return DOWN;
      else if (key == 'C')
	return RIGHT;
      else if (key == 'D')
	return LEFT;
      else
	return getKey();
    }
    else if (key == -1)
      return ESCAPE;
    else if (key == 79)
    {
      key = getchar();
      if (key == 'P')
	return F1;
      else if (key == 'Q')
	return F2;
      else if (key == 'R')
	return F3;
      else if (key == 'S')
	return F4;
      else
	return getKey();
    }
  } 
  else if (key == ' ')
    return SPACE;
  else if (key == '\t')
    return TAB;
  else if (key == 10)
    return ENTER;
  else if (key == 8)
    return BACKSPACE;

  return key;
  
}

#endif // linux

#ifdef WIN32

AREXPORT int ArKeyHandler::getKey(void)
{
  int key;

  if (!myBlocking && !kbhit())
    return -1;

  key = _getch();
  if (key == 224)
  {
    key = _getch();
    if (key == 'H')
      return UP;
    else if (key == 'P')
      return DOWN;
    else if (key == 'K')
      return LEFT;
    else if (key == 'M')
      return RIGHT;
    else 
      return getKey();
  }
  else if (key == 0)
  {
    key = _getch();
    if (key == ';')
      return F1;
    else if (key == '<')
      return F2;
    else if (key == '=')
      return F3;
    else if (key == '>')
      return F4;
    else if (key == 'H')
      return UP;
    else if (key == 'P')
      return DOWN;
    else if (key == 'K')
      return LEFT;
    else if (key == 'M')
      return RIGHT;
    else
      return getKey();
  }
  else if (key == ' ')
    return SPACE;
  else if (key == '\t')
    return TAB;
  else if (key == 13)
    return ENTER;
  else if (key == 8)
    return BACKSPACE;
  else if (key == 27)
	return ESCAPE;

  return key;

}

#endif // WIN32

