/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArNetServer.h"
#include "ArRobot.h"
#include "ArLog.h"
#include "ariaUtil.h"
#include "ArSyncTask.h"

ArNetServer::ArNetServer(void) :
  myTaskCB(this, &ArNetServer::runOnce),
  myHelpCB(this, &ArNetServer::internalHelp),
  myCloseCB(this, &ArNetServer::internalQuit),
  myShutdownCB(this, &ArNetServer::internalShutdown)    
{
  myRobot = NULL;
  myPort = 0;
  myMultipleClients = false;
  myOpened = false;
  addCommand("help", &myHelpCB, "gives the listing of available commands");
  addCommand("quit", &myCloseCB, "closes this connection to the server");
  addCommand("shutdown", &myShutdownCB, "shuts down the server");
}

ArNetServer::~ArNetServer(void)
{
  ArSyncTask *rootTask = NULL;
  ArSyncTask *proc = NULL;
  // get rid of us running on the robot task
  if (myRobot != NULL && (rootTask = myRobot->getSyncTaskRoot()) != NULL)
  {
    proc = rootTask->findNonRecursive(&myTaskCB);
    if (proc != NULL)
      delete proc;
  }
}

/**
   Open the server, if you supply a robot this will run in the robots
   attached, if you do not supply a robot then it will be open and
   you'll have to call runOnce yourself (this is only recommended for
   advanced users)

   @param robot the robot that this should be attached to and run in
   the sync task of or NULL not to run in any robot's task

   @param port the port to start up the service on

   @param password the password needed to use the service

   @param multipleClients if false only one client is allowed to connect, 
   if false multiple clients are allowed to connect or just one

   @return true if the server could be started, false otherwise
**/
AREXPORT bool ArNetServer::open(ArRobot *robot, unsigned int port, 
				const char *password, bool multipleClients)
{
  ArSyncTask *rootTask = NULL;
  ArSyncTask *proc = NULL;
  std::string taskName;

  if (myOpened)
  {
    ArLog::log(ArLog::Terse, "ArNetServer already inited, cannot reinit");
    return false;
  }
  myRobot = robot;
  myPort = port;
  myPassword = password;
  myMultipleClients = multipleClients;
  
  if (myServerSocket.open(myPort, ArSocket::TCP))
  {
    myServerSocket.setLinger(0);
    myServerSocket.setNonBlock();
    ArLog::log(ArLog::Normal, "ArNetServer opened on port %d.", myPort);
    myOpened = true;
  }
  else
  {
    ArLog::log(ArLog::Terse, "ArNetServer failed to open: %s", 
	       myServerSocket.getErrorStr().c_str());
    myOpened = false;
    return false;
  }

  // add ourselves to the robot if we aren't already there
  if (myRobot != NULL && (rootTask = myRobot->getSyncTaskRoot()) != NULL)
  {    
    proc = rootTask->findNonRecursive(&myTaskCB);
    if (proc == NULL)
    {
      // toss in a netserver
      taskName = "Net Servers ";
      taskName += myPort;
      rootTask->addNewLeaf(taskName.c_str(), 60, &myTaskCB, NULL);
    }
  }
  return true;
  
}

/**
   This adds a command to the list, when the command is given the
   broken up argv and argc are given along with the socket it came
   from (so that acks can occur)
**/
AREXPORT bool ArNetServer::addCommand(const char *command, 
			      ArFunctor3<char **, int, ArSocket *> *functor,
				      const char *help)
{
  std::map<std::string, ArFunctor3<char **, int, ArSocket *> *>::iterator it;
  if ((it = myFunctorMap.find(command)) != myFunctorMap.end())
  {
    ArLog::log(ArLog::Normal, "ArNetServer::addCommand: Already a command for %s", command);
    return false;
  }

  myFunctorMap[command] = functor;
  myHelpMap[command] = help;
  return true;
}

/**
   @param command the command to remove
   @return true if the command was there to remove, false otherwise
**/
AREXPORT bool ArNetServer::remCommand(const char *command)
{
  if (myFunctorMap.find(command) == myFunctorMap.end())
  {
    return false;
  }
  myFunctorMap.erase(command);
  myHelpMap.erase(command);
  return true;  
}

/**
   This sends the given string to all the clients, this string cannot
   be more than 2048 number of bytes
**/
AREXPORT void ArNetServer::sendToAllClients(const char *str, ...)
{
  char buf[2049];
  std::list<ArSocket *>::iterator it;
  va_list ptr;
  va_start(ptr, str);
  vsprintf(buf, str, ptr);

  for (it = mySockets.begin(); it != mySockets.end(); ++it)
    (*it)->writeString(buf);

  va_end(ptr);
}

AREXPORT bool ArNetServer::isOpen(void)
{
  return myOpened;
}

AREXPORT void ArNetServer::runOnce(void)
{
  if (!myOpened)
    return;

  ArSocket acceptingSocket;
  ArSocket *socket;
  unsigned char *bytes;
  std::string str;
  char buf[512];
  std::list<ArSocket *> removeList;
  std::list<ArSocket *>::iterator it;
  std::map<std::string, ArFunctor3<char **, int, ArSocket *> *>::iterator fIt;
  unsigned int ui;
  ArArgumentBuilder *args = NULL;
  char **argv;
  int argc;


  // get any new sockets that want to connect
  while (myServerSocket.accept(&acceptingSocket) && acceptingSocket.getFD() >= 0)
  {
    acceptingSocket.setNonBlock();
    // see if we want more sockets
    if (!myMultipleClients && (mySockets.size() > 0 ||
			       myConnectingSockets.size() > 0))
    {
      // we didn't want it, so politely tell it to go away
      acceptingSocket.writeString("Connection refused.\nOnly client allowed and it is already connected.\n");
      acceptingSocket.close();
      bytes = (unsigned char *)acceptingSocket.inAddr();
      ArLog::log(ArLog::Terse, "ArNetServer not taking multiple clients and another client tried to connect from %d.%d.%d.%d.", 
		 bytes[0], bytes[1], bytes[2], bytes[3]);
    }
    else 
    {
      // we want the client so we put it in our list of connecting
      // sockets, which means that it is waiting to give its password
      socket = new ArSocket;
      socket->transfer(&acceptingSocket);
      socket->writeString("Enter password:\n");
      myConnectingSockets.push_front(socket);
      bytes = (unsigned char *)socket->inAddr();
      ArLog::log(ArLog::Normal, 
		 "ArNetServer: Client connecting from %d.%d.%d.%d.",
		 bytes[0], bytes[1], bytes[2], bytes[3]);
    }
  }

  // now we read in data from our connecting sockets and see if
  // they've given us the password
  for (it = myConnectingSockets.begin(); it != myConnectingSockets.end(); ++it)
  {
    socket = (*it);
    // read in what the client has to say
    if (socket->readString(buf, 512))
    {
      if (buf[0] == '\0')
	continue;
      // find the first word for the password
      // set i to the value of the first space or the end of the text
      for (ui = 0; !isspace(buf[ui]) && ui < strlen(buf); ++ui);
      buf[ui] = '\0';
      // now see if the word matchs the password
      if (myPassword == buf)
      {
	bytes = (unsigned char *)socket->inAddr();
	ArLog::log(ArLog::Normal, 
		   "Client from %d.%d.%d.%d gave password and connected.",
		   bytes[0], bytes[1], bytes[2], bytes[3]);
	mySockets.push_front(socket);
	removeList.push_front(socket);
	internalGreeting(socket);
      }
      else
      {
	socket->close();
	myDeleteList.push_front(socket);
	ArLog::log(ArLog::Terse, 
		   "Client from %d.%d.%d.%d gave wrong password and is being disconnected.", 
		   bytes[0], bytes[1], bytes[2], bytes[3]);
      }
    }
    // if we couldn't read a string it means we lost a connection
    else
    {
      ArLog::log(ArLog::Normal, 
		 "Connection to %d.%d.%d.%d lost.",
		 bytes[0], bytes[1], bytes[2], bytes[3]);
      socket->close();
      myDeleteList.push_front(socket);
    }
  }
  // now we clear out the ones we want to remove from our connecting
  // clients list
  while ((it = removeList.begin()) != removeList.end())
  {
    socket = (*it);
    myConnectingSockets.remove(socket);
    removeList.pop_front();
  }


  // now we read in data from our connecting sockets and see if
  // they've given us the password
  for (it = mySockets.begin(); it != mySockets.end() && myOpened; ++it)
  {
    socket = (*it);
    // read in what the client has to say
    if (socket->readString(buf, 512))
    {
      // make sure we read something
      // set up the arguments and then call the function for the
      // argument
      args = new ArArgumentBuilder;
      args->add(buf);
      //args->print();
      argv = args->getArgv();
      argc = args->getArgc();
      // if we have some command see if it has a functor
      if (argc >= 1 && 
	  (fIt = myFunctorMap.find(argv[0])) != myFunctorMap.end())
      {
	fIt->second->invoke(argv, argc, socket);
      }
      // it didn't have a functor so we don't know it as a command
      else if (argc >= 1)
      {
	socket->writeString("Unknown command %s\n", argv[0]);
      }
      delete args;
      args = NULL;
    }
    // we lost connection
    else
    {
      ArLog::log(ArLog::Normal, 
		 "Connection to %d.%d.%d.%d lost.",
		 bytes[0], bytes[1], bytes[2], bytes[3]);
      socket->close();
      myDeleteList.push_front(socket);
    }
  }

  // now we delete the ones we want to delete (we could do this above
  // but then it wouldn't be symetrical with above)
  while ((it = myDeleteList.begin()) != myDeleteList.end())
  {
    socket = (*it);
    myConnectingSockets.remove(socket);
    mySockets.remove(socket);
    socket->close();
    delete socket;
    myDeleteList.pop_front();
  }

  if (!myOpened)
  {
    close();
  }
}

AREXPORT void ArNetServer::close(void)
{
  std::list<ArSocket *>::iterator it;
  ArSocket *socket;

  ArLog::log(ArLog::Normal, "ArNetServer: Shutting down server.");
  sendToAllClients("Shutting down server.\n");
  for (it = myConnectingSockets.begin(); it != myConnectingSockets.end(); ++it)
  {
    (*it)->writeString("Shutting down server.\n");
  }
  myOpened = false;

  while ((it = myConnectingSockets.begin())!= myConnectingSockets.end())
  {
    socket = (*it);
    myConnectingSockets.pop_front();
    delete socket;
  }
  while ((it = mySockets.begin()) != mySockets.end())
  {
    socket = (*it);
    mySockets.pop_front();
    delete socket;
  }
  myServerSocket.close();
}

AREXPORT void ArNetServer::internalGreeting(ArSocket *socket)
{
  socket->writeString("Welcome to the server.\nYou can type 'help' at any time for the following help list.\n");
  internalHelp(socket);
}

AREXPORT void ArNetServer::internalHelp(ArSocket *socket)
{
 std::map<std::string, std::string>::iterator it;
  
 socket->writeString("Commands:\n");
 for (it = myHelpMap.begin(); it != myHelpMap.end(); ++it)
   socket->writeString("%15s%10s%s\n", it->first.c_str(), "", 
		       it->second.c_str());
}

AREXPORT void ArNetServer::internalHelp(char **argv, int argc, 
					ArSocket *socket)
{
  internalHelp(socket);
}

AREXPORT void ArNetServer::internalQuit(char **argv, int argc, 
					 ArSocket *socket)
{
  socket->writeString("Closing connection\n");
  myDeleteList.push_front(socket);
}

AREXPORT void ArNetServer::internalShutdown(char **argv, int argc, 
					    ArSocket *socket)
{
  myOpened = false;
  myRobot->stopRunning();
}
