/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include <stdlib.h>
#include <stdarg.h>
#include "ariaUtil.h"
#include "ArPref.h"
#include "ArPreferences.h"
#include "ArLog.h"


AREXPORT ArPref::ArPref(int section, int pref, const char *name,
			const char *val, const char **validVals,
			ValType valType, const char *comment) :
  myParent(0),
  mySection(0),
  myPref(0),
  myName(),
  myVal(),
  myValidVals(0),
  myComment(),
  myNext(0),
  myPrev(0)
{
  mySection=section;
  myPref=pref;
  if (name)
    myName=name;
  if (val)
    myVal=val;
  if (validVals)
    myValidVals=validVals;
  myValType=valType;
  if (comment)
    myComment=comment;
}

AREXPORT ArPref::ArPref(const ArPref &pref) :
  myParent(0),
  mySection(pref.mySection),
  myPref(pref.myPref),
  myName(pref.myName),
  myVal(pref.myVal),
  myValidVals(pref.myValidVals),
  myValType(pref.myValType),
  myComment(pref.myComment),
  myNext(0),
  myPrev(0)
{
}

AREXPORT ArPref::~ArPref()
{
}

/**
   Get the value, formating it correctly. If the preference is not of
   the boolean type or not found, it will return false.
*/
AREXPORT bool ArPref::getBool()
{
  if (myValType == Boolean)
    return(strcasecmp(myVal.c_str(), "true") == 0);
  else
    return(false);
}

/**
   Get the value, formating it correctly. If the preference is not of
   the integer type or not found, it will return 0.
*/
AREXPORT int ArPref::getInt()
{
  if ((myValType == Integer) || (myValType == Double))
    return(atoi(myVal.c_str()));
  else
    return(0);
}

/**
   Get the value, formating it correctly. If the preference is not of
   the double type or not found, it will return 0.0.
*/
AREXPORT double ArPref::getDouble()
{
  if ((myValType == Integer) || (myValType == Double))
    return(atof(myVal.c_str()));
  else
    return(0.0);
}

/**
   Get the value, formating it correctly. If the preference is not of
   the string type or not found, it will return "".
*/
AREXPORT std::string ArPref::getString()
{
  return(myVal);
}

AREXPORT int ArPref::getValidIndex()
{
  int i;

  for (i=0; myValidVals[i] &&
       strcasecmp(myValidVals[i], myVal.c_str()); ++i)
    ;
  if (myValidVals[i])
    return(i);

  return(-1);
}

AREXPORT bool ArPref::setByValidIndex(int index, bool append)
{
  bool ret;
  ArPref *p=this;

  if (append)
    p=new ArPref(*this);

  if (p->myValidVals)
  {
    p->myVal=p->myValidVals[index];
    ret=true;
  }
  else
    ret=false;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }

  return(ret);
}

/**
   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param val the value to set the preference to
   @param append create a new instance of this preference
*/
AREXPORT bool ArPref::setBool(bool val, bool append)
{
  ArPref *p=this;

  if (append)
    p=new ArPref(*p);

  if (val)
    p->myVal="true";
  else
    p->myVal="false";

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }

  return(true);
}

/**
   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param val the value to set the preference to
   @param append create a new instance of this preference
*/
AREXPORT bool ArPref::setInt(int val, bool append)
{
  char buff[100];
  ArPref *p=this;

  if (append)
    p=new ArPref(*p);

  sprintf(buff, "%d", val);
  p->myVal=buff;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }

  return(true);
}

/**
   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param val the value to set the preference to
   @param append create a new instance of this preference
*/
AREXPORT bool ArPref::setDouble(double val, bool append)
{
  char buff[100];
  ArPref *p=this;

  if (append)
    p=new ArPref(*p);

  sprintf(buff, "%f", val);
  p->myVal=buff;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }

  return(true);
}

/**
   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param val the value to set the preference to
   @param append create a new instance of this preference
*/
AREXPORT bool ArPref::setString(const char *val, bool append)
{
  ArPref *p=this;

  if (append)
    p=new ArPref(*p);

  p->myVal=val;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }

  return(true);
}

/**
   Get the number of values that is contained in this preference. This is
   independent of the type of values. Use this to figure out how big of an
   array that you need to get a set of values.
*/
AREXPORT int ArPref::getSetCount()
{
  std::list<std::string> wordSet;

  ArUtil::splitString(myVal, wordSet);

  return(wordSet.size());
}

/**
   Get the value, formating it correctly. If the preference is not of
   the boolean type or not found, it will return false.
   @param boolArray the array to fill out with the values
   @param size the size of the passed in array
   @return the number of values put into the array
*/
AREXPORT int ArPref::getBoolSet(bool *boolArray, int size)
{
  int i;
  std::list<std::string> wordSet;
  std::list<std::string>::iterator iter;

  if (myValType == Boolean)
  {
    ArUtil::splitString(myVal, wordSet);
    i=0;
    for (iter=wordSet.begin(); (iter != wordSet.end()) && (i < size);
	 ++iter, ++i)
    {
      if (strcasecmp((*iter).c_str(), "true") == 0)
	boolArray[i]=true;
      else if (strcasecmp((*iter).c_str(), "false") == 0)
	boolArray[i]=false;
      else
      {
	ArLog::log(ArLog::Terse, "ArPref::getBoolSet: Unkown word in value for key '%s'. Expected a boolean value.", myName.c_str());
	return(false);
      }
    }
  }

  if (wordSet.size())
    return(i);
  else
    return(0);
}

/**
   Get the value, formating it correctly. If the preference is not of
   the integer type or not found, it will return an empty list.
   @param intArray the array to fill out with the values
   @param size the size of the passed in array
   @return the number of values put into the array
*/
AREXPORT int ArPref::getIntSet(int *intArray, int size)
{
  int i;
  std::list<std::string> wordSet;
  std::list<std::string>::iterator iter;

  if ((myValType == Integer) || (myValType == Double))
  {
    ArUtil::splitString(myVal, wordSet);
    i=0;
    for (iter=wordSet.begin(); (iter != wordSet.end()) && (i<size);
	 ++iter, ++i)
      intArray[i]=atoi((*iter).c_str());
  }

  if (wordSet.size())
    return(i);
  else
    return(0);
}

/**
   Get the value, formating it correctly. If the preference is not of
   the double type or not found, it will return an empty list.
   @param doubleArray the array to fill out with the values
   @param size the size of the passed in array
   @return the number of values put into the array
*/
AREXPORT int ArPref::getDoubleSet(double *doubleArray, int size)
{
  int i;
  std::list<std::string> wordSet;
  std::list<std::string>::iterator iter;

  if ((myValType == Integer) || (myValType == Double))
  {
    ArUtil::splitString(myVal, wordSet);
    i=0;
    for (iter=wordSet.begin(); (iter != wordSet.end()) && (i<size);
	 ++iter, ++i)
      doubleArray[i]=atof((*iter).c_str());
  }

  if (wordSet.size())
    return(i);
  else
    return(0);
}

/**
   Get the value, formating it correctly. If the preference is not of
   the string type or not found, it will return an empty list.
   @param stringArray the array to fill out with the values
   @param size the size of the passed in array
   @return the number of values put into the array
*/
AREXPORT int ArPref::getStringSet(std::string *stringArray, int size)
{
  int i;
  std::list<std::string> wordSet;
  std::list<std::string>::iterator iter;

  if (myValType == String)
  {
    ArUtil::splitString(myVal, wordSet);
    i=0;
    for (iter=wordSet.begin(); (iter != wordSet.end()) && (i<size);
	 ++iter, ++i)
      stringArray[i]=*iter;
  }

  if (wordSet.size())
    return(i);
  else
    return(0);
}

/**
   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param append create a new instance of this preference
   @param count the number of values in the parameter list
*/
AREXPORT bool ArPref::setBoolSet(bool append, int count, ...)
{
  bool val;
  std::string strVal;
  ArPref *p=this;
  va_list argList;

  if (append)
    p=new ArPref(*p);

  va_start(argList, count);

  for (int i=0; i<count; ++i)
  {
    val=va_arg(argList, int);
    if (val)
      strVal+="true ";
    else
      strVal+="false ";
  }
  p->myVal=strVal;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }
  va_end(argList);

  return(true);
}

/**
   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param append create a new instance of this preference
   @param count the number of values in the parameter list
*/
AREXPORT bool ArPref::setIntSet(bool append, int count, ...)
{
  char buff[100];
  int val;
  std::string strVal;
  ArPref *p=this;
  va_list argList;

  if (append)
    p=new ArPref(*p);

  va_start(argList, count);

  for (int i=0; i<count; ++i)
  {
    val=va_arg(argList, int);
    sprintf(buff, "%d ", val);
    strVal+=buff;
  }
  p->myVal=strVal;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }
  va_end(argList);

  return(true);
}

/**
   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param append create a new instance of this preference
   @param count the number of values in the parameter list
*/
AREXPORT bool ArPref::setDoubleSet(bool append, int count, ...)
{
  char buff[100];
  double val;
  std::string strVal;
  ArPref *p=this;
  va_list argList;

  if (append)
    p=new ArPref(*p);

  va_start(argList, count);

  for (int i=0; i<count; ++i)
  {
    val=va_arg(argList, double);
    sprintf(buff, "%f ", val);
    strVal+=buff;
  }
  p->myVal=strVal;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }
  va_end(argList);

  return(true);
}

/* Taking this out since it won't work anyways (string can't go through ...)
   If we need this someday use void *'s to chars or something

   If 'append' is true, a copy of this instance with the supplied value
   will be created and added to the ArPreferences.
   @param append create a new instance of this preference
   @param count the number of values in the parameter list
*/
/*
AREXPORT bool ArPref::setStringSet(bool append, int count, ...)
{
  std::string strVal;
  ArPref *p=this;
  va_list argList;

  if (append)
    p=new ArPref(*p);

  va_start(argList, count);

  for (int i=0; i<count; ++i)
    strVal+=va_arg(argList, std::string);
  p->myVal=strVal;

  if (p != this)
  {
    myParent->addPref(p);
    delete p;
  }
  va_end(argList);

  return(true);
}
*/
