/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include <stdarg.h>
#ifndef WIN32
#include <pwd.h>
#include <dirent.h>
#include <vector.h>
#include <string.h>
#include <algo.h>
#include <fstream.h>
#include <ctype.h>
#include <unistd.h>
#endif
#include "ArPreferences.h"
#include "ariaUtil.h"
#include "ArLog.h"


AREXPORT ArPreferences::ArPreferences(ArPrefSection *sections, int sectEnd,
				      ArPref *prefTable, int prefEnd) :
  mySectionTable(sections),
  mySects(0),
  mySectEnd(sectEnd),
  myPrefTable(prefTable),
  myPrefEnd(prefEnd),
  myError()
{
}

AREXPORT ArPreferences::~ArPreferences()
{
  clearAll();
}

AREXPORT void ArPreferences::init(bool noCreate)
{
  ArPrefSection *psect, *plast;
  int i;

  // Setup the section table.
  for (plast=0, i=0; (mySectionTable[i].myID != mySectEnd); ++i)
  {
    psect=new ArPrefSection(mySectionTable[i]);
    psect->myNext=0;
    if (plast)
    {
      psect->myPrev=plast;
      plast->myNext=psect;
    }
    else
    {
      mySects=psect;
      psect->myPrev=0;
    }
    plast=psect;
  }

  // Setup the preferences map from the pref table. Create sections if they
  // dont exist yet.
  for (i=0; myPrefTable[i].mySection != mySectEnd ; ++i)
    addPref(&myPrefTable[i]);

  // Figure out where the default file is. Also init classes that derive
  // off of this.
  findPrefFile("", noCreate);
}

AREXPORT bool ArPreferences::findPrefFile(const char *fileName, bool noCreate)
{
  myError.empty();

  // Look for the dir. Error out if cant find/open it.
  if (!findDir(fileName))
    return(false);

  // Look for the file.
  if (!findFile(fileName))
  {
    // No file found, lets try to create a default one if we are supposed to.
    // Error out if cant write.
    if (!noCreate && !save())
      return(false);
  }

  return(true);
}

AREXPORT bool ArPreferences::load(const char *fileName, bool noCreate)
{
  char buff[256];
  char *start, *end;
  char *key, *val;
  std::string section, sval, fname;
  ArPrefSection *sect=0;
  int line=0, i;
  bool gotLine=false, startBlock=false;
  ArPref *pref, *blockPref=0, *ppref;
  FILE *fp;

  // Figure out where the file is.
  if (!findPrefFile(fileName, noCreate))
    return(false);

  if (!fileName || (fileName && !*fileName))
  {
    fname=getFileName();
    fileName=fname.c_str();
  }

  myError.empty();

  fp=fopen(fileName, "r+");

  if (!fp)
  {
    std::string err="Can not open preferences file: ";
    err+=fileName;
    //perror(err.c_str());
    return(false);
  }

  while (fgets(buff, sizeof(buff), fp))
  {
    ++line;

    // get rid of spaces
    for (start=buff; *start && isspace(*start); ++start)
      ;
    // ignore any blank lines
    if (!*start)
      continue;
    // ignore comment lines
    if (*start == ';')
      continue;
    // Rip off ending newline
    for (end=start; *end && (*end != '\n') && (*end != '\r'); ++end)
      ;

    if (*end == '\n' || *end == '\r')
      *end=0;

    if (sect && (*start != '['))
    {
      if (handleListSectLine(section.c_str(), start))
	continue;
    }

    // find end of keyword
    for (end=start; *end && !isspace(*end); ++end)
      ;

    // Section identifier?
    if (*start == '[')
    {
      if ((end > start) && (*(end-1) == ']'))
	*(end-1)=0;
      else
      {
	ArLog::log(ArLog::Terse, "ArPreferences::load: Badly formated line in file '%s', line %d", fileName, line);
	continue;
      }
      section=start+1;
      sect=getSection(section.c_str());
      continue;
    }

    if (end)
    {
      *end=0;
      key = start;
      start = end+1;
    }
    else if (end > start)
    {
      key = start;
      start=end;
    }
    else
    {
      ArLog::log(ArLog::Terse, "ArPreferences::load: Badly formated line in preferences file, line %d", line);
      continue;
    }

    // ignore the spaces
    for (; *start && isspace(*start); ++start)
      ;
    // get rid of trailing spaces, newline, and trailing comments
    for (end=start; *end && (*end != ';'); ++end)
      ;
    if (end > start)
      --end;
    for (; (end >= start) && isspace(*end); --end)
      *end='\0';
    val=start;

    // Begining of a block?
    if (strcasecmp(key, "@start") == 0)
    {
      startBlock=true;
      blockPref=0;
      continue;
    }
    // Ending of a block?
    else if (strcasecmp(key, "@end") == 0)
    {
      blockPref=0;
      continue;
    }

    // Store it
    pref=getPref(section.c_str(), key);
    if (pref && *val)
    {
      // Check the validity of the value
      for (i=0; 
	   pref->myValidVals != NULL && pref->myValidVals[i] != NULL &&
	   pref->myValidVals[i][0] != '\0';
	   ++i)
      {
	if (strcasecmp(pref->myValidVals[i], val) == 0)
	  break;
	if (strcasecmp(pref->myValidVals[i], "*") == 0)
	  break;
      }
      
      if (!pref->myValidVals ||
	  (pref->myValidVals != NULL && pref->myValidVals[i] != NULL &&
	   pref->myValidVals[i][0] != '\0'))
      {
	if (startBlock)
	{
	  for (ppref=pref;
	       ppref && (ppref->myPref == pref->myPref) && ppref->myNext &&
	       (ppref->myNext->myPref == pref->myPref);
	       ppref=ppref->myNext)
	    ;
	  if (ppref != pref)
	  {
	    blockPref=pref;
	    clear(ppref->mySection, ppref->myPref, true);
	  }
	  startBlock=false;
	}
	else if (blockPref)
	{
	  pref=new ArPref(*blockPref);
	  sect->addPref(pref);
	}
	pref->myVal=val;
	gotLine=true;
      }
      else
	ArLog::log(ArLog::Terse, "ArPreferences::load: Invalid value in preferences file, line %d", line);
    }
    else if (!pref)
      ArLog::log(ArLog::Terse, "ArPreferences::load: Unknown keyword in preferences file, line %d", line);
  }
  fclose(fp);
  
  if (myError.size())
    return(false);
  else
    return(gotLine);
}

AREXPORT bool ArPreferences::save()
{
  int blockPref=-1;
  ArPrefSection *psect;
  ArPref *ppref, *pfirst, *plast;
  FILE *fp;

  fp=fopen(getFileName().c_str(), "w+");

  myError.empty();

  if (fp)
  {
    fprintf(fp, "%s\n\n", getHeader().c_str());
    for (psect=mySects; psect; psect=psect->myNext)
    {
      fprintf(fp, "[%s]\n", psect->myName.c_str());
      if (psect->myComment.size())
	fprintf(fp, "%s\n", psect->myComment.c_str());
      for (ppref=psect->myPrefs; ppref; ppref=ppref->myNext)
      {
	if (blockPref != ppref->myPref)
	{
	  if (getPrefRange(ppref->mySection, ppref->myPref, &pfirst, &plast) &&
	      (pfirst != plast))
	  {
	    // Starting a block
	    if (blockPref == -1)
	      fprintf(fp, "@start\n");
	    // Ending a block
	    else
	    {
	      blockPref=-1;
	      fprintf(fp, "@end\n");
	    }
	    blockPref=ppref->myPref;
	  }
	  // Ending a block
	  else if (blockPref != -1)
	  {
	    blockPref=-1;
	    fprintf(fp, "@end\n");
	  }
	}
	if (ppref->myComment.size())
	  fprintf(fp, "%s %s   ; %s\n", ppref->myName.c_str(),
		  ppref->myVal.c_str(), ppref->myComment.c_str());
	else
	  fprintf(fp, "%s %s\n", ppref->myName.c_str(), ppref->myVal.c_str());
      }
      // Ending a block
      if (blockPref != -1)
      {
	blockPref=-1;
	fprintf(fp, "@end\n");
      }
      fprintf(fp, "\n");
    }
    extraSave(fp);
    fclose(fp);

    return(true);
  }
  else
  {
    myError="Can not write preferences file: "+getFileName();
    perror(myError.c_str());

    return(false);
  }
}

AREXPORT int ArPreferences::getSectionNum(const char *section)
{
  ArPrefSection *sect;

  for (sect=mySects; sect; sect=sect->myNext)
  {
    if (sect->myName.compare(section) == 0)
      return(sect->myID);
  }

  return(mySectEnd);
}

AREXPORT ArPrefSection * ArPreferences::getSection(const char *section)
{
  ArPrefSection *sect;

  for (sect=mySects; sect; sect=sect->myNext)
  {
    if (sect->myName.compare(section) == 0)
      return(sect);
  }

  return(0);
}

AREXPORT ArPref * ArPreferences::getPref(int section, int pref)
{
  ArPrefSection *psect;
  ArPref *ppref;

  psect=mySects->findSect(section);
  if (psect)
  {
    ppref=psect->findPref(pref);
    if (ppref)
      return(ppref);
  }

  ArLog::log(ArLog::Terse, "ArPreferences::getPref: Failed to find: section=%d pref=%d\n",
	     section, pref);
  return(0);
}

AREXPORT ArPref * ArPreferences::getPref(const char *section, const char *pref)
{
  int sectNum;
  ArPrefSection *psect;
  ArPref *ppref;

  sectNum=getSectionNum(section);
  psect=mySects->findSect(sectNum);
  if (psect)
  {
    ppref=psect->findPref(pref);
    return(ppref);
  }

  ArLog::log(ArLog::Terse, "ArPreferences::getPref: Failed to find: section='%s' pref='%s'\n", section, pref);
  return(0);
}

AREXPORT bool ArPreferences::getPrefRange(int section, int pref,
					  ArPref **first, ArPref **last)
{
  ArPrefSection *psect;
  ArPref *ppref;

  *first=*last=0;
  psect=mySects->findSect(section);
  if (psect)
  {
    ppref=psect->findPref(pref);
    if (ppref)
    {
      *first=ppref;
      for (*last=ppref; *last && ((*last)->myPref == pref);)
      {
	if (((*last)->myNext) && ((*last)->myNext->myPref == pref))
	  *last=(*last)->myNext;
	else
	  break;
      }
      if (*last)
	return(true);
      *first=*last=0;
    }
  }

  ArLog::log(ArLog::Terse, "ArPreferencesgetPrefRange: Failed to find: section=%d pref=%d\n",
	     section, pref);
  return(false);
}

AREXPORT void ArPreferences::addPref(ArPref *pref)
{
  ArPrefSection *psect;
  ArPref *ppref;

  psect=mySects->findSect(pref->mySection);
  if (psect)
  {
    ppref=new ArPref(*pref);
    ppref->myParent=this;
    psect->addPref(ppref);
  }
}

AREXPORT bool ArPreferences::clear(int section, int pref, bool all)
{
  ArPrefSection *psect;
  ArPref *ppref=0, *ptmp;

  psect=mySects->findSect(section);
  if (psect)
    ppref=psect->findPref(pref);
  else
    return(false);

  if (ppref && !all)
  {
    ppref->myVal="";
    return(true);
  }
  else if (ppref && all)
  {
    ppref->myVal="";
    for (ppref=ppref->myNext; ppref && (ppref->myPref == pref);)
    {
      ptmp=ppref;
      ppref=ppref->myNext;
      if (ppref)
	ppref->myPrev=ptmp->myPrev;
      if (ptmp->myPrev)
	ptmp->myPrev->myNext=ppref;
      delete ptmp;
    }
    return(true);
  }
  else
    return(false);
}

AREXPORT bool ArPreferences::clear(const char *section, const char *pref,
				   bool all)
{
  int sectNum;
  ArPrefSection *psect;
  ArPref *ppref;

  sectNum=getSectionNum(section);
  psect=mySects->findSect(sectNum);
  ppref=psect->findPref(pref);
  return(clear(sectNum, ppref->myPref));
}

AREXPORT void ArPreferences::clearAll()
{
  ArPrefSection *psect, *pstmp;
  ArPref *ppref, *pptmp;

  for (psect=mySects; psect;)
  {
    for (ppref=psect->myPrefs; ppref;)
    {
      pptmp=ppref;
      ppref=ppref->myNext;
      delete pptmp;
    }
    pstmp=psect;
    psect=psect->myNext;
    delete pstmp;
  }
}

AREXPORT std::string ArPreferences::getFileName()
{
  std::string ret;

  ret=getDir();
  ArUtil::appendSlash(ret);
  ret+=getFile();

  return(ret);
}
