/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include <time.h>

#include "ArRobot.h"
#include "ArLog.h"
#include "ArDeviceConnection.h"
#include "ArSerialConnection.h"
#include "ArLogFileConnection.h"
#include "ariaUtil.h"
#include "ArSocket.h"
#include "ArCommands.h"
#include "ArRobotTypes.h"
#include "ArSignalHandler.h"
#include "ArPriorityResolver.h"
#include "ArAction.h"
#include "ariaInternal.h"


/**
   @param doStateReflection whether the robot should use direct motion
   command reflection or simply send commands when it gets them.  If
   stateReflection is on then when one of the movement commands is
   given it stores the value and then sends it at the appropriate spot
   in the cycle.  If stateReflection isn't on, then a pulse will be
   sent every other cycle to make sure the watchdog timer doesn't kick
   off.

   @param normalInit whether the robot should initializes its
   structures or the calling program will take care of it.  No one
   will probalby ever use this value, since if they are doing that
   then overriding will probably be more useful, but there it is.

   @param doSigHandle do normal signal handling and have this robot
   instance stopRunning() when the program is signaled

**/

AREXPORT ArRobot::ArRobot(const char *name, bool doStateReflection, 
			  bool doSigHandle, bool normalInit) :
  myMotorPacketCB(this, &ArRobot::processMotorPacket),
  myEncoderPacketCB(this, &ArRobot::processEncoderPacket),
  myIOPacketCB(this, &ArRobot::processIOPacket),
  myPacketHandlerCB(this, &ArRobot::packetHandler),
  myActionHandlerCB(this, &ArRobot::actionHandler),
  myStateReflectorCB(this, &ArRobot::stateReflector),
  myRobotLockerCB(this, &ArRobot::robotLocker),
  myRobotUnlockerCB(this, &ArRobot::robotUnlocker),
  myKeyHandlerExitCB(this, &ArRobot::keyHandlerExit)
{
  setName(name);
  myStateReflecting = doStateReflection;
  myGlobalPose.setPose(0, 0, 0);

  myKeyHandler = NULL;
  myKeyHandlerCB = NULL;

  myConn = NULL;

  myOwnTheResolver = false;

  myBlockingConnectRun = false;
  myAsyncConnectFlag = false;

  myCycleTime = 100;
  myConnectionCycleMultiplier = 6;
  myTimeoutTime = 8000;
  myCounter = 1;
  myResolver = NULL;
  myNumSonar = 0;

  myEncoderCorrectionCB = NULL;

  myCycleChained = true;

  myMoveDoneDist = 40;
  myHeadingDoneDiff = 3;

  reset();
  if (normalInit)
    init();
  
  mySyncLoop.setRobot(this);

  if (doSigHandle)
    Aria::addRobot(this);
}



AREXPORT ArRobot::~ArRobot(void)
{
  ArResolver::ActionMap::iterator it;

  stopRunning();
  delete mySyncTaskRoot;
  ArUtil::deleteSetPairs(mySonars.begin(), mySonars.end());
  Aria::delRobot(this);

  if (myKeyHandlerCB != NULL)
    delete myKeyHandlerCB;

  for (it = myActions.begin(); it != myActions.end(); ++it)
  {
    (*it).second->setRobot(NULL);
  }
}

/**
   Sets up the packet handlers, sets up the sync list and makes the default
   priority resolver.
**/
AREXPORT void ArRobot::init(void)
{
  setUpPacketHandlers();
  setUpSyncList();
  myOwnTheResolver = true;
  myResolver = new ArPriorityResolver;
}

/**
   This starts the list of tasks to be run through until stopped.
   This function doesn't return until something calls stop on this
   instance.  

   @param stopRunIfNotConnected if true, the run will return if there
   is no connection to the robot at any given point, this is good for
   one-shot programs... if it is false the run won't return unless
   stop is called on the instance
**/
AREXPORT void ArRobot::run(bool stopRunIfNotConnected)
{
  if (mySyncLoop.getRunning())
  {
    ArLog::log(ArLog::Terse, 
	       "The robot is already running, cannot run it again.");
    return;
  }
  mySyncLoop.setRunning(true);
  mySyncLoop.stopRunIfNotConnected(stopRunIfNotConnected);
  mySyncLoop.runThread(NULL);
}

/**
   @return true if the robot is currently running in a run or runAsync, 
   otherwise false
**/

AREXPORT bool ArRobot::isRunning(void)
{
  return mySyncLoop.getRunning();
}

/**
   This starts a new thread then has runs through the tasks until stopped.
   This function doesn't return until something calls stop on this instance.
   This function returns immediately
   @param stopRunIfNotConnected if true, the run will stop if there is no 
   connection to the robot at any given point, this is good for one-shot 
   programs... if it is false the run won't stop unless stop is called on the
   instance
**/
AREXPORT void ArRobot::runAsync(bool stopRunIfNotConnected)
{
  if (mySyncLoop.getRunning())
  {
    ArLog::log(ArLog::Terse, 
	       "The robot is already running, cannot run it again.");
    return;
  }
  mySyncLoop.stopRunIfNotConnected(stopRunIfNotConnected);
  // Joinable, but do NOT lower priority. The robot thread is the most
  // important one around.
  mySyncLoop.create(true, false);
}

/**
   This stops this robot from running anymore.  If it is stopping from a 
   runAsync it will cause the thread to return (exit), if it is running 
   from a normal run, it will just cause the run function to return.
   @param doDisconnect Disconnect from the robot. Defaulted to true.
**/
AREXPORT void ArRobot::stopRunning(bool doDisconnect)
{
  if (myKeyHandler != NULL)
    myKeyHandler->restore();
  mySyncLoop.stopRunning();
  myBlockingConnectRun = false;
  if (doDisconnect && isConnected())
  {
    waitForRunExit();
    disconnect();
  }
  wakeAllWaitingThreads();
}

AREXPORT void ArRobot::setUpSyncList(void)
{
  mySyncTaskRoot = new ArSyncTask("SyncTasks");
  mySyncTaskRoot->addNewLeaf("Packet Handler", 85, &myPacketHandlerCB);
  mySyncTaskRoot->addNewLeaf("Robot Locker", 70, &myRobotLockerCB);
  mySyncTaskRoot->addNewBranch("Sensor Interp", 65);
  mySyncTaskRoot->addNewLeaf("Action Handler", 55, &myActionHandlerCB);
  mySyncTaskRoot->addNewLeaf("State Reflector", 45, &myStateReflectorCB);
  mySyncTaskRoot->addNewBranch("User Tasks", 25);
  mySyncTaskRoot->addNewLeaf("Robot Unlocker", 20, &myRobotUnlockerCB);
}


AREXPORT void ArRobot::setUpPacketHandlers(void)
{
  addPacketHandler(&myMotorPacketCB, ArListPos::FIRST);
  addPacketHandler(&myEncoderPacketCB, ArListPos::LAST);
  addPacketHandler(&myIOPacketCB, ArListPos::LAST);
}

void ArRobot::reset(void)
{
  myInterpolation.reset();

  myFirstEncoderPose = true;
  //myEncoderPose.setPose(0, 0, 0);
  //myEncoderPoseTaken.setToNow();
  //myRawEncoderPose.setPose(0, 0, 0);
  //myGlobalPose.setPose(0, 0, 0);
  //myEncoderTransform.setTransform(myEncoderPose, myGlobalPose);


  myLeftVel = 0;
  myRightVel = 0;
  myBatteryVoltage = 0;
  myStallValue = 0;
  myControl = 0;
  myFlags = 0;
  myCompass = 0;
  myAnalogPortSelected = 0;
  myAnalog = 0;
  myDigIn = 0;
  myDigOut = 0;
  myIOAnalogSize = 0;
  myIODigInSize = 0;
  myIODigOutSize = 0;
  myLastIOPacketReceivedTime.setSec(0);
  myLastIOPacketReceivedTime.setMSec(0);
  myVel = 0;
  myRotVel = 0;
  myLastX = 0;
  myLastY = 0;
  myLastTh = 0;
  mySentPulse = false;
  myIsConnected = false;
  
  myTransVal = 0;
  myTransVal2 = 0;
  myTransType = TRANS_NONE;
  myLastTransVal = 0;
  myLastTransVal2 = 0;
  myLastTransType = TRANS_NONE;
  myLastTransSent.setToNow();
  myActionTransSet = false;

  myRotVal = 0;
  myLastRotVal = 0;
  myRotType = ROT_NONE;
  myLastRotType = ROT_NONE;
  myLastRotSent.setToNow();
  myActionRotSet = false;

  myLastPulseSent.setToNow();

  myDirectPrecedenceTime = 0;
  myStateReflectionRefreshTime = 500;

  myActionDesired.reset();

  myMotorPacCurrentCount = 0;
  myMotorPacCount = 0;
  myTimeLastMotorPacket = 0;

  mySonarPacCurrentCount = 0;
  mySonarPacCount = 0;
  myTimeLastSonarPacket = 0;

}

/**
   Sets the connection this instance uses to the actual robot.  This is where
   commands will be sent and packets will be received from

   @param connection The deviceConnection to use for this robot

   @see ArDeviceConnection, ArSerialConnection, ArTcpConnection
**/
AREXPORT void ArRobot::setDeviceConnection(ArDeviceConnection *connection)
{
  myConn = connection;
  mySender.setDeviceConnection(myConn);
  myReceiver.setDeviceConnection(myConn);
}

/**
   Gets the connection this instance uses to the actual robot.  This is where
   commands will be sent and packets will be received from
   @return the deviceConnection used for this robot
   @see ArDeviceConnection
   @see ArSerialConnection
   @see ArTcpConnection
**/
AREXPORT ArDeviceConnection *ArRobot::getDeviceConnection(void)
{
  return myConn;
}

/**
   Sets the number of seconds to go without a response from the robot
   until it is assumed that the connection with the robot has been
   broken and the disconnect on error events will happen.  Note that
   this will only happen with the default packet handler.

   @param seconds if seconds is 0 then the connection timeout feature
   will be disabled, otherwise disconnect on error will be triggered
   after this number of seconds...  
**/
AREXPORT void ArRobot::setConnectionTimeoutTime(int mSecs)
{
  if (mSecs > 0)
    myTimeoutTime = mSecs;
  else
    myTimeoutTime = 0;
}

/**
   Gets the number of seconds to go without response from the robot
   until it is assumed tha tthe connection with the robot has been
   broken and the disconnect on error events will happen.

**/
AREXPORT int ArRobot::getConnectionTimeoutTime(void)
{
  return myTimeoutTime;
}

/**
   This gets the ArTime that the last packet was received at

   @return the time the last packet was received
**/
AREXPORT ArTime ArRobot::getLastPacketTime(void)
{
  return myLastPacketReceivedTime;
}
/** 
    Sets up the robot to connect, then returns, but the robot must be
    running (ie from runAsync) before you do this.  Also this will
    fail if the robot is already connected.  If you want to know what
    happened because of the connect then look at the callbacks.  NOTE,
    this will not lock robot before setting values, so you MUST lock
    the robot before you call this function and unlock the robot after
    you call this function.  If you fail to lock the robot, you'll may
    wind up with wierd behavior.  Other than the aspect of blocking or
    not the only difference between async and blocking connects (other
    than the blocking) is that async is run every robot cycle, whereas
    blocking runs as fast as it can... also blocking will try to
    reconnect a radio modem if it looks like it didn't get connected
    in the first place, so blocking can wind up taking 10 or 12
    seconds to decide it can't connect, whereas async doesn't try hard
    at all to reconnect the radio modem (beyond its first try) (under
    the assumption the async connect is user driven, so they'll just
    try again, and so that it won't mess up the sync loop by blocking
    for so long).

    @return true if the robot is running and the robot will try to
    connect, false if the robot isn't running so won't try to connect
    or if the robot is already connected

    @see addConnectCB

    @see addFailedConnectCB

    @see runAsync
**/

AREXPORT bool ArRobot::asyncConnect(void)
{
  if (!mySyncLoop.getRunning() || isConnected())
    return false;
  
  myAsyncConnectFlag = true;
  myBlockingConnectRun = false;
  myAsyncConnectState = -1;
  return true;
}

/** 
    Connects to the robot, returning only when a connection has been
    made or it has been established a connection can't be made.  This
    connection usually is fast, but can take up to 30 seconds if the
    robot is in a wierd state (this is not often).  If the robot is
    connected via ArSerialConnection then the connect will also
    connect the radio modems.  Upon a successful connection all of the
    Connection Callback Functors that have been registered will be
    called.  NOTE, this will lock the robot before setting values, so
    you MUST not have the robot locked from where you call this
    function.  If you do, you'll wind up in a deadlock.  This behavior
    is there because otherwise you'd have to lock the robot before
    calling this function, and normally blockingConnect will be called
    from a seperate thread, and that thread won't be doing anything
    else with the robot at that time.  Other than the aspect of
    blocking or not the only difference between async and blocking
    connects (other than the blocking) is that async is run every
    robot cycle, whereas blocking runs as fast as it can... also
    blocking will try to reconnect a radio modem if it looks like it
    didn't get connected in the first place, so blocking can wind up
    taking 10 or 12 seconds to decide it can't connect, whereas async
    doesn't try hard at all to reconnect the radio modem (under the
    assumption the async connect is user driven, so they'll just try
    again, and so that it won't mess up the sync loop by blocking for
    so long).

    @return true if a connection could be made, false otherwise 

**/

AREXPORT bool ArRobot::blockingConnect(void)
{
  int ret = 0;

  lock();
  myBlockingConnectRun = true;
  myAsyncConnectState = -1;
  while ((ret = asyncConnectHandler(true)) == 0 && myBlockingConnectRun)
    ArUtil::sleep(ArMath::roundInt(getCycleTime() * .80));
  unlock();
  if (ret == 1)
    return true;
  else
    return false;
}

/**
   This is an internal function that is used both for async connects and 
   blocking connects use to connect.  It does about the same thing for both,
   and it should only be used by asyncConnect and blockingConnect really.
   But here it is.  The only difference between when its being used 
   by blocking/async connect is that in blocking mode if it thinks there
   may be problems with the radio modem it pauses for two seconds trying
   to deal with this... whereas in async mode it tries to deal with this in a
   simpler way.
   @param tryHarderToConnect if this is true, then if the radio modems look
   like they aren't working, it'll take about 2 seconds to try and connect 
   them, whereas if its false, it'll do a little try, but won't try very hard
   @return 0 if its still trying to connect, 1 if it connected, 2 if it failed
**/

AREXPORT int ArRobot::asyncConnectHandler(bool tryHarderToConnect)
{
  int ret = 0;
  ArRobotPacket *packet;
  ArRobotPacket *tempPacket = NULL;
  char robotSubType[255];
  int i;
  int len;
  std::string str;
  ArTime endTime;
  int timeToWait;

  endTime.setToNow();
  endTime.addMSec(getCycleTime()*myConnectionCycleMultiplier);
  
  // if this is -1, then we're doing initialization stuff, then returning
  if (myAsyncConnectState == -1)
  {
    myAsyncConnectNoPacketCount = 0;
    myAsyncConnectTimesTried = 0;
    reset();
    if (myConn == NULL)
    {
      ArLog::log(ArLog::Terse, "Cannot connect, no connection has been set.");
      failedConnect();
      return 2;
    }
    
    if (myConn->getStatus() != ArDeviceConnection::STATUS_OPEN) 
    {
      if (!myConn->openSimple())
      {
	/*str = myConn->getStatusMessage(myConn->getStatus());
	  ArLog::log(ArLog::Terse, "Trying to connect to robot but connection not opened, it is %s", str.c_str());*/
        ArLog::log(ArLog::Terse, 
                   "Could not connect, because open on the device connection failed.");
        failedConnect();
        return 2;
      }
    }

    // check for log file connection: just return success
    if (dynamic_cast<ArLogFileConnection *>(myConn))
    {
      ArLogFileConnection *con = dynamic_cast<ArLogFileConnection *>(myConn);
      myRobotName = con->myName;
      myRobotType = con->myType;
      myRobotSubType = con->mySubtype;
      if (con->havePose)        // we know where to move
        moveTo(con->myPose);
      setCycleChained(false);   // don't process packets all at once
      madeConnection();
      return 1;
    }


    if (dynamic_cast<ArSerialConnection *>(myConn))
    {
      myConn->write("WMS2\15", strlen("WMS2\15"));
    }
    // here we're flushing out all previously received packets
    while ( endTime.mSecTo() > 0 && myReceiver.receivePacket(0) != NULL);

    myAsyncConnectState = 0;
    return 0;
  }

  // here we've gone beyond the initialization, so read set a time limit,
  // read in one packet, then if its a bad packet type, read in all the 
  // packets there are to read... if its a good packet, continue with sequence
  myAsyncConnectTimesTried++;
  ArLog::log(ArLog::Normal, "Syncing %d", myAsyncConnectState);
  mySender.com(myAsyncConnectState);
  //  packet = myReceiver.receivePacket(endTime.mSecTo());
  packet = myReceiver.receivePacket(3000);
  
  if (packet != NULL) 
  {
    ret = packet->getID();
    //printf("Got a packet %d\n", ret);
	
    if (ret == 50)
    {
      ArLog::log(ArLog::Normal, "Attempting to close previous connection.");
      comInt(ArCommands::CLOSE,1);
      while (endTime.mSecTo() > 0)
      {
	timeToWait = endTime.mSecTo();
	if (timeToWait < 0)
	  timeToWait = 0;
	tempPacket = myReceiver.receivePacket(timeToWait);
	if (tempPacket != NULL)
	  ArLog::log(ArLog::Verbose, "Got in another packet!");
	if (tempPacket != NULL && tempPacket->getID() == 50)
	  comInt(ArCommands::CLOSE,1);
      }
      myAsyncConnectState = 0;
    } 
    else if (ret == 255)
    {
      ArLog::log(ArLog::Normal, "Attempting to correct syncCount");
      /*
      while (endTime.mSecTo() > 0)
      {
	timeToWait = endTime.mSecTo();
	if (timeToWait < 0)
	  timeToWait = 0;
	tempPacket = myReceiver.receivePacket(timeToWait);
	if (tempPacket != NULL)
	  ArLog::log(ArLog::Verbose, "Got in another packet!");
      }
      */
      while ((tempPacket = myReceiver.receivePacket(0)) != NULL);

      if (tempPacket != NULL && tempPacket->getID() == 0)
	myAsyncConnectState = 1;
      else
	myAsyncConnectState = 0;
      return 0;
    } 
    else if  (ret != myAsyncConnectState++)
    {
      myAsyncConnectState = 0;
    }
  }
  else 
  {
    ArLog::log(ArLog::Normal, "No packet.");
    myAsyncConnectNoPacketCount++;
    if ((myAsyncConnectNoPacketCount > 8 
	 && myAsyncConnectNoPacketCount >= myAsyncConnectTimesTried)
	|| (myAsyncConnectNoPacketCount > 15))
    {
      ArLog::log(ArLog::Terse, "Could not connect, no robot responding.");
      failedConnect();
      return 2;
    }
    if (myAsyncConnectNoPacketCount >= 4)
    {
      if (tryHarderToConnect && dynamic_cast<ArSerialConnection *>(myConn))
      {
	ArLog::log(ArLog::Normal, 
		   "Radio modem may not connected, trying to connect it.");
	ArUtil::sleep(1000);
	myConn->write("|||\15", strlen("!!!\15"));
	ArUtil::sleep(60);
	myConn->write("WMN\15", strlen("WMN\15"));
	ArUtil::sleep(60);
	myConn->write("WMS2\15", strlen("WMS2\15"));
	ArUtil::sleep(1000);
      } 
      else if (dynamic_cast<ArSerialConnection *>(myConn))
      {
	ArLog::log(ArLog::Normal,
		   "Radio modem may not connected, trying to connect it.");
	myConn->write("WMS2\15", strlen("WMS2\15"));
	ArUtil::sleep(60);
      }
    }

    if (myAsyncConnectNoPacketCount > 1)
    {
      ArLog::log(ArLog::Normal,
		 " Robot may be connected but not open, trying to dislodge.");
      mySender.comInt(ArCommands::OPEN,1);
    }
    
    myAsyncConnectState = 0;
  }
  if (myAsyncConnectState > 2 && packet != NULL) 
  {
    char nameBuf[512];
    ArLog::log(ArLog::Terse, "Connected to robot.");
    packet->bufToStr(nameBuf, 512);
    myRobotName = nameBuf;
    ArLog::log(ArLog::Normal, "Name: %s", myRobotName.c_str());
    packet->bufToStr(nameBuf, 512);
    myRobotType = nameBuf;
    ArLog::log(ArLog::Normal, "Type: %s", myRobotType.c_str());
    packet->bufToStr(nameBuf, 512);
    myRobotSubType = nameBuf;
    strcpy(robotSubType, myRobotSubType.c_str());
    len = strlen(robotSubType);
    for (i = 0; i < len; i++)
      robotSubType[i] = tolower(robotSubType[i]);
    myRobotSubType = robotSubType;
    ArLog::log(ArLog::Normal, "Subtype: %s", myRobotSubType.c_str());
    ArUtil::sleep(getCycleTime());
    mySender.comInt(ArCommands::OPEN,1);
    
    madeConnection();
    return 1;
  }
  if (myAsyncConnectTimesTried > 50)
  {
    failedConnect();
    return 2;
  }
  return 0;
}

/**
   @return true if the file could be loaded, false otherwise
 **/
AREXPORT bool ArRobot::loadParamFile(const char *file)
{
  ArRobotParamFile *paramFile;
  paramFile = new ArRobotGeneric("");;
  if (!paramFile->load(file, true))
  {
    ArLog::log(ArLog::Normal, "ArRobot::loadParamFile: Could not find file '%s' to load.", file);
    return false;
  }
  processParamFile(paramFile);
  return true;
}

AREXPORT void ArRobot::processParamFile(ArRobotParamFile *paramFile)
{
  myParams.init(paramFile);
  for (int i = 0; i < myParams.getNumSonar(); ++i)
  {
    //printf("sonar %d %d %d %d\n", i, myParams.getSonarX(i),
    //myParams.getSonarY(i), myParams.getSonarTh(i));
    if (mySonars.find(i) == mySonars.end())
    {
      mySonars[i] = new ArSensorReading(myParams.getSonarX(i),
					myParams.getSonarY(i), 
					myParams.getSonarTh(i));
    }
    else
    {
      mySonars[i]->resetSensorPosition(myParams.getSonarX(i),
				      myParams.getSonarY(i), 
				      myParams.getSonarTh(i));
    }
    if ((i + 1) > myNumSonar)
      myNumSonar = i + 1;
  }
  myRobotType = myParams.getClass();
  myRobotSubType = myParams.getSubClass();
  myMaxTransVel = myParams.getMaxVelocity();
  myMaxRotVel = myParams.getMaxRotVelocity();
}

AREXPORT void ArRobot::madeConnection(void)
{
  std::list<ArFunctor *>::iterator it;
  ArRobotParamFile *paramFile;
  std::string subTypeParamName;
  std::string paramFileName;
  bool loadedSubTypeParam;
  bool loadedNameParam;
  bool hadDefault = true;

  myIsConnected = true;
  myAsyncConnectFlag = false;
  myBlockingConnectRun = false;
  // Find the robot parameters to load and get them into the structure we have

  if (ArUtil::strcmp(myRobotSubType, "p2dx") == 0)
    paramFile = new ArRobotP2DX;
  else if (ArUtil::strcmp(myRobotSubType, "p2ce") == 0)
    paramFile = new ArRobotP2CE;
  else if (ArUtil::strcmp(myRobotSubType, "p2de") == 0)
    paramFile = new ArRobotP2DXe;
  else if (ArUtil::strcmp(myRobotSubType, "p2df") == 0)
    paramFile = new ArRobotP2DF;
  else if (ArUtil::strcmp(myRobotSubType, "p2d8") == 0)
    paramFile = new ArRobotP2D8;
  else if (ArUtil::strcmp(myRobotSubType, "amigo") == 0)
    paramFile = new ArRobotAmigo;
  else if (ArUtil::strcmp(myRobotSubType, "p2at") == 0)
    paramFile = new ArRobotP2AT;
  else if (ArUtil::strcmp(myRobotSubType, "p2at8") == 0)
    paramFile = new ArRobotP2AT8;
  else if (ArUtil::strcmp(myRobotSubType, "p2it") == 0)
    paramFile = new ArRobotP2IT;
  else if (ArUtil::strcmp(myRobotSubType, "p2pb") == 0)
    paramFile = new ArRobotP2PB;
  else if (ArUtil::strcmp(myRobotSubType, "p2pp") == 0)
    paramFile = new ArRobotP2PP;
  else if (ArUtil::strcmp(myRobotSubType, "perfpb") == 0)
    paramFile = new ArRobotPerfPB;
  else if (ArUtil::strcmp(myRobotSubType, "pion1m") == 0)
    paramFile = new ArRobotPion1M;
  else if (ArUtil::strcmp(myRobotSubType, "pion1x") == 0)
    paramFile = new ArRobotPion1X;
  else if (ArUtil::strcmp(myRobotSubType, "psos1m") == 0)
    paramFile = new ArRobotPsos1M;
  else if (ArUtil::strcmp(myRobotSubType, "psos43m") == 0)
    paramFile = new ArRobotPsos43M;
  else if (ArUtil::strcmp(myRobotSubType, "psos1x") == 0)
    paramFile = new ArRobotPsos1X;
  else if (ArUtil::strcmp(myRobotSubType, "pionat") == 0)
    paramFile = new ArRobotPionAT;
  else if (ArUtil::strcmp(myRobotSubType, "mappr") == 0)
    paramFile = new ArRobotMapper;
  else if (ArUtil::strcmp(myRobotSubType, "powerbot") == 0)
    paramFile = new ArRobotPowerBot;
  else
  {
    hadDefault = false;
    paramFile = new ArRobotGeneric(myRobotName.c_str());
  }
  
  
  paramFile->init();
  // load up the param file for the subtype
  paramFileName = Aria::getDirectory();
  paramFileName += "params/";
  paramFileName += myRobotSubType;
  paramFileName += ".p";
  if ((loadedSubTypeParam = paramFile->load(paramFileName.c_str(), true)))
      ArLog::log(ArLog::Normal, 
		 "Loaded robot parameters from %s.p", 
		 myRobotSubType.c_str());
  /* If the above line was replaced with this one line
     paramFile->load(); 
     then the sonartest (and lots of other stuff probably) would break
  */
  // then the one for the particular name, if we can
  paramFileName = Aria::getDirectory();
  paramFileName += "params/";
  paramFileName += myRobotName;
  paramFileName += ".p";
  if ((loadedNameParam = paramFile->load(paramFileName.c_str(), true)))
  {
    if (loadedSubTypeParam)
      ArLog::log(ArLog::Normal, 
 "Loaded robot parameters from %s.p on top of %s.p robot parameters", 
		 myRobotName.c_str(), myRobotSubType.c_str());
    else
      ArLog::log(ArLog::Normal, "Loaded robot parameters from %s.p", 
		 myRobotName.c_str());
  }
   
  if (!loadedSubTypeParam && !loadedNameParam)
  {
    if (hadDefault)
      ArLog::log(ArLog::Normal, "Using default parameters for a %s robot", 
		 myRobotSubType.c_str());
    else
      ArLog::log(ArLog::Terse, "Error: Have no parameters for this robot");
  }

  processParamFile(paramFile);

  if (myParams.getRequestIOPackets())
    comInt(ArCommands::IOREQUEST, 2);

  for (it = myConnectCBList.begin(); it != myConnectCBList.end(); it++)
    (*it)->invoke();
  myLastPacketReceivedTime.setToNow();

  wakeAllConnWaitingThreads();
}

AREXPORT void ArRobot::failedConnect(void)
{
  std::list<ArFunctor *>::iterator it;  
  
  myAsyncConnectFlag = false;
  myBlockingConnectRun = false;
  ArLog::log(ArLog::Terse, "Failed to connect to robot.");
  myIsConnected = false;
  for (it = myFailedConnectCBList.begin(); 
       it != myFailedConnectCBList.end(); 
       it++)
    (*it)->invoke();

  if (myConn != NULL)
    myConn->close();
  wakeAllConnOrFailWaitingThreads();
}

/** 
    Disconnects from a robot.  This also calls of the DisconnectNormally 
    Callback Functors if the robot was actually connected to a robot when 
    this member was called.
    @return true if not connected to a robot (so no disconnect can happen, but 
    it didn't failed either), also true if the command could be sent to the 
    robot (ie connection hasn't failed)
**/

AREXPORT bool ArRobot::disconnect(void)
{
  std::list<ArFunctor *>::iterator it;  
  bool ret;

  if (!myIsConnected)
    return true;
  
  ArLog::log(ArLog::Terse, "Disconnecting from robot.");
  myIsConnected = false;
  for (it = myDisconnectNormallyCBList.begin(); 
       it != myDisconnectNormallyCBList.end(); 
       it++)
    (*it)->invoke();
  ret = mySender.comInt(2, 1);
  ArUtil::sleep(1000);
  if (ret == true)
  {
    if (myConn != NULL)
    {
      ret = myConn->close();
    } 
    else
      ret = false;
  } 
  else if (myConn != NULL)
    myConn->close();
    
  return ret;
}

AREXPORT void ArRobot::dropConnection(void)
{
  std::list<ArFunctor *>::iterator it;  

  if (!myIsConnected)
    return;

  ArLog::log(ArLog::Terse, "Lost connection to the robot because of error.");
  myIsConnected = false;
  for (it = myDisconnectOnErrorCBList.begin(); 
       it != myDisconnectOnErrorCBList.end(); 
       it++)
    (*it)->invoke();

  if (myConn != NULL)
    myConn->close();
  return;
}

/**
   Stops the robot, by telling it to have a translational velocity and 
   rotational velocity of 0.  Also note that if you are using actions, this 
   will cause the actions to be ignored until the direct motion precedence 
   timeout has been exceeded or clearDirectMotion is called.
   @see setDirectMotionPrecedenceTime
   @see getDirectMotionPrecedenceTime
   @see clearDirectMotion
**/
AREXPORT void ArRobot::stop(void)
{
  comInt(ArCommands::VEL, 0);
  comInt(ArCommands::RVEL, 0);
  setVel(0);
  setRotVel(0);
}

/**
   Sets the velocity of the robot, if the constructor was created with state
   reflecting enabled then it caches this value, and sends it during the next
   cycle.  If state reflecting is disabled it sends this value instantly.
   @param velocity the desired translational velocity of the robot
**/
AREXPORT void ArRobot::setVel(double velocity)
{
  if (!myStateReflecting)
    comInt(ArCommands::VEL, ArMath::roundInt(velocity));
  else
  {
    myTransType = TRANS_VEL;
    myTransVal = velocity;
    myTransVal2 = 0;
    myTransSetTime.setToNow();
  }
}

/** Sets the velocity of each of the wheels on the robot independently.  
    if the constructor was created with state reflecting enabled then it 
    caches this value, and sends it during the next cycle.  If state
    reflecting is disabled it sends this value instantly.  Note that this
    cancels both translational velocity AND rotational velocity, and is 
    canceled by any of the other direct motion commands.
    @param leftVelocity the desired velocity of the left wheel
    @param rightVelocity the desired velocity of the right wheel
**/
AREXPORT void ArRobot::setVel2(double leftVelocity, double rightVelocity)
{
  char leftVel, rightVel;

  if (!myStateReflecting)
  {
    if (ArMath::roundInt(leftVelocity/myParams.getVel2Divisor()) > 127)
      leftVel = 127;
    else if (ArMath::roundInt(leftVelocity/myParams.getVel2Divisor()) < -127)
      leftVel = -127;
    else 
      leftVel = ArMath::roundInt(leftVelocity/myParams.getVel2Divisor());
    if (ArMath::roundInt(rightVelocity/myParams.getVel2Divisor()) > 127)
      rightVel = 127;
    else if (ArMath::roundInt(rightVelocity/myParams.getVel2Divisor()) < -127)
      rightVel = -127;
    else 
      rightVel = ArMath::roundInt(rightVelocity/myParams.getVel2Divisor());
    com2Bytes(ArCommands::VEL2, leftVel, rightVel);
  }
  else
  {
    myTransType = TRANS_VEL2;
    myTransVal = leftVelocity;
    myTransVal2 = rightVelocity;
    myRotType = ROT_IGNORE;
    myRotVal = 0;
    myTransSetTime.setToNow();
  }
}

/**
   Tells the robot to move the specified distance forward/backwards, if the
   constructor was created with state reflecting enabled then it caches this
   value, and sends it during the next cycle.  If state reflecting is disabled
   it sends this value instantly.
   @param distance the distance for the robot to move
**/
AREXPORT void ArRobot::move(double distance)
{
  if (!myStateReflecting)
    comInt(ArCommands::MOVE, ArMath::roundInt(distance));
  else
  {
    myTransType = TRANS_DIST_NEW;
    myTransDistStart = getPose();
    myTransVal = distance;
    myTransVal2 = 0;
    myTransSetTime.setToNow();
  }
}

/**
   Determines if a move command is finished, to within a small
   distance.  If delta = 0 (default), the delta distance is what was
   set with setMoveDoneDist, you can get the distnace with
   getMoveDoneDist

   @param delta how close to the goal distance the robot must be

   @return true if the robot has finished the distance given in a move
   command or if the robot is no longer in a move mode (because its
   now running off of actions, setVel, or setVel2 was called).
**/
AREXPORT bool ArRobot::isMoveDone(double delta)
{
  if (fabs(delta) < 0.001)
    delta = myMoveDoneDist;
  if (myTransType != TRANS_DIST && myTransType != TRANS_DIST_NEW)
    return true;		// no distance command operative
  if (myTransDistStart.findDistanceTo(getPose()) <
      fabs(myTransVal) - delta)
    return false;
  return true;
}


/**
   Determines if a setHeading command is finished, to within a small
   distance.  If delta = 0 (default), the delta distance is what was
   set with setHeadingDoneDiff, you can get the distnace with
   getHeadingDoneDiff

   @param delta how close to the goal distance the robot must be

   @return true if the robot has achieved the heading given in a move
   command or if the robot is no longer in heading mode mode (because
   its now running off of actions, setDHeading, or setRotVel was called).
**/
AREXPORT bool ArRobot::isHeadingDone(double delta)
{
  if (fabs(delta) < 0.001)
    delta = myHeadingDoneDiff;
  if (myRotType != ROT_HEADING)
    return true;		// no heading command operative
  if(fabs(ArMath::subAngle(getTh(), myRotVal)) > delta)
    return false;
  return true;
}



/**
   Sets the heading of the robot, if the constructor was created with state
   reflecting enabled then it caches this value, and sends it during the next
   cycle.  If state reflecting is disabled it sends this value instantly.
   @param heading the desired heading of the robot
**/
AREXPORT void ArRobot::setHeading(double heading)
{
  if (!myStateReflecting)
  {
    heading = ArMath::addAngle(heading, 
			       ArMath::subAngle(myRawEncoderPose.getTh(),
						myEncoderPose.getTh()));
    comInt(ArCommands::HEAD, ArMath::roundInt(heading));
  }
  else
  {
    myRotVal = heading;
    myRotType = ROT_HEADING;
    myRotSetTime.setToNow();
    if (myTransType == TRANS_VEL2)
    {
      myTransType = TRANS_IGNORE;
      myTransVal = 0;
      myTransVal2 = 0;
    }
  }

}

/**
   Sets the rotational velocity of the robot, if the constructor was created 
   with state reflecting enabled then it caches this value, and sends it during
   the next cycle.  If state reflecting is disabled it sends this value 
   instantly.
   @param velocity the desired rotational velocity of the robot
**/
AREXPORT void ArRobot::setRotVel(double velocity)
{
  if (!myStateReflecting)
    comInt(ArCommands::RVEL, ArMath::roundInt(velocity));
  else
  {
    myRotVal = velocity;
    myRotType = ROT_VEL;
    myRotSetTime.setToNow();
    if (myTransType == TRANS_VEL2)
    {
      myTransType = TRANS_IGNORE;
      myTransVal = 0;
      myTransVal2 = 0;
    }
  }
}

/**
   Sets a delta heading to the robot, if the constructor was created with state
   reflecting enabled then it caches this value, and sends it during the next
   cycle.  If state reflecting is disabled it sends this value instantly.
   @param deltaHeading the desired amount to change the heading of the robot by
**/
AREXPORT void ArRobot::setDeltaHeading(double deltaHeading)
{
  if (!myStateReflecting)
    comInt(ArCommands::HEAD, 
	   ArMath::roundInt(ArMath::addAngle(getTh(), deltaHeading)));
  else
  {
    myRotVal = ArMath::addAngle(getTh(), deltaHeading);
    myRotType = ROT_HEADING;
    myRotSetTime.setToNow();
    if (myTransType == TRANS_VEL2)
    {
      myTransType = TRANS_IGNORE;
      myTransVal = 0;
      myTransVal2 = 0;
    }
  }
}

/**
   This sets the maximum velocity the robot will go... the maximum velocity
   can also be set by the actions, but it will not be allowed to go higher
   than this value.
   @param maxVel the maximum velocity to be set, it must be a non-zero number
   @return true if the value is good, false othrewise
**/

AREXPORT bool ArRobot::setMaxTransVel(double maxVel)
{
  if (maxVel < 0)
    return false;
  myMaxTransVel = maxVel;
  if (isConnected() && ArMath::fabs(myLastSentMaxTransVel - myMaxTransVel) > 1)
  {
    myLastSentMaxTransVel = myMaxTransVel;
    comInt(ArCommands::SETV, ArMath::roundInt(myMaxTransVel));
  }
  return true;
}

/**
   This sets the maximum velocity the robot will go... the maximum velocity
   can also be set by the actions, but it will not be allowed to go higher
   than this value.
   @param maxVel the maximum velocity to be set, it must be a non-zero number
   @return true if the value is good, false othrewise
**/

AREXPORT bool ArRobot::setMaxRotVel(double maxVel)
{
  if (maxVel < 0)
    return false;
  myMaxRotVel = maxVel;
  if (isConnected() && ArMath::fabs(myLastSentMaxRotVel - myMaxRotVel) > 1)
  {
    myLastSentMaxRotVel = myMaxRotVel;
    comInt(ArCommands::SETRV, ArMath::roundInt(myMaxRotVel));
  }
  return true;
}

int ArRobot::getIOAnalog(int num)
{
  if (num <= getIOAnalogSize())
    return myIOAnalog[num];
  else
    return 0;
}

unsigned char ArRobot::getIODigIn(int num)
{
  if (num <= getIODigInSize())
    return myIODigIn[num];
  else
    return (unsigned char) 0;
}

unsigned char  ArRobot::getIODigOut(int num)
{
  if (num <= getIODigOutSize())
    return myIODigOut[num];
  else
    return (unsigned char) 0;
}

/** 
    @return the ArRobotParams instance the robot is using for its parameters
**/
AREXPORT ArRobotParams *ArRobot::getRobotParams(void) 
{
  return &myParams;
}
  
/**
   Adds a packet handler.  A packet handler is an ArRetFunctor1, created as 
   an instance of ArRetFunctor1C.  The return is a boolean, while the functor
   takes an ArRobotPacket pointer as the argument.  This functor is placed in
   the list of functors to call when a packet arrives.  This list is gone
   through by order until one of the handlers returns true.
   @argument functor the functor to call when the packet comes in
   @argument position whether to place the functor first or last
   @see remPacketHandler
**/
AREXPORT void ArRobot::addPacketHandler(
	ArRetFunctor1<bool, ArRobotPacket *> *functor, 
	ArListPos::Pos position) 
{
  if (position == ArListPos::FIRST)
    myPacketHandlerList.push_front(functor);
  else if (position == ArListPos::LAST)
    myPacketHandlerList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, "ArRobot::addPacketHandler: Invalid position.");

}

/**
   @param functor the functor to remove from the list of packet handlers
   @see addPacketHandler
**/
AREXPORT void ArRobot::remPacketHandler(
	ArRetFunctor1<bool, ArRobotPacket *> *functor)
{
  myPacketHandlerList.remove(functor);
}

/**
   Adds a connect callback, which is an ArFunctor, created as an ArFunctorC.
   The entire list of connect callbacks is called when a connection is made
   with the robot.  If you have some sort of module that adds a callback, 
   that module must remove the callback when the module is removed.
   @param functorfunctor created from ArFunctorC which refers to the 
   function to call.
   @param position whether to place the functor first or last
   @see remConnectCB
**/
AREXPORT void ArRobot::addConnectCB(ArFunctor *functor, 
				    ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myConnectCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myConnectCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArRobot::myConnectCallbackList: Invalid position.");
}

/** 
    @param functor the functor to remove from the list of connect callbacks
    @see addConnectCB
**/
AREXPORT void ArRobot::remConnectCB(ArFunctor *functor)
{
  myConnectCBList.remove(functor);
}


/** Adds a failed connect callback,which is an ArFunctor, created as an 
    ArFunctorC.  This whole list of failed connect callbacks is called when
    an attempt is made to connect to the robot, but fails.   The usual reason 
    for this failure is either that there is no robot/sim where the connection
    was tried to be made, the robot wasn't given a connection, or the radio
    modems that communicate with the robot aren't on.  If you have some sort
    of module that adds a callback, that module must remove the callback
    when the module removed.
    @param functor functor created from ArFunctorC which refers to the 
    function to call.
    @param position whether to place the functor first or last
    @see remFailedConnectCB
**/
AREXPORT void ArRobot::addFailedConnectCB(ArFunctor *functor, 
					  ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myFailedConnectCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myFailedConnectCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArRobot::myConnectCallbackList: Invalid position.");
}

/** 
    @param functor the functor to remove from the list of connect callbacks
    @see addFailedConnectCB
**/
AREXPORT void ArRobot::remFailedConnectCB(ArFunctor *functor)
{
  myFailedConnectCBList.remove(functor);
}

/** Adds a disconnect normally callback,which is an ArFunctor, created as an 
    ArFunctorC.  This whole list of disconnect normally callbacks is called 
    when something calls disconnect if the instance isConnected.  If there is 
    no connection and disconnect is called nothing is done.  If you have some
    sort of module that adds a callback, that module must remove the callback
    when the module is removed.
    @param functor functor created from ArFunctorC which refers to the 
    function to call.
    @param position whether to place the functor first or last
    @see remFailedConnectCB
**/
AREXPORT void ArRobot::addDisconnectNormallyCB(ArFunctor *functor, 
					       ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myDisconnectNormallyCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myDisconnectNormallyCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArRobot::myConnectCallbackList: Invalid position.");
}

/** 
    @param functor the functor to remove from the list of connect callbacks
    @see addDisconnectNormallyCB
**/
AREXPORT void ArRobot::remDisconnectNormallyCB(ArFunctor *functor)
{
  myDisconnectNormallyCBList.remove(functor);
}

/** Adds a disconnect on error callback, which is an ArFunctor, created as an 
    ArFunctorC.  This whole list of disconnect on error callbacks is called 
    when ARIA loses connection to a robot because of an error.  This can occur
    if the physical connection (ie serial cable) between the robot and the 
    computer is severed/disconnected, if one of a pair of radio modems that 
    connect the robot and computer are disconnected, if someone presses the
    reset button on the robot, or if the simulator is closed while ARIA
    is connected to it.  Note that if the link between the two is lost the 
    ARIA assumes it is temporary until it reaches a timeout value set with
    setConnectionTimeoutTime.  If you have some sort of module that adds a 
    callback, that module must remove the callback when the module removed.
    @param functor functor created from ArFunctorC which refers to the 
    function to call.
    @param position whether to place the functor first or last
    @see remFailedConnectCB
**/
AREXPORT void ArRobot::addDisconnectOnErrorCB(ArFunctor *functor, 
					      ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myDisconnectOnErrorCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myDisconnectOnErrorCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArRobot::myConnectCallbackList: Invalid position");
}

/** 
    @param functor the functor to remove from the list of connect callbacks
    @see addDisconnectOnErrorCB
**/
AREXPORT void ArRobot::remDisconnectOnErrorCB(ArFunctor *functor)
{
  myDisconnectOnErrorCBList.remove(functor);
}

/**
   Adds a callback that is called when the run loop exits. The functor is
   which is an ArFunctor, created as an ArFunctorC. The whole list of
   functors is called when the run loop exits. This is most usefull for
   threaded programs that run the robot using ArRobot::runAsync. This will
   allow user threads to know when the robot loop has exited.
   @param functor functor created from ArFunctorC which refers to the 
   function to call.
   @param position whether to place the functor first or last
   @see remRunExitCB
**/   
AREXPORT void ArRobot::addRunExitCB(ArFunctor *functor,
				    ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myRunExitCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myRunExitCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, "ArRobot::addRunExitCB: Invalid position");
}

/** 
    @param functor the functor to remove from the list of run exit callbacks
    @see addRunExitCB
**/
AREXPORT void ArRobot::remRunExitCB(ArFunctor *functor)
{
  myRunExitCBList.remove(functor);
}

AREXPORT std::list<ArFunctor *> * ArRobot::getRunExitListCopy()
{
  return(new std::list<ArFunctor *>(myRunExitCBList));
}

/**
   This will suspend the calling thread until the ArRobot's run loop has
   managed to connect with the robot. There is an optional paramater of
   milliseconds to wait for the ArRobot to connect. If msecs is set to 0,
   it will wait until the ArRobot connects. This function will never
   return if the robot can not be connected with. If you want to be able
   to handle that case within the calling thread, you must call
   waitForConnectOrConnFail().
   @param msecs milliseconds in which to wait for the ArRobot to connect
   @return WAIT_CONNECTED for success
   @see waitForConnectOrConnFail
   @see wakeAllWaitingThreads
   @see wakeAllConnWaitingThreads
   @see wakeAllRunExitWaitingThreads
**/
AREXPORT ArRobot::WaitState ArRobot::waitForConnect(unsigned int msecs)
{
  int ret;

  if (isConnected())
    return(WAIT_CONNECTED);

  if (msecs == 0)
    ret=myConnectCond.wait();
  else
    ret=myConnectCond.timedWait(msecs);

  if (ret == ArCondition::STATUS_WAIT_INTR)
    return(WAIT_INTR);
  else if (ret == ArCondition::STATUS_WAIT_TIMEDOUT)
    return(WAIT_TIMEDOUT);
  else if (ret == 0)
    return(WAIT_CONNECTED);
  else
    return(WAIT_FAIL);
}

/**
   This will suspend the calling thread until the ArRobot's run loop has
   managed to connect with the robot or fails to connect with the robot.
   There is an optional paramater of milliseconds to wait for the ArRobot
   to connect. If msecs is set to 0, it will wait until the ArRobot connects.
   @param msecs milliseconds in which to wait for the ArRobot to connect
   @return WAIT_CONNECTED for success
   @see waitForConnect
**/
AREXPORT ArRobot::WaitState
ArRobot::waitForConnectOrConnFail(unsigned int msecs)
{
  int ret;

  if (isConnected())
    return(WAIT_CONNECTED);

  if (msecs == 0)
    ret=myConnOrFailCond.wait();
  else
    ret=myConnOrFailCond.timedWait(msecs);

  if (ret == ArCondition::STATUS_WAIT_INTR)
    return(WAIT_INTR);
  else if (ret == ArCondition::STATUS_WAIT_TIMEDOUT)
    return(WAIT_TIMEDOUT);
  else if (ret == 0)
  {
    if (isConnected())
      return(WAIT_CONNECTED);
    else
      return(WAIT_FAILED_CONN);
  }
  else
    return(WAIT_FAIL);
}

/**
   This will suspend the calling thread until the ArRobot's run loop has
   exited. There is an optional paramater of milliseconds to wait for the
   ArRobot run loop to exit . If msecs is set to 0, it will wait until
   the ArRrobot run loop exits.
   @param msecs milliseconds in which to wait for the robot to connect
   @return WAIT_RUN_EXIT for success
**/
AREXPORT ArRobot::WaitState ArRobot::waitForRunExit(unsigned int msecs)
{
  int ret;

  if (!isRunning())
    return(WAIT_RUN_EXIT);

  if (msecs == 0)
    ret=myRunExitCond.wait();
  else
    ret=myRunExitCond.timedWait(msecs);

  if (ret == ArCondition::STATUS_WAIT_INTR)
    return(WAIT_INTR);
  else if (ret == ArCondition::STATUS_WAIT_TIMEDOUT)
    return(WAIT_TIMEDOUT);
  else if (ret == 0)
    return(WAIT_RUN_EXIT);
  else
    return(WAIT_FAIL);
}

/**
   This will wake all the threads waiting for various major state changes
   in this particular ArRobot. This includes all threads waiting for the
   robot to be connected and all threads waiting for the run loop to exit.
   @see wakeAllConnWaitingThreads
   @see wakeAllRunExitWaitingThreads
**/
AREXPORT void ArRobot::wakeAllWaitingThreads()
{
  wakeAllConnWaitingThreads();
  wakeAllRunExitWaitingThreads();
}

/**
   This will wake all the threads waiting for the robot to be connected.
   @see wakeAllWaitingThreads
   @see wakeAllRunExitWaitingThreads
**/
AREXPORT void ArRobot::wakeAllConnWaitingThreads()
{
  myConnectCond.broadcast();
  myConnOrFailCond.broadcast();
}

/**
   This will wake all the threads waiting for the robot to be connected or
   waiting for the robot to fail to connect.
   @see wakeAllWaitingThreads
   @see wakeAllRunExitWaitingThreads
**/
AREXPORT void ArRobot::wakeAllConnOrFailWaitingThreads()
{
  myConnOrFailCond.broadcast();
}

/**
   This will wake all the threads waiting for the run loop to exit.
   @see wakeAllWaitingThreads
   @see wakeAllConnWaitingThreads
**/
AREXPORT void ArRobot::wakeAllRunExitWaitingThreads()
{
  myRunExitCond.broadcast();
}

/**
   This gets the root of the synchronous task tree, so that someone can add
   their own new types of tasks, or find out more information about 
   each task... only serious developers should use this.
   @return the root of the sycnhronous task tree
   @see ArSyncTask
**/
AREXPORT ArSyncTask *ArRobot::getSyncTaskRoot(void)
{
  return mySyncTaskRoot;
}

/**
   The synchronous tasks get called every robot cycle (every 100 ms by 
   default).  

   @param name the name to give to the task, should be unique

   @param position the place in the list of user tasks to place this
   task, this can be any integer, though by convention 0 to 100 is
   used.  The tasks are called in order of highest number to lowest
   position number.

   @param functor functor created from ArFunctorC which refers to the 
   function to call.

   @see remUserTask
**/
AREXPORT bool ArRobot::addUserTask(const char *name, int position, 
				      ArFunctor *functor, 
				      ArTaskState::State *state)
{
  ArSyncTask *proc;
  if (mySyncTaskRoot == NULL)
    return false;

  proc = mySyncTaskRoot->findNonRecursive("User Tasks");
  if (proc == NULL)
    return false;
  
  proc->addNewLeaf(name, position, functor, state);
  return true;	
}

/**
   @see addUserTask
   @see remUserTask(ArFunctor *functor)
**/
AREXPORT void ArRobot::remUserTask(const char *name)
{
  ArSyncTask *proc;
  ArSyncTask *userProc;

  if (mySyncTaskRoot == NULL)
    return;

  proc = mySyncTaskRoot->findNonRecursive("User Tasks");
  if (proc == NULL)
    return;
  
  userProc = proc->findNonRecursive(name);
  if (userProc == NULL)
    return;

  
  delete userProc;

}

/**
   @see addUserTask
   @see remUserTask(std::string name)
**/
AREXPORT void ArRobot::remUserTask(ArFunctor *functor)
{
  ArSyncTask *proc;
  ArSyncTask *userProc;

  if (mySyncTaskRoot == NULL)
    return;

  proc = mySyncTaskRoot->findNonRecursive("User Tasks");
  if (proc == NULL)
    return;
  
  
  userProc = proc->findNonRecursive(functor);
  if (userProc == NULL)
    return;

  
  delete userProc;

}

/**
   The synchronous tasks get called every robot cycle (every 100 ms by 
   default).  
   @param name the name to give to the task, should be unique
   @param position the place in the list of user tasks to place this 
   task, this can be any integer, though by convention 0 to 100 is used.
   The tasks are called in order of highest number to lowest number.
   @param functor functor created from ArFunctorC which refers to the 
   function to call.
   @see remSensorInterpTask
**/
AREXPORT bool ArRobot::addSensorInterpTask(const char *name, int position, 
					      ArFunctor *functor,
					      ArTaskState::State *state)
{
  ArSyncTask *proc;
  if (mySyncTaskRoot == NULL)
    return false;

  proc = mySyncTaskRoot->findNonRecursive("Sensor Interp");
  if (proc == NULL)
    return false;
  
  proc->addNewLeaf(name, position, functor, state);
  return true;	
}

/**
   @see addSensorInterpTask
   @see remSensorInterpTask(ArFunctor *functor)
**/
AREXPORT void ArRobot::remSensorInterpTask(const char *name)
{
  ArSyncTask *proc;
  ArSyncTask *sensorInterpProc;

  if (mySyncTaskRoot == NULL)
    return;

  proc = mySyncTaskRoot->findNonRecursive("Sensor Interp");
  if (proc == NULL)
    return;
  
  sensorInterpProc = proc->findNonRecursive(name);
  if (sensorInterpProc == NULL)
    return;

  
  delete sensorInterpProc;

}

/**
   @see addSensorInterpTask
   @see remSensorInterpTask(std::string name)
**/
AREXPORT void ArRobot::remSensorInterpTask(ArFunctor *functor)
{
  ArSyncTask *proc;
  ArSyncTask *sensorInterpProc;

  if (mySyncTaskRoot == NULL)
    return;

  proc = mySyncTaskRoot->findNonRecursive("Sensor Interp");
  if (proc == NULL)
    return;
  
  
  sensorInterpProc = proc->findNonRecursive(functor);
  if (sensorInterpProc == NULL)
    return;

  
  delete sensorInterpProc;

}

/** 
    @see ArLog
**/
AREXPORT void ArRobot::printUserTasks(void)
{
  ArSyncTask *proc;
  if (mySyncTaskRoot == NULL)
    return;

  proc = mySyncTaskRoot->findNonRecursive("User Tasks");
  if (proc == NULL)
    return;

  proc->print();
}

/**
   @see ArLog
**/
AREXPORT void ArRobot::printAllTasks(void)
{
  if (mySyncTaskRoot != NULL)
    mySyncTaskRoot->print();
}

/**
   Finds a user task by its name, searching the entire space of tasks
   @return NULL if no user task of that name found, otherwise a pointer to 
   the ArSyncTask for the first task found with that name
**/
AREXPORT ArSyncTask *ArRobot::findUserTask(const char *name)
{
  ArSyncTask *proc;
  if (mySyncTaskRoot == NULL)
    return NULL;

  proc = mySyncTaskRoot->findNonRecursive("User Tasks");
  if (proc == NULL)
    return NULL;

  return proc->find(name);
}

/**
   Finds a user task by its functor, searching the entire space of tasks
   @return NULL if no user task with that functor found, otherwise a pointer
   to the ArSyncTask for the first task found with that functor
**/
AREXPORT ArSyncTask *ArRobot::findUserTask(ArFunctor *functor)
{
  ArSyncTask *proc;
  if (mySyncTaskRoot == NULL)
    return NULL;

  proc = mySyncTaskRoot->findNonRecursive("User Tasks");
  if (proc == NULL)
    return NULL;

  return proc->find(functor);
}

/**
   Finds a task by its name, searching the entire space of tasks
   @return NULL if no task of that name found, otherwise a pointer to the
   ArSyncTask for the first task found with that name
**/
AREXPORT ArSyncTask *ArRobot::findTask(const char *name)
{
  if (mySyncTaskRoot != NULL)
    return mySyncTaskRoot->find(name);
  else
    return NULL;

}

/**
   Finds a task by its functor, searching the entire space of tasks
   @return NULL if no task with that functor found, otherwise a pointer
   to the ArSyncTask for the first task found with that functor
**/
AREXPORT ArSyncTask *ArRobot::findTask(ArFunctor *functor)
{
  if (mySyncTaskRoot != NULL)
    return mySyncTaskRoot->find(functor);
  else
    return NULL;

}

/** 
    Adds an action to the list of actions at the given priority, in
    the case of two (or more) actions with the same priority, the
    default resolver (ArPriorityResolver) averages the the multiple
    readings... the priority can be any integer, but as a convention 0
    to 100 is used, with 100 being the highest priority.

    @param action the action to add @param priority what importance to
    give the action
**/
AREXPORT void ArRobot::addAction(ArAction *action, int priority)
{
  if (action == NULL)
  {
    ArLog::log(ArLog::Terse, 
      "ArRobot::addAction: an attempt was made to add a NULL action pointer");
    return;
  }
  
  action->setRobot(this);
  myActions.insert(std::pair<int, ArAction *>(priority, action));
}

/**
   Finds the action with the given name and removes it from the actions... 
   if more than one action has that name it find the one with the lowest
   priority
   @param actionName the name of the action we want to find
   @return whether remAction found anything with that action to remove or not
**/
AREXPORT bool ArRobot::remAction(const char *actionName)
{
  ArResolver::ActionMap::iterator it;
  ArAction *act;

  for (it = myActions.begin(); it != myActions.end(); ++it)
  {
    act = (*it).second;
    if (strcmp(actionName, act->getName().c_str()) == 0)
      break;
  }
  if (it != myActions.end())
  {
    myActions.erase(it);
    return true;
  }
  return false;

}

/**
   Finds the action with the given pointer and removes it from the actions... 
   if more than one action has that pointer it find the one with the lowest
   priority
   @param action the action we want to remove
   @return whether remAction found anything with that action to remove or not
**/
AREXPORT bool ArRobot::remAction(ArAction *action)
{
  ArResolver::ActionMap::iterator it;
  ArAction *act;

  for (it = myActions.begin(); it != myActions.end(); ++it)
  {
    act = (*it).second;
    if (act == action)
      break;
  }
  if (it != myActions.end())
  {
    myActions.erase(it);
    return true;
  }
  return false;

}


/**
   Finds the action with the given name... if more than one action
   has that name it find the one with the highest priority
   @param actionName the name of the action we want to find
   @return the action, if found.  If not found, NULL
**/
AREXPORT ArAction *ArRobot::findAction(const char *actionName)
{
  ArResolver::ActionMap::reverse_iterator it;
  ArAction *act;
  
  for (it = myActions.rbegin(); it != myActions.rend(); ++it)
  {
    act = (*it).second;
    if (strcmp(actionName, act->getName().c_str()) == 0)
      return act;
  }
  return NULL;
}

/**
   This returns the actionMap the robot has... do not mess with this
   list except by using ArRobot::addAction and ArRobot::remAction...
   This is jsut for the things like ArActionGroup that want to
   deactivate or activate all the actions (well, only deactivating
   everything makes sense).
   @return the actions the robot is using
**/

AREXPORT ArResolver::ActionMap *ArRobot::getActionMap(void)
{
  return &myActions;
}

AREXPORT void ArRobot::printActions(void)
{
  ArResolver::ActionMap::reverse_iterator it;
  int lastPriority;
  bool first = true;

  ArLog::log(ArLog::Terse, "The action list (%d of them):\n", myActions.size());
  for (it = myActions.rbegin(); it != myActions.rend(); ++it)
  {
    if (first || lastPriority != (*it).first) 
    {
      ArLog::log(ArLog::Terse, "Priority %d:", (*it).first);
      first = false;
      lastPriority = (*it).first;
    }
    (*it).second->print();
  }
    
}

AREXPORT ArResolver *ArRobot::getResolver(void)
{
  return myResolver;
}

AREXPORT void ArRobot::setResolver(ArResolver *resolver)
{
  if (myOwnTheResolver)
  {
    delete myResolver;
    myResolver = NULL;
  }

  myResolver = resolver;
}

/**
   If state reflecting (really direct motion command reflecting) was
   enabled in the constructor (ArRobot::ArRobot) then this will see if
   there are any direct motion commands to send, and if not then send
   the command given by the actions.  If state reflection is disabled
   this will send a pulse to the robot every state reflection refresh
   time (setStateReflectionRefreshTime), if you don't wish this to happen
   simply set this to a very large value.
**/
AREXPORT void ArRobot::stateReflector(void)
{
  int transVal;
  int transVal2;
  int rotVal;
  bool rotStopped = false;
  int maxVel;
  int maxNegVel;

  if (!myIsConnected)
    return;

  if (!myStateReflecting)
  {
    if (myLastPulseSent.mSecSince() > myStateReflectionRefreshTime)
    {
      com(0);
      myLastPulseSent.setToNow();
    }
    return;
  }


  // if this is true actions can't go
  if ((myTransType != TRANS_NONE && myDirectPrecedenceTime == 0) ||
      (myTransType != TRANS_NONE && myDirectPrecedenceTime != 0 && 
       myTransSetTime.mSecSince() < myDirectPrecedenceTime))
  {
    myActionTransSet = false;
    transVal = ArMath::roundInt(myTransVal);
    transVal2 = 0;
    if (ArMath::fabs(myLastSentMaxTransVel - myMaxTransVel) > 1)
    {
      comInt(ArCommands::SETV,
	     ArMath::roundInt(myMaxTransVel));
      myLastSentMaxTransVel = myMaxTransVel;
    }
    if (myTransType == TRANS_VEL)
    {
      maxVel = ArMath::roundInt(myMaxTransVel);
      if (transVal > maxVel)
	transVal = maxVel;
      if (transVal < -maxVel)
	transVal = -maxVel;
      if (myLastTransVal != transVal || myLastTransType != myTransType ||
	  (myLastTransSent.mSecSince() >= myStateReflectionRefreshTime))
      {
	comInt(ArCommands::VEL, ArMath::roundInt(transVal));
	myLastTransSent.setToNow();
	//printf("Sent command vel!\n");
      }
    }
    else if (myTransType == TRANS_VEL2)
    {
      if (ArMath::roundInt(myTransVal/myParams.getVel2Divisor()) > 128)
	transVal = 128;
      else if (ArMath::roundInt(myTransVal/myParams.getVel2Divisor()) < -128)
	transVal = -128;
      else 
	transVal = ArMath::roundInt(myTransVal/myParams.getVel2Divisor());
      if (ArMath::roundInt(myTransVal2/myParams.getVel2Divisor()) > 128)
	transVal2 = 128;
      else if (ArMath::roundInt(myTransVal2/myParams.getVel2Divisor()) < -128)
	transVal2 = -128;
      else 
	transVal2 = ArMath::roundInt(myTransVal2/myParams.getVel2Divisor());
      if (myLastTransVal != transVal || myLastTransVal2 != transVal2 || 
	  myLastTransType != myTransType ||
	  (myLastTransSent.mSecSince() >= myStateReflectionRefreshTime))
      {
	com2Bytes(ArCommands::VEL2, transVal, transVal2);
	myLastTransSent.setToNow();
	//printf("Sent command vel2!\n");
      }
    }
    else if (myTransType == TRANS_DIST_NEW || myTransType == TRANS_DIST)
    {
      // if the robot doesn't have its own distance command
      if (!myParams.hasMoveCommand())
      {
	double distGone;
	double distToGo;
	double vel;

	myTransType = TRANS_DIST;
	distGone = myTransDistStart.findDistanceTo(getPose());
	distToGo = fabs(fabs(myTransVal) - distGone);
	if (distGone > fabs(myTransVal) || 
	    (distToGo < 10 && fabs(getVel()) < 30))
	{
	  comInt(ArCommands::VEL, 0);
	  myTransType = TRANS_VEL;
	  myTransVal = 0;
	}
	vel = sqrt(distToGo * 200 * 2);
	if (vel > getMaxTransVel())
	  vel = getMaxTransVel();
	if (myTransVal < 0)
	  vel *= -1;
	comInt(ArCommands::VEL, ArMath::roundInt(vel));
      }
      else if (myParams.hasMoveCommand() && myTransType == TRANS_DIST_NEW) 
      {
	comInt(ArCommands::MOVE, transVal);
	myLastTransSent.setToNow();
	myTransType = TRANS_DIST;
      }
      //printf("Sent command move!\n");
    }
    else if (myTransType == TRANS_IGNORE)
    {
      //printf("No trans command sent\n");
    }
    else
      ArLog::log(ArLog::Terse, 
		 "ArRobot::stateReflector: Invalid translational type %d.",
		 myTransType);
    myLastTransVal = transVal;
    myLastTransVal2 = transVal2;
    myLastTransType = myTransType;
  }
  else // if actions can go
  {
    if (myActionDesired.getMaxVelStrength() > 0) 
    {
      myActionMaxTransVel = myActionDesired.getMaxVel();
      if (myActionMaxTransVel > myMaxTransVel)
	myActionMaxTransVel = myMaxTransVel;
    }
    else
      myActionMaxTransVel = myMaxTransVel;

    if (myActionDesired.getMaxNegVelStrength() > 0) 
    {
      myActionMaxNegTransVel = myActionDesired.getMaxNegVel();
      if (myActionMaxNegTransVel < -myMaxTransVel)
	myActionMaxNegTransVel = -myMaxTransVel;
    }
    else
      myActionMaxNegTransVel = -myMaxTransVel;
    
    if (myActionDesired.getVelStrength() > 0)
    {
      transVal = ArMath::roundInt(myActionDesired.getVel());
      myActionTransSet = true;
    }
    else
      transVal = myLastActionTransVal;
    maxVel = ArMath::roundInt(myActionMaxTransVel);
    maxNegVel = ArMath::roundInt(myActionMaxNegTransVel);
    if (transVal > maxVel)
      transVal = maxVel;
    if (transVal < maxNegVel)
      transVal = maxNegVel;
    if (myActionTransSet && 
	(myLastTransSent.mSecSince() >= myStateReflectionRefreshTime ||
	 transVal != myLastActionTransVal))			     
    {
      comInt(ArCommands::VEL, ArMath::roundInt(transVal));
      myLastTransSent.setToNow();
    }
    myLastActionTransVal = transVal;
  }

  // if this is true actions can't go
  if ((myRotType != ROT_NONE && myDirectPrecedenceTime == 0) ||
      (myRotType != ROT_NONE && myDirectPrecedenceTime != 0 && 
       myRotSetTime.mSecSince() < myDirectPrecedenceTime))

  {
    myActionRotSet = false;
    rotVal = ArMath::roundInt(myRotVal);
    if (ArMath::fabs(myLastSentMaxRotVel - myMaxRotVel) > 1)
    {
      //printf("Sent RV\n");
      comInt(ArCommands::SETRV,
		      ArMath::roundInt(myMaxRotVel));
      myLastSentMaxRotVel = myMaxRotVel;
    }
    if (myRotType == ROT_HEADING)
    {
      double th = ArMath::subAngle(myRotVal, myEncoderTransform.getTh());
      th = ArMath::addAngle(th, ArMath::subAngle(myRawEncoderPose.getTh(),
						 myEncoderPose.getTh()));
      rotVal = ArMath::roundInt(th);

      if (myLastRotVal != rotVal || myLastRotType != myRotType ||
	  (myLastRotSent.mSecSince() >= myStateReflectionRefreshTime))
      {
	comInt(ArCommands::HEAD, rotVal);
	myLastRotSent.setToNow();
	//printf("sent command, heading\n");
      }
    }
    else if (myRotType == ROT_VEL)
    {
      if (myLastRotVal != rotVal || myLastRotType != myRotType ||
	  (myLastRotSent.mSecSince() >= myStateReflectionRefreshTime))
      {
	comInt(ArCommands::RVEL, rotVal);
	myLastRotSent.setToNow();
	//printf("sent command, rot vel\n");
      }
    }
    else if (myRotType == ROT_IGNORE)
    {
      //printf("Not sending any command, rot is set to ignore");
    }
    else
      ArLog::log(ArLog::Terse, 
		 "ArRobot::stateReflector: Invalid rotation type %d.",
		 myRotType);
    myLastRotVal = rotVal;
    myLastRotType = myRotType;
  }
  else // if the action can fire
  {
    if (myActionDesired.getMaxRotVelStrength() > ArActionDesired::MIN_STRENGTH)
    {
      myActionMaxRotVel = myActionDesired.getMaxRotVelStrength();
      if (myActionMaxRotVel > myMaxRotVel)
	myActionMaxRotVel = myMaxRotVel;
      if (ArMath::fabs(myLastSentMaxRotVel - myActionMaxRotVel) > 1)
      {
	myLastSentMaxRotVel = myActionMaxRotVel;
	comInt(ArCommands::SETRV, 
	       ArMath::roundInt(myActionMaxRotVel));
	//printf("Sent RV here()\n");
      }
    }
    if (myActionDesired.getDeltaHeadingStrength() > 
	                                     ArActionDesired::MIN_STRENGTH)
    {
      if (ArMath::roundInt(myActionDesired.getDeltaHeading()) == 0)
      {
	rotStopped = true;
	rotVal = 0;
      }
      else
      {
	//printf("delta %.0f\n", myActionDesired.getDeltaHeading());
	double th = ArMath::addAngle(myActionDesired.getDeltaHeading(), 
				     getTh()) - myEncoderTransform.getTh();
	//printf("final th %.0f\n", th);
	th = ArMath::addAngle(th, ArMath::subAngle(myRawEncoderPose.getTh(),
						   myEncoderPose.getTh()));
	rotVal = ArMath::roundInt(th);
	rotStopped = false;
      }
      myActionRotSet = true;
    } 
    else
    {
      rotStopped = myLastActionRotStopped;
      rotVal = myLastActionRotVal;
    }

    if (myActionRotSet && 
	(myLastRotSent.mSecSince() > myStateReflectionRefreshTime ||
	 rotStopped != myLastActionRotStopped || rotVal != myLastActionRotVal))
    {
      if (rotStopped)
      {
	comInt(ArCommands::RVEL, 0);
      }
      else
      {
	comInt(ArCommands::HEAD, rotVal);
      }
      myLastRotSent.setToNow();
    }		    
    myLastActionRotVal = rotVal;
    myLastActionRotStopped = rotStopped;
  }

  if (myLastRotSent.mSecSince() > myStateReflectionRefreshTime &&
      myLastTransSent.mSecSince() > myStateReflectionRefreshTime &&
      myLastPulseSent.mSecSince() > myStateReflectionRefreshTime)
  {
    com(ArCommands::PULSE);
    myLastPulseSent.setToNow();
  }
}

AREXPORT bool ArRobot::handlePacket(ArRobotPacket *packet)
{
  std::list<ArRetFunctor1<bool, ArRobotPacket *> *>::iterator it;
  bool handled;

  lock();
  myLastPacketReceivedTime = packet->getTimeReceived();
  if (packet->getID() == 0xff) 
  {
    ArLog::log(ArLog::Terse, "Loosing connection because robot was reset.");
    dropConnection();
    unlock();
    return false;
  }
  for (handled = false, it = myPacketHandlerList.begin(); 
       it != myPacketHandlerList.end() && handled == false; 
       it++)
  {
    if ((*it) != NULL && (*it)->invokeR(packet)) 
      handled = true;
    else
      packet->resetRead();
  }
  if (!handled)
    ArLog::log(ArLog::Normal, 
	       "No packet handled wanted packet with ID: 0x%x", 
	       packet->getID());
  unlock();
  return handled;
}

/**
   This just locks the robot, so that its locked for all the user tasks
**/
AREXPORT void ArRobot::robotLocker(void)
{
  lock();
}

/**
   This just unlocks the robot
**/
AREXPORT void ArRobot::robotUnlocker(void)
{
  unlock();
}

/**
   Reads in all of the packets that are available to read in, then runs through
   the list of packet handlers and tries to get each packet handled.
   @see addPacketHandler
   @see remPacketHandler
**/
AREXPORT void ArRobot::packetHandler(void)
{
  ArRobotPacket *packet;
  int timeToWait;
  ArTime start;
  bool sipHandled = false;

  if (myAsyncConnectFlag)
  {
    lock();
    asyncConnectHandler(false);
    unlock();
    return;
  }

  if (!isConnected()) 
    return;

  start.setToNow();
  /*
    The basic idea is that if we're chained to the sip we run through and see
    if we have any available now (like if we got backed up), if we don't
    then we chill and wait for it, if we got one, just get on with it 
  */
  // read all the packets that are available
  while ((packet = myReceiver.receivePacket(0)) != NULL)
  {
    handlePacket(packet);
    //printf("Packet 0x%x at %d\n", packet->getID(), start.mSecSince());
    if ((packet->getID() & 0xf0) == 0x30)
      sipHandled = true;
  }

  if (isCycleChained())
    timeToWait = getCycleTime() * 2 - start.mSecSince();
  
  // if we didn't get a sip and we're chained to the sip, wait for the sip
  while (isCycleChained() && !sipHandled && isRunning() && 
	 (packet = myReceiver.receivePacket(timeToWait)) != NULL)
  {
    handlePacket(packet);
    //printf("Packet 0x%x at %d\n", packet->getID(), start.mSecSince());
    if ((packet->getID() & 0xf0) == 0x30)
      break;
    timeToWait = getCycleTime() * 2 - start.mSecSince();
    if (timeToWait < 0)
      timeToWait = 0;
  }

  if (myTimeoutTime > 0 && 
      ((-myLastPacketReceivedTime.mSecTo()) > myTimeoutTime))
  {
    ArLog::log(ArLog::Terse, 
	       "Loosing connection because nothing received from robot in %d milliseconds.", 
	       myTimeoutTime);
    dropConnection();
  }
}

/**
   Runs the resolver on the actions, if state reflection (direct
   motion reflection really) is enabled in the ArRobot::ArRobot
   constructor then it just saves these values for use by the
   stateReflector, otherwise it sends these values straight down to
   the robot.
   @see addAction
   @see remAction
**/
AREXPORT void ArRobot::actionHandler(void)
{
  ArActionDesired *actDesired;
  double transVel;
  

  if (myResolver == NULL || myActions.size() == 0 || !isConnected())
    return;
  
  actDesired = myResolver->resolve(&myActions, this);
  
  if (actDesired == NULL)
    return;

  myActionDesired.reset();
  if (!myStateReflecting)
  {
    if (actDesired->getVelStrength() > 0)
    {
      transVel = actDesired->getVel();
      if (actDesired->getMaxVelStrength() > 0)
      {
	myActionMaxTransVel = actDesired->getMaxVel();
	if (myActionMaxTransVel > myMaxTransVel)
	  myActionMaxTransVel = myMaxTransVel;
	if (transVel > myActionMaxTransVel)
	  transVel = myActionMaxTransVel;
	if (transVel < -myActionMaxTransVel) 
	  transVel = -myActionMaxTransVel;
      }
      comInt(ArCommands::VEL, 
		      ArMath::roundInt(transVel));

    }
    if (actDesired->getDeltaHeadingStrength() > 0)
    {
      double th = ArMath::addAngle(actDesired->getDeltaHeading(), getTh());
      th = ArMath::addAngle(th, ArMath::subAngle(myRawEncoderPose.getTh(),
						 myEncoderPose.getTh()));
      comInt(ArCommands::HEAD, ArMath::roundInt(th));
		     
    }
    if (actDesired->getMaxRotVelStrength() > 0)
    {
      myActionMaxRotVel = actDesired->getMaxRotVel();
      if (myActionMaxRotVel > myMaxRotVel)
	myActionMaxRotVel = myMaxRotVel;
      if (ArMath::fabs(myLastSentMaxRotVel - myActionMaxRotVel) > 1)
      {
	myLastSentMaxRotVel = myActionMaxRotVel;
	comInt(ArCommands::SETRV, ArMath::roundInt(myActionMaxRotVel));
      }
    }
  }
  else
    myActionDesired.merge(actDesired);  
}

/**
   Sets the number of milliseconds between cycles, at each cycle is 
   when all packets are processed, all sensors are interpretted, all
   actions are called, and all user tasks are serviced.  Be warned,
   if you set this too small you could overflow your serial connection.
   @param ms the number of milliseconds between cycles
 **/
AREXPORT void ArRobot::setCycleTime(unsigned int ms)
{
  myCycleTime = ms;
}

/**
   Finds the number of milliseconds between cycles, at each cycle is 
   when all packets are processed, all sensors are interpretted, all
   actions are called, and all user tasks are serviced.  Be warned,
   if you set this too small you could overflow your serial connection.
   @return the number of milliseconds between cycles
**/
AREXPORT unsigned int ArRobot::getCycleTime(void)
{
  return myCycleTime;
}

/**
   @param multiplier when the ArRobot is waiting for a connection
   packet back from a robot, it waits for this multiplier times the
   cycle time for the packet to come back before it gives up on it...
   This should be small for normal connections but if doing something
   over a slow network then you may want to make it larger

 **/
AREXPORT void ArRobot::setConnectionCycleMultiplier(unsigned int multiplier)
{
  myConnectionCycleMultiplier = multiplier;
}

/**
   @return when the ArRobot is waiting for a connection packet back
   from a robot, it waits for this multiplier times the cycle time for
   the packet to come back before it gives up on it...  This should be
   small for normal connections but if doing something over a slow
   network then you may want to make it larger
**/
AREXPORT unsigned int ArRobot::getConnectionCycleMultiplier(void)
{
  return myConnectionCycleMultiplier;
}


/** 
    This function is only for serious developers, it basically runs the 
    loop once.  You would use this function if you were wanting to use robot
    control in some other monolithic program, so you could work within its 
    framework, rather than trying to get it to work in ARIA.
**/
AREXPORT void ArRobot::loopOnce(void)
{
  if (mySyncTaskRoot != NULL)
    mySyncTaskRoot->run();
  
  incCounter();
}


// DigIn IR logic is reverse.  0 means broken, 1 means not broken

AREXPORT bool ArRobot::isLeftTableSensingIRTriggered(void)
{
  if (myParams.haveTableSensingIR())
  {
    if (myParams.haveNewTableSensingIR() && myIODigInSize > 3)
      return !(getIODigIn(3) & ArUtil::BIT1);
    else
      return !(getDigIn() & ArUtil::BIT0);
  }
  return 0;
}

AREXPORT bool ArRobot::isRightTableSensingIRTriggered(void)
{
  if (myParams.haveTableSensingIR())
  {
    if (myParams.haveNewTableSensingIR() && myIODigInSize > 3) 
      return !(getIODigIn(3) & ArUtil::BIT0);
    else
      return !(getDigIn() & ArUtil::BIT1);
  }
  return 0;
}

AREXPORT bool ArRobot::isLeftBreakBeamTriggered(void)
{
  if (myParams.haveTableSensingIR())
  {
    if (myParams.haveNewTableSensingIR() && myIODigInSize > 3) 
      return !(getIODigIn(3) & ArUtil::BIT2);
    else
      return !(getDigIn() & ArUtil::BIT3);
  }
  return 0;
}

AREXPORT bool ArRobot::isRightBreakBeamTriggered(void)
{
  if (myParams.haveTableSensingIR())
  {
    if (myParams.haveNewTableSensingIR() && myIODigInSize > 3) 
      return !(getIODigIn(3) & ArUtil::BIT3);
    else
      return !(getDigIn() & ArUtil::BIT2);
  }
  return 0;
}

AREXPORT int ArRobot::getMotorPacCount(void)
{
  if (myTimeLastMotorPacket == time(NULL))
    return myMotorPacCount;
  if (myTimeLastMotorPacket == time(NULL) - 1)
    return myMotorPacCurrentCount;
  return 0;
}

AREXPORT int ArRobot::getSonarPacCount(void)
{
  if (myTimeLastSonarPacket == time(NULL))
    return mySonarPacCount;
  if (myTimeLastSonarPacket == time(NULL) - 1)
    return mySonarPacCurrentCount;
  return 0;
}


AREXPORT bool ArRobot::processMotorPacket(ArRobotPacket *packet)
{
  int x, y, th, qx, qy, qth; 
  double deltaX, deltaY, deltaTh;

  int numReadings;
  int sonarNum;
  int sonarRange;

  if (packet->getID() != 0x32 && packet->getID() != 0x33) 
    return false;

  // upkeep the counting variable
  if (myTimeLastMotorPacket != time(NULL)) 
  {
    myTimeLastMotorPacket = time(NULL);
    myMotorPacCount = myMotorPacCurrentCount;
    myMotorPacCurrentCount = 0;
  }
  myMotorPacCurrentCount++;

  x = (packet->bufToUByte2() & 0x7fff);
  y = (packet->bufToUByte2() & 0x7fff);
  th = packet->bufToByte2();

  if (myFirstEncoderPose)
  {
    qx = 0;
    qy = 0;
    qth = 0;
    myFirstEncoderPose = false;
    myRawEncoderPose.setPose(
	    myParams.getDistConvFactor() * x,
	    myParams.getDistConvFactor() * y, 
	    ArMath::radToDeg(myParams.getAngleConvFactor() * (double)th));
    myEncoderPose = myRawEncoderPose;
    myEncoderTransform.setTransform(myEncoderPose, myGlobalPose);
  }
  else 
  {
    qx = x - myLastX;
    qy = y - myLastY;
    qth = th - myLastTh;
  }


  //ArLog::log(ArLog::Terse, "Damnit qx %d qy %d,  x %d y %d,  lastx %d lasty %d", qx, qy, x, y, myLastX, myLastY);  
  myLastX = x;
  myLastY = y;
  myLastTh = th;

  if (qx > 0x1000) 
    qx -= 0x8000;
  if (qx < -0x1000)
    qx += 0x8000;

  if (qy > 0x1000) 
    qy -= 0x8000;
  if (qy < -0x1000)
    qy += 0x8000;
  
  deltaX = myParams.getDistConvFactor() * (double)qx;
  deltaY = myParams.getDistConvFactor() * (double)qy;
  deltaTh = ArMath::radToDeg(myParams.getAngleConvFactor() * (double)qth);
  //encoderTh = ArMath::radToDeg(myParams.getAngleConvFactor() * (double)(th));

  /*
    Okay how this works is like so.  

    We keep around the raw encoder position, because we must use this
    to find differences between last position and this position.
    
    We find the difference in x and y positions (deltaX and deltaY)
    and keep these around for later use, but we also add these to our
    raw encoder readings for X and Y.  We also find the change in
    angle (deltaTh), which is used for inertial corrections, and added
    to the raw encoder heading to find which the current raw encoder
    heading.


    From here there are two paths:

    Path 1) Have a callback.  If we have a callback it means that we
    have an inertial nav device of some kind.  If this is the case,
    then we pass the callback the delta between last position and
    current position, along with the time of the current position,
    then the callback gives us back a new delta theta (deltaTh).  We
    then need to rotate the deltaX and deltaY into our corrected
    encoder space.  We do this by making a transform that takes the
    raw encoder heading and transforms it to what our new heading is
    (adding deltaTh to our current encoder th), and then applying that
    transform, taking the results as our new deltaX and deltaY.

    Path 2) We have no callback, we just use the heading that came
    back from the robot as our delta theta (deltaTh);

    From here the two paths unify again.  deltaX and deltaY are added
    to the encoder pose (this is the corrected encoder pose), and the
    encoder heading is set to the newTh.

    Note that this leaves a difference between rawEncoder heading and
    our heading, which is fine, BUT if you are sending heading
    commands to the robot you need to compenstate for the difference
    between these. 
    
    Note above that we return deltaTh instead of just heading so that
    we can turn inertial on and off without losing track of where
    we're at... since we're just adding in deltas from the heading it
    doesn't matter how we switch around the callback.
    
  **/

  myRawEncoderPose.setX(myRawEncoderPose.getX() + deltaX);
  myRawEncoderPose.setY(myRawEncoderPose.getY() + deltaY);
  myRawEncoderPose.setTh(myRawEncoderPose.getTh() + deltaTh);

  // check if there is a correction callback, if there is get the new
  // heading out of it instead of using the raw encoder heading
  if (myEncoderCorrectionCB != NULL)
  {
    ArPoseWithTime deltaPose(deltaX, deltaY, deltaTh,
			     packet->getTimeReceived());
    deltaTh = myEncoderCorrectionCB->invokeR(deltaPose);   
    ArTransform trans(ArPose(0, 0, myRawEncoderPose.getTh()),
		      ArPose(0, 0,
			     ArMath::addAngle(myEncoderPose.getTh(), 
					      deltaTh)));

    ArPose rotatedDelta = trans.doTransform(ArPose(deltaX, deltaY, 0));

    deltaX = rotatedDelta.getX();
    deltaY = rotatedDelta.getY();
  }

  myEncoderPose.setTime(packet->getTimeReceived());
  myEncoderPose.setX(myEncoderPose.getX() + deltaX);
  myEncoderPose.setY(myEncoderPose.getY() + deltaY);
  myEncoderPose.setTh(ArMath::addAngle(myEncoderPose.getTh(), deltaTh));

  myGlobalPose = myEncoderTransform.doTransform(myEncoderPose);

  //ArLog::log(ArLog::Terse, "(%.0f %.0f) (%.0f %.0f)", deltaX, deltaY, myGlobalPose.getX(),	     myGlobalPose.getY());
  
  myInterpolation.addReading(packet->getTimeReceived(), myGlobalPose);

  myLeftVel = myParams.getVelConvFactor() * packet->bufToByte2();
  myRightVel = myParams.getVelConvFactor() * packet->bufToByte2();
  myVel = (myLeftVel + myRightVel)/2.0;
  myRotVel = ArMath::radToDeg((myRightVel - myLeftVel) / 2.0 * 
				    myParams.getDiffConvFactor());

  myBatteryVoltage = packet->bufToUByte() * .1;
  myStallValue = packet->bufToByte2();
  
  //ArLog::log("x %.1f y %.1f th %.1f vel %.1f voltage %.1f", myX, myY, myTh, 
  //myVel, myBatteryVoltage);

  myControl = ArMath::fixAngle(ArMath::radToDeg(myParams.getAngleConvFactor() *
				  (packet->bufToByte2() - th)));
  myFlags = packet->bufToUByte2();
  myCompass = 2*packet->bufToUByte();
  
  for (numReadings = packet->bufToByte(); numReadings > 0; numReadings--)
  {
    sonarNum = packet->bufToByte();
    sonarRange = ArMath::roundInt(
	    (double)packet->bufToUByte2() * myParams.getRangeConvFactor());
    processNewSonar(sonarNum, sonarRange, packet->getTimeReceived());
  }

  if (packet->getLength() - packet->getReadLength() > 2) 
  {
    myAnalogPortSelected = packet->bufToUByte2();
    myAnalog = packet->bufToByte();
    myDigIn = packet->bufToByte();
    myDigOut = packet->bufToByte();
  }
  return true;
}

AREXPORT void ArRobot::processNewSonar(char number, int range, 
				       ArTime timeReceived)
{
  std::map<int, ArSensorReading *>::iterator it;
  ArSensorReading *sonar;
  ArTransform encoderTrans;
  ArPose encoderPose;

  if ((it = mySonars.find(number)) != mySonars.end())
  {
    sonar = (*it).second;
  }
  else
  {
    if ((number + 1) > myNumSonar)
      myNumSonar = number + 1;
    sonar = new ArSensorReading;
    mySonars[number] = sonar;
  }
    
  encoderTrans = getEncoderTransform();
  encoderPose = encoderTrans.doInvTransform(getPose());
  sonar->newData(range, getPose(), encoderPose, getToGlobalTransform(), 
		 getCounter(), timeReceived); 
  if (myTimeLastSonarPacket != time(NULL)) 
  {
    myTimeLastSonarPacket = time(NULL);
    mySonarPacCount = mySonarPacCurrentCount;
    mySonarPacCurrentCount = 0;
  }
  mySonarPacCurrentCount++;
}

AREXPORT bool ArRobot::processEncoderPacket(ArRobotPacket *packet)
{
  if (packet->getID() == 0x90) 
  {
    return true;
  } 
  return false;
}

AREXPORT bool ArRobot::processIOPacket(ArRobotPacket *packet)
{
  int i, num;

  if (packet->getID() != 0xf0)
    return false;

  myLastIOPacketReceivedTime = packet->getTimeReceived();

  // number of DigIn bytes
  num = packet->bufToUByte();
  for (i = 0; i < num; ++i)
    myIODigIn[i] = packet->bufToUByte();
  myIODigInSize = num;

  // number of DigOut bytes
  num = packet->bufToUByte();
  for (i = 0; i < num; ++i)
    myIODigOut[i] = packet->bufToUByte();
  myIODigOutSize = num;

  // number of A/D bytes
  num = packet->bufToUByte();
  for (i = 0; i < num; ++i)
    myIOAnalog[i] = packet->bufToByte2();
  myIOAnalogSize = num;

  return true;
}

/**
   @param num the sonar number to check, should be between 0 and the number of
   sonar, the function won't fail if a bad number is given, will just return
   -1
   @return -1 if the sonar has never returned a reading, otherwise the sonar 
   range, which is the distance from the physical sonar disc to where the sonar
   bounced back
   @see getNumSonar
**/
AREXPORT int ArRobot::getSonarRange(int num)
{
  std::map<int, ArSensorReading *>::iterator it;
  
  if ((it = mySonars.find(num)) != mySonars.end())
    return (*it).second->getRange();
  else
    return -1;
}

/**
   @param num the sonar number to check, should be between 0 and the number of
   sonar, the function won't fail if a bad number is given, will just return
   false
   @return false if the sonar reading is old, or if there is no reading from
   that sonar
**/

AREXPORT bool ArRobot::isSonarNew(int num)
{
  std::map<int, ArSensorReading *>::iterator it;
  
  if ((it = mySonars.find(num)) != mySonars.end())
    return (*it).second->isNew(getCounter());
  else
    return false;
}

/**
   @param num the sonar number to check, should be between 0 and the number of
   sonar, the function won't fail if a bad number is given, will just return
   false
   @return NULL if there is no sonar defined for the given number, otherwise
   it returns a pointer to an instance of the ArSensorReading, note that this 
   class retains ownership, so the instance pointed to should not be deleted
   and no pointers to it should be stored.  Note that often there will be sonar
   defined but no readings for it (since the readings may be created by the 
   parameter reader), if there has never been a reading from the sonar then
   the range of that sonar will be -1 and its counterTaken value will be 0
**/
AREXPORT ArSensorReading *ArRobot::getSonarReading(int num)
{
  std::map<int, ArSensorReading *>::iterator it;
  
  if ((it = mySonars.find(num)) != mySonars.end())
    return (*it).second;
  else
    return NULL;
}


/**
   @param command the command number to send
   @return whether the command could be sent or not
**/
AREXPORT bool ArRobot::com(unsigned char command)
{
  return mySender.com(command);
}

/**
   @param command the command number to send
   @param argument the integer argument to send with the command
   @return whether the command could be sent or not
**/
AREXPORT bool ArRobot::comInt(unsigned char command, short int argument)
{
  return mySender.comInt(command, argument);
}

/**
   @param command the command number to send
   @param high the high byte to send with the command
   @param low the low byte to send with the command
   @return whether the command could be sent or not
**/
AREXPORT bool ArRobot::com2Bytes(unsigned char command, char high, char low)
{
  return mySender.com2Bytes(command, high, low);
}

/**
   @param command the command number to send
   @param str the string to send with the command
   @return whether the command could be sent or not
**/
AREXPORT bool ArRobot::comStr(unsigned char command, const char *argument)
{
  return mySender.comStr(command, argument);
}

/**
   @param command the command number to send
   @param str the character array to send with the command
   @param size length of the array to send
   @return whether the command could be sent or not
**/
AREXPORT bool ArRobot::comStrN(unsigned char command, const char *str, 
			       int size)
{
  return mySender.comStrN(command, str, size);
}

AREXPORT int ArRobot::getClosestSonarRange(double startAngle, double endAngle)
{
  int num;
  num = getClosestSonarNumber(startAngle, endAngle);
  if (num == -1)
    return -1;
  else 
    return getSonarRange(num);
}

AREXPORT int ArRobot::getClosestSonarNumber(double startAngle, double endAngle)
{
  int i;
  ArSensorReading *sonar;
  int closestReading;
  int closestSonar;
  bool noReadings = true;

  for (i = 0; i < getNumSonar(); i++) 
  {
    sonar = getSonarReading(i);
    if (sonar == NULL)
    {
      ArLog::log(ArLog::Terse, "Have an empty sonar at number %d, there should be %d sonar.", i, getNumSonar());
      continue;
    }
    if (ArMath::angleBetween(sonar->getSensorTh(), startAngle, endAngle))
    {
      if (noReadings)
      {
	closestReading = sonar->getRange();
	closestSonar = i;
	noReadings = false;
      }
      else if (sonar->getRange() < closestReading)
      {
	closestReading = sonar->getRange();
	closestSonar = i;
      }
    }
  }

  if (noReadings)
    return -1;
  else
    return closestSonar;
}

AREXPORT void ArRobot::addRangeDevice(ArRangeDevice *device)
{
  device->setRobot(this);
  myRangeDeviceList.push_front(device);
}

/**
   @param name remove the first device with this name
**/
AREXPORT void ArRobot::remRangeDevice(const char *name)
{
  std::list<ArRangeDevice *>::iterator it;
  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    if (strcmp(name, (*it)->getName().c_str()) == 0)
    {
      myRangeDeviceList.erase(it);
      return;
    }
  }
}

/**
   @param device remove the first device with this pointer value
**/
AREXPORT void ArRobot::remRangeDevice(ArRangeDevice *device)
{
  std::list<ArRangeDevice *>::iterator it;
  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    if ((*it) == device)
    {
      myRangeDeviceList.erase(it);
      return;
    }
  }
}

/**
   @param name remove the first device with this name
   @return if found, a range device with the given name, if not found NULL
**/
AREXPORT ArRangeDevice *ArRobot::findRangeDevice(const char *name)
{
  std::list<ArRangeDevice *>::iterator it;
  ArRangeDevice *device;

  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    device = (*it);
    if (strcmp(name, device->getName().c_str()) == 0)
    {
      return device;
    }
  }
  return NULL;
}

/** 
    This gets the list of range devices attached to this robot, do NOT
    manipulate this list directly.  If you want to manipulate use the 
    appropriate addRangeDevice, or remRangeDevice
    @return the list of range dvices attached to this robot
**/
AREXPORT std::list<ArRangeDevice *> *ArRobot::getRangeDeviceList(void)
{
  return &myRangeDeviceList;
}

/**
   @param device the device to check for
**/
AREXPORT bool ArRobot::hasRangeDevice(ArRangeDevice *device)
{
  std::list<ArRangeDevice *>::iterator it;
  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    if ((*it) == device)
      return true;
  }
  return false;
}

/**
   This goes through all of the registered range devices and locks each,
   calls currentReadingPolar on it, and then unlocks it.

   Gets the closest reading in a region defined by startAngle going to 
   endAngle... going counterclockwise (neg degrees to poseitive... with 
   how the robot is set up, thats counterclockwise)... from -180 to 180... 
   this means if you want the slice between 0 and 10 degrees,
   you must enter it as 0, 10, if you do 10, 0 you'll get the 350 degrees
   between 10 and 0... be especially careful with negative... for example
   -30 to -60 is everything from -30, around through 0, 90, and 180 back to 
   -60... since -60 is actually to clockwise of -30
   @param startAngle where to start the slice
   @param endAngle where to end the slice, going clockwise from startAngle
   @param angle a pointer return of the angle to the found reading
   @return if the return is >= 0 then this is the distance to the closest
   reading, if it is < 0 then there were no readings in the given region
**/

AREXPORT double ArRobot::checkRangeDevicesCurrentPolar(double startAngle,
						       double endAngle,
						       double *angle)
{
  double closest, closeAngle, tempDist, tempAngle;
  std::list<ArRangeDevice *>::iterator it;
  ArRangeDevice *device;
  bool foundOne = false;

  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    device = (*it);
    device->lockDevice();
    if (!foundOne || 
	(tempDist = device->currentReadingPolar(startAngle, endAngle,
						&tempAngle)) < closest)
    {
      if (!foundOne)
      {
	closest = device->currentReadingPolar(startAngle, endAngle, 
					      &closeAngle);
      }
      else
      {
	closest = tempDist;
	closeAngle = tempAngle;
      }
      foundOne = true;
    }
    device->unlockDevice();
  }
  if (!foundOne)
    return -1;
  if (angle != NULL)
    *angle = closeAngle;
  return closest;
  
}

/**
   This goes through all of the registered range devices and locks each,
   calls cumulativeReadingPolar on it, and then unlocks it.

   Gets the closest reading in a region defined by startAngle going to 
   endAngle... going counterclockwise (neg degrees to poseitive... with 
   how the robot is set up, thats counterclockwise)... from -180 to 180... 
   this means if you want the slice between 0 and 10 degrees,
   you must enter it as 0, 10, if you do 10, 0 you'll get the 350 degrees
   between 10 and 0... be especially careful with negative... for example
   -30 to -60 is everything from -30, around through 0, 90, and 180 back to 
   -60... since -60 is actually to clockwise of -30
   @param startAngle where to start the slice
   @param endAngle where to end the slice, going clockwise from startAngle
   @param angle a pointer return of the angle to the found reading
   @return if the return is >= 0 then this is the distance to the closest
   reading, if it is < 0 then there were no readings in the given region
**/

AREXPORT double ArRobot::checkRangeDevicesCumulativePolar(double startAngle,
						       double endAngle,
						       double *angle)
{
  double closest, closeAngle, tempDist, tempAngle;
  std::list<ArRangeDevice *>::iterator it;
  ArRangeDevice *device;
  bool foundOne = false;

  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    device = (*it);
    device->lockDevice();
    if (!foundOne || 
	(tempDist = device->cumulativeReadingPolar(startAngle, endAngle,
						 &tempAngle)) < closest)
    {
      if (!foundOne)
      {
	closest = device->cumulativeReadingPolar(startAngle, endAngle, 
					      &closeAngle);
      }
      else
      {
	closest = tempDist;
	closeAngle = tempAngle;
      }
      foundOne = true;
    }
    device->unlockDevice();
  }
  if (!foundOne)
    return -1;
  if (angle != NULL)
    *angle = closeAngle;
  return closest;
  
}

/**
   This goes through all of the registered range devices and locks each,
   calls currentReadingBox on it, and then unlocks it.

   Gets the closest reading in a region defined by the two points of a 
   rectangle.
   @param x1 the x coordinate of one of the rectangle points
   @param y1 the y coordinate of one of the rectangle points
   @param x2 the x coordinate of the other rectangle point
   @param y2 the y coordinate of the other rectangle point
   @param readingPos a pointer to a position in which to store the location of
   the closest position
   @return if the return is >= 0 then this is the distance to the closest
   reading, if it is < 0 then there were no readings in the given region
**/

AREXPORT double ArRobot::checkRangeDevicesCurrentBox(double x1, double y1,
						     double x2, double y2,
						     ArPose *readingPos)
{

  double closest, tempDist;
  ArPose closestPos, tempPos;
  std::list<ArRangeDevice *>::iterator it;
  ArRangeDevice *device;
  bool foundOne = false;

  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    device = (*it);
    device->lockDevice();
    if (!foundOne || 
	(tempDist = device->currentReadingBox(x1, y1, x2, y2, &tempPos)) < closest)
    {
      if (!foundOne)
      {
	closest = device->currentReadingBox(x1, y1, x2, y2, &closestPos);
      }
      else
      {
	closest = tempDist;
	closestPos = tempPos;
      }
      foundOne = true;
    }
    device->unlockDevice();
  }
  if (!foundOne)
    return -1;
  if (readingPos != NULL)
    *readingPos = closestPos;
  return closest;
}

/**
   This goes through all of the registered range devices and locks each,
   calls cumulativeReadingBox on it, and then unlocks it.

   Gets the closest reading in a region defined by the two points of a 
   rectangle.
   @param x1 the x coordinate of one of the rectangle points
   @param y1 the y coordinate of one of the rectangle points
   @param x2 the x coordinate of the other rectangle point
   @param y2 the y coordinate of the other rectangle point
   @param readingPos a pointer to a position in which to store the location of
   the closest position
   @return if the return is >= 0 then this is the distance to the closest
   reading, if it is < 0 then there were no readings in the given region
**/

AREXPORT double ArRobot::checkRangeDevicesCumulativeBox(double x1, double y1,
						     double x2, double y2,
						     ArPose *readingPos)
{

  double closest, tempDist;
  ArPose closestPos, tempPos;
  std::list<ArRangeDevice *>::iterator it;
  ArRangeDevice *device;
  bool foundOne = false;

  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); ++it)
  {
    device = (*it);
    device->lockDevice();
    if (!foundOne || 
	(tempDist = device->cumulativeReadingBox(x1, y1, x2, y2, &tempPos)) < closest)
    {
      if (!foundOne)
      {
	closest = device->cumulativeReadingBox(x1, y1, x2, y2, &closestPos);
      }
      else
      {
	closest = tempDist;
	closestPos = tempPos;
      }
      foundOne = true;
    }
    device->unlockDevice();
  }
  if (!foundOne)
    return -1;
  if (readingPos != NULL)
    *readingPos = closestPos;
  return closest;
}

/** 
    @param pose the absolute real world position to place the robot
    @param doCumulative whether to update the cumulative buffers or not
**/
AREXPORT void ArRobot::moveTo(ArPose pose, bool doCumulative)
{
  applyTransform(getToLocalTransform(), doCumulative);
  myEncoderTransform.setTransform(myEncoderPose, pose);
  myGlobalPose = myEncoderTransform.doTransform(myEncoderPose);
  applyTransform(getToGlobalTransform(), doCumulative);
}

/** 
    @param poseTo the absolute real world position to move to
    @param poseFrom the original absolute real world position
    @param doCumulative whether to update the cumulative buffers or not
**/
AREXPORT void ArRobot::moveTo(ArPose poseTo, ArPose poseFrom,
			      bool doCumulative)
{
  applyTransform(getToLocalTransform(), doCumulative);
  ArPose tp = myEncoderTransform.doInvTransform(poseFrom);
  myEncoderTransform.setTransform(tp, poseTo);
  myGlobalPose = myEncoderTransform.doTransform(myEncoderPose);
  applyTransform(getToGlobalTransform(), doCumulative);
}

/**
   @param deadReconPos the dead recon position to transform from
   @param realWorldPos the real world global position to transform to
**/
AREXPORT void ArRobot::setEncoderTransform(ArPose deadReconPos, 
				    ArPose globalPos)
{
  myEncoderTransform.setTransform(deadReconPos, globalPos);
  myGlobalPose = myEncoderTransform.doTransform(myEncoderPose);
}

/**
   @param transformPos the position to transform to
**/
AREXPORT void ArRobot::setEncoderTransform(ArPose transformPos)
{
  myEncoderTransform.setTransform(transformPos);
  myGlobalPose = myEncoderTransform.doTransform(myEncoderPose);
}

/**
   @return the transform from encoder to global coords
**/
AREXPORT ArTransform ArRobot::getEncoderTransform(void)
{
  return myEncoderTransform;
}

/**
   @param pose the position to set the dead recon position to
**/
AREXPORT void ArRobot::setDeadReconPose(ArPose pose)
{
  myEncoderPose.setPose(pose);
  myEncoderTransform.setTransform(myEncoderPose, myGlobalPose);
  myGlobalPose = myEncoderTransform.doTransform(myEncoderPose);
}


/** 
    @return an ArTransform which can be used for transforming a position
    in local coordinates to one in global coordinates
**/
AREXPORT ArTransform ArRobot::getToGlobalTransform(void)
{
  ArTransform trans;
  ArPose origin(0, 0, 0);
  ArPose pose = getPose();

  trans.setTransform(origin, pose);
  return trans;
}

/** 
    @return an ArTransform which can be used for transforming a position
    in global coordinates to one in local coordinates
**/
AREXPORT ArTransform ArRobot::getToLocalTransform(void)
{
  ArTransform trans;
  ArPose origin(0, 0, 0);
  ArPose pose = getPose();

  trans.setTransform(pose, origin);
  return trans;
}

/** 
    Applies a transform to the range devices... this is mostly useful for
    translating to/from local/global coords, but may have other uses
    @param trans the transform to apply
    @param doCumulative whether to transform the cumulative buffers or not
**/    
AREXPORT void ArRobot::applyTransform(ArTransform trans, bool doCumulative)
{
  std::list<ArRangeDevice *>::iterator it;
  ArSensorReading *son;
  int i;

  for (it = myRangeDeviceList.begin(); it != myRangeDeviceList.end(); it++)
    (*it)->applyTransform(trans, doCumulative);

  for (i = 0; i < getNumSonar(); i++)
  {
    son = getSonarReading(i);
    if (son != NULL)
      son->applyTransform(trans);
  }
}

AREXPORT std::string ArRobot::getName(void)
{
  return myName;
}

AREXPORT void ArRobot::setName(const char * name)
{
  if (name != NULL)
  {
    myName = name;
  }
  else
  {
    std::list<ArRobot *> *robotList;
    std::list<ArRobot *>::iterator it;
    bool found;
    int i;
    char buf[1024];

    robotList = Aria::getRobotList();
    for (i = 1, it = robotList->begin(); it != robotList->end(); it++, i++)
    {
      if (this == (*it))
      {
	found = true;
	break;
      }
    }
    if (found && i == 1)
      myName = "robot";
    else if (found)
    {
      sprintf(buf, "robot%d", i);
      myName = buf;
    }
    else
    {
      sprintf(buf, "robot%d", robotList->size());
      myName = buf;
    }
  }
}

/**
   This sets the encoderCorrectionCB, this callback returns the robots
   change in heading, it takes in the change in heading, x, and y,
   between the previous and current readings.  
   
   @param functor an ArRetFunctor1 created as an ArRetFunctor1C, that
   will be the callback... call this function NULL to clear the
   callback @see getEncoderCorrectionCallback
**/

AREXPORT void ArRobot::setEncoderCorrectionCallback(
	ArRetFunctor1<double, ArPoseWithTime> *functor)
{
  myEncoderCorrectionCB = functor;
}
/**
   This gets the encoderCorrectionCB, see setEncoderCorrectionCallback
   for details.

   @return the callback, or NULL if there isn't one
**/
AREXPORT ArRetFunctor1<double, ArPoseWithTime> * 
ArRobot::getEncoderCorrectionCallback(void)
{
  return myEncoderCorrectionCB;
}

/**
   The direct motion precedence time determines how long actions will be 
   ignored after a direct motion command is given.  If the direct motion
   precedence time is 0, then direct motion will take precedence over actions 
   until a clearDirectMotion command is issued.  This value defaults to 0.
   @param the number of milliseconds direct movement should trump actions, 
   if a negative number is given, then the value will be 0
   @see setDirectMotionPrecedenceTime
   @see clearDirectMotion
**/
AREXPORT void ArRobot::setDirectMotionPrecedenceTime(int mSec)
{
  if (mSec < 0) 
    myDirectPrecedenceTime = 0;
  else
    myDirectPrecedenceTime = mSec;
}

/**
   The direct motion precedence time determines how long actions will be 
   ignored after a direct motion command is given.  If the direct motion
   precedence time is 0, then direct motion will take precedence over actions 
   until a clearDirectMotion command is issued.  This value defaults to 0.
   @return the number of milliseconds direct movement will trump actions
   @see setDirectMotionPrecedenceTime
   @see clearDirectMotion
**/
AREXPORT unsigned int ArRobot::getDirectMotionPrecedenceTime(void)
{
  return myDirectPrecedenceTime;
}


/**
   This clears the direct motion commands so that actions will be allowed to 
   control the robot again.
   @see setDirectMotionPrecedenceTime
   @see getDirectMotionPrecedenceTime
**/
AREXPORT void ArRobot::clearDirectMotion(void)
{
  myTransType = TRANS_NONE;
  myLastTransType = TRANS_NONE;
  myRotType = ROT_NONE;
  myLastRotType = ROT_NONE;
}

/**
   Returns the state of direct motion commands: whether actions are allowed or not
   @see clearDirectMotion
**/
AREXPORT bool ArRobot::isDirectMotion(void)
{
  if (myTransType ==  TRANS_NONE && myLastTransType == TRANS_NONE &&
      myRotType == ROT_NONE && myLastRotType == ROT_NONE)
    return false;
  else
    return true;
}


/**
   This command enables the motors on the robot, if it is connected.
**/
AREXPORT void ArRobot::enableMotors()
{
  comInt(ArCommands::ENABLE, 1);
}

/**
   This command disables the motors on the robot, if it is connected.
**/
AREXPORT void ArRobot::disableMotors()
{
  comInt(ArCommands::ENABLE, 0);
}


/**
   The state reflection refresh time is the number of milliseconds between 
   when the state reflector will refresh the robot, if the command hasn't 
   changed.  The default is 500 milliseconds.  If this number is less than 
   the cyle time, it'll simply happen every cycle.
   @param mSec the refresh time, in milliseconds, non-negative, if negative is 
   given, then the value will be 0
**/
AREXPORT void ArRobot::setStateReflectionRefreshTime(int mSec)
{
  if (mSec < 0)
    myStateReflectionRefreshTime = 0;
  else
    myStateReflectionRefreshTime = mSec;
}

/**
   The state reflection refresh time is the number of milliseconds between 
   when the state reflector will refresh the robot, if the command hasn't 
   changed.  The default is 500 milliseconds.  If this number is less than 
   the cyle time, it'll simply happen every cycle.
   @return the state reflection refresh time
**/
AREXPORT int ArRobot::getStateReflectionRefreshTime(void)
{
  return myStateReflectionRefreshTime;
}

/**
   This will attach a key handler to a robot, by putting it into the
   robots sensor interp task list (a keyboards a sensor of users will,
   right?).  By default exitOnEscape is true, which will cause this
   function to add an escape key handler to the key handler, this will
   make the program exit when escape is pressed... if you don't like
   this you can pass exitOnEscape in as false.
   @param keyHandler the key handler to attach
   @param exitOnEscape whether to exit when escape is pressed or not
**/
AREXPORT void ArRobot::attachKeyHandler(ArKeyHandler *keyHandler, 
					bool exitOnEscape)
{
  if (myKeyHandlerCB != NULL)
    delete myKeyHandlerCB;
  myKeyHandlerCB = new ArFunctorC<ArKeyHandler>(keyHandler, 
					    &ArKeyHandler::checkKeys);
  addSensorInterpTask("Key Handler", 50, myKeyHandlerCB);

  myKeyHandler = keyHandler;
  if (exitOnEscape)
    keyHandler->addKeyHandler(ArKeyHandler::ESCAPE, &myKeyHandlerExitCB);
}

AREXPORT void ArRobot::keyHandlerExit(void)
{
  ArLog::log(ArLog::Terse, "Escape was pressed, program is exiting.");
  stopRunning();
  Aria::shutdown();
}
