/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"

#include "ariaOSDef.h"
#include <stdlib.h>
#include "ArRobotParamFile.h"
#include "ArLog.h"
#include "ariaInternal.h"


const char *ArRobotParamFile::ourClassValues[] =
{
  "Pioneer",
  "*",
  ""
};

const char *ArRobotParamFile::ourSubclassValues[] =
{
  "amigo",
  "p2at",
  "p2ce",
  "p2dx",
  "p2de",
  "p2df",
  "p2d8",
  "p2it",
  "p2pb",
  "p2pp",
  "perfpb",
  "pion41m",
  "pion41x",
  "psos41m",
  "psos41x",
  "*",
  ""
};

ArPrefSection ArRobotParamFile::ourSections[] =
{
  ArPrefSection(ArRobotParamFile::General, "General",
		";\n; Generic settings\n;"),
  ArPrefSection(ArRobotParamFile::ConvFactors, "ConvFactors",
		";\n; Conversion factors\n;"),
  ArPrefSection(ArRobotParamFile::Accessories, "Accessories",
		";\n; Accessories the robot has\n;"),
  ArPrefSection(ArRobotParamFile::Sonar, "Sonar",
			 "\
;\n\
; Sonars\n\
;\n\
; SonarUnit values: <sonarNumber> <x position, mm> <y position, mm> <heading of disc, degrees>\n\
;"),
  ArPrefSection(ArRobotParamFile::Laser, "Laser", ";\n;Settings relating to the laser\n;"),
  ArPrefSection(ArRobotParamFile::LASTSECTION, "", "")
};

ArPref ArRobotParamFile::ourPrefs[] =
{
  // General parameters
  ArPref(ArRobotParamFile::General, ArRobotParamFile::Class, "Class", "",
	 ArRobotParamFile::ourClassValues, ArPref::String, ""),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::Subclass, "Subclass", "",
	 ArRobotParamFile::ourSubclassValues, ArPref::String, ""),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::RobotRadius,
	 "RobotRadius", "", 0, ArPref::Double, "radius in mm"),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::RobotDiagonal,
	 "RobotDiagonal", "", 0, ArPref::Double,
	 "half-height to diagonal of octagon"),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::Holonomic, "Holonomic",
	 "", 0, ArPref::Double, "turns in own radius"),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::MaxRVelocity,
	 "MaxRVelocity", "", 0, ArPref::Double, "degrees per second"),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::MaxVelocity,
	 "MaxVelocity", "", 0, ArPref::Double, "mm per second"),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::HasMoveCommand, 
	 "HasMoveCommand", "", 0, ArPref::Double, "has built in move command"),
  ArPref(ArRobotParamFile::General, ArRobotParamFile::RequestIOPackets,
         "RequestIOPackets", "", 0, ArPref::Boolean, "automatically request IO packets"),

  // Conversion factors
  ArPref(ArRobotParamFile::ConvFactors, ArRobotParamFile::AngleConvFactor,
	 "AngleConvFactor", "", 0, ArPref::Double,
	 "radians per angular unit (2PI/4096)"),
  ArPref(ArRobotParamFile::ConvFactors, ArRobotParamFile::DistConvFactor,
	 "DistConvFactor", "", 0, ArPref::Double, "mm returned by P2"),
  ArPref(ArRobotParamFile::ConvFactors, ArRobotParamFile::VelConvFactor,
	 "VelConvFactor", "", 0, ArPref::Double, "mm/sec returned by P2"),
  ArPref(ArRobotParamFile::ConvFactors, ArRobotParamFile::RangeConvFactor,
	 "RangeConvFactor", "", 0, ArPref::Double,
	 "sonar range returned in mm"),
  ArPref(ArRobotParamFile::ConvFactors, ArRobotParamFile::DiffConvFactor,
	 "DiffConvFactor", "", 0, ArPref::Double,
	 "ratio of angular velocity to wheel velocity difference"),
  ArPref(ArRobotParamFile::ConvFactors, ArRobotParamFile::Vel2Divisor,
	 "Vel2Divisor", "", 0, ArPref::Double,
	 "divisor for sending VEL2 commands to the robot"),
  
  // Accessories
  ArPref(ArRobotParamFile::Accessories, ArRobotParamFile::TableSensingIR,
	  "TableSensingIR", "", 0, ArPref::Boolean,
	  "if the robot has upwards facing table sensing IR"),
  ArPref(ArRobotParamFile::Accessories, ArRobotParamFile::NewTableSensingIR,
          "NewTableSensingIR", "", 0, ArPref::Boolean,
          "if the robot's table sensing IR are sent in the IO packet"),
  ArPref(ArRobotParamFile::Accessories, ArRobotParamFile::FrontBumpers,
	  "FrontBumpers", "", 0, ArPref::Boolean,
	 "if the robot has a front bump ring"),
  ArPref(ArRobotParamFile::Accessories, ArRobotParamFile::NumFrontBumpers,
          "NumFrontBumpers", "", 0, ArPref::Integer,
         "Number of front bumpers on the robot"),
  ArPref(ArRobotParamFile::Accessories, ArRobotParamFile::RearBumpers,
	  "RearBumpers", "", 0, ArPref::Boolean,
	 "if the robot has a rear bump ring"),
  ArPref(ArRobotParamFile::Accessories, ArRobotParamFile::NumRearBumpers,
          "NumRearBumpers", "", 0, ArPref::Integer,
         "Number of rear bumpers on the robot"),

  // Sonar parameters
  ArPref(ArRobotParamFile::Sonar, ArRobotParamFile::SonarNum, "SonarNum", "",
	 0, ArPref::Integer, "Number of sonar on the robot"),
  ArPref(ArRobotParamFile::Sonar, ArRobotParamFile::SonarUnit, "SonarUnit",
	 "", 0, ArPref::Integer, ""),

  // Laser parameters
  ArPref(ArRobotParamFile::Laser, ArRobotParamFile::LaserPossessed,
	 "LaserPossessed", "", 0, ArPref::Boolean,
	 "if there is a laser on the robot or not"),
  ArPref(ArRobotParamFile::Laser, ArRobotParamFile::LaserPort, "LaserPort", "",
	 0, ArPref::String, "port the laser is on"),
  ArPref(ArRobotParamFile::Laser, ArRobotParamFile::LaserFlipped,
	 "LaserFlipped", "", 0, ArPref::Boolean,
	 "if the laser on the robot is flipped (upside-down) or not"),
  ArPref(ArRobotParamFile::Laser, ArRobotParamFile::LaserPowerControlled,
	 "LaserPowerControlled", "", 0, ArPref::Boolean,
	 "if the power to the laser is controlled by serial"),
  ArPref(ArRobotParamFile::Laser, ArRobotParamFile::LaserX, "LaserX", "",
	 0, ArPref::Integer, "X location of laser (mm)"),
  ArPref(ArRobotParamFile::Laser, ArRobotParamFile::LaserY, "LaserY", "",
	 0, ArPref::Integer, "Y location of laser (mm)"),

  // The End
  ArPref(ArRobotParamFile::LASTSECTION, ArRobotParamFile::LASTPREF, 0,
	 "", 0, ArPref::Integer, "")

};


AREXPORT ArRobotParamFile::ArRobotParamFile(const char *dir,
					    const char *file) :
  ArPreferences(ourSections, LASTSECTION, ourPrefs, LASTPREF),
  myDir(),
  myFile()
{
  if (dir)
    myDir=dir;
  if (file)
    myFile=file;
}

AREXPORT ArRobotParamFile::ArRobotParamFile() :
  ArPreferences(ourSections, LASTSECTION, ourPrefs, LASTPREF),
  myDir(),
  myFile()
{
}

AREXPORT ArRobotParamFile::~ArRobotParamFile()
{
}

AREXPORT void ArRobotParamFile::init()
{
  ArPreferences::init(true);
  setDefaults();
}

AREXPORT std::string ArRobotParamFile::getHeader()
{
  return("; Robot parameter file");
}

AREXPORT std::string ArRobotParamFile::getDir()
{
  return(myDir);
}

AREXPORT std::string ArRobotParamFile::getFile()
{
  if (myTmpFile.size())
    return(myTmpFile);
  else
    return(myFile);
}

AREXPORT bool ArRobotParamFile::findDir(const char *fileName)
{
  if (Aria::getDirectory().length() > 0)
  {
    myDir=Aria::getDirectory();
    ArUtil::appendSlash(myDir);
    myDir+="params";
    return(true);
  }
  else
  {
    //ArLog::log(ArLog::Terse, "The ARIA environment variable is not set. Default robot parameters are being used.");
    return(false);
  }
}

AREXPORT bool ArRobotParamFile::findFile(const char *fileName)
{
  std::string path;

  path=myDir;
  ArUtil::appendSlash(path);
  if (fileName && *fileName)
  {
    myTmpFile=fileName;
    if (!strstr(myTmpFile.c_str(), ".p"))
      myTmpFile+=".p";
    path+=myTmpFile;
  }
  else
  {
    myTmpFile="";
    path+=myFile;
  }

  return(ArUtil::findFile(path.c_str()));
}

AREXPORT bool ArRobotParamFile::handleListSectLine(const char *section,
						   const char *line)
{
  return(false);
}

AREXPORT void ArRobotParamFile::extraSave(FILE *fp)
{
}
