/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"

#include "ariaOSDef.h"
#include "ArRobotTypes.h"
#include "ArLog.h"


// Generic robot class

AREXPORT ArRobotGeneric::ArRobotGeneric(const char *name, const char *dir) :
  ArRobotParamFile(dir, name)
{
}

AREXPORT ArRobotGeneric::~ArRobotGeneric()
{
}

AREXPORT void ArRobotGeneric::setDefaults()
{
  ArPref *pref;
  pref=getPref(General, RobotRadius);
  pref->setDouble(250.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(Sonar, SonarNum);
  pref->setInt(0);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0);
  pref->setIntSet(true, 4, 0);
}


// AmigoBot robot class

AREXPORT ArRobotAmigo::ArRobotAmigo(const char *dir) :
  ArRobotParamFile(dir, "amigo.p")
{
}

AREXPORT ArRobotAmigo::~ArRobotAmigo()
{
}

AREXPORT void ArRobotAmigo::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("amigo");
  pref=getPref(General, RobotRadius);
  pref->setDouble(180.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setInt(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(300.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(1000.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.5083);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(0.6154);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(1.000);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.011);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(8);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 73, 105, 90);
  pref->setIntSet(true, 4, 1, 130, 78, 41);
  pref->setIntSet(true, 4, 2, 154, 30, 15);
  pref->setIntSet(true, 4, 3, 154, -30, -15);
  pref->setIntSet(true, 4, 4, 130, -78, -41);
  pref->setIntSet(true, 4, 5, 73, -105, -90);
  pref->setIntSet(true, 4, 6, -146, -60, -145);
  pref->setIntSet(true, 4, 7, -146, 60, 145);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);
}


// P2AT robot class

AREXPORT ArRobotP2AT::ArRobotP2AT(const char *dir) :
  ArRobotParamFile(dir, "p2at.p")
{
}

AREXPORT ArRobotP2AT::~ArRobotP2AT()
{
}

AREXPORT void ArRobotP2AT::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2at");
  pref=getPref(General, RobotRadius);
  pref->setDouble(500.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(300.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(1200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(1.32);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0034);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 145, 130, 90);
  pref->setIntSet(true, 4, 1, 185, 115, 50);
  pref->setIntSet(true, 4, 2, 220, 80, 30);
  pref->setIntSet(true, 4, 3, 240, 25, 10);
  pref->setIntSet(true, 4, 4, 240, -25, -10);
  pref->setIntSet(true, 4, 5, 220, -80, -30);
  pref->setIntSet(true, 4, 6, 185, -115, -50);
  pref->setIntSet(true, 4, 7, 145, -130, -90);
  pref->setIntSet(true, 4, 8, -145, -130, -90);
  pref->setIntSet(true, 4, 9, -185, -115, -130);
  pref->setIntSet(true, 4, 10, -220, -80, -150);
  pref->setIntSet(true, 4, 11, -240, -25, -170);
  pref->setIntSet(true, 4, 12, -240, 25, 170);
  pref->setIntSet(true, 4, 13, -220, 80, 150);
  pref->setIntSet(true, 4, 14, -185, 115, 130);
  pref->setIntSet(true, 4, 15, -145, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(7);
  pref=getPref(Laser, LaserY);
  pref->setInt(160);
}


// P2AT8 robot class

AREXPORT ArRobotP2AT8::ArRobotP2AT8(const char *dir) :
  ArRobotParamFile(dir, "p2at8.p")
{
}

AREXPORT ArRobotP2AT8::~ArRobotP2AT8()
{
}

AREXPORT void ArRobotP2AT8::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2at8");
  pref=getPref(General, RobotRadius);
  pref->setDouble(500.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(300.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(1200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(1.32);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(1.00);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0034);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 145, 130, 90);
  pref->setIntSet(true, 4, 1, 185, 115, 50);
  pref->setIntSet(true, 4, 2, 220, 80, 30);
  pref->setIntSet(true, 4, 3, 240, 25, 10);
  pref->setIntSet(true, 4, 4, 240, -25, -10);
  pref->setIntSet(true, 4, 5, 220, -80, -30);
  pref->setIntSet(true, 4, 6, 185, -115, -50);
  pref->setIntSet(true, 4, 7, 145, -130, -90);
  pref->setIntSet(true, 4, 8, -145, -130, -90);
  pref->setIntSet(true, 4, 9, -185, -115, -130);
  pref->setIntSet(true, 4, 10, -220, -80, -150);
  pref->setIntSet(true, 4, 11, -240, -25, -170);
  pref->setIntSet(true, 4, 12, -240, 25, 170);
  pref->setIntSet(true, 4, 13, -220, 80, 150);
  pref->setIntSet(true, 4, 14, -185, 115, 130);
  pref->setIntSet(true, 4, 15, -145, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(7);
  pref=getPref(Laser, LaserY);
  pref->setInt(160);
}


// P2IT robot class

AREXPORT ArRobotP2IT::ArRobotP2IT(const char *dir) :
  ArRobotParamFile(dir, "p2it.p")
{
}

AREXPORT ArRobotP2IT::~ArRobotP2IT()
{
}

AREXPORT void ArRobotP2IT::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2it");
  pref=getPref(General, RobotRadius);
  pref->setDouble(500.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(300.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(1200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(1.136);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0032);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 145, 130, 90);
  pref->setIntSet(true, 4, 1, 185, 115, 50);
  pref->setIntSet(true, 4, 2, 220, 80, 30);
  pref->setIntSet(true, 4, 3, 240, 25, 10);
  pref->setIntSet(true, 4, 4, 240, -25, -10);
  pref->setIntSet(true, 4, 5, 220, -80, -30);
  pref->setIntSet(true, 4, 6, 185, -115, -50);
  pref->setIntSet(true, 4, 7, 145, -130, -90);
  pref->setIntSet(true, 4, 8, -145, -130, -90);
  pref->setIntSet(true, 4, 9, -185, -115, -130);
  pref->setIntSet(true, 4, 10, -220, -80, -150);
  pref->setIntSet(true, 4, 11, -240, -25, -170);
  pref->setIntSet(true, 4, 12, -240, 25, 170);
  pref->setIntSet(true, 4, 13, -220, 80, 150);
  pref->setIntSet(true, 4, 14, -185, 115, 130);
  pref->setIntSet(true, 4, 15, -145, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);
}


// P2DX robot class

AREXPORT ArRobotP2DX::ArRobotP2DX(const char *dir) :
  ArRobotParamFile(dir, "p2dx.p")
{
}

AREXPORT ArRobotP2DX::~ArRobotP2DX()
{
}

AREXPORT void ArRobotP2DX::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2dx");
  pref=getPref(General, RobotRadius);
  pref->setDouble(250.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.84);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0056);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);
  pref->setIntSet(true, 4, 8, -115, -130, -90);
  pref->setIntSet(true, 4, 9, -155, -115, -130);
  pref->setIntSet(true, 4, 10, -190, -80, -150);
  pref->setIntSet(true, 4, 11, -210, -25, -170);
  pref->setIntSet(true, 4, 12, -210, 25, 170);
  pref->setIntSet(true, 4, 13, -190, 80, 150);
  pref->setIntSet(true, 4, 14, -155, 115, 130);
  pref->setIntSet(true, 4, 15, -115, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(8);
  pref=getPref(Laser, LaserY);
  pref->setInt(17);
}
// P2DXe robot class

AREXPORT ArRobotP2DXe::ArRobotP2DXe(const char *dir) :
  ArRobotParamFile(dir, "p2de.p")
{
}

AREXPORT ArRobotP2DXe::~ArRobotP2DXe()
{
}

AREXPORT void ArRobotP2DXe::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2de");
  pref=getPref(General, RobotRadius);
  pref->setDouble(250.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.969);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0056);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);
  pref->setIntSet(true, 4, 8, -115, -130, -90);
  pref->setIntSet(true, 4, 9, -155, -115, -130);
  pref->setIntSet(true, 4, 10, -190, -80, -150);
  pref->setIntSet(true, 4, 11, -210, -25, -170);
  pref->setIntSet(true, 4, 12, -210, 25, 170);
  pref->setIntSet(true, 4, 13, -190, 80, 150);
  pref->setIntSet(true, 4, 14, -155, 115, 130);
  pref->setIntSet(true, 4, 15, -115, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(8);
  pref=getPref(Laser, LaserY);
  pref->setInt(17);
}

// P2DF robot class

AREXPORT ArRobotP2DF::ArRobotP2DF(const char *dir) :
  ArRobotParamFile(dir, "p2df.p")
{
}

AREXPORT ArRobotP2DF::~ArRobotP2DF()
{
}

AREXPORT void ArRobotP2DF::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2df");
  pref=getPref(General, RobotRadius);
  pref->setDouble(250.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.485);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0060);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);

  pref->setIntSet(true, 4, 8, -115, -130, -90);
  pref->setIntSet(true, 4, 9, -155, -115, -130);
  pref->setIntSet(true, 4, 10, -190, -80, -150);
  pref->setIntSet(true, 4, 11, -210, -25, -170);
  pref->setIntSet(true, 4, 12, -210, 25, 170);
  pref->setIntSet(true, 4, 13, -190, 80, 150);
  pref->setIntSet(true, 4, 14, -155, 115, 130);
  pref->setIntSet(true, 4, 15, -115, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(8);
  pref=getPref(Laser, LaserY);
  pref->setInt(17);
}

// P2D8 robot class

AREXPORT ArRobotP2D8::ArRobotP2D8(const char *dir) :
  ArRobotParamFile(dir, "p2d8.p")
{
}

AREXPORT ArRobotP2D8::~ArRobotP2D8()
{
}

AREXPORT void ArRobotP2D8::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2d8");
  pref=getPref(General, RobotRadius);
  pref->setDouble(250.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.485);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0056);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);

  pref->setIntSet(true, 4, 8, -115, -130, -90);
  pref->setIntSet(true, 4, 9, -155, -115, -130);
  pref->setIntSet(true, 4, 10, -190, -80, -150);
  pref->setIntSet(true, 4, 11, -210, -25, -170);
  pref->setIntSet(true, 4, 12, -210, 25, 170);
  pref->setIntSet(true, 4, 13, -190, 80, 150);
  pref->setIntSet(true, 4, 14, -155, 115, 130);
  pref->setIntSet(true, 4, 15, -115, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(21);
}


// P2CE robot class

AREXPORT ArRobotP2CE::ArRobotP2CE(const char *dir) :
  ArRobotParamFile(dir, "p2ce.p")
{
}

AREXPORT ArRobotP2CE::~ArRobotP2CE()
{
}

AREXPORT void ArRobotP2CE::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2ce");
  pref=getPref(General, RobotRadius);
  pref->setDouble(250.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.826);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0057);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(16);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);

  pref->setIntSet(true, 4, 8, -115, -130, -90);
  pref->setIntSet(true, 4, 9, -155, -115, -130);
  pref->setIntSet(true, 4, 10, -190, -80, -150);
  pref->setIntSet(true, 4, 11, -210, -25, -170);
  pref->setIntSet(true, 4, 12, -210, 25, 170);
  pref->setIntSet(true, 4, 13, -190, 80, 150);
  pref->setIntSet(true, 4, 14, -155, 115, 130);
  pref->setIntSet(true, 4, 15, -115, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);
}


// P2PP robot class

AREXPORT ArRobotP2PP::ArRobotP2PP(const char *dir) :
  ArRobotParamFile(dir, "p2pp.p")
{
}

AREXPORT ArRobotP2PP::~ArRobotP2PP()
{
}

AREXPORT void ArRobotP2PP::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2pp");
  pref=getPref(General, RobotRadius);
  pref->setDouble(300.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.485);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0060);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(true);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(5);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(5);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(24);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);

  pref->setIntSet(true, 4, 8, 50, 130, 90);
  pref->setIntSet(true, 4, 9, 90, 115, 50);
  pref->setIntSet(true, 4, 10, 125, 80, 30);
  pref->setIntSet(true, 4, 11, 145, 25, 10);
  pref->setIntSet(true, 4, 12, 145, -25, -10);
  pref->setIntSet(true, 4, 13, 125, -80, -30);
  pref->setIntSet(true, 4, 14, 90, -115, -50);
  pref->setIntSet(true, 4, 15, 50, -130, -90);

  pref->setIntSet(true, 4, 16, -115, -130, -90);
  pref->setIntSet(true, 4, 17, -155, -115, -130);
  pref->setIntSet(true, 4, 18, -190, -80, -150);
  pref->setIntSet(true, 4, 19, -210, -25, -170);
  pref->setIntSet(true, 4, 20, -210, 25, 170);
  pref->setIntSet(true, 4, 21, -190, 80, 150);
  pref->setIntSet(true, 4, 22, -155, 115, 130);
  pref->setIntSet(true, 4, 23, -115, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(1);
  pref=getPref(Laser, LaserY);
  pref->setInt(16);
}

// P2PB robot class

AREXPORT ArRobotP2PB::ArRobotP2PB(const char *dir) :
  ArRobotParamFile(dir, "p2pb.p")
{
}

AREXPORT ArRobotP2PB::~ArRobotP2PB()
{
}

AREXPORT void ArRobotP2PB::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("p2pb");
  pref=getPref(General, RobotRadius);
  pref->setDouble(300.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.424);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.268);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.0056);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(5);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(5);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(24);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);

  pref->setIntSet(true, 4, 8, 50, 130, 90);
  pref->setIntSet(true, 4, 9, 90, 115, 50);
  pref->setIntSet(true, 4, 10, 125, 80, 30);
  pref->setIntSet(true, 4, 11, 145, 25, 10);
  pref->setIntSet(true, 4, 12, 145, -25, -10);
  pref->setIntSet(true, 4, 13, 125, -80, -30);
  pref->setIntSet(true, 4, 14, 90, -115, -50);
  pref->setIntSet(true, 4, 15, 50, -130, -90);

  pref->setIntSet(true, 4, 16, -115, -130, -90);
  pref->setIntSet(true, 4, 17, -155, -115, -130);
  pref->setIntSet(true, 4, 18, -190, -80, -150);
  pref->setIntSet(true, 4, 19, -210, -25, -170);
  pref->setIntSet(true, 4, 20, -210, 25, 170);
  pref->setIntSet(true, 4, 21, -190, 80, 150);
  pref->setIntSet(true, 4, 22, -155, 115, 130);
  pref->setIntSet(true, 4, 23, -115, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(8);
  pref=getPref(Laser, LaserY);
  pref->setInt(17);
}

// PerfPB robot class

AREXPORT ArRobotPerfPB::ArRobotPerfPB(const char *dir) :
  ArRobotParamFile(dir, "perfpb.p")
{
}

AREXPORT ArRobotPerfPB::~ArRobotPerfPB()
{
}

AREXPORT void ArRobotPerfPB::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("perfpb");
  pref=getPref(General, RobotRadius);
  pref->setDouble(340.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2200.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(true);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.485);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(1);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.006);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(true);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(true);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(5);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(5);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(32);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 115, 130, 90);
  pref->setIntSet(true, 4, 1, 155, 115, 50);
  pref->setIntSet(true, 4, 2, 190, 80, 30);
  pref->setIntSet(true, 4, 3, 210, 25, 10);
  pref->setIntSet(true, 4, 4, 210, -25, -10);
  pref->setIntSet(true, 4, 5, 190, -80, -30);
  pref->setIntSet(true, 4, 6, 155, -115, -50);
  pref->setIntSet(true, 4, 7, 115, -130, -90);

  pref->setIntSet(true, 4, 8, 50, 130, 90);
  pref->setIntSet(true, 4, 9, 90, 115, 50);
  pref->setIntSet(true, 4, 10, 125, 80, 30);
  pref->setIntSet(true, 4, 11, 145, 25, 10);
  pref->setIntSet(true, 4, 12, 145, -25, -10);
  pref->setIntSet(true, 4, 13, 125, -80, -30);
  pref->setIntSet(true, 4, 14, 90, -115, -50);
  pref->setIntSet(true, 4, 15, 50, -130, -90);

  pref->setIntSet(true, 4, 16, -115, -130, -90);
  pref->setIntSet(true, 4, 17, -155, -115, -130);
  pref->setIntSet(true, 4, 18, -190, -80, -150);
  pref->setIntSet(true, 4, 19, -210, -25, -170);
  pref->setIntSet(true, 4, 20, -210, 25, 170);
  pref->setIntSet(true, 4, 21, -190, 80, 150);
  pref->setIntSet(true, 4, 22, -155, 115, 130);
  pref->setIntSet(true, 4, 23, -115, 130, 90);

  pref->setIntSet(true, 4, 24, -195, -130, -90);
  pref->setIntSet(true, 4, 25, -235, -115, -130);
  pref->setIntSet(true, 4, 26, -270, -80, -150);
  pref->setIntSet(true, 4, 27, -290, -25, -170);
  pref->setIntSet(true, 4, 28, -290, 25, 170);
  pref->setIntSet(true, 4, 29, -270, 80, 150);
  pref->setIntSet(true, 4, 30, -235, 115, 130);
  pref->setIntSet(true, 4, 31, -195, 130, 90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(21);
}

AREXPORT ArRobotPion1M::ArRobotPion1M(const char *dir) :
  ArRobotParamFile(dir, "pion1m.p")
{
}

AREXPORT ArRobotPion1M::~ArRobotPion1M()
{
}

AREXPORT void ArRobotPion1M::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("pion1m");
  pref=getPref(General, RobotRadius);
  pref->setDouble(220.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(90.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(100.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(400.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(0);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.0061359);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.05066);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(2.5332);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.1734);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(0.0033);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(4.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(7);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 100, 100, 90);
  pref->setIntSet(true, 4, 1, 120, 80, 30);
  pref->setIntSet(true, 4, 2, 130, 40, 15);
  pref->setIntSet(true, 4, 3, 130, 0, 0);
  pref->setIntSet(true, 4, 4, 130, -40, -15);
  pref->setIntSet(true, 4, 5, 120, -80, -30);
  pref->setIntSet(true, 4, 6, 100, -100, -90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);
}

AREXPORT ArRobotPsos1M::ArRobotPsos1M(const char *dir) :
  ArRobotParamFile(dir, "psos1m.p")
{
}

AREXPORT ArRobotPsos1M::~ArRobotPsos1M()
{
}

AREXPORT void ArRobotPsos1M::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("psos1m");
  pref=getPref(General, RobotRadius);
  pref->setDouble(220.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(90.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(100.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(400.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(0);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.0061359);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.05066);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(2.5332);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.1734);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(0.0033);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(4.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(7);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 100, 100, 90);
  pref->setIntSet(true, 4, 1, 120, 80, 30);
  pref->setIntSet(true, 4, 2, 130, 40, 15);
  pref->setIntSet(true, 4, 3, 130, 0, 0);
  pref->setIntSet(true, 4, 4, 130, -40, -15);
  pref->setIntSet(true, 4, 5, 120, -80, -30);
  pref->setIntSet(true, 4, 6, 100, -100, -90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);
}

AREXPORT ArRobotPsos43M::ArRobotPsos43M(const char *dir) :
  ArRobotParamFile(dir, "psos1m.p")
{
}

AREXPORT ArRobotPsos43M::~ArRobotPsos43M()
{
}

AREXPORT void ArRobotPsos43M::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("psos1m");
  pref=getPref(General, RobotRadius);
  pref->setDouble(220.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(90.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(100.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(400.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(0);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.0061359);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.05066);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(2.5332);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.1734);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(0.0033);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(4.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(7);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 100, 100, 90);
  pref->setIntSet(true, 4, 1, 120, 80, 30);
  pref->setIntSet(true, 4, 2, 130, 40, 15);
  pref->setIntSet(true, 4, 3, 130, 0, 0);
  pref->setIntSet(true, 4, 4, 130, -40, -15);
  pref->setIntSet(true, 4, 5, 120, -80, -30);
  pref->setIntSet(true, 4, 6, 100, -100, -90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);
}


// PionAT robot class

AREXPORT ArRobotPionAT::ArRobotPionAT(const char *dir) :
  ArRobotParamFile(dir, "pionat.p")
{
}

AREXPORT ArRobotPionAT::~ArRobotPionAT()
{
}

AREXPORT void ArRobotPionAT::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("pionat");
  pref=getPref(General, RobotRadius);
  pref->setDouble(330.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(100.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(500.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(0);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.0061359);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.07);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(2.5332);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.1734);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(1.0/300.0);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(4.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(7);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 100, 100, 90);
  pref->setIntSet(true, 4, 1, 120, 80, 30);
  pref->setIntSet(true, 4, 2, 130, 40, 15);
  pref->setIntSet(true, 4, 3, 130, 0, 0);
  pref->setIntSet(true, 4, 4, 130, -40, -15);
  pref->setIntSet(true, 4, 5, 120, -80, -30);
  pref->setIntSet(true, 4, 6, 100, -100, -90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);

}


AREXPORT ArRobotPion1X::ArRobotPion1X(const char *dir) :
  ArRobotParamFile(dir, "pion1x.p")
{
}

AREXPORT ArRobotPion1X::~ArRobotPion1X()
{
}

AREXPORT void ArRobotPion1X::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("pion1x");
  pref=getPref(General, RobotRadius);
  pref->setDouble(220.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(90.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(100.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(400.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(0);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.0061359);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.05066);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(0.07979);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.1734);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(1.0/300.0);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(4.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(7);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 100, 100, 90);
  pref->setIntSet(true, 4, 1, 120, 80, 30);
  pref->setIntSet(true, 4, 2, 130, 40, 15);
  pref->setIntSet(true, 4, 3, 130, 0, 0);
  pref->setIntSet(true, 4, 4, 130, -40, -15);
  pref->setIntSet(true, 4, 5, 120, -80, -30);
  pref->setIntSet(true, 4, 6, 100, -100, -90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);

}

AREXPORT ArRobotPsos1X::ArRobotPsos1X(const char *dir) :
  ArRobotParamFile(dir, "psos1x.p")
{
}

AREXPORT ArRobotPsos1X::~ArRobotPsos1X()
{
}

AREXPORT void ArRobotPsos1X::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("psos1x");
  pref=getPref(General, RobotRadius);
  pref->setDouble(220.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(90.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(100.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(400.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(0);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.0061359);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(0.05066);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(0.07979);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(0.1734);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(1.0/300.0);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(4.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(7);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 100, 100, 90);
  pref->setIntSet(true, 4, 1, 120, 80, 30);
  pref->setIntSet(true, 4, 2, 130, 40, 15);
  pref->setIntSet(true, 4, 3, 130, 0, 0);
  pref->setIntSet(true, 4, 4, 130, -40, -15);
  pref->setIntSet(true, 4, 5, 120, -80, -30);
  pref->setIntSet(true, 4, 6, 100, -100, -90);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(false);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(0);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);

}


AREXPORT ArRobotMapper::ArRobotMapper(const char *dir) :
  ArRobotParamFile(dir, "mappr.p")
{
}

AREXPORT ArRobotMapper::~ArRobotMapper()
{
}

AREXPORT void ArRobotMapper::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("mappr");
  pref=getPref(General, RobotRadius);
  pref->setDouble(180.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(120.0);
  pref=getPref(General, Holonomic);
  pref->setInt(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(0.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(0.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(0);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(6.77); //Solid Foam Tires
  //pref->setDouble(7.39); //Pneumatic Tires at 40 psi
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(0.6154);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(1.000);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.011);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(0);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(false);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(0);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(2);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 0, 0, 0);
  pref->setIntSet(true, 4, 0, 0, 0, 0);

  // laser parameters
  pref=getPref(Laser, LaserPossessed);
  pref->setBool(true);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM3");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(false);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(false);
  pref=getPref(Laser, LaserX);
  pref->setInt(312);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);

}

// PowerBot robot class

AREXPORT ArRobotPowerBot::ArRobotPowerBot(const char *dir) :
  ArRobotParamFile(dir, "powerbot.p")
{
}

AREXPORT ArRobotPowerBot::~ArRobotPowerBot()
{
}

AREXPORT void ArRobotPowerBot::setDefaults()
{
  ArPref *pref;

  // General parameters
  pref=getPref(General, Class);
  pref->setString("Pioneer");
  pref=getPref(General, Subclass);
  pref->setString("powerbot");
  pref=getPref(General, RobotRadius);
  pref->setDouble(550.0);
  pref=getPref(General, RobotDiagonal);
  pref->setDouble(400.0);
  pref=getPref(General, Holonomic);
  pref->setDouble(1);
  pref=getPref(General, MaxRVelocity);
  pref->setDouble(360.0);
  pref=getPref(General, MaxVelocity);
  pref->setDouble(2000.0);
  pref=getPref(General, HasMoveCommand);
  pref->setInt(1);
  pref=getPref(General, RequestIOPackets);
  pref->setBool(false);

  // Conversion factors
  pref=getPref(ConvFactors, AngleConvFactor);
  pref->setDouble(0.001534);
  pref=getPref(ConvFactors, DistConvFactor);
  pref->setDouble(.5813);
  pref=getPref(ConvFactors, VelConvFactor);
  pref->setDouble(1.0);
  pref=getPref(ConvFactors, RangeConvFactor);
  pref->setDouble(1.000);
  pref=getPref(ConvFactors, DiffConvFactor);
  pref->setDouble(.00373);
  pref=getPref(ConvFactors, Vel2Divisor);
  pref->setDouble(20.0);

  // Accessories
  pref=getPref(Accessories, TableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, NewTableSensingIR);
  pref->setBool(false);
  pref=getPref(Accessories, FrontBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumFrontBumpers);
  pref->setInt(7);
  pref=getPref(Accessories, RearBumpers);
  pref->setBool(true);
  pref=getPref(Accessories, NumRearBumpers);
  pref->setInt(5);

  // Sonar parameters
  pref=getPref(Sonar, SonarNum);
  pref->setInt(31);
  pref=getPref(Sonar, SonarUnit);
  pref->setIntSet(false, 4, 0, 152, 278, 90);
  pref->setIntSet(true, 4, 1, 200, 267, 65);
  pref->setIntSet(true, 4, 2, 241, 238, 45);
  pref->setIntSet(true, 4, 3, 274, 200, 35);
  pref->setIntSet(true, 4, 4, 300, 153, 25);
  pref->setIntSet(true, 4, 5, 320, 96, 15);
  pref->setIntSet(true, 4, 6, 332, 33, 5);
  pref->setIntSet(true, 4, 7, 0, 0, -180);

  pref->setIntSet(true, 4, 8, 332, -33, -5);
  pref->setIntSet(true, 4, 9, 320, -96, -15);
  pref->setIntSet(true, 4, 10, 300, -153, -25);
  pref->setIntSet(true, 4, 11, 274, -200, -35);
  pref->setIntSet(true, 4, 12, 241, -238, -45);
  pref->setIntSet(true, 4, 13, 200, -267, -65);
  pref->setIntSet(true, 4, 14, 152, -278, -90);
  pref->setIntSet(true, 4, 15, 0, 0, -180);

  pref->setIntSet(true, 4, 16, -298, -278, -90);
  pref->setIntSet(true, 4, 17, -347, -267, -115);
  pref->setIntSet(true, 4, 18, -388, -238, -135);
  pref->setIntSet(true, 4, 19, -420, -200, -145);
  pref->setIntSet(true, 4, 20, -447, -153, -155);
  pref->setIntSet(true, 4, 21, -467, -96, -165);
  pref->setIntSet(true, 4, 22, -478, -33, -175);
  pref->setIntSet(true, 4, 23, 0, 0, -180);

  pref->setIntSet(true, 4, 24, -478, 33, 175);
  pref->setIntSet(true, 4, 25, -467, 96, 165);
  pref->setIntSet(true, 4, 26, -447, 153, 155);
  pref->setIntSet(true, 4, 27, -420, 200, 145);
  pref->setIntSet(true, 4, 28, -388, 238, 135);
  pref->setIntSet(true, 4, 29, -347, 267, 115);
  pref->setIntSet(true, 4, 30, -298, 278, 90);

  pref=getPref(Laser, LaserPossessed);
  pref->setBool(true);
  pref=getPref(Laser, LaserPort);
  pref->setString("COM2");
  pref=getPref(Laser, LaserPowerControlled);
  pref->setBool(true);
  pref=getPref(Laser, LaserFlipped);
  pref->setBool(true);
  pref=getPref(Laser, LaserX);
  pref->setInt(251);
  pref=getPref(Laser, LaserY);
  pref->setInt(0);
}

