/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include <errno.h>
#include "ArSemaphore.h"
#include "ArLog.h"





ArSemaphore::ArSemaphore() :
  myFailedInit(false),
  mySema(),
  myStrMap()
{
  myStrMap[STATUS_FAILED_INIT]="failed to initialize";
  myStrMap[STATUS_TRY_AGAIN]="try again";
  myStrMap[STATUS_FAILED_UNKNOWN]="failed";

  if (sem_init(&mySema, 0, 0) < 0)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::ArSemaphore: Failed to create semaphore");
    myFailedInit=true;
  }
}

ArSemaphore::~ArSemaphore()
{
  if (!myFailedInit && (sem_destroy(&mySema) < 0))
  {
    if (errno == EBUSY)
      ArLog::log(ArLog::Terse, "ArSemaphore::~ArSemaphore: Failed to destroy semaphore. A thread is currently blocked waiting for this semaphore.");
    else
      ArLog::log(ArLog::Terse, "ArSemaphore::~ArSemaphore: Failed to destroy semaphore. Unknown error.");
  }      
}

int ArSemaphore::wait()
{
  if (myFailedInit)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::wait: Initialization of semaphore failed, failed wait");
    return(STATUS_FAILED_INIT);
  }

  sem_wait(&mySema);

  return(0);
}

int ArSemaphore::tryWait()
{
  if (myFailedInit)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::tryWait: Initialization of semaphore failed, failed tryWait");
    return(STATUS_FAILED_INIT);
  }

  if (sem_trywait(&mySema) < 0)
  {
    if (errno == EAGAIN)
    {
      ArLog::log(ArLog::Terse, "ArSemaphore::tryWait: The wait would block. Returning");
      return(STATUS_TRY_AGAIN);
    }
    else
    {
      ArLog::log(ArLog::Terse, "ArSemaphore::tryWait: Failed the wait with unknown error");
      return(STATUS_FAILED_UNKNOWN);
    }
  }

  return(0);
}

int ArSemaphore::post()
{
  if (myFailedInit)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::post: Initialization of semaphore failed, failed post");
    return(STATUS_FAILED_INIT);
  }
    
  if (sem_post(&mySema) < 0)
  {
    if (errno == ERANGE)
      ArLog::log(ArLog::Terse, "ArSemaphore::post: Failed to post. Semaphore value excedded maximum value.");
    else
      ArLog::log(ArLog::Terse, "ArSemaphore::post: Failed the wait with unknown error");
    return(STATUS_FAILED_UNKNOWN);
  }

  return(0);
}

int ArSemaphore::getValue(int &value)
{
  if (myFailedInit)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::getValue: Initialization of semaphore failed, failed to get value");
    return(STATUS_FAILED_INIT);
  }

  sem_getvalue(&mySema, &value);

  return(0);
}

std::string ArSemaphore::getError(int messageNumber)
{
  return(myStrMap[messageNumber]);
}
