/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArSemaphore.h"
#include "ArLog.h"


ArSemaphore::ArSemaphore() :
  myFailedInit(false),
  mySema(),
  myStrMap()
#ifdef WIN32
  ,myMutex(),
  myCount(0)
#endif
{
  myStrMap[STATUS_FAILED_INIT]="failed to initialize";
  myStrMap[STATUS_TRY_AGAIN]="try again";
  myStrMap[STATUS_FAILED_UNKNOWN]="failed";

  // I guess 100 for a maximum count is okay...Dont
  // want to go to high cause I bet windows will get
  // screwy with kernel resource allocation...
  mySema=CreateSemaphore(0, 0, 100, 0);
  if (!mySema)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::ArSemaphore: Failed to create semaphore");
    myFailedInit=true;
  }
}

ArSemaphore::~ArSemaphore()
{
  if (!myFailedInit && !CloseHandle(mySema))
    ArLog::log(ArLog::Terse, "ArSemaphore::~ArSemaphore: Failed to destroy semaphore.");
}

int ArSemaphore::wait()
{
  DWORD ret;

  if (myFailedInit)
  {
    ArLog::logNoLock(ArLog::Terse, "ArSemaphore::wait: Initialization of semaphore failed, failed wait");
    return(STATUS_FAILED_INIT);
  }

  ret=WaitForSingleObject(mySema, INFINITE);
  if (ret == WAIT_ABANDONED)
  {
    ArLog::logNoLock(ArLog::Terse, "ArSemaphore::wait: Tried to wait on a semaphore which was locked by a different thread and never unlocked before that thread exited. This is a recoverable error");
    return(STATUS_FAILED_UNKNOWN);
  }
  else if (ret == WAIT_OBJECT_0)
    return(0);
  else
  {
    ArLog::logNoLock(ArLog::Terse, "ArSemaphore::wait: Failed to wait due to an unknown error");
    return(STATUS_FAILED_UNKNOWN);
  }

#ifdef WIN32
  myMutex.lock();
#endif
  myCount--;
#ifdef WIN32
  myMutex.unlock();
#endif

  return(0);
}

int ArSemaphore::tryWait()
{
  DWORD ret;

  if (myFailedInit)
  {
    ArLog::logNoLock(ArLog::Terse, "ArSemaphore::tryWait: Initialization of semaphore failed, failed wait");
    return(STATUS_FAILED_INIT);
  }

  // Attempt to wait as little as posesible
  ret=WaitForSingleObject(mySema, 1);
  if (ret == WAIT_ABANDONED)
  {
    ArLog::logNoLock(ArLog::Terse, "ArSemaphore::tryWait: Tried to wait on a semaphore which was locked by a different thread and never unlocked before that thread exited. This is a recoverable error");
    return(STATUS_FAILED_UNKNOWN);
  }
  else if (ret == WAIT_OBJECT_0)
    return(0);
  else
  {
    ArLog::logNoLock(ArLog::Terse, "ArSemaphore::tryWait: Failed to wait due to an unknown error");
    return(STATUS_FAILED_UNKNOWN);
  }

#ifdef WIN32
  myMutex.lock();
#endif
  myCount--;
#ifdef WIN32
  myMutex.unlock();
#endif

  return(0);
}

int ArSemaphore::post()
{
  if (myFailedInit)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::post: Initialization of semaphore failed, failed post");
    return(STATUS_FAILED_INIT);
  }

  if (!ReleaseSemaphore(mySema, 1, &myCount))
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::post: Failed to post due to an unknown error");
    return(STATUS_FAILED_UNKNOWN);
  }

  return(0);
}

int ArSemaphore::getValue(int &value)
{
  if (myFailedInit)
  {
    ArLog::log(ArLog::Terse, "ArSemaphore::getValue: Initialization of semaphore failed, failed post");
    return(STATUS_FAILED_INIT);
  }

#ifdef WIN32
  myMutex.lock();
#endif
  value=myCount;
#ifdef WIN32
  myMutex.unlock();
#endif

  return(0);
}

std::string ArSemaphore::getError(int messageNumber)
{
  return(myStrMap[messageNumber]);
}
