/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArSick.h"
#include "ArRobot.h"
#include "ArSerialConnection.h"
#include <time.h>

AREXPORT ArSick::ArSick(size_t currentBufferSize, size_t cumulativeBufferSize, 
			const char *name) :
  ArRangeDeviceThreaded(currentBufferSize, cumulativeBufferSize, name, 32768),
  myRobotConnectCB(this, &ArSick::robotConnectCallback),
  mySimPacketHandler(this, &ArSick::simPacketHandler),
  mySensorInterpCB(this, &ArSick::sensorInterpCallback),
  mySickPacketReceiver(0, true)
{
  configure();
  setSensorPosition(0, 0, 0);
  myAssembleReadings = new std::list<ArSensorReading *>;
  myCurrentReadings = new std::list<ArSensorReading *>;
  myIter = myAssembleReadings->begin();
  myConn = NULL;
  myRobot = NULL;
  myStartConnect = false;
  myRunningOnRobot = false;
  switchState(STATE_NONE);
  myProcessImmediately = false;
  myInterpolation = true;
  myTimeoutTime = 8;
  // default filter params
  //myFilterNearDist = 50.0;	// 5 cm distance between saved readings
  myFilterNearDist = 0;	// 5 cm distance between saved readings
  myRealConfigured = false;
}

AREXPORT ArSick::~ArSick(void)
{
  if (myRobot != NULL)
  {
    myRobot->remPacketHandler(&mySimPacketHandler);
    myRobot->remSensorInterpTask(&mySensorInterpCB);
    myRobot->addConnectCB(&myRobotConnectCB, ArListPos::FIRST);
  }
  lockDevice();
  if (isConnected())
  {
    disconnect();
  }
  unlockDevice();
}

AREXPORT void ArSick::robotConnectCallback(void)
{
  ArRobotParams params;
  if (myRealConfigured || !myRobot->isConnected()) 
    return;
  params = myRobot->getParams();
  myLaserFlipped = params.getLaserFlipped();
  myPowerControl = params.getLaserPowerControlled();
  
  if (myDegrees == DEGREES180)
    myOffsetAmount = 90;
  else if (myDegrees == DEGREES100)
    myOffsetAmount = 50;
  else
  {
    myOffsetAmount = 0;
    ArLog::log(ArLog::Terse,"ArSick::robotConnectCallback: bad degrees configured.\n");
  }
  

  if (myLaserFlipped)
    myOffsetAmount *= -1;

  if (myIncrement == INCREMENT_ONE)
    myIncrementAmount = 1.0;
  else if (myIncrement == INCREMENT_HALF)
    myIncrementAmount = 0.5;
  else
  {
    myIncrementAmount = 0;
    ArLog::log(ArLog::Terse,"ArSick::robotConnectCallback: bad increment configured.\n");
  }
  if (myLaserFlipped)
    myIncrementAmount *= -1;

  setSensorPosition(params.getLaserX(), params.getLaserY(), 0);
}

AREXPORT bool ArSick::isUsingSim(void)
{
  return myUseSim;
}

AREXPORT bool ArSick::isControllingPower(void)
{
  return myPowerControl;
}

AREXPORT bool ArSick::isLaserFlipped(void)
{
  return myLaserFlipped;
}

AREXPORT ArSick::Degrees ArSick::getDegrees(void)
{
  return myDegrees;
}

AREXPORT ArSick::Increment ArSick::getIncrement(void)
{
  return myIncrement;
}

AREXPORT void ArSick::setSensorPosition(double x, double y, double th)
{
  mySensorPose.setPose(x, y, th);
}

AREXPORT void ArSick::setSensorPosition(ArPose pose)
{
  mySensorPose = pose;
}

AREXPORT ArPose ArSick::getSensorPosition(void)
{
  return mySensorPose;
}

AREXPORT double ArSick::getSensorPositionX(void)
{
  return mySensorPose.getX();
}

AREXPORT double ArSick::getSensorPositionY(void)
{
  return mySensorPose.getY();
}

AREXPORT double ArSick::getSensorPositionTh(void)
{
  return mySensorPose.getTh();
}


AREXPORT void ArSick::setDeviceConnection(ArDeviceConnection *conn)
{
  myConnLock.lock();
  myConn = conn; 
  mySickPacketReceiver.setDeviceConnection(conn); 
  myConnLock.unlock();
}

AREXPORT ArDeviceConnection *ArSick::getDeviceConnection(void)
{
  return myConn;
}

/**
   Sets the number of seconds to go without a response from the laser
   until it is assumed that the connection with the laser has been
   broken and the disconnect on error events will happen.

   @param seconds if seconds is 0 then the connection timeout feature
   will be disabled, otherwise disconnect on error will be triggered
   after this number of seconds...  
**/
AREXPORT void ArSick::setConnectionTimeoutTime(int mSecs)
{
  if (mSecs > 0)
    myTimeoutTime = mSecs;
  else
    myTimeoutTime = 0;
}

/**
   Gets the number of seconds to go without response from the laser
   until it is assumed tha tthe connection with the laser has been
   broken and the disconnect on error events will happen.
**/
AREXPORT int ArSick::getConnectionTimeoutTime(void)
{
  return myTimeoutTime;
}

AREXPORT ArTime ArSick::getLastReadingTime(void)
{
  return myLastReading;
}

AREXPORT void ArSick::setRobot(ArRobot *robot)
{
  myRobot = robot;
  if (myRobot != NULL)
  {
    myRobot->addPacketHandler(&mySimPacketHandler, ArListPos::LAST);
    myRobot->addSensorInterpTask("sick", 10, &mySensorInterpCB);
    myRobot->addConnectCB(&myRobotConnectCB, ArListPos::FIRST);
  }
}

/**
   You must lockDevice the laser or not have the laser being poked at
   by multiple threads before you use htis function call
**/
AREXPORT void ArSick::configure(bool useSim, bool powerControl,
				bool laserFlipped, BaudRate baud,
				Degrees deg, Increment incr)
{

  myUseSim = useSim;
  myPowerControl = powerControl;
  myLaserFlipped = laserFlipped;
  myBaud = baud;
  myDegrees = deg;
  myIncrement = incr;

  if (myDegrees == DEGREES180)
    myOffsetAmount = 90;
  else if (myDegrees == DEGREES100)
    myOffsetAmount = 50;
  else
  {
    myOffsetAmount = 0;
    ArLog::log(ArLog::Terse,"ArSick::configure: bad degrees configured.\n");
  }
  
  if (myLaserFlipped)
    myOffsetAmount *= -1;

  if (myIncrement == INCREMENT_ONE)
    myIncrementAmount = 1.0;
  else if (myIncrement == INCREMENT_HALF)
    myIncrementAmount = 0.5;
  else
  {
    myIncrementAmount = 0;
    ArLog::log(ArLog::Terse,"ArSick::configure: bad increment configured.\n");
  }

  if (myLaserFlipped)
    myIncrementAmount *= -1;

  myRealConfigured = true;
}

/**
   You must lockDevice the laser or not have the laser being poked at
   by multiple threads before you use htis function call
**/
AREXPORT void ArSick::configureShort(bool useSim, BaudRate baud,
				Degrees deg, Increment incr)
{

  myUseSim = useSim;
  myPowerControl = true;
  myLaserFlipped = false;
  myBaud = baud;
  myDegrees = deg;
  myIncrement = incr;

  if (myDegrees == DEGREES180)
    myOffsetAmount = 90;
  else if (myDegrees == DEGREES100)
    myOffsetAmount = 50;
  else
  {
    myOffsetAmount = 0;
    ArLog::log(ArLog::Terse,"ArSick::configureShort: bad degrees configured.\n");
  }
  
  if (myLaserFlipped)
    myOffsetAmount *= -1;

  if (myIncrement == INCREMENT_ONE)
    myIncrementAmount = 1.0;
  else if (myIncrement == INCREMENT_HALF)
    myIncrementAmount = 0.5;
  else
  {
    myIncrementAmount = 0;
    ArLog::log(ArLog::Terse,"ArSick::configureShort: bad increment configured.\n");
  }
  if (myLaserFlipped)
    myIncrementAmount *= -1;

  myRealConfigured = false;
  // if we're connected, just have this set things
  if (myRobot != NULL && myRobot->isConnected())
    robotConnectCallback();
}

AREXPORT bool ArSick::simPacketHandler(ArRobotPacket *packet)
{
  unsigned int totalNumReadings;
  unsigned int readingNumber;
  double atDeg;
  unsigned int i;
  ArSensorReading *reading;
  std::list<ArSensorReading *>::iterator tempIt;
  unsigned int dist;
  unsigned int newReadings;
  ArPose encoderPose;
  
  if (packet->getID() != 0x60)
    return false;

  // if we got here, its the right type of packet

  //printf("Got in a packet from the simulator\n");
  lockDevice();
  //printf("1\n");
  if (!myUseSim)
  {
    ArLog::log(ArLog::Terse, "ArSick: Got a packet from the simulator with laser information, but the laser is not being simulated, major trouble.");
    unlockDevice();
    return true;
  }
  // ignore the positional information
  packet->bufToByte2();
  packet->bufToByte2();
  packet->bufToByte2();
  totalNumReadings = packet->bufToByte2(); // total for this reading
  readingNumber = packet->bufToByte2(); // which one we're on in this packet
  newReadings = packet->bufToUByte(); // how many are in this packet
  if (readingNumber == 0)
  {
    mySimPacketStart = myRobot->getPose();
    mySimPacketTrans = myRobot->getToGlobalTransform();
    mySimPacketEncoderTrans = myRobot->getEncoderTransform();
    mySimPacketCounter = myRobot->getCounter();
  }
  //printf("On reading number %d out of %d, new %d\n", readingNumber, totalNumReadings, newReadings);
  // if we have too many readings in our list of raw readings, pop the extras
  while (myAssembleReadings->size() > totalNumReadings)
  {
    ArLog::log(ArLog::Verbose, "ArSick::simPacketHandler, too many readings, popping one.\n");
    tempIt = myAssembleReadings->begin();
    if (tempIt != myAssembleReadings->end())
      delete (*tempIt);
    myAssembleReadings->pop_front();
  }
  
  // If we don't have any sensor readings created at all, make 'em all now
  if (myAssembleReadings->size() == 0)
    for (i = 0; i < totalNumReadings; i++)
      myAssembleReadings->push_back(new ArSensorReading);
  
  // Okay, we know where we're at, so get an iterator to the right spot, or 
  // make sure the one we keep around is in the right spot... if neither of
  // these trigger, then the iter should be in the right spot
  if ((readingNumber != myWhichReading + 1) || 
      totalNumReadings != myTotalNumReadings)
  {
    //printf("2\n");
    myWhichReading = readingNumber;
    myTotalNumReadings = totalNumReadings;
    for (i = 0, myIter = myAssembleReadings->begin(); i < readingNumber; i++)
    {
      tempIt = myIter;
      tempIt++;
      if (tempIt == myAssembleReadings->end() && (i + 1 != myTotalNumReadings))
	myAssembleReadings->push_back(new ArSensorReading);
      myIter++;
    }
  }
  else
  {
    //printf("3\n");
    myWhichReading = readingNumber;
  }

  atDeg = (mySensorPose.getTh() - myOffsetAmount + 
	   readingNumber * myIncrementAmount);
  //printf("4\n");
  encoderPose = mySimPacketEncoderTrans.doInvTransform(mySimPacketStart);
  // while we have in the readings and have stuff left we can read 
  for (i = 0; 
       //	 (myWhichReading < myTotalNumReadings && 
       //	  packet->getReadLength() < packet->getLength() - 4);
       i < newReadings;
       i++, myWhichReading++, atDeg += myIncrementAmount)
  {
    reading = (*myIter);
    reading->resetSensorPosition(ArMath::roundInt(mySensorPose.getX()),
				 ArMath::roundInt(mySensorPose.getY()),
				 atDeg);
    dist = packet->bufToUByte2();
    //      printf("dist %d\n", dist);
    reading->newData(dist, mySimPacketStart, 
		     encoderPose,
		     mySimPacketTrans,
		     mySimPacketCounter, packet->getTimeReceived());
    //addReading(reading->getX(), reading->getY());
    tempIt = myIter;
    tempIt++;
    if (tempIt == myAssembleReadings->end() && 
	myWhichReading + 1 != myTotalNumReadings)
    {
      myAssembleReadings->push_back(new ArSensorReading);
    }
    myIter++;
  }

  // check if the sensor set is complete
  //printf("%d %d %d\n", newReadings, readingNumber, totalNumReadings);
  if (newReadings + readingNumber >= totalNumReadings)
  {
    // set ArRangeDevice buffer
    myRawReadings = myAssembleReadings;
    // switch internal buffers
    myAssembleReadings = myCurrentReadings;
    myCurrentReadings = myRawReadings;
    // We have in all the readings, now sort 'em and update the current ones
    filterReadings();

    if (myTimeLastSickPacket != time(NULL)) 
      {
	myTimeLastSickPacket = time(NULL);
	mySickPacCount = mySickPacCurrentCount;
	mySickPacCurrentCount = 0;
      }
    mySickPacCurrentCount++;
    myLastReading.setToNow();
  }	
  
  unlockDevice();
  return true;
}

/**
 filter readings here, from raw current buffer to filtered current
     buffer of the range device object, and then to the cumulative
     buffer

 current buffer filtering is to eliminate max (null) readings, and
     compress close readings

 cumulative buffer filtering is to replace readings within the scope of the
     current sensor set
**/

void ArSick::filterReadings()
{
  std::list<ArSensorReading *>::iterator sensIt;
  ArSensorReading *sReading;
  double x, y, d;
  double ox = 0.0, oy = 0.0;
  unsigned int i;
  int drange = (int)getMaxRange();

  sensIt = myRawReadings->begin();
  sReading = (*sensIt);
  myCurrentBuffer.setPoseTaken(sReading->getPoseTaken());
  double dd = myFilterNearDist * myFilterNearDist; // squared nearness dist
  //myCurrentBuffer.reset();

  if (myCurrentBuffer.getBuffer()->size() == 0)
    for (i = 0; i < myRawReadings->size(); i++)
      myCurrentBuffer.getBuffer()->push_back(new ArPose);
  
  i = 0;
  for (myCurrentBuffer.beginRedoBuffer();	  
       sensIt != myRawReadings->end(); 
       ++sensIt)
  {
    sReading = (*sensIt);
    if (sReading->getRange() < drange && sReading->getRange() > 100) // ok, have a valid reading
    {
      x = sReading->getX();
      y = sReading->getY();
      
      if (dd > 0.0000001)
      {
	d = (x-ox)*(x-ox) + (y-oy)*(y-oy);
	if (d > dd)		// moved enough
	{
	  ox = x;
	  oy = y;
	}
	else
	{
	  continue;		// nope, look at next one
	}
      }
      myCurrentBuffer.redoReading(x, y);
      i++;
    }
  }
  myCurrentBuffer.endRedoBuffer();
}

AREXPORT void ArSick::switchState(State state)
{
  myState = state;
  myStateStart.setToNow();
}

/**
   @return 0 if its still trying to connect, 1 if it connected, 2 if it failed
**/
AREXPORT int ArSick::internalConnectHandler(void)
{
  ArSickPacket *packet;
  ArSerialConnection *conn;
  int value;

  switch (myState)
  {
  case STATE_INIT:
    if (myConn->getStatus() != ArDeviceConnection::STATUS_OPEN)
    {
      if ((conn = dynamic_cast<ArSerialConnection *>(myConn)) != NULL)
      {
	conn->setBaud(9600);
      }
      if (!myConn->openSimple())
      {
	ArLog::log(ArLog::Terse,
		   "ArSick: Failed to connect to laser, could not open port.");
	switchState(STATE_NONE);
	failedConnect();
	return 2;
      }
    }
    if (!myPowerControl)
    {
      /*
      myPacket.empty();
      myPacket.uByteToBuf(0x20);
      myPacket.uByteToBuf(0x25);
      myPacket.finalize();
      myConn->write(myPacket.getBuf(), myPacket.getLength());
      */
      switchState(STATE_CHANGE_BAUD);
      return internalConnectHandler();
    }
    ArLog::log(ArLog::Terse, "ArSick: waiting for laser to power on.");
    myPacket.empty();
    myPacket.uByteToBuf(0x10);
    myPacket.finalize();
    if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
    {
      switchState(STATE_WAIT_FOR_POWER_ON);
      return 0;
    }
    else
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, could not send init.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_WAIT_FOR_POWER_ON:
    while ((packet = mySickPacketReceiver.receivePacket()) != NULL)
    {
      if (packet->getID() == 0x90)
      {
	switchState(STATE_CHANGE_BAUD);
	return 0;
      }
    }
    if (myStateStart.secSince() > 65)
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, no poweron received.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_CHANGE_BAUD:
    myPacket.empty();
    myPacket.byteToBuf(0x20);
    if (myBaud == BAUD9600)
      myPacket.byteToBuf(0x42);
    else if (myBaud == BAUD19200)
      myPacket.byteToBuf(0x41);
    else if (myBaud == BAUD38400)
      myPacket.byteToBuf(0x40);
    myPacket.finalize();
    if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
    {
      ArUtil::sleep(20);
      if ((conn = dynamic_cast<ArSerialConnection *>(myConn)))
      {
	if (myBaud == BAUD9600)
	  conn->setBaud(9600);
	else if (myBaud == BAUD19200)
	  conn->setBaud(19200);
	else if (myBaud == BAUD38400)
	  conn->setBaud(38400);
      }
      switchState(STATE_CONFIGURE);
      return 0;
    }
    else
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, could not send baud command.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_CONFIGURE:
    // wait at least a 100 ms for the baud to change
    if (myStateStart.mSecSince() < 300)
      return 0;
    myPacket.empty();
    myPacket.byteToBuf(0x3b);
    myPacket.uByte2ToBuf(abs(ArMath::roundInt(myOffsetAmount * 2)));
    myPacket.uByte2ToBuf(abs(ArMath::roundInt(myIncrementAmount * 100)));
    myPacket.finalize();
    if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
    {
      switchState(STATE_WAIT_FOR_CONFIGURE_ACK);
      return 0;
    }
    else
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, could not send configure command.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_WAIT_FOR_CONFIGURE_ACK:
    while ((packet = mySickPacketReceiver.receivePacket()) != NULL)
    {
      if (packet->getID() == 0xbb)
      {
	value = packet->bufToByte();
	if (value == 0)
	{
	  ArLog::log(ArLog::Terse, 
		     "ArSick: Could not configure laser, failed connect.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
	else if (value == 1)
	{
	  // here
	  //switchState(STATE_START_READINGS);
	  switchState(STATE_INSTALL_MODE);
	  return 0;
	}
	else
	{
	  ArLog::log(ArLog::Terse, 
		     "ArSick: Could not configure laser, failed connect.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
      }
      else if (packet->getID() == 0xb0)
      {

	ArLog::log(ArLog::Terse, "ArSick: extra data packet while waiting for configure ack");
	myPacket.empty();
	myPacket.uByteToBuf(0x20);
	myPacket.uByteToBuf(0x25);
	myPacket.finalize();
	if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
	{
	  switchState(STATE_CONFIGURE);
	  return 0;
	}
      }
      else
	ArLog::log(ArLog::Terse, "ArSick: Got a 0x%x\n", packet->getID());
    }
    if (myStateStart.mSecSince() > 10000)
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, no configure acknowledgement received.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_INSTALL_MODE:
    if (myStateStart.mSecSince() < 200)
      return 0;
    myPacket.empty();
    myPacket.byteToBuf(0x20);
    myPacket.byteToBuf(0x00);
    myPacket.strNToBuf("SICK_LMS", strlen("SICK_LMS"));
    myPacket.finalize();
    if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
    {
      switchState(STATE_WAIT_FOR_INSTALL_MODE_ACK);
      return 0;
    }
    else
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, could not send start command.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_WAIT_FOR_INSTALL_MODE_ACK:
    while ((packet = mySickPacketReceiver.receivePacket()) != NULL)
    {
      if (packet->getID() == 0xa0)
      {
	value = packet->bufToByte();
	if (value == 0)
	{
	  //printf("Um, should set mode?\n");
	  switchState(STATE_SET_MODE);
	  return 0;
	}
	else if (value == 1)
	{
	  ArLog::log(ArLog::Terse, 
		     "ArSick: Could not start laser, incorrect password.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
	else if (value == 2)
	{
	  ArLog::log(ArLog::Terse, 
		     "ArSick: Could not start laser, LMI fault.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
	else
	{
	  ArLog::log(ArLog::Terse, 
		     "ArSick: Could not start laser, unknown problem.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
      }
      else if (packet->getID() == 0xb0)
      {
	
	ArLog::log(ArLog::Terse, "ArSick: extra data packet\n");
	myPacket.empty();
	myPacket.uByteToBuf(0x20);
	myPacket.uByteToBuf(0x25);
	myPacket.finalize();
	if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
	{
	  switchState(STATE_INSTALL_MODE);
	  return 0;
	}
      }
      else
	ArLog::log(ArLog::Terse, "ArSick: bad packet 0x%x\n", packet->getID());
    }
    if (myStateStart.mSecSince() > 10000)
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, no install mode ack received.");
      switchState(STATE_NONE);
      return 2;
    }
    break;
  case STATE_SET_MODE:
    if (myStateStart.mSecSince() < 200)
      return 0;
    myPacket.empty();
    // type of packet
    myPacket.byteToBuf(0x77);
    // blanking
    myPacket.uByte2ToBuf(0);
    // peak threshhold, stop threshold
    myPacket.uByte2ToBuf(70);
    // the old peak threshhold thats probably broken
    // myPacket.uByte2ToBuf(0);
    // fog correction
    myPacket.uByteToBuf(0);
    // measurement Mode (fun one)
    myPacket.uByteToBuf(6);
    // unit value (fun one)
    myPacket.uByteToBuf(1);
    // temporary field set
    myPacket.uByteToBuf(0);
    // fields A & B as subtractive
    myPacket.uByteToBuf(0);
    // multiple evaluation
    myPacket.uByteToBuf(2);
    // restart
    myPacket.uByteToBuf(2);
    // restart time
    myPacket.uByteToBuf(0);
    // contour A as reference object size
    myPacket.uByteToBuf(0);
    // contour A positive range of tolerance
    myPacket.uByteToBuf(0);
    // contour A negative range of tolerance
    myPacket.uByteToBuf(0);
    // contour A starting angle
    myPacket.uByteToBuf(0);
    // contour A stopping angle
    myPacket.uByteToBuf(0);
    // contour B as reference object size
    myPacket.uByteToBuf(0);
    // contour B positive range of tolerance
    myPacket.uByteToBuf(0);
    // contour B negative range of tolerance
    myPacket.uByteToBuf(0);
    // contour B starting angle
    myPacket.uByteToBuf(0);
    // contour B stopping angle
    myPacket.uByteToBuf(0);
    // contour C as reference object size
    myPacket.uByteToBuf(0);
    // contour C positive range of tolerance
    myPacket.uByteToBuf(0);
    // contour C negative range of tolerance
    myPacket.uByteToBuf(0);
    // contour C starting angle
    myPacket.uByteToBuf(0);
    // contour C stopping angle
    myPacket.uByteToBuf(0);
    // pixel oriented evaluation
    myPacket.uByteToBuf(0);
    // mode for single meas value eval
    myPacket.uByteToBuf(0);
    // restart times for field b and field c
    myPacket.byte2ToBuf(0);
    // um, an extra one (sick quickstart manual says its 21 not 20 long)
    myPacket.uByteToBuf(0);
    
    myPacket.finalize();
    //myPacket.print();
    //printf("Sending mode!\n");
    if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
    {
      //printf("Set mode!\n");
      switchState(STATE_WAIT_FOR_SET_MODE_ACK);
      return 0;
    }
    else
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, could not send set mode command.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_WAIT_FOR_SET_MODE_ACK:
    while ((packet = mySickPacketReceiver.receivePacket()) != NULL)
    {
      if (packet->getID() == 0xF7)
      {
	//value = packet->bufToByte();
	//printf("YAY %d\n", value);
	//packet->print();
	switchState(STATE_START_READINGS);
	return 0;
      }
      else if (packet->getID() == 0xb0)
      {
	
	ArLog::log(ArLog::Terse, "ArSick: extra data packet\n");
	myPacket.empty();
	myPacket.uByteToBuf(0x20);
	myPacket.uByteToBuf(0x25);
	myPacket.finalize();
	if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
	{
	  switchState(STATE_INSTALL_MODE);
	  return 0;
	}
      }
      else if (packet->getID() == 0x92)
      {
	switchState(STATE_INSTALL_MODE);
	return 0;
      }
      else
	ArLog::log(ArLog::Terse, "ArSick: Got a 0x%x\n", packet->getID());
    }
    if (myStateStart.mSecSince() > 14000)
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, no set mode acknowledgement received.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
    case STATE_START_READINGS: 
      if (myStateStart.mSecSince() < 200)
      return 0;
    myPacket.empty();
    myPacket.byteToBuf(0x20);
    myPacket.byteToBuf(0x24);
    myPacket.finalize();
    if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
    {
      switchState(STATE_WAIT_FOR_START_ACK);
      return 0;
    }
    else
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, could not send start command.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  case STATE_WAIT_FOR_START_ACK:
    while ((packet = mySickPacketReceiver.receivePacket()) != NULL)
    {
      if (packet->getID() == 0xa0)
      {
	value = packet->bufToByte();
	if (value == 0)
	{
	  ArLog::log(ArLog::Terse, "ArSick: Connected to the laser.");
	  switchState(STATE_CONNECTED);
	  madeConnection();
	  return 1;
	}
	else if (value == 1)
	{
	  ArLog::log(ArLog::Terse, 
	     "ArSick: Could not start laser laser, incorrect password.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
	else if (value == 2)
	{
	  ArLog::log(ArLog::Terse, 
		     "ArSick: Could not start laser laser, LMI fault.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
	else
	{
	  ArLog::log(ArLog::Terse, 
		     "ArSick: Could not start laser laser, unknown problem.");
	  switchState(STATE_NONE);
	  failedConnect();
	  return 2;
	}
      }
    }
    if (myStateStart.mSecSince() > 1000)
    {
      ArLog::log(ArLog::Terse,
		 "ArSick: Failed to connect to laser, no start acknowledgement received.");
      switchState(STATE_NONE);
      failedConnect();
      return 2;
    }
    break;
  default:
    ArLog::log(ArLog::Verbose, "ArSick: In bad connection state\n");
    break;
  }
  return 0;
}

/**
   Sends the commands to the sim to start up the connection

   @return true if the commands were sent, false otherwise
**/
AREXPORT bool ArSick::internalConnectSim(void)
{
  lockDevice();
  double offset = myOffsetAmount;
  double increment = myIncrementAmount;
  unlockDevice();

  myRobot->lock();
  // return true if we could send all the commands
  if (myRobot->comInt(36, -ArMath::roundInt(offset)) &&
      myRobot->comInt(37, ArMath::roundInt(offset)) &&
      myRobot->comInt(38, ArMath::roundInt(increment * 100.0)) &&
      myRobot->comInt(35, 1))
  {
    myRobot->unlock();
    myState = STATE_CONNECTED;
    madeConnection();
    ArLog::log(ArLog::Terse, "ArSick: Connected to simulated laser.\n");
    return true;
  }
  else
  {
    myState = STATE_NONE;
    failedConnect();
    ArLog::log(ArLog::Terse, 
	       "ArSick: Failed to connect to simulated laser.\n");
    return false;
  }
}

/**
   Adds a connect callback, which is an ArFunctor, created as an
   ArFunctorC.  The entire list of connect callbacks is called when a
   connection is made with the laser.  If you have some sort of module
   that adds a callback, that module must remove the callback when the
   module is removed.

   @param functorfunctor created from ArFunctorC which refers to the 
   function to call.
   @param position whether to place the functor first or last
   @see remConnectCB
**/
AREXPORT void ArSick::addConnectCB(ArFunctor *functor, 
				    ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myConnectCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myConnectCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArSick::myConnectCallbackList: Invalid position.");
}

/** 
    @param functor the functor to remove from the list of connect callbacks
    @see addConnectCB
**/
AREXPORT void ArSick::remConnectCB(ArFunctor *functor)
{
  myConnectCBList.remove(functor);
}


/** Adds a failed connect callback,which is an ArFunctor, created as
    an ArFunctorC.  This whole list of failed connect callbacks is
    called when an attempt is made to connect to the laser, but fails.
    The usual reason for this failure is either that there is no
    laser/sim where the connection was tried to be made.  If you have
    some sort of module that adds a callback, that module must remove
    the callback when the module removed.

    @param functor functor created from ArFunctorC which refers to the
    function to call.
    @param position whether to place the functor first or last
    @see remFailedConnectCB
**/
AREXPORT void ArSick::addFailedConnectCB(ArFunctor *functor, 
					  ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myFailedConnectCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myFailedConnectCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArSick::myConnectCallbackList: Invalid position.");
}

/** 
    @param functor the functor to remove from the list of connect callbacks
    @see addFailedConnectCB
**/
AREXPORT void ArSick::remFailedConnectCB(ArFunctor *functor)
{
  myFailedConnectCBList.remove(functor);
}

/** Adds a disconnect normally callback,which is an ArFunctor, created
    as an ArFunctorC.  This whole list of disconnect normally
    callbacks is called when something calls disconnect if the
    instance isConnected.  If there is no connection and disconnect is
    called nothing is done.  If you have some sort of module that adds
    a callback, that module must remove the callback when the module
    is removed.

    @param functor functor created from ArFunctorC which refers to the 
    function to call.
    @param position whether to place the functor first or last
    @see remFailedConnectCB
**/
AREXPORT void ArSick::addDisconnectNormallyCB(ArFunctor *functor, 
					       ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myDisconnectNormallyCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myDisconnectNormallyCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArSick::myConnectCallbackList: Invalid position.");
}

/** 
    @param functor the functor to remove from the list of connect callbacks
    @see addDisconnectNormallyCB
**/
AREXPORT void ArSick::remDisconnectNormallyCB(ArFunctor *functor)
{
  myDisconnectNormallyCBList.remove(functor);
}

/** Adds a disconnect on error callback, which is an ArFunctor,
    created as an ArFunctorC.  This whole list of disconnect on error
    callbacks is called when ARIA loses connection to a laser because
    of an error.  This can occur if the physical connection (ie serial
    cable) between the laser and the computer is severed/disconnected,
    or if the laser is turned off.  Note that if the link between the
    two is lost the ARIA assumes it is temporary until it reaches a
    timeout value set with setConnectionTimeoutTime.  If you have some
    sort of module that adds a callback, that module must remove the
    callback when the module removed.

    @param functor functor created from ArFunctorC which refers to the 
    function to call.
    @param position whether to place the functor first or last
    @see remFailedConnectCB
**/
AREXPORT void ArSick::addDisconnectOnErrorCB(ArFunctor *functor, 
					      ArListPos::Pos position)
{
  if (position == ArListPos::FIRST)
    myDisconnectOnErrorCBList.push_front(functor);
  else if (position == ArListPos::LAST)
    myDisconnectOnErrorCBList.push_back(functor);
  else
    ArLog::log(ArLog::Terse, 
	       "ArSick::myConnectCallbackList: Invalid position");
}

/** 
   @param functor the functor to remove from the list of connect callbacks
   @see addDisconnectOnErrorCB
**/
AREXPORT void ArSick::remDisconnectOnErrorCB(ArFunctor *functor)
{
  myDisconnectOnErrorCBList.remove(functor);
}

AREXPORT void ArSick::dropConnection(void)
{
  std::list<ArFunctor *>::iterator it;  

  if (myState != STATE_CONNECTED)
    return;

  myCurrentBuffer.reset();
  myCumulativeBuffer.reset();
  ArLog::log(ArLog::Terse, 
	     "ArSick:  Lost connection to the laser because of error.");
  myState = STATE_NONE;
  for (it = myDisconnectOnErrorCBList.begin(); 
       it != myDisconnectOnErrorCBList.end(); 
       it++)
    (*it)->invoke();
  if (myConn != NULL)
    myConn->close();
}

AREXPORT void ArSick::failedConnect(void)
{
  std::list<ArFunctor *>::iterator it;  
  
  myState = STATE_NONE;
  for (it = myFailedConnectCBList.begin(); 
       it != myFailedConnectCBList.end(); 
       it++)
    (*it)->invoke();
  if (myConn != NULL)
    myConn->close();
}

AREXPORT void ArSick::madeConnection(void)
{
  std::list<ArFunctor *>::iterator it;

  myLastReading.setToNow();
  for (it = myConnectCBList.begin(); it != myConnectCBList.end(); it++)
    (*it)->invoke();
}

/**
   Disconnects from the laser.  You should lockDevice the laser before
   calling this function.  Also if you are using the simulator it will
   lock the robot so it can send the command to the simulator, so you
   should make sure the robot is unlocked.  
   
   @param doNotLockRobotForSim if this is true, this will not lock the
   robot if its trying to send a command to the sim... ONLY do this if
   you are calling this from within the robots sync loop (ie from a
   sync task, sensor interp task, or user task)
   
@return true if it could disconnect from the laser cleanly
**/
AREXPORT bool ArSick::disconnect(bool doNotLockRobotForSim)
{
  std::list<ArFunctor *>::iterator it;  
  bool ret;
  ArSerialConnection *conn;

  if (myState == STATE_NONE)
    return true;
    
  if (myState != STATE_CONNECTED)
  {
    lockDevice();
    myConnLock.lock();
    myState = STATE_NONE;
    ret = myConn->close();
    myConnLock.unlock();
    unlockDevice();
    ArLog::log(ArLog::Terse, "ArSick: Disconnecting from laser that was not fully connected to...");
    ArLog::log(ArLog::Terse, "this may cause problems later.");
    return ret;
  }

  myCurrentBuffer.reset();
  myCumulativeBuffer.reset();
  ArLog::log(ArLog::Terse, "ArSick: Disconnecting from laser.");
  myState = STATE_NONE;
  for (it = myDisconnectNormallyCBList.begin(); 
       it != myDisconnectNormallyCBList.end(); 
       it++)
    (*it)->invoke();
  if (myUseSim)
  {
    if (myRobot == NULL)
      return false;
    if (!doNotLockRobotForSim)
      myRobot->lock();
    ret = myRobot->comInt(35, 1);
    if (!doNotLockRobotForSim)
      myRobot->unlock();
    return ret;
  }
  else
  {
    myConnLock.lock();
    while (mySickPacketReceiver.receivePacket() != NULL);
    myPacket.empty();
    myPacket.uByteToBuf(0x20);
    myPacket.uByteToBuf(0x25);
    myPacket.finalize();
    ret = myConn->write(myPacket.getBuf(), myPacket.getLength());
    // put the thing back to 9600 baud
    ArUtil::sleep(1000);
    myPacket.empty();
    myPacket.byteToBuf(0x20);
    myPacket.byteToBuf(0x42);
    myPacket.finalize();
    if (myConn->write(myPacket.getBuf(), myPacket.getLength()))
    {
      ArUtil::sleep(20);
      if ((conn = dynamic_cast<ArSerialConnection *>(myConn)))
	  conn->setBaud(9600);
    } else
      ret = false;
    ret = ret && myConn->close();
    myConnLock.unlock();
    ArUtil::sleep(300);
    return ret;
  }
}

/**
   lockDevice s the laser, and then makes a connection.  If it is
   connecting to the simulator (set with the useSim flag in configure)
   then it will lock the laser and send the commands to the sim.  If
   where you are calling from has the laser locked, make sure you
   unlock it before calling this function.

   @return true if a connection was made, false otherwise
**/
 
AREXPORT bool ArSick::blockingConnect(void)
{
  int ret;
  // if we're using the sim
  if (myUseSim)
  {
    return internalConnectSim();
  }
  // if we're talking to a real laser
  else 
  {
    lockDevice();
    myConnLock.lock();
    myState = STATE_INIT;
    unlockDevice();
    while (getRunningWithLock() && (ret = internalConnectHandler()) == 0)
      ArUtil::sleep(100);
    myConnLock.unlock();
    if (ret == 1)
      return true;
    else 
      return false;
  }
  return false;
}

/**
  This does not lockDevice the laser, but you should lockDevice the
  laser before you try to connect.  Also note that if you are
  connecting to the sim the laser MUST be unlocked so that this can
  lock the laser and send the commands to the sim.  To be connected
  successfully, either the useSim must be set from configure (and the
  laser must be connected to a simulator, or this will return true but
  connection will fail), the device must have been run or runasync, or
  the device must have been runOnLaser.

  @return true if a connection will be able to be tried, false
  otherwise 

  @see configure, ArRangeDeviceThreaded::run,
  ArRangeDeviceThreaded::runAsync, runOnRobot
**/
AREXPORT bool ArSick::asyncConnect(void)
{
  if (myState == STATE_CONNECTED)
  {
    ArLog::log(ArLog::Terse,"ArSick: already connected to laser.");
    return false;
  }
  if (myUseSim || getRunning() || myRunningOnRobot)
  {
    myStartConnect = true;
    return true;
  } 
  else
  {
    ArLog::log(ArLog::Terse, "ArSick: Could not connect, to make an async connection either the sim needs to be used, the device needs to be run or runAsync, or the device needs to be runOnRobot.");
    return false;
  }
}

/**
   This sets up a sensor interp task on the robot, which is where the
   robot will be driven from.  Note that the device must have been added to
   the robot already so that the device has a pointer to the robot.
   You should lock the robot and lockDevice the laser before doing this if 
   other things are running already.
**/
AREXPORT bool ArSick::runOnRobot(void)
{
  if (myRobot == NULL)
    return false;
  else
  {  
    myRunningOnRobot = true;
    if (getRunning())
      stopRunning();
    return true;
  }
}

AREXPORT void ArSick::processPacket(ArSickPacket *packet, ArPose pose,
				    ArPose encoderPose,
				    unsigned int counter)
{
  unsigned int value;
  unsigned int numReadings;
  unsigned int i;
  double atDeg;
  unsigned int onReading;
  ArSensorReading *reading;
  int dist;
  std::list<ArSensorReading *>::iterator tempIt;
  int multiplier;
  ArTransform transform;

  //if (packet->getID() != 0xb0)
  //printf("Got in packet of type 0x%x\n", packet->getID());
  if (packet->getID() == 0xb0)
  {
    value = packet->bufToUByte2();
    numReadings = value & 0x3ff;
    //printf("numreadings %d\n", numReadings);
    if (!(value & ArUtil::BIT14) && !(value & ArUtil::BIT15))
      multiplier = 10;
    else if ((value & ArUtil::BIT14) && !(value & ArUtil::BIT15))
      multiplier = 1;
    else if (!(value & ArUtil::BIT14) && (value & ArUtil::BIT15))
      multiplier = 100;
    else
    {
      ArLog::log(ArLog::Terse, 
		 "ArSick::processPacket: bad distance configuration in packet\n");
      multiplier = 0;
    }
    //printf("%ld ms after last reading.\n", myLastReading.mSecSince());
    /*printf("Reading number %d, complete %d, unit: %d %d:\n", numReadings,
      !(bool)(value & ArUtil::BIT13), (bool)(value & ArUtil::BIT14),
      (bool)(value & ArUtil::BIT15));*/
    while (myAssembleReadings->size() > numReadings)
    {
      ArLog::log(ArLog::Verbose, "ArSick::processPacket, too many readings, popping one.\n");
      tempIt = myAssembleReadings->begin();
      if (tempIt != myAssembleReadings->end())
	delete (*tempIt);
      myAssembleReadings->pop_front();
    }
    
    // If we don't have any sensor readings created at all, make 'em all 
    if (myAssembleReadings->size() == 0)
      for (i = 0; i < numReadings; i++)
	myAssembleReadings->push_back(new ArSensorReading);
    
    for (atDeg = mySensorPose.getTh() - myOffsetAmount, onReading = 0,
	 myIter = myAssembleReadings->begin();
	 (onReading < numReadings && 
	  packet->getReadLength() < packet->getLength() - 4);
	 myWhichReading++, atDeg += myIncrementAmount, myIter++, onReading++)
    {
      reading = (*myIter);
      reading->resetSensorPosition(ArMath::roundInt(mySensorPose.getX()),
				   ArMath::roundInt(mySensorPose.getY()),
				   atDeg); 
      //reading->resetSensorPosition(0, 0, 0);

      //value = packet->bufToUByte2() & 0x1fff;
      //dist = (value & 0x1fff) * multiplier ;
      value = packet->bufToUByte2() & 0x7fff;
      dist = value * multiplier ;
      //printf("%d ", dist);
      transform.setTransform(pose);

      reading->newData(dist, pose, encoderPose, transform, counter, 
	packet->getTimeReceived());
      /*
      reading->newData(onReading, 0, 0, 0,
		       ArTransform(), counter, 
		       packet->getTimeReceived());
      */
      tempIt = myIter;
      tempIt++;
      if (tempIt == myAssembleReadings->end() && 
	  onReading + 1 != numReadings)
      {
	myAssembleReadings->push_back(new ArSensorReading);
      }
    }
    // set ArRangeDevice buffer, switch internal buffers
    myRawReadings = myAssembleReadings;
    //printf("Readings? 0x%x\n", myRawReadings);
    myAssembleReadings = myCurrentReadings;
    myCurrentReadings = myRawReadings;
    //printf("\n");
    myLastReading.setToNow();
    filterReadings();
    
    if (myTimeLastSickPacket != time(NULL)) 
    {
      myTimeLastSickPacket = time(NULL);
      mySickPacCount = mySickPacCurrentCount;
      mySickPacCurrentCount = 0;
    }
    mySickPacCurrentCount++;
  }
}

AREXPORT void ArSick::runOnce(bool lockRobot)
{
  ArSickPacket *packet;
  unsigned int counter;
  int ret;
  ArPose pose;
  ArTransform encoderTransform;
  ArPose encoderPose;

  if (myProcessImmediately && myRobot != NULL)
  {
    if (lockRobot)
      myRobot->lock();
    pose = myRobot->getPose();
    counter = myRobot->getCounter();
    if (lockRobot)
      myRobot->unlock();
  }

  lockDevice();
  if (myState == STATE_CONNECTED && myTimeoutTime > 0 && 
      myLastReading.mSecSince() > myTimeoutTime * 1000)
  {
    dropConnection();
    unlockDevice();
    return;
  }
  if (myUseSim)
  {
    unlockDevice();
    return;
  }
  if (myState == STATE_CONNECTED)
  {
    unlockDevice();
    myConnLock.lock();
    packet = mySickPacketReceiver.receivePacket();
    myConnLock.unlock();
    lockDevice();
    // if we're attached to a robot and have a packet
    if (myRobot != NULL && packet != NULL && !myProcessImmediately)
    {
      myPackets.push_back(packet);
    }
    else if (myRobot != NULL && packet != NULL && myProcessImmediately)
    {
      unlockDevice();
      if (lockRobot && myInterpolation)
	myRobot->lock();
      if (myInterpolation && (ret = myRobot->getPoseInterpPosition(
	      packet->getTimeReceived(), &pose)) < 0)
	pose = myRobot->getPose();
      encoderTransform = myRobot->getEncoderTransform();
      encoderPose = encoderTransform.doInvTransform(pose);
      if (lockRobot && myInterpolation)
	myRobot->unlock();
      lockDevice();
      processPacket(packet, pose, encoderPose, counter);
    }
    else if (packet != NULL) // if there's no robot
    {
      processPacket(packet, pose, encoderPose, 0);
      delete packet;
    }
  }
  unlockDevice();
  return;

}

AREXPORT int ArSick::getSickPacCount()
{
  if (myTimeLastSickPacket == time(NULL))
    return mySickPacCount;
  if (myTimeLastSickPacket == time(NULL) - 1)
    return mySickPacCurrentCount;
  return 0;
}


AREXPORT void ArSick::sensorInterpCallback(void)
{
  std::list<ArSickPacket *>::iterator it;
  std::list<ArSickPacket *> processed;
  ArSickPacket *packet;
  ArPose pose;
  int ret;
  ArTransform encoderTrans;
  ArPose encoderPose;

  if (myRunningOnRobot)
    runOnce(false);
  
  lockDevice();
  for (it = myPackets.begin(); it != myPackets.end(); it++)
  {
    packet = (*it);
    if ((ret = myRobot->getPoseInterpPosition(
	    packet->getTimeReceived(), &pose)) == 1)
    {
      //printf("Good interp (%.0f %.0f %.0f) (%.0f %.0f %.0f)\n", pose.getX(),
      //     pose.getY(), pose.getTh(), myRobot->getX(), myRobot->getY(),
      //     myRobot->getTh());
      encoderTrans = myRobot->getEncoderTransform();
      encoderPose = encoderTrans.doInvTransform(pose);
      processPacket(packet, pose, encoderPose, myRobot->getCounter());
      processed.push_back(packet);
    }
    else if (ret < -1)
    {
      if (myRobot->isConnected())
	ArLog::log(ArLog::Normal, "ArSick::processPacket: too old to process\n");
      else
      {
	encoderTrans = myRobot->getEncoderTransform();
	encoderPose = encoderTrans.doInvTransform(pose);
	processPacket(packet, pose, encoderPose, myRobot->getCounter());
      }
      processed.push_back(packet);
    }
    else 
    {
      //ArLog::log(ArLog::Terse, "ArSick::processPacket: error %d from interpolation\n", ret);
      //printf("$$$ ret = %d\n", ret);
    }
  }
  while ((it = processed.begin()) != processed.end())
  {
    packet = (*it);
    myPackets.remove(packet);
    processed.pop_front();
    delete packet;
  }
  unlockDevice();
}

AREXPORT void *ArSick::runThread(void *arg)
{
  while (getRunningWithLock())
  {
    lockDevice();
    if (myStartConnect)
    {
      myStartConnect = false;
      myState = STATE_INIT;
      if (myUseSim)
      {
	unlockDevice();
	internalConnectSim();
      }
      else 
      {
	unlockDevice();
	while (getRunningWithLock())
	{
	  lockDevice();
	  myConnLock.lock();
	  if (internalConnectHandler() != 0)
	  {
	    myConnLock.unlock();
	    unlockDevice();
	    break;
	  }
	  myConnLock.unlock();
	  unlockDevice();
	  ArUtil::sleep(1);
	}
      }
    } else
      unlockDevice();
    runOnce(true);
    ArUtil::sleep(1);
  }
  lockDevice();
  if (isConnected())
  {
    disconnect();
  }
  unlockDevice();

  return NULL;
}


