/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArSickLogger.h"
#include "ArRobot.h"
#include "ArSick.h"

/**
   Make sure you have called ArSick::configure on your laser before
   you make this class

   @param robot The robot to attach to
   @param sick the laser to log from
   @param distDiff the distance traveled at which to take a new reading
   @param degDiff the degrees turned at which to take a new reading
   @param filename the file name in which to put the log 
**/
AREXPORT ArSickLogger::ArSickLogger(ArRobot *robot, ArSick *sick, 
				    double distDiff, double degDiff, 
				    const char *filename) :
  mySectors(18), 
  myTaskCB(this, &ArSickLogger::robotTask)
{
  ArSick::Degrees degrees;
  ArSick::Increment increment;
  double deg, incr;

  myRobot = robot;
  mySick = sick;
  sick->setFilterNearDist(0);
  myFile = fopen(filename, "w+");
  degrees = mySick->getDegrees();
  increment = mySick->getIncrement();
  if (degrees == ArSick::DEGREES180)
    deg = 180;
  else 
    deg = 100;
  if (increment == ArSick::INCREMENT_ONE)
    incr = 1;
  else
    incr = .5;
  if (myFile != NULL)
  {
    ArRobotParams params;
    params = robot->getParams();
    fprintf(myFile, "LaserOdometryLog\n");
    fprintf(myFile, "#Created by ARIA's ArSickLogger\n");
    fprintf(myFile, "version: 1\n");
    fprintf(myFile, "sick1pose: %d %d 0\n", params.getLaserX(), 
	    params.getLaserY());
    fprintf(myFile, "sick1conf: %d %d %d\n", ArMath::roundInt(0.0 - deg / 2.0),
	    ArMath::roundInt(deg / 2.0), ArMath::roundInt(deg / incr + 1.0));
  }
  myDistDiff = distDiff;
  myDegDiff = degDiff;
  myFirstTaken = false;
  myScanNumber = 0;
  myLastVel = 0;
  myStartTime.setToNow();
  myRobot->addUserTask("Sick Logger", 1, &myTaskCB);

}

AREXPORT ArSickLogger::~ArSickLogger(void)
{
  myRobot->remUserTask(&myTaskCB);
  fprintf(myFile, "# End of log\n");
  if (myFile != NULL)
    fclose(myFile);
}

/**
   The robot MUST be locked before you call this function, so that
   this function is not adding to a list as the robotTask is using it.
   
   This function takes the given tag
**/
AREXPORT void ArSickLogger::addTagToLog(const char *str, ...)
{
  char buf[2048];
  va_list ptr;
  va_start(ptr, str);
  vsprintf(buf, str, ptr);
  myTags.push_front(buf);
  va_end(ptr);
}

AREXPORT void ArSickLogger::robotTask(void)
{
  std::list<ArPose *> *readings;
  std::list<ArPose *>::iterator it;
  std::list<ArPose *>::reverse_iterator rit;
  ArRangeBuffer *buffer;
  ArPose poseTaken;
  time_t msec;

  // now put the tags into the file
  while (myTags.size() > 0)
  {
    if (myFile != NULL)
    {
      msec = myStartTime.mSecSince();
      fprintf(myFile, "time: %ld.%ld\n", msec / 1000, msec % 1000);
      fprintf(myFile, "robot: %.0f %.0f %.2f\n", 
	      myRobot->getX(), 
	      myRobot->getY(), 
	      myRobot->getTh());
      fprintf(myFile, "%s\n", (*myTags.begin()).c_str());
    }
    myTags.pop_front();
  }

  // we take readings in any of the following cases if we haven't
  // taken one yet if we've gone further than myDistDiff if we've
  // turned more than myDegDiff if we've switched sign on velocity and
  // gone more than 50 mm (so it doesn't oscilate and cause us to
  // trigger)

  if (myRobot->isConnected() && (!myFirstTaken ||
      myLast.findDistanceTo(myRobot->getPose()) > myDistDiff ||
      fabs(ArMath::subAngle(myLast.getTh(), myRobot->getTh())) > myDegDiff ||
      ((myLastVel < 0 && myRobot->getVel() > 0 ||
	myLastVel > 0 && myRobot->getVel() < 0) && 
       myLast.findDistanceTo(myRobot->getPose()) > 50)))
  {
    mySick->lock();
    buffer = mySick->getCurrentRangeBuffer();
    if (buffer != NULL)
      readings = buffer->getBuffer();
    if (buffer == NULL || readings == NULL || readings->size() <= 0)
    {
      mySick->unlock();
      fflush(myFile);
#ifndef WIN32
      fsync(fileno(myFile));
#endif
      return;
    }
    myScanNumber++;
    printf("Taking a reading from the %d laser values\n", readings->size());
    myFirstTaken = true;
    myLast = myRobot->getPose();
    poseTaken = buffer->getPoseTaken();
    myLastVel = myRobot->getVel();
    if (myFile != NULL)
    {
      msec = myStartTime.mSecSince();
      fprintf(myFile, "#Scan %d\n", myScanNumber);
      fprintf(myFile, "time: %ld.%ld\n", msec / 1000, msec % 1000);
      fprintf(myFile, "robot: %.0f %.0f %.2f\n", 
	      poseTaken.getX(), 
	      poseTaken.getY(), 
	      poseTaken.getTh());
      fprintf(myFile, "sick1: ");
    } else
      printf("Can't print to log file!\n");

    if (!mySick->isLaserFlipped())
    {
      // make sure that the list is in increasing order
      for (it = readings->begin(); it != readings->end(); it++)
      {
	if (myFile != NULL)
	  fprintf(myFile, "%.0f ", 
		  (*it)->findDistanceTo(poseTaken));
      }
    }
    else
    {
      for (rit = readings->rbegin(); rit != readings->rend(); rit++)
      {
	if (myFile != NULL)
	  fprintf(myFile, "%.0f ", 
		  (*rit)->findDistanceTo(poseTaken));
      }
    }
    fprintf(myFile, "\n");
    mySick->unlock();
  }
  
  fflush(myFile);
#ifndef WIN32
  fsync(fileno(myFile));
#endif


}




