/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArTaskPool.h"
#include "ArLog.h"


/**
   Threads will be allocated as needed. If all the threads in the pool are
   busy with a task when a task needs to be run, then another thread will be
   allocated. The maxLimit will limit the number of threads to that number.
   If maxLimit is 0, there will be no limit to threads and threads will be
   allocated as needed. See also setTaskMinLimit() for the behavior of how
   to destroy threads.
   @param maxLimit maximum number of threads that exist at once
   @see setTaskMinLimit
*/
/**
   When tasks need to be run, threads will be created based upon the need.
   See setTaskMaxLimit() about this. If minLimit is greater than 0 and there
   are more threads than minLimit, the extra threads will be destroyed. If
   minLimit is 0, then all threads will be immediately destroyed after the
   task is done.
   @param minLimit minimum number of threads that exist at once
   @see setTaskMaxLimit
*/


ArTaskPool * ArTaskPool::ourPool=0;


AREXPORT ArTaskPool::ArTaskPool() :
  myRunning(false),
  myMaxLimit(0),
  myMinLimit(0),
  myThreads(),
  myFreeThreads(),
  myBusyThreads(),
  myTaskQueue(),
  myMutex(),
  myThreadIdlerSem()
{
}

AREXPORT ArTaskPool::~ArTaskPool()
{
}

AREXPORT bool ArTaskPool::runTask(ArFunctor *task)
{
  return(runTask(std::pair<ArFunctor*,ArPeriodicTask*>(task, 0)));
}

AREXPORT bool ArTaskPool::runTask(ArPeriodicTask *task)
{
  return(runTask(std::pair<ArFunctor*,ArPeriodicTask*>(0, task)));
}

AREXPORT bool ArTaskPool::runTask(std::pair<ArFunctor*,ArPeriodicTask*> task)
{
  myMutex.lock();

  if (!myRunning)
    return(false);

  myTaskQueue.push_back(task);
  // We gotta make a thread?
  if (!myFreeThreads.size() &&
      (!myMaxLimit || (myMaxLimit && 
		       (((signed int)myThreads.size()) < myMaxLimit))))
    makeThread(true);
  else if (myMaxLimit && (((signed int)myThreads.size()) > myMaxLimit))
  {
    ArLog::log(ArLog::Terse, "ArTaskPool::runTask: While trying to run a task, the maximum thread limit was reached. Please try again, raise the maxLimit, or disable the maxLimit");
    return(false);
  }
  // Lets put a slacker thread to work
  myThreadIdlerSem.post();

  myMutex.unlock();

  return(true);
}

AREXPORT void ArTaskPool::makeThread(bool makeBusy)
{
  ArTaskPoolTask *thread;

  thread=new ArTaskPoolTask(this);
  thread->create();
  myThreads.insert(thread);
  if (makeBusy)
    myBusyThreads.insert(thread);
  else
    myFreeThreads.insert(thread);
}

AREXPORT void ArTaskPool::init(int minLimit, int maxLimit)
{
  int i;

  // Nothing should be using the data now, but lets just be safe
  myMutex.lock();

  myRunning=true;
  myMinLimit=minLimit;
  myMaxLimit=maxLimit;

  // Create the initial threads
  for (i=0; i<minLimit; ++i)
    makeThread(false);

  myMutex.unlock();
}

AREXPORT void ArTaskPool::uninit()
{
  int start, end;
  std::set<ArTaskPoolTask*>::iterator iter;

  myMutex.lock();
  myRunning=false;
  for (iter=myThreads.begin(); iter != myThreads.end(); ++iter)
    (*iter)->stopRunning();
  myThreadIdlerSem.getValue(end);
  for (start=0; start<end; ++start)
    myThreadIdlerSem.post();
  reset();
  myMutex.unlock();
}

AREXPORT void ArTaskPool::forceUninit()
{
  std::set<ArTaskPoolTask*>::iterator iter;

  myMutex.lock();
  myRunning=false;
  for (iter=myThreads.begin(); iter != myThreads.end(); ++iter)
    (*iter)->cancel();
  reset();
  myMutex.unlock();
}

AREXPORT void ArTaskPool::reset()
{
  myRunning=false;
  myMaxLimit=0;
  myMinLimit=0;
  myThreads.clear();
  myFreeThreads.clear();
  myBusyThreads.clear();
  myTaskQueue.clear();
}

AREXPORT ArTaskPool * ArTaskPool::getPool()
{
  if (!ourPool)
  {
    ourPool=new ArTaskPool;
    ourPool->init();
  }
  return(ourPool);
}

AREXPORT void ArTaskPool::setPool(ArTaskPool *pool)
{
  ourPool=pool;
}
