/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include "ArVCC4.h"
#include "ArCommands.h"
#include "ArRobot.h"

AREXPORT ArVCC4Packet::ArVCC4Packet(ArTypes::UByte2 bufferSize) :
  ArBasePacket(bufferSize, 0)
{
}

AREXPORT ArVCC4Packet::~ArVCC4Packet(void)
{

}


AREXPORT void ArVCC4Packet::byte2ToBuf(ArTypes::Byte4 val)
{
  int i;
  char buf[5];
  if (myLength + 4 > myMaxLength)
  {
    ArLog::log(ArLog::Terse, "ArVCC4Packet::uByte2ToBuf: Trying to add beyond length of buffer.");
    return;
  }

  sprintf(buf, "%X", val);
  for (i=0;i<(int)strlen(buf);i++)
  {
    myBuf[myLength] = buf[i];
    ++myLength;
  }
}

AREXPORT void ArVCC4Packet::finalize(void)
{
  uByteToBuf(ArVCC4Commands::FOOTER);  //Automatically tacks on footer char
}

//Creates new packet with default header, device id, and delimeter - FE 30 30 00

AREXPORT void ArVCC4::preparePacket(ArVCC4Packet *myPacket)		
{
  myPacket->uByteToBuf(ArVCC4Commands::HEADER);
  myPacket->uByteToBuf(ArVCC4Commands::DEVICEID);
  myPacket->uByteToBuf(ArVCC4Commands::DEVICEID);
  myPacket->uByteToBuf(ArVCC4Commands::DELIM);
}

/**
   @param robot the robot this camera is attached to

   @param inverted if this camera is inverted or not, the only time a
   camera will normally be inverted is on a robot where it's mounted
   on the underside of something, ie like in a peoplebot
**/

AREXPORT ArVCC4::ArVCC4(ArRobot *robot, bool inverted) :
  ArPTZ(robot)
{
  myInverted = inverted;
  myRobot = robot;
  myPanSlew = getMaxPanSlew(); //Default to max
  myTiltSlew = getMaxTiltSlew(); //Defaults to max
  myPan = 0;
  myTilt = 0;
  myZoom = 0;
}

AREXPORT ArVCC4::~ArVCC4(void)
{
}

AREXPORT void ArVCC4::connectHandler(void)
{
}


AREXPORT bool ArVCC4::power(int status)
{
  myPacket.empty();
  preparePacket(&myPacket);
  myPacket.uByteToBuf(ArVCC4Commands::POWER);
  myPacket.uByteToBuf(ArVCC4Commands::DEVICEID + status);

  return sendPacket(&myPacket);
}

AREXPORT bool ArVCC4::init(void)
{
  power(0);

  myTime.setToNow();
  while(myTime.mSecSince() < 200) {
  }

  power(1);

  myTime.setToNow();  
  while(myTime.mSecSince() < 4000) {
  }

  myPacket.empty();		//Send Control command
  preparePacket(&myPacket);
  myPacket.uByteToBuf(ArVCC4Commands::CONTROL);
  myPacket.uByteToBuf(ArVCC4Commands::DEVICEID);

  if (!sendPacket(&myPacket))
    return false;

  myPacket.empty();		// Send Init command
  preparePacket(&myPacket);
  myPacket.uByteToBuf(ArVCC4Commands::INIT);
  myPacket.uByteToBuf(ArVCC4Commands::DEVICEID);

  if (!sendPacket(&myPacket))
    return false;

  myTime.setToNow();	//Allow initialization time
  while(myTime.mSecSince() < 2000) {
  }

  myPan = -1;  // Set these to -1 for initial positioning.
  myTilt = -1; // If they are initialized to 0, then a panTilt(0,0)
  myZoom = -1; // will not move the camera at all, because it will
	       // already think it's at (0,0).  By setting them to
	       // (-1,-1), then a panTilt(0,0) forces a command to be
	       // sent to the camera, forcing it to move to (0,0)
	       // This will go away when reponse packets are implemented.

  myPanSlew = 0;
  myTiltSlew = 0;

  panSlew(getMaxPanSlew());  //  These will set myPanSlew and myTiltSlew
  tiltSlew(getMaxTiltSlew());

  myPacket.empty();   //  This packet is necessary because camera defaults to 
  preparePacket(&myPacket);   //  max tilt range of +30, instead of +90

  myPacket.uByteToBuf(ArVCC4Commands::SETRANGE);
  myPacket.uByteToBuf(ArVCC4Commands::DEVICEID + 1);

  // Note the conversion from degrees to camera units:
  //    units = degrees / 0.1125

  myPacket.byte2ToBuf((int) floor(getMaxNegTilt()/.1125) + 0x8000);	// Set min tilt range
  myPacket.byte2ToBuf((int) floor(getMaxPosTilt()/.1125) + 0x8000);	// Set max tilt range

  if (!sendPacket(&myPacket))
    return false;

  if (!panTilt(0,0))
    return false;

  if (!zoom(0))
    return false;

  return true;
}

AREXPORT bool ArVCC4::panTilt(int pdeg, int tdeg)
{
  if (pdeg > getMaxPosPan())
    pdeg = getMaxPosPan();
  if (pdeg < getMaxNegPan())
    pdeg = getMaxNegPan();

  if (tdeg > getMaxPosTilt())
    tdeg = getMaxPosTilt();
  if (tdeg < getMaxNegTilt())
    tdeg = getMaxNegTilt();

  if (pdeg != myPan || tdeg != myTilt)
  {
//  haltPanTilt();	// Causes annoying 200ms blocking, but leads to accuracy
    myPan = pdeg;
    myTilt = tdeg;
    if (myInverted)
    {
      tdeg *= -1;
      pdeg *= -1;
    }
    myPacket.empty();
    preparePacket(&myPacket);
    myPacket.uByteToBuf(ArVCC4Commands::PANTILT);
    myPacket.byte2ToBuf((int) floor(invert(myPan)/.1125) + 0x8000);
    myPacket.byte2ToBuf((int) floor(invert(myTilt)/.1125) + 0x8000);

    return sendPacket(&myPacket);
  }

  return true;
}

AREXPORT bool ArVCC4::zoom(int deg)
{
  char buf[3];
  if (deg > getMaxZoom())
    deg = getMaxZoom();
  if (deg < getMinZoom())
    deg = getMinZoom();

  if (deg != myZoom)
  {
//  haltZoom();		// Causes 200ms blocking....
    myZoom = deg;

    myPacket.empty();
    preparePacket(&myPacket);
    myPacket.uByteToBuf(ArVCC4Commands::ZOOM);
    sprintf(buf, "%2X", deg);
    if (buf[0] == ' ')
      buf[0] = '0';

    myPacket.byteToBuf(buf[0]);
    myPacket.byteToBuf(buf[1]);

    return sendPacket(&myPacket);
  }

  return true;
}


AREXPORT bool ArVCC4::haltPanTilt(void)
{
  myPacket.empty();
  preparePacket(&myPacket);
  myPacket.uByteToBuf(ArVCC4Commands::STOP);
  myPacket.uByteToBuf(ArVCC4Commands::DEVICEID);

  if (!sendPacket(&myPacket))
    return false;

  myTime.setToNow();		// Delay to allow camera to process stop command.  ~200ms
  while(myTime.mSecSince() < 200) {
  }
  return true;
}


AREXPORT bool ArVCC4::haltZoom(void)
{
  myPacket.empty();
  preparePacket(&myPacket);
  myPacket.uByteToBuf(ArVCC4Commands::ZOOMSTOP);
  myPacket.uByteToBuf(ArVCC4Commands::DEVICEID);

  if (!sendPacket(&myPacket))
    return false;

  myTime.setToNow();		// Delay to allow camera to process stop command.  ~200ms
  while(myTime.mSecSince() < 200) {
  }
  return true;
}


AREXPORT bool ArVCC4::panSlew(int slew)
{
  char buf[4];

  if (slew > getMaxPanSlew())
    slew = getMaxPanSlew();
  if (slew < getMinPanSlew())
    slew = getMinPanSlew();

  if (slew != myPanSlew)
  {
    myPanSlew = slew;

    myPacket.empty();
    preparePacket(&myPacket);
    myPacket.uByteToBuf(ArVCC4Commands::PANSLEW);

    sprintf(buf,"%3X", (int) floor(slew/.1125));

    if (buf[0] < '0')
      buf[0] = '0';
    if (buf[1] < '0')
      buf[1] = '0';
    if (buf[2] < '0')
      buf[2] = '0';

    myPacket.byteToBuf(buf[0]);
    myPacket.byteToBuf(buf[1]);
    myPacket.byteToBuf(buf[2]);

    return sendPacket(&myPacket);
  }
  return true;
}


AREXPORT bool ArVCC4::tiltSlew(int slew)
{
  char buf[4];

  if (slew > getMaxTiltSlew())
    slew = getMaxTiltSlew();
  if (slew < getMinTiltSlew())
    slew = getMinTiltSlew();

  if (slew != myTiltSlew)
  {
    myTiltSlew = slew;

    myPacket.empty();
    preparePacket(&myPacket);
    myPacket.uByteToBuf(ArVCC4Commands::TILTSLEW);

    sprintf(buf,"%3X", (int) floor(slew/.1125));

    if (buf[0] == ' ')
      buf[0] = '0';
    if (buf[1] == ' ')
      buf[1] = '0';

    myPacket.byteToBuf(buf[0]);
    myPacket.byteToBuf(buf[1]);
    myPacket.byteToBuf(buf[2]);

    return sendPacket(&myPacket);
  }
  return true;
}
