/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#include "ArExport.h"
#include "ariaOSDef.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#ifdef linux
#include <sys/time.h>
#include <ctype.h>
#include <unistd.h>
#endif

#include "ariaTypedefs.h"
#include "ariaUtil.h"

#ifdef WIN32
AREXPORT const char *ArUtil::COM1 = "COM1";
AREXPORT const char *ArUtil::COM2 = "COM2";
AREXPORT const char *ArUtil::COM3 = "COM3";
AREXPORT const char *ArUtil::COM4 = "COM4";
#else // ifndef WIN32
const char *ArUtil::COM1 = "/dev/ttyS0";
const char *ArUtil::COM2 = "/dev/ttyS1";
const char *ArUtil::COM3 = "/dev/ttyS2";
const char *ArUtil::COM4 = "/dev/ttyS3";
#endif  // WIN32

/**
   This sleeps for the given number of milliseconds... Note in linux it 
   tries to sleep for 10 ms less than the amount given, which should wind up 
   close to correct... 
   Linux is broken in this regard and sleeps for too long...
   it sleeps for the ceiling of the current 10 ms range,
   then for an additional 10 ms... so:
   11 to 20 ms sleeps for 30 ms...
   21 to 30 ms sleeps for 40 ms...
   31 to 40 ms sleeps for 50 ms... 
   this continues on up to the values we care about of..
   81 to 90 ms sleeps for 100 ms...
   91 to 100 ms sleeps for 110 ms...
   so we'll sleep for 10 ms less than we want to, which should put us about 
   right... guh
   @param ms the number of milliseconds to sleep for
*/
AREXPORT void ArUtil::sleep(unsigned int ms)
{
#ifdef WIN32
  Sleep(ms);
#endif // WIN32
#ifdef linux
  if (ms > 10)
    ms -= 10;
  usleep(ms * 1000);
#endif // linux

}

/**
   Get the time in milliseconds, counting from some arbitrary point.
   This time is only valid within this run of the program.
   @return millisecond time
*/
AREXPORT unsigned int ArUtil::getTime(void)
{
#ifdef WIN32
  return timeGetTime();
#endif // WIN32

#ifdef linux
  struct timeval tv;
  gettimeofday(&tv,NULL);
  return tv.tv_usec/1000 + (tv.tv_sec % 1000000)*1000;
#endif
}

/**
   Takes a string and splits it into a list of words. It appends the words
   to the outList. If there is nothing found, it will not touch the outList.
   @param inString the input string to split
   @param ourList the list in which to store the words that are found
*/
AREXPORT void ArUtil::splitString(std::string inString,
				  std::list<std::string> &outList)
{
  const char *start, *end;

  // Strip off leading white space
  for (end=inString.c_str(); *end && isspace(*end); ++end)
    ;
  while (*end)
  {
    // Mark start of the word then find end
    for (start=end; *end && !isspace(*end); ++end)
      ;
    // Store the word
    if (*start && ((*end && isspace(*end)) || !*end))
      outList.push_back(std::string(start, (int)(end-start)));
    for (; *end && isspace(*end); ++end)
      ;
  }
}

#ifdef WIN32

/**
   @return size in bytes. -1 on error.
   @param fileName name of the file to size
*/
AREXPORT long ArUtil::sizeFile(std::string fileName)
{
  struct _stat buf;

  if (_stat(fileName.c_str(), &buf) < 0)
    return(-1);

  if (!(buf.st_mode | _S_IFREG))
    return(-1);

  return(buf.st_size);
}

#else // WIN32

/**
   @return size in bytes. -1 on error.
   @param fileName name of the file to size
*/
AREXPORT long ArUtil::sizeFile(std::string fileName)
{
  struct stat buf;

  if (stat(fileName.c_str(), &buf) < 0)
  {
    perror("stat");
    return(-1);
  }

  if (!S_ISREG(buf.st_mode))
    return(-1);

  return(buf.st_size);
}

#endif // WIN32

/**
   @return true if file is found
   @param fileName name of the file to size
*/
AREXPORT bool ArUtil::findFile(const char *fileName)
{
  FILE *fp;

  if ((fp=fopen(fileName, "r")))
  {
    fclose(fp);
    return(true);
  }
  else
    return(false);
}

/**
   Works for \ and /. Returns true if something was actualy done. Sets
   fileOut to be what ever the answer is.
   @return true if the path contains a file
   @param fileIn input path/filename
   @param fileOut output filename
*/
AREXPORT bool ArUtil::stripDir(std::string fileIn, std::string &fileOut)
{
  const char *ptr;

  for (ptr=fileIn.c_str(); *ptr; ++ptr)
    ;
  for (--ptr; (ptr > fileIn.c_str()) && (*ptr != '/') && (*ptr != '\\'); --ptr)
    ;
  if ((*ptr == '/') || (*ptr == '\\'))
  {
    fileOut=ptr+1;
    return(true);
  }
  else
  {
    fileOut=fileIn;
    return(false);
  }
}

/**
   Works for \ and /. Returns true if something was actualy done. Sets
   fileOut to be what ever the answer is.
   @return true if the file contains a path
   @param fileIn input path/filename
   @param fileOut output path
*/
AREXPORT bool ArUtil::stripFile(std::string fileIn, std::string &fileOut)
{
  const char *start, *end;

  for (start=end=fileIn.c_str(); *end; ++end)
  {
    if ((*end == '/') || (*end == '\\'))
    {
      start=end;
      for (; *end && ((*end == '/') || (*end == '\\')); ++end)
        ;
    }
  }

  if (start < end)
  {
    fileOut.assign(fileIn, 0, start-fileIn.c_str());
    return(true);
  }

  fileOut=fileIn;
  return(false);
}

/**
   @param path the path to append a slash to
*/
AREXPORT void ArUtil::appendSlash(std::string &path)
{
  if ((*(path.c_str()+path.size()-1) != '\\') && (*(path.c_str()+path.size()-1) != '/'))
#ifdef WIN32
    path+="\\";
#else
    path+="/";
#endif
}

/**
   @param path the path in which to fix the orientation of the slashes
*/
AREXPORT void ArUtil::fixSlashes(std::string &path)
{
#ifdef WIN32
  fixSlashesBackward(path);
#else
  fixSlashesForward(path);
#endif
}

/**
   @param path the path in which to fix the orientation of the slashes
*/
AREXPORT void ArUtil::fixSlashesBackward(std::string &path)
{
  for (int i=0; path[i]; ++i)
  {
    if (path[i] == '/')
      path[i]='\\';
  }
}

/**
   @param path the path in which to fix the orientation of the slashes
*/
AREXPORT void ArUtil::fixSlashesForward(std::string &path)
{
  for (int i=0; path[i]; ++i)
  {
    if (path[i] == '\\')
      path[i]='/';
  }
}

/** 
    This compares two strings, it returns an integer less than, equal to, 
    or greater than zero  if  str  is  found, respectively, to be less than, to
    match, or be greater than str2.
    @param str the string to compare
    @param str2 the second string to compare
    @return an integer less than, equal to, or greater than zero if str is 
    found, respectively, to be less than, to match, or be greater than str2.
*/
AREXPORT int ArUtil::strcmp(std::string str, std::string str2)
{
  return ::strcmp(str.c_str(), str2.c_str());
}

/** 
    This compares two strings, it returns an integer less than, equal to, 
    or greater than zero  if  str  is  found, respectively, to be less than, to
    match, or be greater than str2.
    @param str the string to compare
    @param str2 the second string to compare
    @return an integer less than, equal to, or greater than zero if str is 
    found, respectively, to be less than, to match, or be greater than str2.
*/
AREXPORT int ArUtil::strcmp(std::string str, const char *str2)
{
  return ::strcmp(str.c_str(), str2);
}

/** 
    This compares two strings, it returns an integer less than, equal to, 
    or greater than zero  if  str  is  found, respectively, to be less than, to
    match, or be greater than str2.
    @param str the string to compare
    @param str2 the second string to compare
    @return an integer less than, equal to, or greater than zero if str is 
    found, respectively, to be less than, to match, or be greater than str2.
*/
AREXPORT int ArUtil::strcmp(const char *str, std::string str2)
{
  return ::strcmp(str, str2.c_str());
}

/** 
    This compares two strings, it returns an integer less than, equal to, 
    or greater than zero  if  str  is  found, respectively, to be less than, to
    match, or be greater than str2.
    @param str the string to compare
    @param str2 the second string to compare
    @return an integer less than, equal to, or greater than zero if str is 
    found, respectively, to be less than, to match, or be greater than str2.
*/
AREXPORT int ArUtil::strcmp(const char *str, const char *str2)
{
  return ::strcmp(str, str2);
}

/**
   This copies src into dest but puts a \ before any spaces in src,
   escaping them... its mostly for use with ArArgumentBuilder... 
   make sure you have enough space in the arrays that you're passing 
   as dest... this allocates no memory
**/
AREXPORT void ArUtil::escapeSpaces(char *dest, const char *src)
{
  size_t i, adj;

  // walk it, when we find one toss in the slash and incr adj so the
  // next characters go in the right space
  for (i = 0, adj = 0; i < strlen(src); i++)
  {
    if (src[i] == ' ')
    {
      dest[i+adj] = '\\';
      adj++;
    }
    dest[i+adj] = src[i];
  }
  // make sure its null terminated
  dest[i+adj] = '\0';
}

/**
   This function looks in the given filename and extracts a string
   from the file.  The string can contain spaces or tabs, but a '\r'
   or '\n' will be treated as the end of the string, and the string
   cannot have more than 1024 characters.  This is mostly for use with
   Linux to pick up the Aria directory from a file in /etc, but will
   work with Linux or Windows.

   @param filename the filename to look in

   @return the string that was in the file, or a string with length 0
   if the file was not found or if the file was empty
**/
AREXPORT std::string ArUtil::getStringFromFile(const char *fileName)
{
  FILE *dirFile;
  std::string ret;
  char dir[1025];
  int i;

  if ((dirFile = fopen(fileName, "r")) != NULL)
  {
    fscanf(dirFile, "%1024c", dir);
    for (i = 0; i < 1024; i++)
    {
      if (dir[i] == '\r' || dir[i] == '\n' || dir[i] == '\0')
      {
	dir[i] = '\0';
	break;
      }
    }
  }
  else
    dir[0] = '\0';
  ret = dir;
  return ret;
}

/**
   This takes a root key, and looks up the given <key> within that root,
   then finds the string given to <value> and returns it.

   @param root the root key to use, one of the REGKEY enums

   @param key the name of the key to find

 @param value the value to find the string contained in

   @param str where to put the string sought, or if it could not be
   found for some reason an empty (length() == 0) string

   @param len the length of the allocated memory in str

   @return true if the string was found, false if it was not found or if there was a problem such as the string not being long enough 
 **/

AREXPORT bool ArUtil::getStringFromRegistry(REGKEY root,
						   const char *key,
						   const char *value,
						   char *str,
						   int len)
{
#ifndef WIN32
  return false;
#else // WIN32

  HKEY hkey;
  int err;
  int i;
  unsigned long numKeys;
  unsigned long longestKey;
  unsigned long numValues;
  unsigned long longestValue;
  unsigned long longestDataLength;
  char *valueName;
  unsigned long valueLength;
  unsigned long type;
  char *data;
  unsigned long dataLength;
  HKEY rootKey;


  switch (root)
  {
  case REGKEY_CLASSES_ROOT:
    rootKey = HKEY_CLASSES_ROOT;
    break;
  case REGKEY_CURRENT_CONFIG:
    rootKey = HKEY_CURRENT_CONFIG;
    break;
  case REGKEY_CURRENT_USER:
    rootKey = HKEY_CURRENT_USER;
    break;
  case REGKEY_LOCAL_MACHINE:
    rootKey = HKEY_LOCAL_MACHINE;
    break;
  case REGKEY_USERS:
    rootKey=HKEY_USERS;
    break;
  default:
    ArLog::log(ArLog::Terse, 
	       "ArUtil::getStringFromRegistry: Bad root key given.");
    return false;
  }


  if ((err = RegOpenKeyEx(rootKey, key, 0, KEY_READ, &hkey)) == ERROR_SUCCESS)
  {
    //printf("Got a key\n");
    if (RegQueryInfoKey(hkey, NULL, NULL, NULL, &numKeys, &longestKey, NULL, 
			&numValues, &longestValue, &longestDataLength, NULL, NULL) == ERROR_SUCCESS)
    {
	/*
      printf("Have %d keys longest is %d, have %d values longest name is %d, longest data is %d\n",
	     numKeys, longestKey, numValues, longestValue, longestDataLength);
	*/	 
      data = new char[longestDataLength+2];
      valueName = new char[longestValue+2];
      for (i = 0; i < numValues; ++i)
      {
	dataLength = longestDataLength+1;
	valueLength = longestValue+1;
	if ((err = RegEnumValue(hkey, i, valueName, &valueLength, NULL, 
				&type, (unsigned char *)data, &dataLength)) == ERROR_SUCCESS)
	{
		//printf("Enumed value %d, name is %s, value is %s\n", i, valueName, data);
	  if (strcmp(value, valueName) == 0)
	  {
	    if (len < dataLength)
	    {
	      ArLog::log(ArLog::Terse,"ArUtil::getStringFromRegistry: str passed in not long enough for data.");
	      delete data;
	      delete valueName;
	      return false;
	    }
	    strncpy(str, data, len);
	    delete data;
	    delete valueName;
	    return true;
	  }
	}
	/*
	else
		printf("Couldn't enum value %d cause %d\n",i,  err);
		*/
	    }
      delete data;
      delete valueName;
    }
	/*
    else
      printf("QueryInfoKey failed\n");
	  */
  }
  /*
  else
    printf("No key %d\n", err);
  */
  return false;
#endif
}

AREXPORT ArArgumentBuilder::ArArgumentBuilder(size_t argvLen)
{
  myArgc = 0;
  myArgvLen = argvLen;
  myArgv = new char *[myArgvLen];
}

AREXPORT ArArgumentBuilder::~ArArgumentBuilder(void)
{
  size_t i;
  for (i = 0; i < myArgc; ++i)
    delete[] myArgv[i];
  delete[] myArgv;
}

AREXPORT void ArArgumentBuilder::add(char *str, ...)
{
  char buf[2048];
  int i;
  int j;
  bool findingSpace = true;
  int startNonSpace;
  int totalLen;
  
  // basically we set up the var arg stuff stuff now
  va_list ptr;
  va_start(ptr, str);
  vsprintf(buf, str, ptr);
  totalLen = strlen(buf);
  // can do whatever you want with the buf now
  // first we advance to non-space
  for (i = 0; i < totalLen; ++i)
  {
    if (!isspace(buf[i]))
      break;
  }
  // see if we're done
  if (i == totalLen)
  {
    ArLog::log(ArLog::Verbose, "All white space add for argument builder.");
    va_end(ptr);
    return;
  }
  // walk through the line until we get to the end of the buffer...
  // we keep track of if we're looking for white space or non-white...
  // if we're looking for white space when we find it we have finished
  // one argument, so we toss that into argv, reset pointers and moveon
  for (startNonSpace = i; ; ++i)
  {
    // take out the slash of escaped spaces
    if (buf[i] == '\\' && i + 1 < totalLen && buf[i + 1] == ' ')
    {
      for (j = i; j < totalLen && j != '\0'; j++)
      {
	buf[j] = buf[j + 1];
      }
      --totalLen;
    }
    // if we're not finding space and we see a non space (or the end),
    // set us into finding space mode and denote where it started
    else if (!findingSpace && 
	     !(i == totalLen || isspace(buf[i]) || buf[i] == '\0'))
    {
      startNonSpace = i;
      findingSpace = true;
    }
    // if we're finding space, and we see it (or the end), we're at
    // the end of one arg, so toss it into the list
    else if (findingSpace && 
	     (i == totalLen || isspace(buf[i]) || buf[i] == '\0'))
    {
      // see if we have room in our argvLen
      if (myArgc + 1 >= myArgvLen)
      {
	ArLog::log(ArLog::Terse, "ArArgumentBuilder::Add: could not add argument since argc (%u) has grown beyond the argv given in the constructor (%u)", myArgc, myArgvLen);
      }
      else
      {
	myArgv[myArgc] = new char[i - startNonSpace + 1];
	strncpy(myArgv[myArgc], &buf[startNonSpace], i - startNonSpace);
	myArgv[myArgc][i - startNonSpace] = '\0';
	myArgc++;
      }
      findingSpace = false;
    }
    // if we're at the end or its a null, we're at the end of the buf
    if (i == totalLen || buf[i] == '\0')
      break;
  }
  va_end(ptr);
}

AREXPORT size_t ArArgumentBuilder::getArgc(void)
{
  return myArgc;
}

AREXPORT char** ArArgumentBuilder::getArgv(void)
{
  return myArgv;
}

AREXPORT void ArArgumentBuilder::print(void)
{
  size_t i;
  ArLog::log(ArLog::Terse, "Num arguments: %d", myArgc);
  for (i = 0; i < myArgc; ++i)
    ArLog::log(ArLog::Terse, "Arg %d: %s", i, myArgv[i]);
}

/**
   @param argc a pointer to the argc used
   @param argv argv
**/
AREXPORT ArArgumentParser::ArArgumentParser(int *argc, char **argv)
{
  myArgc = argc;
  myArgv = argv;
}

AREXPORT ArArgumentParser::~ArArgumentParser(void)
{
}

/**
   @param argument the string to check for, if the argument is found
   its pulled from the list of arguments
   
   @return true if the argument was found, false otherwise
**/
AREXPORT bool ArArgumentParser::checkArgument(char *argument)
{
  int i;
  for (i = 1; i < *myArgc; i++)
  {
    if (strcmp(argument, myArgv[i]) == 0)
    {
      for (; i < *myArgc - 1; i++)
	myArgv[i] = myArgv[i+1];
      *myArgc -= 1;
      return true;
    }
  }
  return false;
}

/**
   @param argument the string to check for, if the argument is found
   its pulled from the list of arguments
   
   @return NULL if the argument wasn't found, the argument after the
   one given if the argument was found, or NULL again if the argument
   was found as the last item
**/
AREXPORT char * ArArgumentParser::checkParameterArgument(char *argument)
{
  char *ret;
  int i;
  for (i = 1; i < *myArgc; i++)
  {
    if (strcmp(argument, myArgv[i]) == 0)
    {
      // see if we have a ret, we don't if the ret would be beyond argc
      if (*myArgc > i+1)
	ret = myArgv[i+1];
      else
	ret = NULL;
      // now toss our arguments closer, just by 1 if we don't have a ret
      if (ret == NULL)
      {
	for (; i < *myArgc - 1; i++)
	  myArgv[i] = myArgv[i+1];
      } 
      // by 2 if we do have a ret though
      else
      {
	for (; i < *myArgc - 2; i++)
	  myArgv[i] = myArgv[i+2];
      }
      // now decrement argc, again by 1 if we have no ret, by 2 if we
      // have a ret
      if (ret == NULL)
	*myArgc -= 1;
      else
	*myArgc -= 2;
      return ret;
    }
  }
  return NULL;
}






