/*#########################################
 * The localization and gradient example: starts all the basic
 * processes and then gradient and localization.  Note that in windows you 
 * need to copy sfGrad.dll and sfLoc.dll into the bin directory for this 
 * to run.
 *#########################################
 */


/**
 ** SfSaphira.cpp
 **
 ** Copyright 2001 by Kurt Konolige
 **
 ** The author hereby grants to SRI permission to use this software.
 ** The author also grants to SRI permission to distribute this software
 ** to schools for non-commercial educational use only.
 **
 ** The author hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial
 ** educational use only.  This software may not be distributed to others
 ** except by SRI, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of the author.
 **
 ** Neither the author nor SRI make any representations about the 
 ** suitability of this software for any purpose.  It is provided 
 ** "as is" without express or implied warranty.
 **
 ** Kurt Konolige
 ** Senior Computer Scientist
 ** SRI International
 ** 333 Ravenswood Avenue
 ** Menlo Park, CA 94025
 ** E-mail:  konolige@ai.sri.com
 **
 **/

#include "Saphira.h"
#include "SfMonte.h"
#include "SfGrad.h"

// This is a task that is set up to drive around to different points

class GotoTask
{
public:
  // It needs a constructor of course
  GotoTask(ArRobot *robot);
  // And a destructor too
  ~GotoTask(void);
  // Then it has a function here to call
  void task(void);
protected:
  // the goal we're on
  int myGoal;
  // our pointer to the robot object
  ArRobot *myRobot;
  // a callback functor
  ArFunctorC<GotoTask> myTaskCB;
  // a boolean to denote if we've started or not
  bool myStarted;
};

// the constructor, it also chains to create the callback
GotoTask::GotoTask(ArRobot *robot) : 
  myTaskCB(this, &GotoTask::task)
{
  // set our robot pointer
  myRobot = robot;
  // add ourself as a user task so we get called once per cycle
  myRobot->addUserTask("gotoTask", 50, &myTaskCB);
  // initialize variable so we start correctly
  myStarted = false;
}

// the destructor
GotoTask::~GotoTask(void)
{
  // remove ourself from the robot's user task list
  myRobot->remUserTask(&myTaskCB);
}

// where we drive
void GotoTask::task(void)
{
  // if we're done, if we failed, or if we haven't started yet we need
  // to give the gradient module a new place to drive
  if (my_grad->goalStatus == statusDone || 
      my_grad->goalStatus == statusFailed || 
      !myStarted)
  {
    // if we haven't started yet start the beginning
    if (!myStarted)
    {
      myStarted = true;
      myGoal = 1;
    }
    // if we've finished this goal move on to the next one
    if (my_grad->goalStatus == statusDone)
    {
      // just increment the variable for the next goal
      myGoal++;
      // but here catch if we're beyond our number of goals to start
      // over... you should change this > as you add goals below
      if (myGoal > 2)
        myGoal = 1;
    }
    // if we failed then print that out for information but try to get
    // to the same goal again (could have lost the path for a moment
    // as we turned or someone could have been in the way
    if (my_grad->goalStatus == 3)
      sfMessage("Failed getting to goal %d, trying again", myGoal);
    // let everyone know whats going on
    sfMessage("Going to goal %d", myGoal);
    // now just a sequence of goals to go to
    if (myGoal == 1)
      sfGradSetGoal(1000, 0);
    if (myGoal == 2)
      sfGradSetGoal(0, 0);

  }
}

// main function, sets up a robot and inits ARIA and Saphira
// only returns when exit is explicitly signaled
#ifndef WIN32
int
main(int argc, char **argv)
#else
int PASCAL
WinMain(HINSTANCE hInst, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
#endif
{
  ArSerialConnection robotCon;

  Sf::init();			// start up Aria and Saphira  
  
  // Load the graphical files... only needed for debugging
  sfLoadLibraryFile("fltk");
  sfLoadLibraryFile("fltkwin");
  sfLoadLibraryFile("sfLocFl");
  sfLoadLibraryFile("sfGradFl");

  sfLoadWorldFile("worlds/newoffice.wld");
  // load a world file
  sfLoadWorldFile("worlds/newofficelines.wld");
  // initialize the monte carlo module
  mcSonarInit();
  // we don't need to see all that extra data
  mcPrintDuringUpdates(0);
  // initialize gradient
  sfGradInit();
  // don't use the map (thats from the laser) when driving
  sfGradUseMap(0);
  // use those forbidden lines when driving
  sfGradUseArtifacts(1);
  // use the sonar when driving, but not the laser
  sfGradUseSonar(1);
  // don't use the laser
  sfGradUseLaser(0);
  // actually update the robot position
  mcUpdateRobotPose(1);

  // start up the robot, don't need a connection
  SfROBOT->runAsync(false);
  // set what port we're going to connect to
  robotCon.setPort(ArUtil::COM1);
  // attach the serial connection to the robot
  SfROBOT->setDeviceConnection(&robotCon);
  // connection to the robot
  if (!SfROBOT->blockingConnect())
  {
    sfMessage("Could not connect to the robot, exiting");
    printf("Could not connect to the robot, exiting\n");
    exit(1);
  }
  
  // Enable the motors
  SfROBOT->comInt(ArCommands::ENABLE, 1);

  // make the task that drives around, comment out to drive yourself
  GotoTask task(SfROBOT);
  // loop while the robot's running
  while (Sf::robot()->isRunning())
    ArUtil::sleep(100);
  // now finish up and leave
  Aria::shutdown();		
  return 0;
}
