/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARDPPTU_H
#define ARDPPTU_H

#include "ariaTypedefs.h"
#include "ArRobot.h"
#include "ArPTZ.h"

/** This class is for controlling the Directed Perceptions Pan-Tilt Unit

Note that there are far too many functions enabled in here, most of which are extraneous.  The important ones are defined in the ArPTZ class and include the basic pan, tilt commands.

The DPPTU's pan and tilt commands work on a number of units equal to (degrees / 0.514).  The panTilt function always rounds the conversion closer to zero, so that a magnitude greater than the allowable range of movement is not sent to the camera.

*/

/// A class with the commands for the DPPTU
class ArDPPTUCommands
{
public:
  enum {
    DELIM = 0x20, ///<Space - Carriage return delimeter
    INIT = 0x40, ///<Init character
    ACCEL = 0x61, ///<Acceleration, Await position-command completion
    BASE = 0x62, ///<Base speed
    CONTROL = 0x63, ///<Speed control
    DISABLE = 0x64, ///<Disable character, Delta, Default
    ENABLE = 0x65, ///<Enable character, Echoing
    FACTORY = 0x66, ///<Restore factory defaults
    HALT = 0x68, ///<Halt, Hold, High
    IMMED = 0x69, ///<Immediate position-command execution mode, Independent control mode
    LIMIT = 0x6C, ///<Position limit character, Low
    MONITOR = 0x6D, ///<Monitor, In-motion power mode
    OFFSET = 0x6F, ///<Offset position, Off
    PAN = 0x70, ///<Pan
    RESET = 0x72, ///<Reset calibration, Restore stored defaults, Regular
    SPEED = 0x73, ///<Speed, Slave
    TILT = 0x74, ///<Tilt
    UPPER = 0x75, ///<Upper speed limit
    VELOCITY = 0x76 ///<Velocity control mode
  };

};

/// A class for for making commands to send to the DPPTU
/** 
    There are only a few functioning ways to put things into this packet, you
    MUST use these, if you use anything else your commands won't work.  You 
    must use byteToBuf and byte2ToBuf.  
*/
class ArDPPTUPacket: public ArBasePacket
{
public:
  /// Constructor
  AREXPORT ArDPPTUPacket(ArTypes::UByte2 bufferSize = 30);
  /// Destructor
  AREXPORT virtual ~ArDPPTUPacket(void);

  AREXPORT virtual void byte2ToBuf(int val);

  AREXPORT virtual void finalize(void);

protected:
};

/// Driver for the DPPTU
class ArDPPTU : public ArPTZ
{
public:
  /// Constructor
  AREXPORT ArDPPTU(ArRobot *robot);
  /// Destructor
  AREXPORT virtual ~ArDPPTU(void);

  AREXPORT bool init(void);

  AREXPORT bool canZoom(void) { return 0; }

  /// Sends a delimiter only
  AREXPORT bool blank(void);

  /// Perform reset calibration
  AREXPORT bool resetCalib(void);
  /// Disable power-on reset
  AREXPORT bool disableReset(void);
  /// Reset tilt axis
  AREXPORT bool resetTilt(void);
  /// Reset pan axis only
  AREXPORT bool resetPan(void);
  /// Reset pan and tilt axes on power-on
  AREXPORT bool resetAll(void);

  /// Save current settings as defaults
  AREXPORT bool saveSet(void);
  /// Restore stored defaults
  AREXPORT bool restoreSet(void);
  /// Restore factory defaults
  AREXPORT bool factorySet(void);

  AREXPORT bool panTilt(int pdeg, int tdeg);

  AREXPORT bool pan(int deg) { return panTilt(deg, myTilt); }
  AREXPORT bool panRel(int deg) { return panTilt(myPan+deg, myTilt); }
  AREXPORT bool tilt(int deg) { return panTilt(myPan, deg); }
  AREXPORT bool tiltRel(int deg) { return panTilt(myPan, myTilt+deg); }
  AREXPORT bool panTiltRel(int pdeg, int tdeg) { return panTilt(myPan+pdeg, myTilt+tdeg); }

  /// Enables or disables the position limit enforcement
  AREXPORT bool limitEnforce(bool val);
  /// Sets unit to immediate-execution mode for positional commands
  AREXPORT bool immedExec(void);
  /// Sets unit to slaved-execution mode for positional commands
  AREXPORT bool slaveExec(void);
  /// Instructs unit to await completion of the last issued command
  AREXPORT bool awaitExec(void);
  /// Halts all pan-tilt movement
  AREXPORT bool haltAll(void);
  /// Halts pan axis movement
  AREXPORT bool haltPan(void);
  /// Halts tilt axis movement
  AREXPORT bool haltTilt(void);

  AREXPORT int getMaxPosPan(void) { return MAX_PAN; }
  AREXPORT int getMaxNegPan(void) { return MIN_PAN; }
  AREXPORT int getMaxPosTilt(void) { return MAX_TILT; }
  AREXPORT int getMaxNegTilt(void) { return MIN_TILT; }
  AREXPORT int getMaxPanSlew(void) { return MAX_PAN_SLEW; }
  AREXPORT int getMinPanSlew(void) { return MIN_PAN_SLEW; }
  AREXPORT int getMaxTiltSlew(void) { return MAX_TILT_SLEW; }
  AREXPORT int getMinTiltSlew(void) { return MIN_TILT_SLEW; }
  AREXPORT int getMaxPanAccel(void) { return MAX_PAN_ACCEL; }
  AREXPORT int getMinPanAccel(void) { return MIN_PAN_ACCEL; }
  AREXPORT int getMaxTiltAccel(void) { return MAX_TILT_ACCEL; }
  AREXPORT int getMinTiltAccel(void) { return MIN_TILT_ACCEL; }

  /// Sets monitor mode - pan pos1/pos2, tilt pos1/pos2
  AREXPORT bool initMon(int deg1, int deg2, int deg3, int deg4);
  /// Enables monitor mode at power up
  AREXPORT bool enMon(void);
  /// Disables monitor mode at power up
  AREXPORT bool disMon(void);

  /// Sets stationary power mode to off
  AREXPORT bool offStatPower(void);
  /// Sets regular stationary power mode
  AREXPORT bool regStatPower(void);
  /// Sets low stationary power mode
  AREXPORT bool lowStatPower(void);
  /// Sets high in-motion power mode
  AREXPORT bool highMotPower(void);
  /// Sets regular in-motion power mode
  AREXPORT bool regMotPower(void);
  /// Sets low in-motion power mode
  AREXPORT bool lowMotPower(void);

  /// Sets acceleration for pan axis
  AREXPORT bool panAccel(int deg);
  /// Sets acceleration for tilt axis
  AREXPORT bool tiltAccel(int deg);

  /// Sets the start-up pan slew
  AREXPORT bool basePanSlew(int deg);
  /// Sets the start-up tilt slew
  AREXPORT bool baseTiltSlew(int deg);

  /// Sets the upper pan slew
  AREXPORT bool upperPanSlew(int deg);
  /// Sets the lower pan slew
  AREXPORT bool lowerPanSlew(int deg);
  /// Sets the upper tilt slew
  AREXPORT bool upperTiltSlew(int deg);
  /// Sets the lower pan slew
  AREXPORT bool lowerTiltSlew(int deg);

  /// Sets motion to indenpendent control mode
  AREXPORT bool indepMove(void);
  /// Sets motion to pure velocity control mode
  AREXPORT bool velMove(void);

  /// Sets the rate that the unit pans at
  AREXPORT bool panSlew(int deg);
  /// Sets the rate the unit tilts at 
  AREXPORT bool tiltSlew(int deg);

  /// Sets the rate that the unit pans at, relative to current slew
  AREXPORT bool panSlewRel(int deg) { return panSlew(myPanSlew+deg); }
  /// Sets the rate the unit tilts at, relative to current slew
  AREXPORT bool tiltSlewRel(int deg) { return tiltSlew(myTiltSlew+deg); }

  int getPan(void) { return myPan; }
  int getTilt(void) { return myTilt; }

  /// Gets the current pan slew
  int getPanSlew(void) { return myPanSlew; }
  /// Gets the current tilt slew
  int getTiltSlew(void) { return myTiltSlew; }
  /// Gets the base pan slew
  int getBasePanSlew(void) { return myBasePanSlew; }
  /// Gets the base tilt slew
  int getBaseTiltSlew(void) { return myBaseTiltSlew; }
  /// Gets the current pan acceleration rate
  int getPanAccel(void) { return myPanAccel; }
  /// Gets the current tilt acceleration rate
  int getTiltAccel(void) { return myTiltAccel; }

  enum {
    MAX_PAN = 158, /// Maximum pan range of 3090 positions
    MIN_PAN = -158, /// Minimum pan range of -3090 positions
    MAX_TILT = 30, /// Maximum tilt range of 600 positions
    MIN_TILT = -46, /// Minimum tilt range of -900 positions
    MAX_PAN_SLEW = 149, /// Maximum pan slew of 2902 positions/sec
    MIN_PAN_SLEW = 2, /// Minimum tilt slew of 31 positions/sec
    MAX_TILT_SLEW = 149, /// Maximum tilt slew of 2902 positions/sec
    MIN_TILT_SLEW = 2, /// Minimum tilt slew of 31 positions/sec
// These aren't correct, because the manual doesn't specify
// the min/max acceleration
    MAX_PAN_ACCEL = 102, /// Maximum pan acceleration of 2000 positions/sec^2
    MIN_PAN_ACCEL = 2, /// Minimum pan acceleration of 0 positions/sec^2
    MAX_TILT_ACCEL = 102, /// Maximum tilt acceleration of 2000 positions/sec^2
    MIN_TILT_ACCEL = 2, /// Minimum tilt acceleration of 0 positions/sec^2
  };
protected:
  ArRobot *myRobot;
  ArDPPTUPacket myPacket;
  int myPan;
  int myTilt;
  int myPanSlew;
  int myTiltSlew;
  int myBasePanSlew;
  int myBaseTiltSlew;
  int myPanAccel;
  int myTiltAccel;
  static const double CONVERT; /// 0.0514 degrees per position
};

#endif // ARDPPTU_H

