/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARJOYHANDLER_H
#define ARJOYHANDLER_H

#include "ariaTypedefs.h"

#ifdef WIN32
#include <mmsystem.h>
#endif // WIN32

#ifdef linux
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/joystick.h>
#endif


/// Interfaces to a joystick
/** 
    The joystick handler keeps track of the minimum and maximums for both
    axes, updating them to constantly be better calibrated.  The speeds set 
    influence what is returned by getAdjusted...
    
    The joystick is not opened until init is called.  What should basically 
    be done to use this class is to 'init' a joystick, do a 'setSpeed' so you 
    can use 'getAdusted', then at some point do a 'getButton' to see if a
    button is pressed, and then do a 'getAdjusted' to get the values to act
    on.
    
    Also note that x is usually rotational velocity (since it right/left),
    whereas Y is translational (since it is up/down).
    
    You can also use this to do multiple uses with the joystick, for example to
    have button 1 drive the robot while to have button 2 move the camera, 
    you can get the different values you want (don't want to move the camera
    as quickly or as far as the robot) by using setSpeed before doing 
    getAdjusted since setSpeed is fast and won't take any time.
*/
class ArJoyHandler
{
 public:
  /// Constructor
  AREXPORT ArJoyHandler(bool useOSCal = true);
  /// Destructor
  AREXPORT ~ArJoyHandler(void);
  /// Intializes the joystick, returns true if successful
  AREXPORT bool init(void);
  /// Sets whether to just use OS calibration or not
  AREXPORT void setUseOSCal(bool useOSCal);
  /// Gets whether to just use OS calibration or not
  AREXPORT bool getUseOSCal(void);
  /// Returns if the joystick was successfully initialized or not
  AREXPORT bool haveJoystick(void) { return myInitialized; }
  /// Sets the max that X or Y will return
  AREXPORT void setSpeeds(int x, int y) { myTopX = x; myTopY = y; }
  /// Gets the adjusted reading, as integers, based on the setSpeed
  AREXPORT void getAdjusted(int *x, int *y);
  /// Gets the adjusted reading, as floats, between -1.0 and 1.0
  AREXPORT void getDoubles(double *x, double *y);
  /// Checkes whether the given button is pressed or not
  AREXPORT bool getButton(int button);

  /// Starts the calibration process
  AREXPORT void startCal(void);
  /// Ends the calibration process
  AREXPORT void endCal(void);
  /// Gets the unfilitered reading, mostly for internal use, maybe
  /// useful for Calibration
  AREXPORT void getUnfiltered(int *x, int *y);
  /// Gets the stats for the joystick, useful after calibrating to save values
  AREXPORT void getStats(int *maxX, int *minX, int *maxY, int*minY, 
		 int *cenX, int *cenY);
  /// Sets the stats for the joystick, useful for restoring calibrated settings
  AREXPORT void setStats(int maxX, int minX, int maxY, int minY, 
		int cenX, int cenY);
  /// Gets the speeds that X and Y are set to
  AREXPORT void getSpeeds(int *x, int *y);

 protected:
  int myMaxX, myMinX, myMaxY, myMinY, myCenX, myCenY, myTopX, myTopY;
  int myPhysMax;
  bool myInitialized;
  bool myUseOSCal;
#ifdef WIN32
  JOYINFO myJoyInfo;
  JOYCAPS myJoyCaps;
#endif // WIN32
#ifdef linux
  struct JS_DATA_TYPE myJoyData; // structure for the buttons and x,y coords
  FILE * myJoyDesc;
#endif // linux
};


#endif // ARJOYHANDLER_H

