/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef _ArPreferences_h
#define _ArPreferences_h


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "ariaTypedefs.h"
#include "ArPref.h"
#include "ArPrefSection.h"


/*
  Abstract preferences class. This class manages the preferences file.
  You must derive off this class and define the functions to find the
  preferences file and define the preferences table
  
  The preferences are organized by section. Each section can have any
  number of key/value pairs. The key name from one section can be the same
  as from another section. When getting a value, the section name and
  keyword name must be supplied. A value can be one of 4 types: Integer,
  Double, Boolean, String. There are accessor functions for all 4 types that
  will parse the value into the correct form.
  
  To add a preference, add the keyword name to the PrefType enumeration.
  Then add an entry in the ArPreferences::ourPrefs array of Pref structures.
  The ArPreferences::ourPrefs array contains the default values in case the
  preferences file is not found.
  
  To add a section, add to the PrefSection enum. Add the string that will
  match the section name in the preferences file to ourSections[].
  
  If a keyword name is used in multiple sections, just define it once in
  the PrefType enumeration and it can be used in as many sections as are
  defined.
 */


class ArPreferences
{
public:

  AREXPORT ArPreferences(ArPrefSection *sections, int sectEnd, ArPref *prefTable,
			 int prefEnd);
  AREXPORT virtual ~ArPreferences();

  // Initialize the preferences file. This is all the program needs to call
  // to prepare the preferences class. It will return false if there are
  // any unrecoverable errors. Check alreadyCreated() to see if the init
  // had to create the prefs file.
  AREXPORT virtual void init(bool noCreate = false);

  // go through the pref file, and retrieve the preference values
  AREXPORT virtual bool load(const char *fileName="", bool noCreate=true);

  // Save the file, creating it if need be.
  AREXPORT virtual bool save();

  // Get the preference structure
  AREXPORT virtual ArPref * getPref(int section, int pref);
  AREXPORT virtual ArPref * getPref(const char *section, const char *pref);
  AREXPORT virtual bool getPrefRange(int section, int pref,
				     ArPref **first, ArPref **last);
  AREXPORT virtual void addPref(ArPref *pref);

  AREXPORT virtual bool clear(int section, int pref, bool all=false);
  AREXPORT virtual bool clear(const char *section, const char *pref,
			      bool all=false);
  AREXPORT virtual void clearAll();

  AREXPORT virtual std::string getHeader() = 0;
  AREXPORT virtual std::string getDir() = 0;
  AREXPORT virtual std::string getFile() = 0;
  AREXPORT virtual std::string getFileName();
  AREXPORT virtual std::string getError() {return(myError);}

  AREXPORT virtual ArPrefSection * getSection(const char *section);
  // Get the section in enumeration form
  AREXPORT virtual int getSectionNum(const char *section);

protected:

  // Find the location of the preferences file.
  AREXPORT virtual bool findPrefFile(const char *fileName="",
				     bool noCreate=false);
  AREXPORT virtual bool findDir(const char *fileName) = 0;
  AREXPORT virtual bool findFile(const char *fileName) = 0;
  AREXPORT virtual bool handleListSectLine(const char *section,
					   const char *line) = 0;
  AREXPORT virtual void extraSave(FILE *fp) = 0;


  ArPrefSection *mySectionTable;
  ArPrefSection *mySects;
  int mySectEnd;
  ArPref *myPrefTable;
  int myPrefEnd;
  std::string myError;
};


#endif // _ArPreferences_h
