/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARSENSORREADING_H
#define ARSENSORREADING_H

#include "ariaTypedefs.h"
#include "ariaUtil.h"
#include "ArTransform.h"

/// A class to hold a sensor reading, should be one instance per sensor
/** This class holds sensor data and a sensor reading... it can happen that
    it contains the data for a sonar, but not the reading, in which case the 
    range (from getRange) will be -1, and the counter it was taken (from
    getCounterTaken) will be 0, also it will never be new (from isNew)
*/
class ArSensorReading
{
public:
  /// Constructor, the three args are the physical location of the sonar
  AREXPORT ArSensorReading(double xPos = 0.0, double yPos = 0.0, double thPos = 0.0);
  AREXPORT virtual ~ArSensorReading(void);

  /// Gets the range of the reading
  /**
     @return the distance return from the sensor (how far from the robot)
  */
  int getRange(void) { return myRange; }

  /// Given the counter from the robot, it returns whether the reading is new
  /**
     @param counter the counter from the robot at the current time
     @return true if the reading was taken on the current loop
     @see getCounter
  */
  bool isNew(unsigned int counter) { return counter == myCounterTaken; }
  /// Gets the X location of the sonar reading
  double getX(void) { return myReading.getX(); }
  /// Gets the Y location of the sonar reading
  double getY(void) { return myReading.getY(); }
  /// Gets the position of the reading 
  /// @return the position of the reading (ie where the sonar pinged back)
  ArPose getPose(void) { return myReading; }

  /// Gets the pose the reading was taken at
  ArPose getPoseTaken(void) { return myReadingTaken; }

  /// Gets the encoder pose the reading was taken at
  ArPose getEncoderPoseTaken(void) { return myEncoderPoseTaken; }

  /// Gets the X location of the sonar on the robot
  double getSensorX(void) { return mySensorPos.getX(); }
  /// Gets the Y location of the sensor on the robot
  double getSensorY(void) { return mySensorPos.getY(); }
  /// Gets the heading of the sensor on the robot
  double getSensorTh(void) { return mySensorPos.getTh(); }
  /// Gets the sensors position on the robot
  /** 
      @return the position of the sensor on the robot
  */
  ArPose getSensorPosition(void) { return mySensorPos; }

  /// Gets the cos component of the heading of the sensor reading
  double getSensorDX(void) { return mySensorCos; }
  /// Gets the sin component of the heading of the sensor reading
  double getSensorDY(void) { return mySensorSin; }

  /// Gets the X locaiton of the robot when the reading was received
  double getXTaken(void) { return myReadingTaken.getX(); }
  /// Gets the Y location of the robot when the reading was received
  double getYTaken(void) { return myReadingTaken.getY(); }
  /// Gets the th (heading) of the robot when the reading was received
  double getThTaken(void) { return myReadingTaken.getTh(); }

  /// Gets the counter from when the reading arrived
  /**
     @return the counter from the robot when the sonar reading was taken
     @see isNew
  */
  unsigned int getCounterTaken(void) { return myCounterTaken; }
  
  /// Takes the data and makes the reading reflect it
  AREXPORT void newData(int range, ArPose robotPose, ArPose encoderPose,
			ArTransform trans, unsigned int counter, 
			ArTime timeTaken);
  /// Takes the data and makes the reading reflect it
  AREXPORT void ArSensorReading::newData(int sx, int sy, ArPose robotPose,
					 ArPose encoderPose,
					 ArTransform trans, 
					 unsigned int counter,
					 ArTime timeTaken);
  /// Resets the sensors idea of its physical location on the robot
  AREXPORT void resetSensorPosition(double xPos, double yPos, double thPos,
				    bool forceComputation = false);

  /// Applies a transform to the reading position, and where it was taken
  AREXPORT void applyTransform(ArTransform trans);
protected:
  unsigned int myCounterTaken;
  ArPose myReading;
  ArPose myReadingTaken;
  ArPose myEncoderPoseTaken;
  ArPose mySensorPos;
  double mySensorCos, mySensorSin;
  double myDistToCenter;
  double myAngleToCenter;
  int myRange;
  ArTime myTimeTaken;
};

#endif
