/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARSICK_H
#define ARSICK_H

#include "ariaTypedefs.h"
#include "ArSickPacket.h"
#include "ArSickPacketReceiver.h"
#include "ArRobotPacket.h"
#include "ArRangeDeviceThreaded.h"   
#include "ArFunctor.h"
#include "ArCondition.h"

/// The sick driver
class ArSick : public ArRangeDeviceThreaded
{
public:
  enum BaudRate {
    BAUD9600, ///< 9600 Baud
    BAUD19200, ///< 19200 Baud
    BAUD38400 ///< 38400 Baud
  };
  enum Degrees {
    DEGREES180, ///< 180 Degrees
    DEGREES100 ///< 100 Degrees
  };
  enum Increment {
    INCREMENT_ONE, ///< One degree increments
    INCREMENT_HALF ///< Half a degree increments
  };
  /// Constructor
  AREXPORT ArSick(size_t currentBufferSize = 361, 
		  size_t cumulativeBufferSize = 1448,
		  const char *name = "laser");
  /// Destructor
  AREXPORT ~ArSick(void);
  /// Configure the laser before connecting to it
  AREXPORT void configure(bool useSim = false, bool powerControl = true, 
			  bool laserFlipped = false, 
			  BaudRate baud = BAUD38400,
			  Degrees deg = DEGREES180, 
			  Increment incr = INCREMENT_ONE);
  /// Shorter configure for the laser
  AREXPORT void configureShort(bool useSim = false, 
			       BaudRate baud = BAUD38400,
			       Degrees deg = DEGREES180, 
			       Increment incr = INCREMENT_ONE);
  /// Sets the position of the laser on the robot
  AREXPORT void setSensorPosition(double x, double y, double th);
  /// Sets the position of the laser on the robot
  AREXPORT void setSensorPosition(ArPose pose);  
  /// Gets the position of the laser on the robot
  AREXPORT ArPose getSensorPosition();
  /// Gets the X position of the laser on the robot
  AREXPORT double getSensorPositionX();
  /// Gets the Y position of the laser on the robot
  AREXPORT double getSensorPositionY();
  /// Gets the heading of the laser on the robot
  AREXPORT double getSensorPositionTh();
  /// Connect to the laser while blocking
  AREXPORT bool blockingConnect(void);
  /// Connect to the laser asyncronously
  AREXPORT bool asyncConnect(void);
  /// Disconnect from the laser
  AREXPORT bool disconnect(bool doNotLockRobotForSim = false);
  /// Sets the device connection
  AREXPORT void setDeviceConnection(ArDeviceConnection *conn);
  /// Gets the device connection
  AREXPORT ArDeviceConnection *getDeviceConnection(void);
  /// Sees if this is connected to the laser
  bool isConnected(void) 
    { if (myState == STATE_CONNECTED) return true; else return false; }
  /// Sees if this is trying to connect to the laser at the moment
  bool tryingToConnect(void) 
    { if (myState != STATE_CONNECTED && myState != STATE_NONE) 
      return true; else return false; }
  /// Runs the laser off of the robot, advised
  AREXPORT bool runOnRobot(void);

  /// Gets the number of laser packets received in the last second
  AREXPORT int getSickPacCount();

  /// Adds a connect callback
  AREXPORT void addConnectCB(ArFunctor *functor, ArListPos::Pos position);
  /// Adds a disconnect callback
  AREXPORT void remConnectCB(ArFunctor *functor);

  /// Adds a callback for when a connection to the robot is failed
  AREXPORT void addFailedConnectCB(ArFunctor *functor, 
				   ArListPos::Pos position);
  /// Removes a callback for when a connection to the robot is failed
  AREXPORT void remFailedConnectCB(ArFunctor *functor);

  /// Adds a callback for when disconnect is called while connected
  AREXPORT void addDisconnectNormallyCB(ArFunctor *functor, 
					ArListPos::Pos position);
  /// Removes a callback for when disconnect is called while connected
  AREXPORT void remDisconnectNormallyCB(ArFunctor *functor);
  
  /// Adds a callback for when disconnection happens because of an error
  AREXPORT void addDisconnectOnErrorCB(ArFunctor *functor, 
				       ArListPos::Pos position);
  /// Removes a callback for when disconnection happens because of an error
  AREXPORT void remDisconnectOnErrorCB(ArFunctor *functor);

  /// Sets the time without a response until connection assumed lost
  AREXPORT void setConnectionTimeoutTime(int mSecs);
  /// Gets the time without a response until connection assumed lost
  AREXPORT int getConnectionTimeoutTime(void);

  /// Gets the time data was last receieved
  AREXPORT ArTime getLastReadingTime(void);

  /// Sets the distance which two readings must be less than to be ignored
  AREXPORT void setFilterNearDist(double dist) { myFilterNearDist = dist; }
  /// Gets the distance which two readings must be less than to be ignored
  AREXPORT double getFilterNearDist(void) { return myFilterNearDist; }

  /// Gets whether the laser is simulated or not
  AREXPORT bool isUsingSim(void);
  /// Gets whether the computer is controling laser power or not
  AREXPORT bool isControllingPower(void);
  /// Gets whether the laser is flipped over or not
  AREXPORT bool isLaserFlipped(void);
  /// Gets the degrees the laser is scanning
  AREXPORT Degrees getDegrees(void);
  /// Gets the amount each scan increments
  AREXPORT Increment getIncrement(void);
  /// The packet handler for when connected to the simulator
  AREXPORT bool simPacketHandler(ArRobotPacket * packet);
  /// The function called if the laser isn't running in its own thread and
  /// isn't simulated
  AREXPORT void sensorInterpCallback(void);
  /// An internal function
  AREXPORT bool internalConnectSim(void);
  /// An internal function, single loop event to connect to laser
  AREXPORT int internalConnectHandler(void);
  /// The internal function used by the ArRangeDeviceThreaded
  AREXPORT virtual void * runThread(void *arg);
  /// The internal function which processes the sickPackets
  AREXPORT void processPacket(ArSickPacket *packet, ArPose pose, 
			      ArPose encoderPose, unsigned int counter);
  /// The internal function that gets does the work
  AREXPORT void runOnce(bool lockRobot);
  AREXPORT virtual void setRobot(ArRobot *robot);
  /// Internal function, shouldn't be used, drops the conn because of error
  AREXPORT void dropConnection(void);
  /// Internal function, shouldn't be used, denotes the conn failed
  AREXPORT void failedConnect(void);
  /// Internal function, shouldn't be used, does the after conn stuff
  AREXPORT void madeConnection(void);
  /// Internal function, shouldn't be used, gets params from the robot
  AREXPORT void robotConnectCallback(void);
protected:
  enum State {
    STATE_NONE, ///< Nothing, haven't tried to connect or anything
    STATE_INIT, ///< Initializing the laser
    STATE_WAIT_FOR_POWER_ON, ///< Waiting for power on
    STATE_CHANGE_BAUD, ///< Change the baud, no confirm here
    STATE_CONFIGURE, ///< Send the width and increment to the laser
    STATE_WAIT_FOR_CONFIGURE_ACK, ///< Wait for the configuration Ack
    STATE_INSTALL_MODE, ///< Switch to install mode
    STATE_WAIT_FOR_INSTALL_MODE_ACK, ///< Wait until its switched to install mode
    STATE_SET_MODE, ///< Set the mode (mm/cm) and extra field bits
    STATE_WAIT_FOR_SET_MODE_ACK, ///< Waiting for set-mode ack
    STATE_START_READINGS, ///< Switch to monitoring mode
    STATE_WAIT_FOR_START_ACK, ///< Waiting for the switch-mode ack
    STATE_CONNECTED ///< We're connected and getting readings
  };
  /// Internal function for filtering the raw readings and updating buffers
  void filterReadings();
  /// Internal function for switching states
  AREXPORT void switchState(State state);
  State myState;
  ArTime myStateStart;
  ArFunctorC<ArSick> myRobotConnectCB;
  ArRetFunctor1C<bool, ArSick, ArRobotPacket *> mySimPacketHandler;
  ArFunctorC<ArSick> mySensorInterpCB;
  std::list<ArSensorReading *>::iterator myIter;
  bool myStartConnect;
  bool myRunningOnRobot;
  int myTimeoutTime;

  // range buffers to hold current range set and assembling range set
  std::list<ArSensorReading *> *myAssembleReadings;
  std::list<ArSensorReading *> *myCurrentReadings;

  bool myProcessImmediately;
  bool myInterpolation;
  // list of packets, so we can process them from the sensor callback
  std::list<ArSickPacket *> myPackets;

  // a lock for the device (so we are mostly unlocked for people to get data, 
  // but so we don't have people trying to read multiple times from the same 
  // device
  ArMutex myConnLock;

  // whether we did a real configure or a short configure
  bool myRealConfigured;
  
  // these two are just for the sim packets
  unsigned int myWhichReading;
  unsigned int myTotalNumReadings;

  // some variables so we don't have to do a tedios if every time
  double myOffsetAmount;
  double myIncrementAmount;

  // packet stuff
  ArSickPacket myPacket;
  bool myRunInOwnThread;
  bool myUseSim;
  bool myLaserFlipped;
  ArPose mySensorPose;
  bool myPowerControl;
  BaudRate myBaud;
  Degrees myDegrees;
  Increment myIncrement;
  ArTime myLastReading;
  
  // stuff for the sim packet
  ArPose mySimPacketStart;
  ArTransform mySimPacketTrans;
  ArTransform mySimPacketEncoderTrans;
  unsigned int mySimPacketCounter;

  // packet count
  time_t myTimeLastSickPacket;
  int mySickPacCurrentCount;
  int mySickPacCount;

  // connection
  ArSickPacketReceiver mySickPacketReceiver;
  ArDeviceConnection *myConn;
  std::list<ArFunctor *> myConnectCBList;
  std::list<ArFunctor *> myFailedConnectCBList;
  std::list<ArFunctor *> myDisconnectNormallyCBList;
  std::list<ArFunctor *> myDisconnectOnErrorCBList;

  // filtering parameters
  double myFilterNearDist;	// we throw out cumulative readings this close to current one
};

#endif //ARSICK_H
