/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARSOCKET_H
#define ARSOCKET_H


#ifdef linux
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <stdio.h>
#include <errno.h>
#endif


#include <string>
#include "ariaTypedefs.h"


/// socket communication wrapper
/**
   ArSocket is a layer which allows people to use the sockets networking
   interface in an operating system independent manner. All of the standard
   commonly used socket functions are implemented. This class also contains
   the file descriptor which identifies the socket to the operating system.

   In Windows, the networking subsystem needs to be initialized and shutdown
   individyaly by each program. So when a program starts they will need to
   call the static function ArSocket::init() and call ArSocket::shutdown()
   when it exits. For programs that use Aria::init() and Aria::uninit()
   calling the ArSocket::init() and ArSocket::shutdown() is unnecessary. The
   Aria initialization functions take care of this. These functions do nothing
   in Linux.
*/
class ArSocket
{
public:

  enum Type {UDP, TCP, Unknown};
  enum Error {NoErr, NetFail, ConBadHost, ConNoRoute, ConRefused};

  /// Constructor
  AREXPORT ArSocket();

  /// Constructor which connects to a server
  AREXPORT ArSocket(const char *host, int port, Type type);

  /// Constructor which opens a server port
  AREXPORT ArSocket(int port, bool doClose, Type type);

  /// Destructor
  AREXPORT ~ArSocket();

  /// Initialize the network layer
  AREXPORT static bool init();

  /// Shutdown the network layer
  AREXPORT static void shutdown();

  AREXPORT static bool ourInitialized;

  /// Copy socket structures
  AREXPORT bool copy(int fd, bool doclose);

  /// Copy socket structures
  AREXPORT void copy(ArSocket *s)
    {myFD=s->myFD; myDoClose=false; mySin=s->mySin;}

  /// Transfer ownership of a socket
  /** transfer() will transfer ownership to this socket. The input socket
      will no longer close the file descriptor when it is destructed.
  */
  AREXPORT void transfer(ArSocket *s)
    {myFD=s->myFD; myDoClose=true; s->myDoClose=false; mySin=s->mySin;
     myType=s->myType;}

  /// Connect as a client to a server
  AREXPORT bool connect(const char *host, int port, Type type);

  /// Open a server port
  AREXPORT bool open(int port, Type type);

  /// Simply create a port.
  AREXPORT bool create(Type type);

  /// Find a valid unused port and bind the socket to it.
  AREXPORT bool findValidPort(int startPort);

  /// Connect the socket to the given address
  AREXPORT bool connectTo(const char *host, int port);

  /// Connect the socket to the given address
  AREXPORT bool connectTo(struct sockaddr_in *sin);

  /// Accept a new connection
  AREXPORT bool accept(ArSocket *sock);

  /// Close the socket
  AREXPORT bool close();

  /// Write to the socket
  /**
     @param buff buffer to write from
     @param len how many bytes to write
     @return number of bytes written
  */
  AREXPORT int write(const void *buff, size_t len)
    {
      struct timeval tval;
      fd_set fdSet;
      tval.tv_sec = 0;
      tval.tv_usec = 0;
      FD_ZERO(&fdSet);
      FD_SET(myFD, &fdSet);
      if (select(myFD + 1, NULL, &fdSet, NULL, &tval) <= 0)
	return 0;
      
#ifdef WIN32
      return(::send(myFD, (char*)buff, len, 0));
#else
      return(::write(myFD, (char*)buff, len));
#endif
    }

  /// Read from the socket
  /**
     @param buff buffer to read into
     @param len how many bytes to read
     @param msWait if 0, don't block, if > 0 wait this long for data
     @return number of bytes read
  */
  AREXPORT int read(void *buff, size_t len, unsigned int msWait = 0)
    {
      if (msWait != 0)
      {
	struct timeval tval;
	fd_set fdSet;
	tval.tv_sec = msWait / 1000;
	tval.tv_usec = (msWait % 1000) * 1000;
	FD_ZERO(&fdSet);
	FD_SET(myFD, &fdSet);
	if (select(myFD + 1, &fdSet, NULL, NULL, &tval) <= 0)
	  return 0;
      }
      return(::recv(myFD, (char*)buff, len, 0));
    }

  /// Send a message on the socket
  AREXPORT int sendTo(const void *msg, int len)
    {return(::sendto(myFD, (char*)msg, len, 0, (struct sockaddr*)&mySin,
		     sizeof(mySin)));}

  /// Send a message on the socket
  AREXPORT int sendTo(const void *msg, int len, struct sockaddr_in *sin)
    {return(::sendto(myFD, (char*)msg, len, 0, (struct sockaddr*)sin,
		     sizeof(struct sockaddr_in)));}

  /// Receive a message from the socket
  AREXPORT int recvFrom(void *msg, int len, sockaddr_in *sin)
#ifdef WIN32
    {int i=sizeof(sockaddr_in);
#else
    {unsigned int i=sizeof(sockaddr_in);
#endif
     return(::recvfrom(myFD, (char*)msg, len, 0, (struct sockaddr*)sin, &i));}

  /// Convert a host string to an address structure
  AREXPORT static bool hostAddr(const char *host, struct in_addr &addr);

  /// Convert an address structure to a host string
  AREXPORT static bool addrHost(struct in_addr &addr, char *host);

  /// Get the localhost address
  AREXPORT static std::string getHostName();

  /// Get the socket name. Stored in ArSocket::mySin
  AREXPORT bool getSockName();

  /// Accessor for the sockaddr
  AREXPORT struct sockaddr_in * sockAddrIn() {return(&mySin);}

  /// Accessor for the in_addr
  AREXPORT struct in_addr * inAddr() {return(&mySin.sin_addr);}

  /// Accessor for the port of the sockaddr
  AREXPORT unsigned short int inPort() {return(mySin.sin_port);}

  /// Convert addr into string numerical address
  AREXPORT static void inToA(struct in_addr *addr, char *buff);

  /// Size of the sockaddr
  AREXPORT static const size_t sockAddrLen() {return(sizeof(struct sockaddr_in));}

#ifdef WIN32
  /// Max host name length
  AREXPORT static const size_t maxHostNameLen() {return(MAXGETHOSTSTRUCT);}
#else
  /// Max host name length
  AREXPORT static const size_t maxHostNameLen() {return(MAXHOSTNAMELEN);}
#endif

  /// Host byte order to network byte order
  AREXPORT static unsigned int hostToNetOrder(int i);

  /// Network byte order to host byte order
  AREXPORT static unsigned int netToHostOrder(int i);

  /// Set the linger value
  AREXPORT bool setLinger(int time);

  /// Set broadcast value
  AREXPORT bool setBroadcast();

  /// Set the reuse address value
  AREXPORT bool setReuseAddress();

  /// Set socket to nonblocking
  AREXPORT bool setNonBlock();

  /// Change the doClose value
  AREXPORT void setDoClose(bool yesno) {myDoClose=yesno;}

  /// Get the file descriptor
  AREXPORT int getFD() const {return(myFD);}

  /// Get the protocol type
  AREXPORT Type getType() const {return(myType);}

  /// Get the last error string
  AREXPORT const std::string & getErrorStr() const {return(myErrorStr);}

  /// Get the last error
  AREXPORT Error getError() const {return(myError);}

  /// Writes a string to the socket
  /**
     This cannot write more than 2048 number of bytes
     @param str the string to write to the socket
     @return number of bytes written
  **/
  AREXPORT int writeString(const char *str, ...) 
  {
    char buf[2049];
    va_list ptr;
    va_start(ptr, str);
    vsprintf(buf, str, ptr);
    va_end(ptr);
    return write(buf, strlen(buf));
  }

  /// Reads a string from the socket
  /**
     @param buf the buffer to read the string into, if there is no
     error but there is no string to read then the first character of
     the buffer is set to the null character
     
     @param len the lenth of the buffer

     @return true if the socket could be read from, false if it
     couldn't (which also means the connection should be closed)
  **/
  AREXPORT bool readString(char *buf, size_t len)
  {
    int n;
    n = read(buf, len - 1);
    if (n > 0)
    {
      buf[n] = '\0';
      return true;
    }
    else if (n == 0)
    {
      return false;
    }
    else // which means (n < 0)
    {
      buf[0] = '\0';
#ifdef WIN32
      if (WSAGetLastError() == WSAEWOULDBLOCK)
	return true;
#endif
#ifndef WIN32
      if (errno == EAGAIN)
	return true;
#endif
      perror("Error in reading from network");
      return false;
    }
  }

protected:

  Type myType;
  Error myError;
  std::string myErrorStr;
  bool myDoClose;
  int myFD;
  bool myNonBlocking;
  struct sockaddr_in mySin;
};


#endif // ARSOCKET_H
