/* 

Copyright(C) 2002, ActivMedia Robotics, LLC. All rights reserved.

This copy of Aria was relicensed for use with Saphira and the Saphira
license by ActivMedia Robotics, LLC.  If you wish to download a real
distribution of Aria under the GPL or a commercial license go to
http://www.activrobots.com/SOFTWARE/aria.html or contact ActivMedia
Robotics, LLC at robots@activmedia.com or ActivMedia Robotics, 44
Concord Street, Peterborough, NH 03458; 603-924-9100

ActivMedia Robotics, LLC hereby grants to SRI permission to use this
software.  ActivMedia Robotics, LLC also grants to SRI permission to
distribute this software to schools for non-commercial educational use
only.

ActivMedia Robotics, LLC hereby grants to other individuals or
organizations permission to use this software for non-commercial
educational use only.  This software may not be distributed to others
except by SRI, under the conditions above.

Other than these cases, no part of this software may be used or
distributed without written permission of ActivMedia Robotics, LLC.

Neither ActivMedia Robotics, LLC nor SRI make any representations
about the suitability of this software for any purpose.  It is
provided "as is" without express or implied warranty.  

*/
#ifndef ARIAUTIL_H
#define ARIAUTIL_H

#include <string>
#include <list>
#include <math.h>
#ifdef WIN32
#include <sys/timeb.h>
#endif
#ifdef linux
#include <sys/time.h>
#include <unistd.h>
#endif
#include "ariaTypedefs.h"
#include "ArLog.h"
#include "ariaInternal.h"

#ifndef M_PI
#define M_PI 3.1415927
#endif // of M_PI, windows has a function call instead of a define

/// This class has utility functions
class ArUtil
{
public:
  /// Values for the bits from 0 to 16
  enum BITS { 
    BIT0 = 0x1, ///< value of BIT0
    BIT1 = 0x2, ///< value of BIT1
    BIT2 = 0x4, ///< value of BIT2
    BIT3 = 0x8, ///< value of BIT3
    BIT4 = 0x10, ///< value of BIT4
    BIT5 = 0x20, ///< value of BIT5
    BIT6 = 0x40, ///< value of BIT6
    BIT7 = 0x80, ///< value of BIT7
    BIT8 = 0x100, ///< value of BIT8
    BIT9 = 0x200, ///< value of BIT9
    BIT10 = 0x400, ///< value of BIT10
    BIT11 = 0x800, ///< value of BIT11
    BIT12 = 0x1000, ///< value of BIT12
    BIT13 = 0x2000, ///< value of BIT13
    BIT14 = 0x4000, ///< value of BIT14
    BIT15 = 0x8000, ///< value of BIT15
  };

  /// Sleep for the given number of milliseconds
  AREXPORT static void sleep(unsigned int ms);
  
  /// Get the time in milliseconds
  AREXPORT static unsigned int getTime(void);

  /// Delete all members of a set. Does NOT empty the set.
  /** 
      Assumes that T is an iterator that supports the operator*, operator!=
      and operator++. The return is assumed to be a pointer to a class that
      needs to be deleted.
  */
  template<class T> static void deleteSet(T begin, T end)
    {
      for (; begin != end; ++begin)
      {
	delete (*begin);
      }
    }
  
  /// Delete all members of a set. Does NOT empty the set.
  /**
     Assumes that T is an iterator that supports the operator**, operator!=
     and operator++. The return is assumed to be a pair. The second value of
     the pair is assumed to be a pointer to a class that needs to be deleted.
  */
  template<class T> static void deleteSetPairs(T begin, T end)
    {
      for (; begin != end; ++begin)
      {
	delete (*begin).second;
      }
    }

  /// Split a string into a set of words.
  AREXPORT static void splitString(std::string inString,
				   std::list<std::string> &outList);

  /// OS-independent way of finding the size of a file.
  AREXPORT static long sizeFile(std::string fileName);

  /// OS-independent way of checking to see if a file exists and is readable.
  AREXPORT static bool findFile(const char *fileName);

  /// OS-independent way of stripping the directory from the filename.
  AREXPORT static bool stripDir(std::string fileIn, std::string &fileOut);

  /// OS-independent way of stripping the filename from the directory.
  AREXPORT static bool stripFile(std::string fileIn, std::string &fileOut);

  /// Appends a slash to a path if there is not one there already
  AREXPORT static void appendSlash(std::string &path);

  /// Fix the slash orientation in file path string for windows or linux
  AREXPORT static void fixSlashes(std::string &path);

  /// Fix the slash orientation in file path string to be all forward
  AREXPORT static void fixSlashesForward(std::string &path);

  /// Fix the slash orientation in file path string to be all backward
  AREXPORT static void fixSlashesBackward(std::string &path);

  /// Finds out if two strings are equal
  AREXPORT static int strcmp(std::string str, std::string str2);

  /// Finds out if two strings are equal
  AREXPORT static int strcmp(std::string str, const char *str2);

  /// Finds out if two strings are equal
  AREXPORT static int strcmp(const char *str, std::string str2);

  /// Finds out if two strings are equal
  AREXPORT static int strcmp(const char *str, const char *str2);

  /// Puts a \ before spaces in src, puts it into dest
  AREXPORT static void escapeSpaces(char *dest, const char *src);

  /// Returns a string contained in an arbitrary file
  AREXPORT static std::string getStringFromFile(const char *fileName);

  /** 
  These are for passing into getStringFromRegistry
  **/
  enum REGKEY {
    REGKEY_CLASSES_ROOT, ///< use HKEY_CLASSES_ROOT
    REGKEY_CURRENT_CONFIG, ///< use HKEY_CURRENT_CONFIG
    REGKEY_CURRENT_USER, ///< use HKEY_CURRENT_USER
    REGKEY_LOCAL_MACHINE, ///< use HKEY_LOCAL_MACHIE
    REGKEY_USERS ///< use HKEY_USERS
  };

  /// Returns a string from the Windows registry
  AREXPORT static bool getStringFromRegistry(REGKEY root,
					     const char *key,
					     const char *value,
					     char *str,
					     int len);
  AREXPORT static const char *COM1;
  AREXPORT static const char *COM2;
  AREXPORT static const char *COM3;
  AREXPORT static const char *COM4;
  
};

///  This class has static members to do common math operations.
class ArMath
{
public:

  /// This adds two angles together and fixes the result to [-180, 180] 
  /**
     @param ang1 first angle
     @param ang2 second angle, added to first
     @return sum of the angles, in range [-180,180]
     @see subAngle
     @see fixAngle */
  static double addAngle(double ang1, double ang2) 
    { return fixAngle(ang1 + ang2); }

  /// This subtracts one angle from another and fixes the result to [-180,180]
  /**
     @param ang1 first angle
     @param ang2 second angle, subtracted from first angle
     @return resulting angle, in range [-180,180]
     @see addAngle
     @see fixAngle
  */
  static double subAngle(double ang1, double ang2) 
    { return fixAngle(ang1 - ang2); }

  /// Takes an angle and returns the angle in range (-180,180]
  /**
     @param angle the angle to fix
     @return the angle in range (-180,180]
     @see addAngle
     @see subAngle
  */
  static double fixAngle(double angle) 
    {
      if (angle >= 360)
	angle = angle - 360.0 * (double)((int)angle / 360);
      if (angle < -360)
	angle = angle + 360.0 * (double)((int)angle / -360);
      if (angle <= -180)
	angle = + 180.0 + (angle + 180.0);
      if (angle > 180)
	angle = - 180.0 + (angle - 180.0);
      return angle;
    }
  
  /// Converts an angle in degrees to an angle in radians
  /**
     @param deg the angle in degrees
     @return the angle in radians
     @see radToDeg
  */     
  static double degToRad(double deg) { return deg * M_PI / 180.0; }

  /// Converts an angle in radians to an angle in degrees
  /**
     @param rad the angle in radians
     @return the angle in degrees
     @see degToRad
  */
  static double radToDeg(double rad) { return rad * 180.0 / M_PI; }

  /// Finds the cos, from angles in degrees
  /**
     @param angle angle to find the cos of, in degrees
     @return the cos of the angle
     @see sin
  */
  static double cos(double angle) { return ::cos(ArMath::degToRad(angle)); }

  /// Finds the sin, from angles in degrees
  /**
     @param angle angle to find the sin of, in degrees
     @return the sin of the angle
     @see cos
  */
  static double sin(double angle) { return ::sin(ArMath::degToRad(angle)); }

  /// Finds the arctan of the given y/x pair
  /**
     @param y the y distance
     @param x the x distance
     @return the angle y and x form
  */
  static double atan2(double y, double x) 
    { return ArMath::radToDeg(::atan2(y, x)); }
  /// Finds if one angle is between two other angles
  static bool angleBetween(double angle, double startAngle, double endAngle)
    {
      angle = fixAngle(angle);
      startAngle = fixAngle(startAngle);
      endAngle = fixAngle(endAngle);
      if ((startAngle < endAngle && angle > startAngle && angle < endAngle) ||
	  (startAngle > endAngle && (angle > startAngle || angle < endAngle)))
	return true;
      else
	return false;
    }

  /// Finds the absolute value of a double
  /**
     @param val the number to find the absolute value of
     @return the absolute value of the number
  */
  static double fabs(double val) 
    {
      if (val < 0.0)
	return -val;
      else
	return val;
    }

  /// Finds the closest integer to double given
  /**
     @param val the double to find the nearest integer to
     @return the integer the value is nearest to
  */
  static int roundInt(double val) 
    { return((int) floor(val + .49)); }
    
  /// Rotates a point around 0 by degrees given
  static void pointRotate(double *x, double *y, double th)
    {
      double cs, sn, xt, yt;
      cs = cos(th);
      sn = sin(th);
      xt = *x;  
      yt = *y;
      *x = cs*xt + sn*yt;
      *y = cs*yt - sn*xt;
    }
  
  /// Returns a long between 0 and some arbitrary huge number
  static long random(void)
    {
#ifdef WIN32
      return(rand());
#else
      return(lrand48());
#endif
    }
  /// Finds the distance between two coordinates
  /**
     @param x1 the first coords x position
     @param y1 the first coords y position
     @param x2 the second coords x position
     @param y2 the second coords y position
     @return the distance between (x1, y1) and (x2, y2)
  **/
  static double distanceBetween(double x1, double y1, double x2, double y2)
    { return sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));  }
};

/// The class which represents a position
/** 
    This class represents a robot position with heading.  The heading defaults
    to 0, and so does not need to be used (this avoids having 2 types of 
    positions).  Everything in the class is inline so it should be fast.
*/
class ArPose
{
public:
  /// Constructor, with optional initial values
  /** 
      Sets the position with the given values, can be used with no variables,
      with just x and y, or with x, y, and th
      @param x the position to set the x position to, default of 0
      @param y the position to set the y position to, default of 0
      @param th the position to set the th position to, default of 0
  */
  ArPose(double x = 0, double y = 0, double th = 0)
    { myX = x; myY = y; myTh = th; }
  /// Copy Constructor
  ArPose(const ArPose &pose) : 
    myX(pose.myX), myY(pose.myY), myTh(pose.myTh) {}

  /// Destructor
  virtual ~ArPose(void) {}
  /// Sets the position to the given values
  /** 
      Sets the position with the given three values, but the theta does not
      need to be given as it defaults to 0.
      @param x the position to set the x position to
      @param y the position to set the y position to
      @param th the position to set the th position to, default of 0
  */
  virtual void setPose(double x, double y, double th = 0) 
    { setX(x); setY(y); setTh(th); }
  /// Sets the position equal to the given position
  /** @param position the position value this instance should be set to */
  virtual void setPose(ArPose position)
    {
      setX(position.getX());
      setY(position.getY());
      setTh(position.getTh());
    }
  /// Sets the x position
  void setX(double x) { myX = x; }
  /// Sets the y position
  void setY(double y) { myY = y; }
  /// Sets the heading
  void setTh(double th) { myTh = ArMath::fixAngle(th); }
  /// Sets the heading, using radians
  void setThRad(double th) { myTh = ArMath::fixAngle(ArMath::radToDeg(th)); }
  /// Gets the x position
  double getX(void) { return myX; }
  /// Gets the y position
  double getY(void) { return myY; }
  /// Gets the heading
  double getTh(void) { return myTh; }
  /// Gets the heading, in radians
  double getThRad(void) { return ArMath::degToRad(myTh); }
  /// Gets the whole position in one function call
  /**
     Gets the whole position at once, by giving it 2 or 3 pointers to 
     doubles.  If you give the function a null pointer for a value it won't
     try to use the null pointer, so you can pass in a NULL if you don't 
     care about that value.  Also note that th defaults to NULL so you can 
     use this with just x and y.
     @param x a pointer to a double to set the x position to
     @param y a pointer to a double to set the y position to
     @param th a pointer to a double to set the heading to, defaults to NULL
   */
  void getPose(double *x, double *y, double *th = NULL) 
    { 
      if (x != NULL) 
	*x = myX;
      if (y != NULL) 
	*y = myY; 
      if (th != NULL) 
	*th = myTh; 
    }
  /// Finds the distance from this position to the given position
  /**
     @param position the position to find the distance to
     @return the distance to the position from this instance
  */
  virtual double findDistanceTo(ArPose position)
    {
      return ArMath::distanceBetween(getX(), getY(), 
				     position.getX(), 
				     position.getY());
    }
  /// Finds the angle between this position and the given position
  /** 
      @param position the position to find the angle to
      @return the angle to the given position from this instance, in degrees
  */
  virtual double findAngleTo(ArPose position)
    {
      return ArMath::radToDeg(atan2(position.getY() - getY(),
				    position.getX() - getX()));
    }
  /// Logs the coordinates using ArLog
  virtual void print(void) 
    { ArLog::log(ArLog::Terse, "%.0f %.0f %.1f", myX, myY, myTh); }
protected:
  double myX;
  double myY;
  double myTh;
};


/// A class for time readings
/** 
    This class is for getting the time of certain events.  This class is not
    for generic time stuff, just for timeStamping, hence the only commands are
    very simple and the accessors for getting the data directly shouldn't 
    really be used.  DON'T use this for keeping track of what time it is, 
    its just for relative timing (ie this loop needs to sleep another 100 ms);
*/

class ArTime
{
public:
  /// Constructor
  ArTime(void) { mySec = 0; myMSec = 0; }
  /// Destructor
  ~ArTime(void) {}
  
  /// Gets the number of milliseconds since the given timestamp to this one
  long mSecSince(ArTime since) 
    {
      long timeSince, timeThis;

      timeSince = since.getSec() * 1000 + since.getMSec();
      timeThis = mySec * 1000 + myMSec;
      return timeSince - timeThis;
    }
  /// Gets the number of seconds since the given timestamp to this one
  long secSince(ArTime since)
    {
      return mSecSince(since)/1000;
    }
  /// Finds the number of millisecs from when this timestamp is set to to now
  long mSecTo(void) 
    {
      ArTime now;
      now.setToNow();
      return -mSecSince(now);
    }
  /// Finds the number of seconds from when this timestamp is set to to now
  long secTo(void)
    {
      return mSecTo()/1000;
    }
  /// Finds the number of milliseconds from this timestamp to now
  long mSecSince(void) 
    {
      ArTime now;
      now.setToNow();
      return mSecSince(now);
    }
  /// Finds the number of seconds from when this timestamp was set to now
  long secSince(void)
    {
      return mSecSince()/1000;
    }
  /// returns whether the given time is before this one or not
  bool isBefore(ArTime testTime)
    {
      if (mSecSince(testTime) < 0)
	return true;
      else
	return false;
    }
  /// returns whether the given time is equal to this time or not
  bool isAt(ArTime testTime)
    {
      if (mSecSince(testTime) == 0)
	return true;
      else
	return false;
    }
  /// returns whether the given time is after this one or not
  bool isAfter(ArTime testTime)
    {
      if (mSecSince(testTime) > 0)
	return true;
      else
	return false;
    }
  /// Sets the time to now
  void setToNow(void)
    {
#ifdef WIN32
      /* this should be the better way, but it doesn't really work...
	 this would be seconds from 1970, but it is based on the
	 hardware timer or something and so winds up not being updated
	 all the time and winds up being some number of ms < 20 ms off
      struct _timeb startTime;
      _ftime(&startTime);
      mySec = startTime.time;
      myMSec = startTime.millitm;*/
      // so we're going with just their normal function, msec since boot
      long timeNow;
      timeNow = timeGetTime();
      mySec = timeNow / 1000;
      myMSec = timeNow % 1000;
#endif // WIN32
      
#ifdef linux
      struct timeval timeNow;
      
      if (gettimeofday(&timeNow, NULL) == 0)
      {
	mySec = timeNow.tv_sec;
	myMSec = timeNow.tv_usec / 1000;
      }
      else
	ArLog::log(ArLog::Terse, "ArTime::setToNow: invalid return from gettimeofday.\n");
#endif //linux
    }
  /// Add some milliseconds (can be negative) to this time
  void addMSec(long ms)
    {
      unsigned long timeThis;
      timeThis = mySec * 1000 + myMSec;
      if (ms < 0 && (unsigned)abs(ms) > timeThis)
      {
	ArLog::log(ArLog::Terse, "ArTime::addMsec: tried to subtract too many milliseconds, would result in a negative time.");
	mySec = 0;
	myMSec = 0;
      }
      else 
      {
	timeThis += ms;
	mySec = timeThis / 1000;
	myMSec = timeThis % 1000;
      }
    }
  /// Sets the seconds since 1970
  void setSec(time_t sec) { mySec = sec; }
  /// Sets the milliseconds
  void setMSec(time_t msec) { myMSec = msec; }
  /// Gets the seconds since 1970
  time_t getSec(void) { return mySec; }
  /// Gets the milliseconds
  time_t getMSec(void) { return myMSec; }
  /// Logs the time
  void print(void) 
    { ArLog::log(ArLog::Terse, "Time: %ld.%ld", getSec(), getMSec()); }
protected:
  time_t mySec;
  time_t myMSec;

};

/// A subclass of pose that also has the time the pose was taken
/**

 */
class ArPoseWithTime : public ArPose
{
public:
  ArPoseWithTime(double x = 0, double y = 0, double th = 0,
	 ArTime thisTime = ArTime()) : ArPose(x, y, th)
    { myTime = thisTime; }
  virtual ~ArPoseWithTime(void) {}
  void setTime(ArTime newTime) { myTime = newTime; }
  ArTime getTime(void) { return myTime; }
protected:
  ArTime myTime;
};

/// A class for keeping track of if a complete revolution has been attained
/**
   This class can be used to keep track of if a complete revolution has been
   done, it is used by doing doing a clearQuadrants when you want to stat
   the revolution.  Then at each point doing an updateQuadrant with the current
   heading of the robot.  When didAllQuadrants returns true, then all the 
   quadrants have been done.
*/
class ArSectors
{
public:
  /// Constructor
  ArSectors(int numSectors = 8) 
    { 
      mySectorSize = 360/numSectors;
      mySectors = new int[numSectors]; 
      myNumSectors = numSectors; 
      clear();
    }
  /// Destructor
  virtual ~ArSectors(void) { delete mySectors; }
  /// Clears all quadrants
  void clear(void) 
    {
      int i;
      for (i = 0; i < myNumSectors; i++)
	mySectors[i] = false;
    }
  /// Updates the appropriate quadrant for the given angle
  void update(double angle)
    {
      int angleInt;
      angleInt = ArMath::roundInt(ArMath::fixAngle(angle) + 180);
      mySectors[angleInt / mySectorSize] = true;
    }
  /// Returns true if the all of the quadrants have been gone through
  bool didAll(void)
    {
      int i;
      for (i = 0; i < myNumSectors; i++)
	if (mySectors[i] == false)
	  return false;
      return true;
    }
protected:
  int *mySectors;
  int myNumSectors;
  int mySectorSize;
};

/// This class is to build arguments for things that require argc and argv
class ArArgumentBuilder
{
public:
  /// Constructor
  AREXPORT ArArgumentBuilder(size_t argvLen = 256);
  /// Destructor
  AREXPORT virtual ~ArArgumentBuilder(void);
  /// Adds the given string, with varargs, seperates if there are spaces
  AREXPORT void add(char *str, ...);
  /// Prints out the arguments
  AREXPORT void print(void);
  /// Gets the argc
  AREXPORT size_t getArgc(void);
  /// Gets the argv
  AREXPORT char** getArgv(void);
protected:
  // how many arguments we have
  size_t myArgc;
  // argument list
  char **myArgv;
  // argv length
  size_t myArgvLen;
};

/// Class for parsing arguments
/**
   This class is made for parsing arguments form an argc/argv
   set... if you're using a winmain you can first toss your string at
   the ArArgumentBuilder above class ArArgumentParser and then use
   this parser on it
**/
class ArArgumentParser
{
public:
  /// Constructor, takes the argc argv
  AREXPORT ArArgumentParser(int *argc, char **argv);
  /// Destructor
  AREXPORT ~ArArgumentParser(void);
  /// Returns true if the argument was found
  AREXPORT bool checkArgument(char *argument);
  /// Returns the word/argument after given argument or NULL if it is
  /// not present
  AREXPORT char *checkParameterArgument(char *argument);
protected:
  int *myArgc;
  char **myArgv;


};


#endif // ARIAUTIL_H
