//
// SfClass.h
//
// Basic Saphira class definitions

//
// Copyright 2000 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

#ifndef SFCLASSES_H
#define SFCLASSES_H

#include <list>
#include <map>


//
// enumerated lists
//

class SfList
{
 public:
  SfList(int mxx, void **pp)
    { n = 0; p = pp; mx = mxx; };
  ~SfList() {};
  void reset()
    { n = 0; };
  SFEXPORT void add(void *p);
  SFEXPORT void addBeginning(void *p);
  SFEXPORT void addUnique(void *p);
  SFEXPORT int  remove(void *p);
  SFEXPORT void **getObjs() 
    { return p; };
  SFEXPORT int  getNum()
    { return n; };

 protected:
  int n;
  int mx;
  void **p;
};



// abstract drawable class, for mixing in
// inheritance from this class means you can draw yourself

class SfWin;			// forward ref

/// SfDrawable class
/// To draw on the LPS graphics window, inherit from the subclass SfArtifact
/// Members variables here can be used to turn drawing on or off, change the color
class SfDrawable
{
public:
  SfDrawable()			// by default we draw
     { visible = true; }
  virtual void draw(SfWin *) {};
  /// True if the object is to be drawn
  bool visible;			// should we draw?
  /// Color of the object; not yet implemented...
  int color;			// what color we are
};




#define MAXKEYFNS 10
#define MAXBUTTONFNS 10


/// Abstract window class
/// Purpose of this class is to make drawing fns available without
///   specifying any implementation
/// E.g., could use FLTK, or a null implementation for no drawing
class SfWin
{
public:
  SFEXPORT SfWin();		// sets up key and button handler lists
  virtual ~SfWin() {};
  enum {
    FIRST, ///< Put a button or key handler at the beginning of the callback list
    LAST,  ///< At the end
    REMOVE ///< Remove it
  };
  /// draws a line
  virtual void Vector(double, double, double, double) {}; 
  /// draws a line relative to a pose; use NULL for the robot
  virtual void Vector(double, double, double, double, ArPose *) {}; 
  /// draws a rectangle
  virtual void Rectangle(double, double, double, double) {}; 
  // draws a rectangle relative to a pose; use NULL for the robot
  virtual void Rectangle(double, double, double, double, ArPose *) {}; 
  /// draws a centered rectangle
  virtual void CRectangle(double, double, double, double) {}; 
  /// draws a centered rectangle relative to a pose; use NULL for the robot
  virtual void CRectangle(double, double, double, double, ArPose *) {}; 
  /// draws a point
  virtual void Point(double, double) {}; 
  /// draws a point relative to a pose; use NULL for the robot
  virtual void Point(double, double, ArPose *) {}; 
  /// draws a polygon, using an array of points
  virtual void Polygon(int, double *, double *) {} 
  /// draws a circle at x,y, with radius r
  virtual void Circle(double x, double y, double r) {}; 
  /// draws a circle relative to a pose; use NULL for the robot
  virtual void Circle(double x, double y, double r, ArPose *) {}; 
  /// draws a text string at location x,y
  virtual void Text(char *str, double x, double y) {}; 
  /// draws a text string at location x,y relative to a pose; use NULL for the robot
  virtual void Text(char *str, double x, double y, ArPose *) {}; 
  virtual void Add(SfDrawable *) {}; // adds a drawable to the display list
  virtual void Scale(double) {}; // scale in mm/pixel
  virtual void Resize(int, int) {}; // in pixels
  virtual void Center(double, double) {}; // center window here in rw coords
  /// Sets the drawing pen color
  /// This holds until another pen color is set
  virtual void PenColor(int) {}; 
  virtual void Transform(ArPose *) {}; // sets the transform for window display, go away
  virtual void Rotate(double) {}; // sets the rotation, in deg
  /// returns RW coords from screen i,j
  virtual void Coords(double *x, double *y, int i, int j) {}; 


  /// adds a keystroke handler to the window
  SFEXPORT void AddKeyHandler(int (*fn)(int, int, SfWin *), int which = FIRST); 
  /// adds a button press handler
  SFEXPORT void AddButtonHandler(int (*fn)(int, int, int, int, SfWin *), int which = FIRST); 

 private:
  void **key_fn_buf[MAXKEYFNS];
  void **button_fn_buf[MAXBUTTONFNS];

 protected:
  SfList keyFnList;
  SfList buttonFnList;
};

// definitions of some graphics constants

#include "FL/Enumerations.H"

/// left button constant
#define sfLeftButton   FL_BUTTON1
/// middle button constant
#define sfMiddleButton FL_BUTTON2
/// right button constant
#define sfRightButton  FL_BUTTON3 

#define sfShiftMask    FL_SHIFT
#define sfControlMask  FL_CTRL
#define sfAltMask      FL_ALT
#define sfMetaMask     FL_META



// Abstract frame class
// Purpose of this class is to make frames available without
//   specifying any implementation
// E.g., could use FLTK, or a null implementation for no drawing

class SfSensorDevice;
class SfRangeDevice;

class SfFr
{
public:
  virtual ~SfFr() {};
  virtual void AddSensor(SfSensorDevice *sd) {}; // adds this device to display list
  virtual void addRangeDevice(SfRangeDevice *) {}; // adds this device to display list
  SFEXPORT static SfFr *Current(); // returns default frame
  SFEXPORT static void Current(SfFr *f); // sets default frame
protected:
  static SFEXPORT SfFr *_current; // pointer to current default
};

/// reasonable macro for returning current frame
#define SfFRAME SfFr::Current()




//
// Geometry definitions
//

/// SfVector objects represent a vector by its endpoints
class SfVector			// represents a vector or rectangle in 2D
{				// dims in mm
public:
  double x1, y1, x2, y2;
  /// Default constructor, all zero coords
  SfVector()
    { x1 = y1 = x2 = y2 = 0; }
  /// Constructor, using endpoints
  SfVector(double xx1, double yy1, double xx2, double yy2)
    { x1 = xx1; y1 = yy1; x2 = xx2; y2 = yy2; }
      
};



/// Unility timing class
class SfTime
{
public:
  /// Constructor, sets time to zero for this object
  SFEXPORT SfTime();
  /// Reset time to zero
  SFEXPORT void Reset();	// sets to zero
  /// returns current time in milliseconds
  SFEXPORT int TimeMS();	
  /// returns current time in microseconds
  SFEXPORT int TimeUS();	
private:
  int time;			// microsecond time
};


//
// Activity interpreter uTask class
// One instance is used as a schema
// Other instances are running activities
// Only used internally for the activity interpreter
//

class SfActTask
{
public:
  SfActTask(char *iname, int pri = 50, bool schema = false);
  ~SfActTask();

  void process();		// what we do

  SfActTask *dad;		// parent, NULL if none
  int timeout;			// if we have a time limit
  int time;			// time since we started

  interpact *ia;		// activity schema or instance
  void *actionNew;		// creates a new action
  ArAction *action;		// current action instance
  int nargs;			// number of arguments
  int type;			// sfACTIVITY or sfACTION
  int priority;			// priority for sfACTION only

  bool trace;			// are we tracing?
  int tracepc;			// last pc traced
  int disp;			// are we displaying?
  char *name;			// name of activity
  char *schemaName;		// name of schema
  beh_params params;		// parameters of activity
  int processState;		// state of the process

protected:
  ArFunctorC<SfActTask> myActCB;
};


/// signalling fns
SFEXPORT void sfSuspendTask(SfActTask *p, int time);
SFEXPORT void sfFailTask(SfActTask *p);
SFEXPORT void sfSucceedTask(SfActTask *p);
SFEXPORT void sfInterruptTask(SfActTask *p);
SFEXPORT void sfResumeTask(SfActTask *p);
SFEXPORT void sfRemoveTask(SfActTask *p);

/// task fns
SFEXPORT bool sfTaskFinished(SfActTask *act);
SFEXPORT bool sfTaskFinished(char *iname);
SFEXPORT bool sfTaskSuspended(SfActTask *act);
SFEXPORT bool sfTaskSuspended(char *iname);
extern "C" {
SFEXPORT int sfStartTask(char *schema, char *iname, int timeout, int priority, 
			 int suspend, ...);
}

//
// Activity schema registration
//

class SfActRegister
{
  friend class Sf;

 public:
  static void add(SfActTask *);	// add an activity as a schema
  static void add(char *name, interpact *ia); // make a new activity schema
  static void add(char *name, int nargs, beh_params pp, void *fn); // make a new action schema
  static void remove(SfActTask *);  // remove an activity
  static void remove(char *name); // remove by name
  static SFEXPORT SfActTask *find(char *name); // find schema by name
  static SFEXPORT SfActTask *getAct(char *iname); // get currently running task by name
  static SFEXPORT SfActTask *getAct(SfActTask *schema, char *iname); // make new instance of schema
  static SfActTask *removeAct(char *iname);// removes it from the activities list
  static void       removeAct(SfActTask *act); // removes it from the activities list
  static SFEXPORT SfActTask *currentTask; // current executing activity, NULL if none
  static std::list<SfActTask *> *getActList() { return activities; };

 protected:
  static std::list<SfActTask *> *schemas;
  static SFEXPORT std::list<SfActTask *> *activities;
};



#endif // SFCLASSES_H

