//
// SfDevices.h
//
// Saphira sonar, laser, and IR range device definitions

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

#ifndef SFDEVICES_H
#define SFDEVICES_H

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

#undef SFLASEREXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LASER_LIBRARY
#define SFLASEREXPORT _declspec( dllexport )
#else
#define SFLASEREXPORT _declspec( dllimport )
#endif
#else
#define SFLASEREXPORT
#endif


/// SfRangeDevice is the Saphira class for range devices, encapsulating
/// the ArRangeDevice class
/** 
    Instead of subclassing ArRangeDevice, we include a pointer
    to one as part of the class data.  This way, the subclasses
    of ArRangeDevice can be accomodated.

    User programs can access the current and accum buffers easily.
    Range device buffers are made available in the Sf class.
    @see Sf
*/
class SfRangeDevice : public SfDrawable
{
public:
  /// Aria range device class
  ArRangeDevice *ourRangeDevice;
  /// Returns the current history buffer for the range device
  ArRangeBuffer *getCurrent() { return ourRangeDevice->getCurrentRangeBuffer(); }
  /// Returns the accumulated history buffer for the range device
  ArRangeBuffer *getAccum() { return ourRangeDevice->getCumulativeRangeBuffer(); }
  /// Lock the range buffers while we fool with them
  void lockDevice() { ourRangeDevice->lockDevice(); }
  /// Unlock the buffers
  void unlockDevice() { ourRangeDevice->unlockDevice(); }
  /// Name of the device
  const char *getName()  { return ourRangeDevice->getName().c_str(); }
  void *paramWin;		// parameter window for GUI

  /// Set this to draw the accum buffer
  bool accumDraw;		// flags for drawing range buffers
  /// Set this to draw the current buffer
  bool curDraw;
};


/// Sonar device class.  Created by initialization of the Sf static class.
class SfSonarDevice : public SfRangeDevice
{
public:
  SFEXPORT SfSonarDevice(SfRobot *robot = NULL);
  void draw(SfWin *);
};

/// Laser device class.  Created by initialization of the Sf static class.
class SfLaserDevice : public SfRangeDevice
{
public:
  SFLASEREXPORT SfLaserDevice(SfRobot *robot = NULL);
  /// Start up the device
  SFLASEREXPORT bool start(char *port); // starts up the device; null port is sim
  SFLASEREXPORT bool startTcp(const char *host, int port);  // starts up the device on a tcp connect, DO NOT USE for the sim, use start with a NULL
  /// Stop the device
  SFLASEREXPORT bool stop();		// shuts down the device
  SFLASEREXPORT virtual void draw(SfWin *);

  // the following were added by benson limketkai (bensonl@ai.sri.com, June 2002)

  /// Start angle of laser scan wrt robot.
  SFLASEREXPORT double getStartAngle();
  /// End angle of laser scan wrt robot.
  SFLASEREXPORT double getEndAngle();
  /// Gets whether the laser is flipped over or not.
  SFLASEREXPORT bool isLaserFlipped();
  /// Gets the degrees the laser is scanning.
  SFLASEREXPORT double getDegrees();
  /// Gets the amount each scan increments.
  SFLASEREXPORT double getIncrement();
};


/// Irrf device class.  Created by initialization of the Sf static class.
class SfIrrfDevice : public SfRangeDevice
{
public:
  SFLASEREXPORT SfIrrfDevice(SfRobot *robot = NULL);

  SFLASEREXPORT virtual void draw(SfWin *);

  /// Start angle of laser scan wrt robot.
  SFLASEREXPORT double getStartAngle();
  /// End angle of laser scan wrt robot.
  SFLASEREXPORT double getEndAngle();
  /// Gets whether the laser is flipped over or not.
  SFLASEREXPORT bool isIrrfFlipped();
  /// Gets the degrees the laser is scanning.
  SFLASEREXPORT double getDegrees();
  /// Gets the amount each scan increments.
  SFLASEREXPORT double getIncrement();
};


#endif // SFDEVICES_H
