//
// SfGrad.h
//
// gradient class definitions

//
// Copyright 1999 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com


#ifndef SFGRAD_H
#define SFGRAD_H

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef GRAD_EXPORTS
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

/**
   \defgroup grad Gradient Navigation Module

   For efficient movement based on local obstacles and world maps,
   Saphira has a realtime path planner based on the gradient method
   [Konolige, IROS 2000].  For planning paths and moving in a world
   map, the gradient module is typically used with Markov Localization
   to keep the robot registered with a map as it moves (sfLoc,
   sfLocLaser, and sfLocFl libraries).

   Gradient Path Planning is a process for determining optimal paths
   for the robot, in real time.  These paths can take into account
   both local obstacles, sensed by sonars and/or laser range-finder
   devices; and global map information such as the location of walls
   and other structural obstacles.  At each sync cycle (100 ms), the
   Gradient module calculates the lowest-cost path from a goal point
   or set of goal points to the robot.  The algorithm starts by
   considering a local neighborhood connecting the robot and the goal
   or goals, and then expands its search if no path is found.  There
   is a user-settable limit on the size of the neighborhood
   considered.

   Gradient uses a square-cell grid as a cost field for determining
   good paths.  You can set the grid resolution; a typical resolution
   is 10 mm.  The maximum size of the grid can also be set.

   Costs are calculated from a set of obstacles, obtained from
   pre-existing maps and from local sensor readings.  Here are the
   obstacles sources:

   1. Artifacts in a world map.  Load a world map, and call
   sfGradUseArtifacts(true). 

   2. A grid map created from the laser navigation software.  A grid
   map is typically loaded into the localization system using
   mcLoadScanMap().  To access this map from the gradient module, use
   sfGradSetMap(mcGetObject()).  Finally, turn on grid map use by
   calling sfGradUseMap(true).

   Grid maps and world maps may both be used at the same time.

   3. Laser and sonar readings.  These can be turned on and off with
   sfGradUseLaser() and sfGradUseSonar().


   The cost field radiates outward from obstacles, to create a safety
   cushion for the robot.  You can adjust this cushion using
   sfGradObsParams(). 

   There is a local controllers, implemented as an action
   (SfGradAction), that drives the robot along the gradient path.
   This action controls the speed of the robot in a parameterized
   fashion, and can be set with sfGradSetSpeed.  A further refinement
   for rectangular robots is the ability to back up when appropriate;
   this behavior is controlled by sfGradSetCanBack and
   sfGradSetTurnRadius.  If the robot needs to turn more than 45
   degrees to follow the path, it will see if its turn radius is
   clear.  If it isn't, it will try to back up if possible, and turn
   around when it is clear of obstacles.
   
   Some relevant samples Colbert load files:
     flgrad.act - basic gradient, without localization
                  for localization with respect to a world, look at
                  flloc.act
     scan.act   - localization and gradient using a grid map
                  (from the Laser Localization/Navigation module).

   You can also use the gradient for a final approach to a given
   position, to do this call sfGradUseFinalApproach... you will also
   want to adjust the gradient obstacle parameters with
   sfGradObsParams and the done distance with sfGradSetDone (the close
   dist is ignored for the final approach).  You may want to set the
   speed with sfGradSetFinalApproachSpeed.
**/
/*@{*/


/// Initializes the gradient
SFEXPORT void sfGradInit(void);
/// Initializes the gradient,
/*
   uses res in mm for grid cell resolution, default is 100

   if turnRadius is 0 then it will use the normal parameters and
   action for movement, buf it turnRadius is not 0 it will make the
   grad action that will back up if the robot needs to travel backwards 
   and a circle with radius turnRadius is obstructed
*/
SFEXPORT void sfGradInitRes(int res, int turnRadius); 

/// Sets the map with an mcObject
SFEXPORT void sfGradSetMap(void *p);

/// Whether the gradient module should use artifacts (.wld files)
SFEXPORT void sfGradUseArtifacts(int useArtifacts);
/// Whether the gradient module should use grid maps (.map files)
SFEXPORT void sfGradUseMap(int useMap);

/// Whether the gradient module should use sonar data
SFEXPORT void sfGradUseSonar(int useSonar);
/// Whether the gradient module should use laser data
SFEXPORT void sfGradUseLaser(int useLaser);

/// Sets the Gradient goal to this global point (in mm).  
/// The goal can be changed at any time. 
SFEXPORT void sfGradSetGoal(float x, float y);

/// Sets the speed the robot travels at
SFEXPORT void sfGradSetSpeed(int high, int mid, int back);

/// Whether the robot should be going to a goal or not
SFEXPORT void sfGradDoGoal(int which);

/// Stops the robot going to goal and stops the robot
SFEXPORT void sfGradStop(void);

/// Gets the status of the gradient module
SFEXPORT int sfGradStatus(void);

/// Gets if the gradient module is active or not
SFEXPORT int sfGradIsActive(void);

/// Resets keep-out distance and decay distance; in mm
SFEXPORT void sfGradObsParams(int keepout, int decay);

/// Set how close we need to be to a goal to slow done or be done
SFEXPORT void sfGradSetDone(int close, int done);

/// Gets if the grad action can back up, 0 = no, 1 = when appropriate,
/// 2 = always
SFEXPORT int sfGradGetCanBack(void);

/// Gets if the grad action can back up, 0 = no, 1 = when appropriate,
/// 2 = always
SFEXPORT void sfGradSetCanBack(int canBack);

/// Sets the turn radius needed to let it turn instead of back, make
/// it back with sfGradSetCanBack
SFEXPORT void sfGradSetTurnRadius(int turnRadius);

/// Gets the turn radius needed to let it turn instead of back, make
/// sure it is allowed to back up with with sfGradGetCanBack
SFEXPORT int sfGradGetTurnRadius(void);

/// Sets the acceleration used in driving the robot
SFEXPORT void sfGradSetAcc(int acc);

/// Set number of propagations, mostly for show
SFEXPORT void sfGradSetPnum(int n);

/// Sets the size of the gradient window
SFEXPORT void sfGradSetMax(int width, int height);

/// Use the final approach action or not
SFEXPORT void sfGradUseFinalApproach(int which);

/// Sets the final approach speed
SFEXPORT void sfGradSetFinalApproachSpeed(int speed);

/// Gets the final approach speed
SFEXPORT int sfGradGetFinalApproachSpeed(void);

/*@}*/

#define WAVEMAX 20000

//
// the SfGradGrid class is a specialized grid for doing gradient movement
//

#define statusIdle   0
#define statusActive 1
#define statusDone   2
#define statusFailed 3
#define statusSearching 4

class SfGradGrid : public SfCellIndex, public SfDrawable
{
 public:
  SFEXPORT SfGradGrid(double res, SfVector *s); // resolution, size and location
  SFEXPORT void SetGradSize(double res, SfVector &s); // reset size, in Process()
  SFEXPORT bool WaitingToSync(void); // returns true if the setsize hasn't been done yet
  SFEXPORT void draw(SfWin *w);	// display it, with outlines
  SFEXPORT void AddToFrame(SfFr *f); // add GUI stuff

  /*
    fcost is the number of grid elements away from an obstacle that
    the gradient won't go within, while fdecay is how far away from
    the obstacle that the robot will try to avoid if possible
  */

  double fcost, fdecay;		// frontier expansion parameters
  SFEXPORT void SetGoal(ArPose &g, bool doit = true); // sets the goal to a point, in Process()
  SFEXPORT void ResetGoal(ArPose &g); // resets the goal, no new grad grid
  SFEXPORT void SetGoal2(ArPose &g, bool doit = true); // sets the goal to two points
  SFEXPORT void SetGoal(SfVector &v, bool doit = true); // sets the goal to a line
  // gradient values
  SFEXPORT double GradValX(double, double);// interpolated gradient value at this RW pos
  SFEXPORT double GradValY(double, double);// interpolated gradient value at this RW pos
  // returns found point p at dist on path from robot
  SFEXPORT bool CalcPathPoint(double dist, ArPose &p); 
  // returns found point p at dist on path from start
  SFEXPORT bool CalcPathPoint(double dist, ArPose &start, ArPose &p); 
  SFEXPORT double GoalDist();		// distance to goal from robot
  SFEXPORT double GoalDist(ArPose &p); // distance to goal from p
  SFEXPORT bool CalcField(int n = -1); // reset and recalculate the field
  SFEXPORT void CalcObs();      // reset and calculate obstacles, for debugging
  // boundaries
  SFEXPORT void SetMaxBounds(double width, double height);
  SFEXPORT void SetSearchBounds(SfVector &s);
  SFEXPORT void UseArtifacts(bool on); // sets grad bounds to world by default
  SFEXPORT void UseGridMap(SfGrid *g); // we want to use our map now
  SFEXPORT bool UseGridMap(bool on); // turn it on and off
  SFEXPORT bool IsMaxSize();
  SFEXPORT void BuildPath(std::list<ArPose> *path);
  /// Adds a callback for when a new goal is called
  SFEXPORT void addNewGoalCallback(ArFunctor *newGoalCB, 
				   ArListPos::Pos position);
  /// Removes a callback 
  SFEXPORT void remNewGoalCallback(ArFunctor *newGoalCB);

  ArPose goal;		// goal position
  ArPose goal2;		// 2nd goal position, for line goal
  ArPose goalCoords;	// goal coordinate system 
  ArTransform goalTrans;	// transform to goal coord system
  bool doGoal;			// if we're trying to get somewhere
  int goalStatus;		// what we're doing

  SFEXPORT void useSonar(bool on); // turns on/off sonar obstacles
  SFEXPORT void useLRF(bool on); // turns on/off LRF obstacles
  bool lrfFlag, sonarFlag;	// use flags, for these sensors

  bool displayGrad, displayAcc; // display these guys
  bool displayGoal, displayPath;
  int gcalcCycleNum;		// how many cycles to wait between gradient calcs

  bool useStraightLine; // if we should use a straight line, for final approach

  bool pathFlag;		// true if shortcut has been found
  double pathDist;		// how far ahead we look
  SFEXPORT bool CheckCompletePath(); // returns true if there is a path from the robot to goal 

  // artifacts and maps as obstacles
  bool useArtifacts;
  bool useGridMap;

  // device
  ArRangeDevice *lr0;		// laser range finder stashed here
  // device
  ArSonarDevice *sd0;		// sonar device stashed here
  // should be private
  int pnum;			// how much to propagate

  // parameter popup window, generic pointer
  void *grad_params;

  // testing...
  void DoExpand()
    { ArPose p; p.setX(goal.getX()); p.setY(goal.getY()); setGoal(p, false); Expand(fcost, fdecay); };
  float *acc, *oacc;		// acceleration array (where we don't want to go) 

 protected:
  std::list<ArFunctor *> newGoalCBs;
  SFEXPORT virtual void AddGoal(); // adds goal, can be subclassed
  SFEXPORT void AddGoalPoint(double x, double y); // adds goal points during AddGoal call
  SFEXPORT void Process();	// UTask process
  void Reset();			// resets arrays
  SFEXPORT bool CheckCompletePathStraight(void);

 private:
  // boundaries
  double gradMaxWidth, gradMaxHeight;	// absolute max gradient grid

  // cost grid, for artifacts and map grid obstacles
  SfGrid *costGrid;

  // propagation functions
  int Propagate(int z);		// send the wavefront on z times
  void CalcCost(int i);		// calculate cost at index i 
  void Expand(double, double);	// expand the current cost frontier
  void Gradient();		// calculate gradient from cost field
  void Grad(int k);		// calculate gradient for a single point
  void SetGoalPose(SfVector &v); // sets line goal coord system
  void setGradSize(double res, SfVector &s); // reset size, only within Process()
  void setGoal(ArPose &g, bool doit = true); // sets the goal to a point, within Process()
  float *cost, *ocost;		// cost array
  float *gradx, *ogradx, *grady, *ogrady; // gradient values
  int *count;			// for histogram
  int gridsize;			// total length of grid
  ArPose pathPoint;		// dx, dy point ahead of robot on path (shortcut)
  bool firstGoal;		// for defining 2 point goals
  int gcalcCycleN;		// current calculation cycle
  SfGrid *gridMap;		// set if we want to use a map from a grid

  // wave propagation arrays
  bool *done, *odone;		// where we've finished
  int wf1[WAVEMAX];		// wavefront array 
  int wf2[WAVEMAX];		// wavefront array 
  int wf3[WAVEMAX];		// wavefront array 
  int *wp, *w, *wshunt;		// wavefront array pts 
  int w_end, wp_end, wshunt_end; // end points of arrays
  bool obcost;			// if we're doing obstacles
  float thresh;			// for smoothing wavefront values
  float inc_thresh;		// how much to increment threshold

  // sync to Process()
  SfVector syncingSetSizeValue;
  double syncingSetSizeGridRes;
  bool needToDoSyncingSetSize;
  
  ArPose syncingSetGoalGoal;
  bool syncingSetGoalDoit;
  bool needToDoSyncingSetGoal;

  // updating GUI
  void update();
};


//
// Aria action for moving robot along the gradient
//

class SfGradAction : public ArAction
{
 public:
  SFEXPORT SfGradAction(int velTop, int velMid, int velBack, int turnRadius, int canBack);	
  SFEXPORT virtual ~SfGradAction(void);
  virtual ArActionDesired *fire(ArActionDesired currentDesired);
  /// Set robot speed parameters
  SFEXPORT virtual void SetSpeed(double high, double mid, double back);
  /// Set robot acceleration parameters
  SFEXPORT virtual void SetAcc(double);	
  /// Sets the grid this action uses
  SFEXPORT virtual void SetGrid(SfGradGrid *g);
  /// When we're near the goal switch speed
  double nearGoal;		
  /// How close to be done with goal
  double doneGoal; 
  /// this returns the turn radius 
  SFEXPORT virtual int getTurnRadius(void);
  /// this sets the turn radius 
  SFEXPORT virtual void setTurnRadius(int turnRadius);
  /// Gets whether the action can back up or not
  SFEXPORT virtual int getCanBack(void);
  /// Sets whether the action can back up or not
  SFEXPORT virtual void setCanBack(int canBack);
  
  // This gets called whenever there is a new goal
  SFEXPORT virtual void newGoal(void); 
 protected:
  ArActionDesired myDesired;
  ArFunctorC<SfGradAction> newGoalCB;
  // things for the turn around
  SFEXPORT bool isRadiusClear(int radius);
  double velBack;
  double turnRadius;
  ArPose lastBacked;
  bool backing;

  // if this is 0 there is no backing, if this is 1 there is backing
  // when appropriate, and 2 there is backing all the time
  int canBack;
  // movement velocities and params
  double velSlow, velMid, velTop;
  double lowAngle, highAngle;	// low is ok, high is off (in degrees)
  SfGradGrid *grad;

  ArPose pathPoint;		// dx, dy point ahead of robot on path (shortcut)
  double pathDist;		// how far ahead we look
  bool pathFlag;		// true if path shortcut ok
  double CurveSpeed(ArPose &begin, ArPose &end); // max velocity on this curve
  double MaxSpeed(double dth);	// max speed for this error angle
  double robotAcc;		// acceleration, in mm / sec ** 2
  int safeDegrees;  // degrees in which we'll check for good degrees
  double clearDist; // the amount we want to clear where the turn radius was bad by
};

class SfGradFinalApproachAction : public ArAction
{
public:
  /// Constructor
  SFEXPORT SfGradFinalApproachAction(int doneDist, int speed);
  /// Destructor
  SFEXPORT virtual ~SfGradFinalApproachAction(void);
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired);

  /// Set robot speed parameter
  SFEXPORT virtual void SetSpeed(int speed) { mySpeed = speed; }
  /// Get robot speed parameter
  SFEXPORT virtual int GetSpeed(void) { return mySpeed; }
  /// Sets the distance to the goal at which the action is done
  SFEXPORT virtual void SetDoneDist(int doneDist) { myDoneDist = doneDist; }
  /// Gets the distance to the goal at which the action is done
  SFEXPORT virtual int GetDoneDist(void) { return myDoneDist; }
  /// Sets the grid this action uses
  SFEXPORT virtual void SetGrid(SfGradGrid *g); 
  // This gets called whenever there is a new goal
  SFEXPORT virtual void newGoal(void); 
 protected:
  ArActionDesired myDesired;
  ArFunctorC<SfGradFinalApproachAction> myNewGoalCB;
  int myDoneDist;
  int mySpeed;
  SfGradGrid *grad;
};


// Callback function for creating GUI items
SFEXPORT extern void (* sfGradFrameFn)(SfGradGrid *x, SfFr *f);
SFEXPORT extern void (* sfGradUpdateFn)(SfGradGrid *x, SfFr *f);

// External grad window
SFEXPORT extern SfWin *my_grad_win;
SFEXPORT extern SfGradGrid *my_grad;		// globals for instances
SFEXPORT extern SfGradAction *theGradAction;	
SFEXPORT extern SfGradFinalApproachAction *theGradFinalApproachAction;	

// C interface fns

#if 0
extern "C" {
  // sets up a single goal, starts robot
  SFEXPORT void sfGradSetGoal(float x, float y);
}
#endif


#endif  // SFGRAD_H
