//
// SfGrid.h
//
// grid class definitions

//
// Copyright 1999 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

#ifndef SFGRID_H
#define SFGRID_H

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif



// SfCellIndex class
// This class forms the basis for any grid class, with indexing services
//   for 2D cell arrays
//

class SfCellIndex
{
 public:
  SFEXPORT SfCellIndex();
  SFEXPORT virtual ~SfCellIndex();

  int getGridRes()              // return grid resolution (mm)
    { return (int)gridres; }
  SFEXPORT virtual bool setGridRes(int res); // set grid resolution (mm) 

  SFEXPORT bool InBounds(double, double); // return TRUE if we're in bounds of the grid

  SFEXPORT void setSize(SfVector &s); // sets the size of the grid area
  int xsize, ysize;             // number of cells in each dimension
  SfVector size;                // RW area of grid space, as rectangle
  double gridres, igridres;     // grid cell size, in mm, plus inverse; should be private

  SFEXPORT int grid_index(double, double); // returns index of RW point
  SFEXPORT bool grid_coords(double, double, int *, int *); // sets grid coords from RW

  SFEXPORT bool rw_coords(int, int, double *, double *); // sets RW coords from grid 
  SFEXPORT bool in_bounds(int, int); // returns TRUE if in bounds, else FALSE
};

// Ray class

class SFEXPORT SfRay
{
public:
  SfRay(double angle, double gridres, double len, int width); 
  // ray angle, length, resolution; width of grid in pixels
  int num;			// number of cell addresses
  int *addr;			// relative addresses for this ray
  double *dd;			// distance to this index
};

// Point array class

class SFEXPORT SfPtArray
{
public:
  SfPtArray(double angle, int n);
  void SfPtArray::Set(double *xb, double *yb, int np, double gridres, int width);
  double cs, sn;		// rotation angle of this pt array (rads)
  int num;			// number of cell addresses
  int *addr;			// relative addresses for this pt array
};


//
// Grid class
// Grids are 2D byte arrays; nominally, a value of 100 represents 
//   a probability of 1.0
//

class SfGrid : public SfDrawable, public SfCellIndex
{
public:
  SFEXPORT SfGrid();		// default constructor
  SFEXPORT virtual ~SfGrid();	// destructor
  SFEXPORT void CreateGrid(double res, SfVector *size = NULL,
			   SfArtifactList * = NULL, double margin = 0.0);
  SFEXPORT void AddSegment(SfWall *w);	// add a wall segment to the grid map
  SFEXPORT  void AddSegments(SfArtifactList *);	// add all wall segments in a list
  SFEXPORT  void EraseGrid();		// erases the grid
  SFEXPORT  void SmearGrid(double smear = -1); // does grid smearing 
  SFEXPORT  void Threshold(int thresh);	// threshold the grid
  SFEXPORT  void MoveGrid(double gx, double gy); // move grid to this lower-left position

  SFEXPORT int GridSmearDist()		// return grid smear distance (mm)
    { return (int)gridsmear; }
  SFEXPORT bool GridSmearDist(int smear); // set grid smear distance (mm) and smear it

  SFEXPORT virtual bool setGridRes(int res); // set grid resolution (mm) and recreate it
  SFEXPORT int GridMaxVal()		// return max grid value
    { return gridmax; }
  SFEXPORT bool GridMaxVal(int v);	// set max value

  SFEXPORT unsigned char GridVal(double, double); // grid value at these RW coords
  SFEXPORT void GridVal(double, double, int v); // set value at these coords
  SFEXPORT void IncGridVal(double, double, int v); // increment grid value at these coords
  SFEXPORT bool SetBounds(ArPose &p, double near, double place); // reset to in bounds

  SFEXPORT void CreateRays(int num, double len); // create ray addresses
  SFEXPORT double NearDist(double x, double y, double th); // distance to nearest wall
  SFEXPORT void CreatePtArrays(int num, int np); 
  SFEXPORT void SetPtArrays(double *xb, double *yb, int np); 
  unsigned char *grid, *ogrid;	// holds map grid

  /////////////////////////////////////////////////////////////////////////////

  bool displaySmear, displayGrid, displayArtifacts; // whether we display these

  // drawing 
  SFEXPORT void draw(SfWin *w);	// draw artifacts and smear 
  double tx, ty, tth;           // test pt array drawing

  // loading a scanstudio map file
  SFEXPORT void LoadScanMap(char *name);

protected:
  SfArtifactList *alist;        // current artifact list for grid creation
  int msize;                    // margin size, for accesses out of bounds
  double gridsmear;             // how much we smear out the map
  int gridmax;                  // max value for grid, nominally 100
  
  // ray storage
  double angleres;              // ray angle resolution, in rads
  int anglenum;
  SfRay **angles;               // ray angle buffers

  // rotated point array storage
  double rotres;                // rotation angle resolution (rads)
  int rotnum;                   // number of angles
  SfPtArray **pts;              // point arrays
};

#endif  // SFGRID_H
