/*#########################################
 * SfInterp.h
 *
 * definitions for interpreter
 *
 *#########################################
 */

/**
 ** interp.h
 **
 ** Copyright 1997 by Kurt Konolige
 **
 ** The author hereby grants to SRI permission to use this software.
 ** The author also grants to SRI permission to distribute this software
 ** to schools for non-commercial educational use only.
 **
 ** The author hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial
 ** educational use only.  This software may not be distributed to others
 ** except by SRI, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of the author.
 **
 ** Neither the author nor SRI make any representations about the 
 ** suitability of this software for any purpose.  It is provided 
 ** "as is" without express or implied warranty.
 **
 ** Kurt Konolige
 ** Senior Computer Scientist
 ** SRI International
 ** 333 Ravenswood Avenue
 ** Menlo Park, CA 94025
 ** E-mail:  konolige@ai.sri.com
 **
 **/

#ifdef __cplusplus
extern "C" {

#endif


/* atoms and expressions */

#define sfUNDEFINED 0		/* these should mimic sfINT, sfFLOAT, sfPTR */
#define sfINT       1
#define sfFLOAT     2
#define sfVOID      3
#define sfVAR       4
#define sfFUNCTION  5
#define sfEND       6
#define sfACTIVITY  7
#define sfSTRING    8
#define sfACTION    9
#define sfREF      10
#define sfDEREF    11
#define sfDOT      12
#define sfSIZEOF   13
#define sfARROW    14
#define sfFUNCPTR  15		/* function address */
#define sfUTASK    16
#define sfPTR     103		/* ptr to void */

#define MAXTYPES 100		/* max number of primitive types */
extern char *ftypes[];
#define FIRSTTYPE 20
extern int lasttype;		/* last struct type defined */

struct slentry			/* offset is where the slot is */
{
  char *name;
  int offset;
  int type;
};

struct stentry
{
  int size;			/* size of struct */
  int nslots;			/* number of slots */
  struct slentry slot1;		/* first slot */
};


/*
 * expression data structures 
 */

typedef union
{
  int n;
  float f;
  void *p;
} fvalue;

typedef struct
{
  int type;
  fvalue v;			/* value of constant */
} fconstant;

typedef struct
{
  int type;
  int index;			/* variable entry */
} fvariable;

typedef struct
{
  int type;
  fconstant *arg;		/* pointer to argument */
  fconstant r;			/* return value */
} fref;

typedef struct
{
  int type;
  fconstant *arg;		/* pointer to argument */
  char *slot;			/* slot name */
  fconstant r;			/* return value */
} fdot;

typedef struct
{
  int type;
  int stype;			/* struct/type index */
  fconstant r;			/* return value */
} fsizeof;

typedef struct
{
  int type;
  int index;			/* function entry */
  fconstant r;			/* return value */
  /* args */
  int nargs;			/* number of arguments */
  fconstant *a1;		/* pointer to argument */
  fvalue r1;			/* return value */
  fconstant *a2;
  fvalue r2;
  fconstant *a3;
  fvalue r3;
  fconstant *a4;
  fvalue r4;
  fconstant *a5;
  fvalue r5;
  fconstant *a6;
  fvalue r6;
  fconstant *a7;
  fvalue r7;
} ffunction;

typedef union
{
  fconstant c;
  fvariable v;
  ffunction f;
  fref      r;
  fsizeof   s;
  fdot      d;
} fexpr;

typedef fexpr * eptr;		/* pointer to expression */
typedef fexpr ** eptrs;		/* array of pointers to expressions */

struct exprlist
{
  eptr e;
  struct exprlist *next;
};

typedef struct exprlist * elist; /* list of fexpr pointers */

/*
 * global vars and fns
 */

int find_gvar(char *name);	/* creates if necessary */
char *name_gvar(int n);		/* return name from index */
int find_fn(char *name);	/* find index from name */
void set_gvar(int index, fvalue v); /* sets it to a new value */
void *get_fn(void *ent);	/* return function pointer from function entry */


/*
 * evaluating expressions
 */

extern char *expr_error;	/* set if error in evaluating expression */
void init_expr_fns(void);	/* setup */
void print_fexpr(fexpr *f);
void p_fexpr(fexpr *f);
char *ps_fexpr(fexpr *f);	/* returns a (non-editable) string */
char *ps_type(fexpr *f);	/* returns a (non-editable) string */
fconstant *lookup_gvar(int index); /* value from index */
fconstant *evaluate_gvar(int index); /* value from index, check local vals */
fconstant *evaluate_gvar(int index); /* value from index, make consistent */
fconstant *val_gvar(char *name); /* value from name */
char *name_gvar(int n);		/* return name from index */
fconstant *eval_fexpr(fexpr *e); /* evaluate arbitrary expression */
int eval_arg(fexpr *e, int type, fvalue *v); /* evaluate and cast to type */
int fexpr_is_zero(fexpr *e);	/* check if it's zero or not */
fexpr *make_constant(int type, fvalue v);
fexpr *make_ref(fexpr *arg, int type);
fexpr *make_variable(char *name);
fexpr *make_dot(fexpr *st, char *slot, int type);
fexpr *make_sizeof(int stype);	/* type index, returns fexpr for size */
fexpr *make_function(char *name, int num, ...);	/* takes variable number of args */
fexpr *make_fnlist(char *name, elist el); /* takes an expr list */
extern fconstant vzero;		/* the zero constant */
struct fnentry *lookup_fn(int n);
char *name_fn(int n);
void new_value(fconstant *val);	/* sets a new value into val */

/* statement data structures */

#define sGOTO       0
#define sACTION     1
#define sWAITFOR    2
#define sIF         3
#define sWHILE      4
#define sASSIGN     5
#define sFUNCTION   6
#define sSTART      7
#define sSIGNAL     8
#define sLABEL      9
#define sELSE      10
#define sDUMMY     11
#define sDECLARE   12
#define sACTIVITY  13
#define sTRACE     14

typedef struct			/* just for inserting into other structs */
{
  int type;
  int level;
  int line;			/* text line number */
  int next;			/* where we go next */
} sDummy;

typedef struct			/* Goto */
{
  int type;
  int level;
  int line;			/* text line number */
  int pc;			/* where we go */
  int index;			/* label index */
} sGoto;

typedef struct			/* Label */
{
  int type;
  int level;			/* always 0 */
  int line;			/* text line number */
  int next;			/* should always be next statement */
  int index;
} sLabel;

#define aMOVE    1		/* action types, no type 0 */
#define aHEAD    2
#define aDHEAD   3
#define aVEL     4
#define aROTATE  5
#define aSTOP    6
#define aDCHEAD  7
#define aACTIVITY 8		/* dummy, says we're doing an activity */
#define aWAITFOR 9		/* monitoring a waitfor */
#define aBEHAVIORS 10		/* turn on behavioral actions */

extern char *atypes[];

typedef struct
{
  int type;
  int level;
  int line;			/* text line number */
  int next;
  int acttype;			/* type of action */
  eptr arg;			/* argument is an expression */
  int noblock;			/* blocking or not */
  int timeout;			/* in ms */
} sAction;

typedef struct
{
  int type;
  int level;
  int line;			/* text line number */
  int pctrue;			/* where to go if true */
  int pcfalse;			/* and if false */
  fexpr *cond;			/* condition expression */
} sIf;

typedef struct			/* nothing here... */
{
  int type;
  int level;
  int line;			/* text line number */
  int next;			/* should be next statement */
} sElse;

typedef struct
{
  int type;
  int level;
  int line;			/* text line number */
  int pctrue;			/* where to go if true */
  int pcfalse;			/* and if false */
  fexpr *cond;			/* condition expression */
} sWhile;

typedef struct
{
  int type;
  int level;
  int line;			/* text line number */
  fexpr *cond;			/* condition expression */
  int timeout;			/* max time allowed */
} sWaitfor;

typedef struct
{
  int type;
  int level;
  int line;			/* text line number */
  int next;
  int sym;			/* 1 if symbol assignment, else 0 */
  fexpr *lvalue;		/* what we're assigning */
  fexpr *value;			/* value to assign */
} sAssign;

typedef struct
{
  int type;
  int level;
  int line;			/* text line number */
  int next;
  fexpr *fn;			/* function to evaluate */
} sFunction;

typedef struct			/* declaration */
{
  int type;
  int level;			/* always 0 */
  int line;			/* text line number */
  int next;			/* should always be next statement */
  char *name;			/* variable name */
  int vtype;			/* type */
  int index;			/* global index */
  fconstant val;		/* value for local vars */
} sDeclare;

typedef struct			/* activity call */
{
  int type;
  int level;
  int line;			/* text line number */
  int next;			/* should always be next statement */
  char *schema;			/* activity schema name */
  char *instance;		/* activity instance name */
  void *proc;			/* uTask instance */
  int timeout;			/* if there's a timeout */
  int noblock;			/* do we block? */
  int suspend;			/* start it suspended */
  int priority;			/* the behavior priority */
  int nargs;			/* number of arguments */
  eptr arg1;			/* first argument; structure extended to include more */
} sActivity;

typedef struct			/* activity termination */
{
  int type;
  int level;
  int line;			/* text line number */
  int next;			/* should always be next statement */
  int signal;			/* what state we're signalling */
  char *instance;		/* activity instance name */
  int time;			/* amount of time to suspend */
} sSignal;

typedef struct			/* activity termination */
{
  int type;
  int level;
  int line;			/* text line number */
  int next;			/* should always be next statement */
  int trace;			/* tracing state */
  char *instance;		/* activity instance name */
} sTrace;


typedef union
{
  sDummy  dummy;
  sGoto   go;
  sLabel  label;
  sAction action;
  sIf     ifs;
  sElse   elses;
  sWhile  whiles;
  sAssign assign;
  sFunction function;
  sDeclare declare;
  sActivity activity;
  sSignal signal;
  sTrace  trace;
  sWaitfor waitfor;
} statement;

typedef statement * sptr;
typedef sptr * sptrs;

statement *make_goto(int level, int line, char *label, int pc); /* Goto statement instance */
statement *make_label(int line, char *label); /* label statement instance */
statement *make_nop(int level, int line, int pc); /* NOP statement instance */
statement *make_assign(int level, int line, eptr el, eptr e, int sym); /* assignment statement */
statement *make_declare(int level, int line, char *name, int vtype); /* declaration */
statement *make_fnstate(int level, int line, eptr e); /* function statement */
statement *make_action(int level, int line, int atype, eptr arg, int noblock, int timeout);
statement *make_if(int level, int line, fexpr *cond, int pctrue, int pcfalse);
statement *make_else(int level, int line, int pc);
statement *make_while(int level, int line, fexpr *cond, int pctrue, int pcfalse);
statement *make_activity(int level, int line, char *schema, char *instance, int timeout, 
			 int noblock, int priority, int nargs, eptr *args, int suspend);
statement *make_signal(int level, int line, char *instance, int sig, int trace, int time);
statement *make_waitfor(int level, int line, fexpr *cond, int timeout);

/*
 * Activity closure definition
 */

typedef struct
{
  int length;			/* number of lines */
  int pc;			/* where we are */
  int monitor;			/* if we're monitoring */
  int timer;			/* timeout for actions */
  //  sfprocess *child;		/* if waiting for a child process */
  void *child;
  sptr *body;			/* body statements */
  int slength;			/* setup length */
  sptr *setup;			/* setup statements */
  int plength;			/* total param length, includes local declares */
  int flength;			/* formal parameter length */
  sptr *params;			/* param statements */
  int rtype;			/* return type, if interpreted as a function */
  int resume;			/* resumption state */
  int interrupt;		/* interrupt state */
} interpact;

int lineof(interpact *ia);	/* returns src line number of ia */
statement *get_current_statement(interpact *ia);
void print_statement(sptr s);
SFEXPORT void sfPrintAct(char *name);

/*
 * prototypes of command dispatchers
 */

int do_one_interp(interpact *ia, int pc, char *act); /* single act step */
void do_setup_interp(interpact *ia, char *act); /* setup forms */
int do_action(sptr s);		/* primitive action */
int do_assign(sptr s);		/* do the assignment */
int do_signal(sptr s);		/* and signal it */
int do_trace(sptr s);		/* and trace it */
void do_dir(char *dir);		/* changes or returns directory */
int do_eval(eptr e);
int do_help(char *s);		/* prints help strings */
char *get_hsub(int *i, char *s); /* returns help topics containing s */
struct slentry *lookup_slot(int type, char *name);


typedef enum			/* process closure state */
{ sfINIT = 0, sfSUSPEND, sfRESUME, sfSUCCESS, sfFAILURE, sfTIMEOUT, 
  sfREMOVE, sfINTERRUPT, sfINACTIVE }
pstate;

extern char *pstates[];

// structures for parameters to activities

typedef struct			/* a param can be either a fp number */
{				/* or a pointer */
  float f;			/* or an integer */
  int i;
  void * p;
  int x;			/* one of sfINT, sfFLOAT, or sfPTR */
} param;

typedef param * beh_params;

extern param *
beh_alloc(int n);

// extern definitions

int lookup_stype(char *stype);	/* checks for type string */
void sout(char *s, ...);
void p_fexpr1(eptr e, int parens);
int get_fexpr_int(fexpr *e);

// evaluation

SFEXPORT void sfLoadActivityFile(char *name); /* loads an evaluator file */
SFEXPORT void sfLoadLibraryFile(char *name); /* loads an evaluator file */

/* All of these load aria ones return an entry in the enum
 ArModuleLoader::Status... 0 is success, anything else is some form of
 failure */
SFEXPORT int sfLoadAriaLibraryFile(char *name); /* loads an Aria library file */
SFEXPORT int sfLoadAriaLibraryFileWithString(char *name, char *argument); /* loads an Aria library file with passing in a string argument */
SFEXPORT int  sfLoadAriaLibraryFileWithVoid(char *name, void *argument); /* loads an Aria library file with passing in a void * argument */
SFEXPORT int sfUnloadAriaLibraryFile(char *name); /* unloads an Aria Library File */

// some helpful types
SFEXPORT int sfTypeRef(int type);/* creates a type which is pointer to type */
SFEXPORT int sfTypeDeref(int type); /* creates a type which is the reference of type */


#ifdef __cplusplus
}
#endif

// make new one if not there, else use old one 
SFEXPORT int sfAddEvalFn(char *name, void *fn, int rtype, int nargs, ...); 
// make new one if not there, else use old one 
SFEXPORT int sfAddEvalConst(char *name, int type, ...);
// make new one if not there, else use old one 
SFEXPORT int sfAddEvalVar(char *name, int type, void *v);
// make new one if not there, else use old one 
SFEXPORT void sfAddEvalAction(char *name, void *fn, int nargs, ...); 
// make new one if not there, else use old one 
SFEXPORT int sfAddEvalStruct(char *name, int size, char *ex, int numslots, ...);
// add help strings
SFEXPORT void sfAddHelp(char *name, char *str); /* adds str as help for name */
SFEXPORT char *sfGetHelp(char *name); /* returns help string for name, NULL if none */

// Turn exception catching on or off
SFEXPORT extern int sfExceptionFlag;

// Initialization on load; these have to be C defs for dlsym
#ifdef __cplusplus 
extern "C" {
#endif
REALLYEXPORT void sfLoadInit(void);
REALLYEXPORT void sfLoadExit(void);
#ifdef __cplusplus 
}
#endif
