//
// SfLps.h
//
// object class definitions for the Local Perceptual Space

//
// Copyright 1999 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

//
// Artifact abstract class
//

#ifndef SFLPS_H
#define SFLPS_H

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

/// SfArtifact objects are Local Perceptual Space objects
/// Inherit from this class if you want to be drawn on the LPS graphics window
/**
    The SfArtifact class is the standard way to draw objects on the LPS graphcis
    window.  Inheriting from this class lets a subclass define the draw(SfWin *)
    function, which is called every time the graphics window is refreshed.
    The SfArtifact class adds its objects to the artifact list, and removes them
    on destruction.  
    To turn off drawing of an artifact, use the visible flag (inherited from the
    SfDrawable class).
    @see SfWin
    @see SfArtifactList
*/
class SfArtifact : public SfDrawable // abstract class
{
 public:
  /// Current pose of the object
  ArPose p;			// where the point is

  typedef enum			// what we are
    { 
      Robot, ///< Robot object
      Point, ///< Point object, draws as a circle
      Wall,  ///< Wall object, draws as a line
      Wallset, ///< Wall set object, not currently used
      Corridor, ///< Corridor object, draws as a parallel pair of double lines
      Goal ///< Goal object, draws as a circle for now
    } Type;
  Type type;
  
  int id;			// ID for general access
  int newflags;			// new flags, for routines that need to know
  /// This function is overridden by the artifact subclass to draw a particular artifact
  void draw(SfWin *w) {}	// by default we don't draw
  void print() {}
				// move incrementally using a pose
  void moveObj(ArPose *q) { p.setX(q->getX() + p.getX()); p.setY(q->getY() + p.getY()); 
                         p.setTh(p.getTh() + q->getTh()); 
                         newflags = -1; }
				// move incrementally using three numbers
  void moveObj(double x, double y, double th) 
    { p.setX(p.getX()+x); p.setY(p.getY()+y); 
      p.setTh(p.getTh() + th); newflags = -1; }
  /// Constructor, adds the artifact object to the artifact list
  SFEXPORT SfArtifact();	// constructor, should add to arts
  /// Destructor, removes the artifact from the artifact list
  SFEXPORT virtual ~SfArtifact(); // deconstructor, removes from list
 private:
 static int newid;
};

//
// Artifacts of different types
//


/// SfRobot inherits the basic ArRobot, plus is an artifact
/// so it can be drawn.  
//  has size, should add color
//
class SfRobot : public ArRobot, public SfArtifact
{
 public:
  SFEXPORT SfRobot();		// constructor with no arguments 
  virtual ~SfRobot() {}

  void print() {}
  SFEXPORT void draw(SfWin *w);	// draws the robot

};


/// Point artifact, which is a position and direction
/// Draws as a circle
class SfPoint : public SfArtifact
{
 public:
  /// Constructor using all parameters
  SfPoint(double x, double y, double th)  // constructor
    { p.setX(x); p.setY(y); p.setTh(th);  type = Point; }
  /// Constructor with default 0,0,0
  SfPoint()			// constructor with no arguments
    { p.setX(0); p.setY(0); p.setTh(0);  type = Point; }
  void print() {}
  SFEXPORT void draw(SfWin *w);		// draws a circle
  /// Destructor
  virtual ~SfPoint() {}
};

/// Goal artifact, which is a pose and a name
/// Draws as a circle
class SfGoal : public SfArtifact
{
 public:
  /// Constructor using all parameters
  SfGoal(double x, double y, double th, const char *name, bool useHeading)  // constructor
    { p.setX(x); p.setY(y); p.setTh(th);  type = Goal; myName = name; myUseHeading = useHeading; }
  /// Constructor with default 0,0,0
  SfGoal()			// constructor with no arguments
    { p.setX(0); p.setY(0); p.setTh(0);  type = Goal; myUseHeading = false; }
  void print() {}
  ArPose getPose(void) { return p; }
  bool useHeading(void) { return myUseHeading; }
  const char *getName(void) { return myName.c_str(); };
  SFEXPORT void draw(SfWin *w);		// draws a circle
  /// Destructor
  virtual ~SfGoal() {}
private:
  std::string myName;
  bool myUseHeading;
};


/// Corridor artifacts: center point, width and length
class SfCorridor : public SfArtifact
{
 public:
  /// Constructor using all parameters
  SfCorridor(double x, double y, double th, double w, double l) // constructor
    { p.setX(x); p.setY(y); p.setTh(th); width = w; length = l; type = Corridor; }
  /// Constructor using default of zero
  SfCorridor()			// constructor with no arguments
    { p.setX(0); p.setY(0); p.setTh(0); width = 0; length = 0; type = Corridor; }
  /// Destructor
  virtual ~SfCorridor() {}
  /// Width and length of the corridor, can be reset 
  double width, length;		// width and length of corridor
  void print() {}
  SFEXPORT void draw(SfWin *w);
};



/// Wall artifacts: center point, length 
class SfWall : public SfArtifact
{
 public:
  /// Constructor, using endpoint arguments
  SfWall(double x1, double y1, double x2, double y2) // constructor using vector
    { 
      v.x1 = x1; v.x2 = x2; v.y1 = y1; v.y2 = y2;
      length = makeWall(&p,x1,y1,x2,y2);
      type = Wall;
    }
  /// Constructor, using defaults of zero
  SfWall()			// constructor with no arguments
    { 
      v.x1 = v.x2 = v.y1 = v.y2 = 0;
      length = makeWall(&p,0,0,0,0);
      type = Wall;
    }
  /// Destructor
  virtual ~SfWall() {}
  /// Length of wall, should not be reset
  double length;		// length of wall
  /// Vector representation, should not be reset
  SfVector v;			// vector representation
  void print() { }
  SFEXPORT void draw(SfWin *w); 

 private:
  double makeWall(ArPose *p, double x1, double y1, double x2, double y2);
};


//
// Wall set: point is moment center, maybe?
//

class SfWallset : public SfArtifact
{
 public:
  SfVector *walls;		// wall vectors
  int n;			// length
  virtual ~SfWallset () {}
  SfWallset(double x, double y, double th, int n) // constructor
    { p.setX(x); p.setY(y); p.setTh(th); walls = new SfVector[n]; type = Wallset; }
  SfWallset(int n)			// constructor with no arguments
    { p.setX(0); p.setY(0); p.setTh(0); walls = new SfVector[n]; type = Wallset; }
  void print() {}
  void draw(SfWin *w) 
    { 
    }
};


//
// Artifact lists
// First one created is default, but can be reset
//

/// This macro retrieves the artifact list
#define SfARTIFACTS SfArtifactList::current()


/// SfArtifactList is a static class that holds the set of current artifacts.
/**
    All artifacts objects, when created, put themselves onto the global artifact
    list; and they remove themselves when destroyed.  User programs should not
    explicitly add or delete artifacts from this list.  To stop an artifact from
    drawing, using the visible flag.
    User programs can request several facts about the artifact list, including its
    current bounds, which is set when a world is read in.  They can also request
    the artifact list itself, to cycle through the artifacts.
*/
class SfArtifactList : public SfDrawable
{
 public:
  /// Artifact list
  SFEXPORT static SfArtifactList *current(); // returns default list
  SFEXPORT static void current(SfArtifactList *a); // sets default list
  SFEXPORT SfArtifactList();// constructor
  SFEXPORT virtual  ~SfArtifactList() {};	// destructor
  SFEXPORT void draw(SfWin *w);		// drawing fn
  void add(SfArtifact *a)	// add onto list
    { arts->push_front(a); }
  void remove(SfArtifact *a)	// remove from list
    { arts->remove(a); }
  SFEXPORT void RemoveWalls();  // remove all walls, i.e., remove a world
  SFEXPORT void SetOrigin(double x, double y); // keep the same width, height
  void SetWidth(double w)
    { bounds.x2 = bounds.x1+w; }
  void SetHeight(double h)
  { bounds.y2 = bounds.y1+h; }
  /// Returns the bounds of the artifacts in the artifact list
  /// The bounds are set from the most recent world file read in
  SfVector &Bounds(void)
    { return bounds; }
  std::list<SfArtifact *> *Artifacts() // returns artifact list
    { return arts; }

 private:
  static SFEXPORT SfArtifactList *_current; // pointer to current default
  std::list<SfArtifact *> *arts;	// list of artifacts
  SfVector bounds;		// bounding box of artifacts
};


/// Loads wall artifacts from a .wld file, returns the robot pose
/// set by the file, if any
SFEXPORT ArPose *sfLoadWorldFile(char *name);  
/// Removes all wall artifacts
SFEXPORT void sfRemoveWorld();	

//
// Geometry classes, useful for working with artifacts
//

// general coordinate system transform
// can be set by two points or an origin and angle
// transforms between global coordinate system and itself
// not sure what this is going to be used for...
//

class SfVectorSystem : public SfVector, public ArPose

{
 public:
  SFEXPORT SfVectorSystem(SfVector &v) { Set(v); };
  SFEXPORT SfVectorSystem(ArPose &p) { Set(p); };
  SFEXPORT void Set(SfVector &v);
  SFEXPORT void Set(ArPose &p);
  void ToLocal(double &x, double &y) 
    { x-=x1; y-=y1; double temp = x; x = cs*x + sn*y; y = cs*y - sn*temp;};
  void ToGlobal(double &x, double &y)
    { double temp = x; x = cs*x - sn*y; y = cs*y + sn*temp; x+=x1; y+=y1;};
 private:
  double sn, cs;
};


#endif // SFLPS_H
