//
// SfMonte.h
//
// monte carlo localization class definitions

//
// Copyright 1999 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

//
// Robot pose class, with probabilities
//

#ifndef SFMONTE_H
#define SFMONTE_H

#include "export.h"

/**
   \defgroup loc Localization Module

   This module is used localization.  By itself it can only be used
   with the sonar to localize in a vector (line) map.  With the Laser
   Localization/Navigation module it can be used with the laser in a
   grid map generated by a robot.

   You can look at/load flloc.act to see how to use this module.
**/
/*@{*/


/// Initializes the localization
SFEXPORT void mcSonarInit(void);
/// Initiailizes the localization using a particular grid size
SFEXPORT void mcSonarInitRes(int res); 
/// Gets an mc object (mostly for use with sfGradSetMap)
SFEXPORT void *mcGetObject(void);
/// Sets the delta angle (da) delta distance (ds) or delta time (tm)
/// before loc fires again
SFEXPORT void mcSetMove(int da, int ds, int tm);

/// call this with true (non-zero) to make the localization update the
/// robot position
SFEXPORT void mcUpdateRobotPose(int on);

/// If true this will print during localization updates, false it won't
SFEXPORT void mcPrintDuringUpdates(int on);

/// Sets the gain of the sensor information in the update step
SFEXPORT void mcSetGain(int pct);

/// Centers the sample distribution on the center of the robot
SFEXPORT void mcSetGauss(float dx, float dth);

/// Sets the number of samples
SFEXPORT void mcSetNumSamples(int n);

/*@}*/

class SfPoseP : public ArPose
{
 public:
  double prob;			// how certain we are
};

// Error parameter class
//   We're using systematic error parameters, because the robot
//   seems to adhere to these statistics

class SfErrParams		// error parameters of the robot
{
 public:
  double range;			// range error, m/m
  double angle;			// angle error, rads/rad
  double drift;			// drift error, rads/m
  SFEXPORT SfErrParams(double r, double a, double d)
    { range = r; angle = a; drift = d; }
  SFEXPORT SfErrParams()
    { range = 0; angle = 0; drift = 0; }
};


//
// Sample class, for manipulating a set of localization samples
//

class SfUpdateSampleTask;		// forward ref, update async process

class SfSamples : public SfGrid
{
 public:
  SfPoseP *Samples;             // array of poses
  SFEXPORT SfSamples(int, SfVector &); // constructor
  ~SfSamples() {};              // destructor
  SFEXPORT void AddToFrame(SfFr *f); // add GUI stuff
  SFEXPORT virtual void draw(SfWin *w); // drawing fn

  int ns;                       // current number of samples, should be private
  int ns_max;                   // max number of samples, ditto
  SFEXPORT void SetNum(int n);	// number of samples
  int GetNum() { return ns; } // what we're using
  int CurrentSamples() { return ns; } // what we're using

  SFEXPORT void SetUniform(double x, double y, double dx, double dy); 
  // initialize samples to uniform
  SFEXPORT void SetUniform();	// use world size
  SFEXPORT void SetGauss(double mx, double my, double mth, // pos (mm, deg), gauss ellipse
                         double th, double axis1, double axis2, double sth);
  SFEXPORT void SmearSamples(double xydev, double thdev); 
  // smears uniformly; in mm and deg
  SFEXPORT void Move(double dr, double da, double dir); // move by dead reckoning
  SFEXPORT void MoveError(SfErrParams &e) // set error parameters
    { err = e; }
  SFEXPORT void Update(bool init); // update pose samples 
  SFEXPORT void Resample();     // resample that puppy 
  SFEXPORT void Best(int n = 20); // find mean pose (approximate, using n samples)
  ArPose *best;                 // what the current best pose is 
  //  SfPoseCovar *bestvar;		// best position variance


  SFEXPORT virtual bool Gain(int g); // sets the gain value, as a percent
  SFEXPORT virtual bool SetGain(int g){ return Gain(g); }; // sets the gain value, as a percent
  int Gain()                    // returns gain as a percent
    { return (int)(100.01*gain); }
  int GetGain()                    // returns gain as a percent
    { return (int)(100.01*gain); }

  bool MaxProb(int g)           // sets max prob value for adaptive sampling, percent
    { if (g >= 0 && g <= 100)
        { maxprob = .01*(double)g; return true; }
      else return false;
      updateGUI();
    }
  bool SetMaxProb(int g) { return MaxProb(g); }

  int MaxProb()                 // returns maxprob as a percent
    { return (int)(100.01*maxprob); }
  int GetMaxProb()
    { return (int)(100.01*maxprob); }

  bool MinProb(int g)           // sets min prob value for adaptive sampling, percent
    { if (g >= 0 && g <= 100)
        { minprob = .01*(double)g; return true; }
      else return false;
    updateGUI();
    }
  bool SetMinProb(int g) { return MinProb(g); }

  int MinProb()                 // returns maxprob as a percent
    { return (int)(100.01*minprob); }
  int GetMinProb()
    { return (int)(100.01*minprob); }


  int MaxSamples()              // return max number of samples allowed
    { return (int)maxsamples; }
  int GetMaxSamples()           // return max number of samples allowed
    { return (int)maxsamples; }

  bool MaxSamples(int n)        // set max samples, less than ns_max
    { if (n >= minsamples)
        { if (n > ns_max) n = ns_max; maxsamples = n; return true; }
      else return false;
      updateGUI();
    }
  bool GetMaxSamples(int n) { return MaxSamples(n); }	// set max samples, less than ns_max

  int MinSamples()              // return min number of samples allowed
    { return (int)minsamples; }
  int GetMinSamples()           // return min number of samples allowed
    { return (int)minsamples; }

  bool MinSamples(int n)        // set min samples
    { if (n >= 100 && n <= maxsamples)
        { minsamples = n; return true; }
      else return false;
      updateGUI();
    }
  bool SetMinSamples(int n) { return MinSamples(n); } // set min samples

  int ProbCalcCycles()          // return number of cycles to calc 
    { return (prob_calc_cycles); }
  int GetProbCalcCycles()       // return number of cycles to calc 
    { return (prob_calc_cycles); }

  void ProbCalcCycles(int n)    // set number of cycles to calc
    { prob_calc_cycles = n; }
  void SetProbCalcCycles(int n) { ProbCalcCycles(n); } // set number of cycles to calc

  // Functions to be subsumed by subclasses
  SFEXPORT virtual void PostProbInit() {} // initialize post prob calculation
  SFEXPORT virtual void PostProb(SfPoseP *, int n) {} // returns posterior prob over poses
  SFEXPORT virtual void DoMapUpdate() {}  // for map building

  SFEXPORT int NumLocalizations(void) { return numLocalizations; } // return number of times we've localized

  // Display flags -- should go through fns
  bool displaySamples;          // sample pt display flag
  bool displayBest;             // best pose display flag
  void *monte_params;           // for parameters pop-up window
  void DisplaySamples(bool on)
    { displaySamples = on; updateGUI(); }
  void DisplayBest(bool on)
    { displayBest = on; updateGUI(); }
  void DisplayGrid(bool on)
    { displayGrid = on; updateGUI(); }

  // Localization params
  bool updateFlag;              // whether we're localizing or not
  bool updatePoseFlag;          // whether we update robot pose after localize step
  void UpdateFlag(bool on)
    { updateFlag = on; updateGUI(); }
  void UpdatePoseFlag(bool on)
    { updatePoseFlag = on; updateGUI(); }

  bool printDuringUpdates;      // whether we want to print out on the updates or not
  bool updateMapFlag;		// whether we do map-building, LRF only...
  void UpdateMapFlag(bool on)
    { updateMapFlag = on; }
  int move_ds, move_da, move_time; // how far we need to move, in mm and deg; timeout after moving
  void SetMoveDs(int x)
    { move_ds = x; updateGUI(); }
  void SetMoveDa(int x)
    { move_da = x; updateGUI(); }
  void SetMoveTime(int x)
    { move_time = x; updateGUI(); }

  int new_update;		// how many updates we've done
  ArPose rpos;			// keep track of where we started
  SFEXPORT void InitUpdate();	// set up parameters
  SFEXPORT void Globalize();	// initializes globalization process
  SFEXPORT void ResetAsyncTask();
  SFEXPORT void KillAsyncTask();

  // update GUI
  SFEXPORT void updateGUI();


 private:
  SfErrParams err;		// movement error params
  SfPoseP *resampled;		// for resampling
  double *probs;		// cumulative probs for resampling
  SfTime *tt;			// timer
  bool reset_flag;		// if we want to reset samples and re-globalize
  int  prob_calc_cycles;	// how many Saphira cycles for async calc, 0 for inf
  int state;

 protected:
  double gain;			// how much we trust the sensor

  int numLocalizations;         // how may times we've localized
  // async break
  bool stop_prob_calc;		// if we want to stop the async prob calculation

  // adaptive sampling
  int minsamples, maxsamples;	// bounds for number of samples
  double minprob, maxprob;	// bounds for deleting or adding samples

  // localization processing
  void Process();		// we check robot position, synchronize with
  // localization processing in a thread
  ArRecurrentTask *utask;	// asynchronous update process
  double move_d, move_dth, move_a; // movement values
};

// Set this function to initialize GUI parameters, on instantiation
extern SFEXPORT void (* sfSampleFrameFn)(SfSamples *x, SfFr *f);
extern SFEXPORT void (* sfSampleUpdateFn)(SfSamples *x, SfFr *f);

//
// SfUpdateSampleTask -- subclass of SfAsyncTask
//

class SfUpdateSampleTask : public ArRecurrentTask
{
 public:
  SFEXPORT SfUpdateSampleTask(SfSamples *s); // sample instance 
  SFEXPORT void task();		// async updating task
 private:
  SfSamples *samples;
};


//
// Subclass SfSonarSamples -- uses sonar incidence function for
//   probability calculation
//

#define MONTENUMSONARS 32

class SfSonarSamples : public SfSamples
{
 public:
  SFEXPORT SfSonarSamples(int num, SfVector &s); // number of samples, size of grid
  SFEXPORT void PostProbInit();		// set up LRF readings
  SFEXPORT void PostProb(SfPoseP *p, int n); // posterior prob calculation
  SFEXPORT bool Gain(int g);	// have to do this to recalc PSD table

 private:
  //  SfSensorReading *sensor;	// which sensor we're using
  int sonarRange[MONTENUMSONARS];
  int start_sonar;              // which sonar range we use for update
};

SFEXPORT extern SfSamples *montePs;
#define MONTERANGE .02          // range error, m/m
#define MONTEANGLE .05          // angle error, rads/rad
#define MONTEDRIFT .00001		// drift error, rads/mm
#define MONTENUM_SAMPLES 2000


#endif // SFMONTE_H
