//
// SfStream.h
//
// Colbert stream class definitions

//
// Copyright 2000 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

//
// Classes and functions for manipulating Colbert input/output streams

#ifndef SFSTREAM_H
#define SFSTREAM_H


#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

// struct stream_entry is used to hold {string, stream index} pairs

struct stream_entry
{ 
  char * str;
  int index;
};
  
//
// SfColbertStream is the underlying class for sending input to Colbert,
//   and displaying output.  Should be subclassed by a window or console
//   for displaying and receiving characters.
//
// Colbert streams are kept in an array of max COLBERT_STREAM_MAX entries, 
//   similar to Unix FILE streams.  Streams are referred to by their entry
//   index.  Writing via sfMessage is to a specified stream, or the default
//   stream, which can be specified.
//
// Each stream keeps its own history list of input.
//
// Stream input is put on a global circular buffer for consumption
//   by the Colbert parsing process.
//
// Haven't figured out issues of multithreading: AddInput and
//    GetNextColbertString should have protected access.
//

// number of open Colbert streams
#define COLBERT_STREAM_MAX 20
// number of strings in global buffer, to be parsed
#define COLBERT_PARSE_BUFFER_MAX 100 
// number of chars allowed per (continued) Colbert line
#define COLBERT_STRING_MAX 1000

class SfColbertStream
{
public:
  SFEXPORT SfColbertStream(int n); // create one with history list size n 
  SFEXPORT virtual ~SfColbertStream();
  SFEXPORT int StreamIndex() { return index; } // gets index of stream
  SFEXPORT static SfColbertStream *GetStream(int i);  // returns stream of index
  SFEXPORT static SfColbertStream *GetDefaultStream();  // returns default stream
  SFEXPORT static void DeleteStream(int i); // deletes stream with this index
  SFEXPORT static int SetDefaultStream(int i); // sets the default stream to this index
  SFEXPORT static char *GetNextColbertString(int *i); // for the parser
  SFEXPORT void AddInput(const char *str); // copies str; if ends with "\", 
				     // concatenates with next line.
  SFEXPORT virtual void AddOutput(const char *str); // saves output, this must be overridden
  SFEXPORT virtual void AddOutputFlush(); // writes all output, must be overridden
  SFEXPORT char *GetHistory(int n); // n is +1 or -1
  // printing on a stream
  SFEXPORT static void sfFPrintf(int i, char *, ...);
  SFEXPORT static void sfFPrint(int i, char *);
  SFEXPORT static void sfPrintf(char *, ...);

private:
  // input strings, in a circular buffer
  char **history_list;		
  int hfirst, hlast, hcur, hsize;
  char continuation[COLBERT_STRING_MAX];  // storage for a continued input line
  int csize;                // where we are in the continuation buffer

  // stream repository  
  static SfColbertStream *streams[COLBERT_STREAM_MAX]; 
  static int default_stream;	// default to this one
  int index;			// where this one is in streams buffer

  // parse strings, in a circular buffer
  static struct stream_entry parse_list[COLBERT_PARSE_BUFFER_MAX]; 
  static int pfirst, plast;
};

/* Window messages */

SFEXPORT char *sfGetColbertString(int *i);
SFEXPORT int  sfSetDefaultOutput(int i);
SFEXPORT void sfMessage(char *s, ...);

#endif // SFSTREAM_H
