//
// SfSystem.h
//
// Saphira system definitions

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com


#ifndef SFSYSTEM_H
#define SFSYSTEM_H

#undef SFEXPORT
#if defined(MS_WINDOWS) && !defined(SF_STATIC)
#ifdef MAKE_LIBRARY
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif


/// Saphira system definition
/**
    The Saphira system class is a static class that holds basic information
    about the single robot server for which Saphira is the client.  It has
    useful functions to get the robot object, device buffers, artifact list,
    connections to the robot server, and so on.
   
    On startup of the standard Saphira client, all the items in the Sf class
    are initialized by calling init(). 
*/

class Sf
{
  Sf() {}; // no constructor!

public:


  /// initializes Saphira system, including Aria; called by the standard Saphira client 
  /// on startup
  static SFEXPORT void init();	
  /// Current robot object; Saphira 8.0 has only one
  /// User programs can access this with the SfROBOT macro
  static SfRobot *robot() { return ourRobot; }; 
  /// sonar device object
  static SfSonarDevice *sonar() { return ourSonar; }
  /// laser device object
  static SfLaserDevice *laser() { return ourLaser; }
  /// IR rangefinder device object
  static SfIrrfDevice *irrf() { return ourIrrf; }
  /// artifact list
  static SFEXPORT SfArtifactList *artList;     // artifact list
  /// main Colbert stream for reading/writing Colbert commands
  static SFEXPORT SfColbertStream *ourColbert; // console Colbert stream

  // values for the current robot

  /// Gets the robot current X value, in mm
  static double getX() { return ourRobot->getX(); }; // in mm
  /// Gets the robot current Y value, in mm
  static double getY() { return ourRobot->getY(); }; // in mm;
  /// Gets the robot current heading, in degrees
  static double getTh() { return ourRobot->getTh(); }; // in deg
  /// Gets the robot current pose object
  static ArPose getRwPose() { return ourRobot->getPose(); }; // robot's pose

  // connections
  static SFEXPORT ArTcpConnection *tcpCon; 
  static SFEXPORT ArSerialConnection *serialCon; 
  static SFEXPORT ArLogFileConnection *logCon; 

  /// display frame; user programs can acces this with the SfFRAME macro
  static SFEXPORT SfFr *frame;
  static SFEXPORT bool isExited;		// true when we have an exit request

  // internal, use the accessor... laser device, currently just a single one
  static SFEXPORT SfLaserDevice *ourLaser;
  // internal, use the accessor... IR rangefinder device, currently just a single one
  static SFEXPORT SfIrrfDevice *ourIrrf;
  // internal, use the accessor, the current robot
  static SFEXPORT SfRobot *ourRobot;

private:
  // sonar device, most robots have this...
  static SFEXPORT SfSonarDevice *ourSonar;


  // parsing stream handler

  static void parser();
};

//
// Global macros
//

/// Standard way to access the current robot
#define SfROBOT Sf::robot()

#define SfSELFTASK  SfActRegister::currentTask


//
// Helper macro for User and Sensor tasks
//

// use these in a constructor or any other member fn for the object
// NAME is a string, double quotes

#define ADDUSERTASK(NAME,PRIORITY,CLASS,PROC) \
  Sf::robot()->addUserTask(NAME, PRIORITY,   \
   new ArFunctorC<CLASS>(this, &CLASS::PROC));

#define ADDSENSORTASK(NAME,PRIORITY,CLASS,PROC) \
  Sf::robot()->addSensorInterpTask(NAME, PRIORITY,   \
   new ArFunctorC<CLASS>(this, &CLASS::PROC));


//
// Various connection globals
//
SFEXPORT void sfSetSerialBaud(int rate);
SFEXPORT void sfSetSerialPort(char *dev);


#endif // SFSYSTEM_H

