//
// flframe.h
//
// Saphira widget classes, e.g., frames
//

//
// Copyright 2000 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

//
// Classes and functions for manipulating Colbert input/output streams

#ifndef FLFRAME_H
#define FLFRAME_H

#ifdef MS_WINDOWS
#ifdef SFEXPORT
#undef SFEXPORT
#endif
#ifdef FLTKWIN_EXPORTS
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

// "index" is a macro in some Unix header...
#ifdef index
#undef index
#endif

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Tile.H>
#include <FL/Fl_Light_Button.H>

//
// Draws a single string, int, or double value, with an associated label
// Can change formatting and type at any time; value() fn is overloaded
// value(char *) acts as formatter for numeric values
// If formatter is changed, the numeric value must be resent
//

class Fl_Value : public Fl_Widget 
{

public:
  SFEXPORT Fl_Value(int x, int y, int w, int h, const char *l=0, const char *val = "");
  SFEXPORT void value(char *v) { if (v) { strcpy(str,v); strcpy(buf,v); } redraw(); }
  SFEXPORT void value(int x) { sprintf(buf, str, x); redraw(); }
  SFEXPORT void value(double x) { sprintf(buf, str, x); redraw(); }
  SFEXPORT void setcolor(Fl_Color x) { c = x; }

private:
  void draw()
  { 
    fl_color(color());
    fl_rectf(x(),y(),w(),h());  // erase background
    fl_font(FL_HELVETICA_BOLD, labelsize());
    fl_color(c);
    fl_draw(buf, x(), y()+ h()/2 + fl_height()/2 - fl_descent() );
  }
  Fl_Color c;
  char str[100];
  char buf[100];
};


//
// SfFrame is a top-level window containing a menubar (empty)
//   and perhaps a graphics window (flWin)
//

#define MENUBAR_HEIGHT 30

class SfActivityFrame;  // forward ref

class SfFrame : public SfFr, public Fl_Window
{
public:
  SFEXPORT SfFrame(int w, int h, const char *title = "Saphira window");
  SFEXPORT ~SfFrame() {}
  SFEXPORT Fl_Menu_Bar *menubar() { return mb; }  // returns the embedded menubar
  SfWindow *Win() { return win; }
  SfWindow *win;
  SfActivityFrame *actWin;	// this is the activity frame window 
  SFEXPORT void addRangeDevice(SfRangeDevice *sd); // adds this device to display list
  SFEXPORT void AddAllSensors(); // adds all current sensors
  SFEXPORT Fl_Menu_Item *AddMenuItem(char *name, Fl_Callback *cb, int flags, void *data = 0);
  SFEXPORT Fl_Menu_Item *FindMenuItem(char *name, Fl_Callback *cb = NULL);
  SFEXPORT bool UpdateMenuItem(bool val, char *name, Fl_Callback *cb = NULL);

private:
  Fl_Menu_Bar *mb;
};


//
// SfColbertFrame is a top-level frame with just a Colbert input window
//

class SfColbertFrame : public SfFrame
{
public:
  SFEXPORT SfColbertFrame(int w = 400, int h = 300, const char *title = "Colbert interaction window")
    : SfFrame(w,h,title)
      { new SfColbertGraphicStream(0,MENUBAR_HEIGHT,w,h-MENUBAR_HEIGHT); }
  SFEXPORT ~SfColbertFrame() {}
};


//
// SfMainFrame is a top-level controller, with a graphics window, Colbert window, and stats
//

class SfMainFrame : public SfFrame
{
public:
  SFEXPORT SfMainFrame(int w = 500, int h = 500, const char *title = "Saphira main window");
  SFEXPORT ~SfMainFrame() {}
  void Process();

private:
  Fl_Value *statlist[11];
  Fl_Light_Button *behavior_button;
  Fl_Button *motor_button;
};


// reasonable macro for returning current frame
#define SfMAINFRAME ((SfMainFrame *)SfFr::Current())


#endif // FLFRAME_H
