//
// flwin.h
//
// Display window class 
// Generic GUI frame class for FLTK
//

//
// Copyright 2000 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com

#ifndef FLWIN_H
#define FLWIN_H

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Menu_Button.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/Fl_File_Chooser.H>

#ifdef MS_WINDOWS
#ifdef SFEXPORT
#undef SFEXPORT
#endif
#ifdef FLTKWIN_EXPORTS
#define SFEXPORT _declspec( dllexport )
#else
#define SFEXPORT _declspec( dllimport )
#endif
#else
#define SFEXPORT
#endif

// Define a new window type for drawing

#define XRW_TO_PIX(x,y) (iscale * ((cs*x+sn*y)-x_origin))
#define YRW_TO_PIX(x,y) (iscale * (y_origin-(cs*y-sn*x)))

class SfWindow : public Fl_Double_Window, public SfWin
{
public:
  // constructor
  SFEXPORT SfWindow(int x, int y, int w, int h, const double scale = 20.0);

  SFEXPORT void Scale(double z);	// scale factor in mm/pixel
  SFEXPORT void Center(double x, double y); // center window here in rw coords
  SFEXPORT void Center();		// center on transform pose
  void Rotate(double th)	// rotation, in deg
    { cs = cos(-DEG_TO_RAD*th); sn = sin(-DEG_TO_RAD*th); }
  void Transform(ArPose *p);	// sets the 2D transform to this pose; NULL for none
  SFEXPORT void Coords(double *x, double *y, int i, int j);
				// returns RW coords from pixel indices

  void Add(SfDrawable *d)	// add to display list
    { if (d) dlist->push_front(d); }
  void Remove(SfDrawable *d)	// remove from display list
    { dlist->remove(d); }
  SFEXPORT void Empty();			// empty the display list
  double Offx()
    { return offx; }
  double Offy()
    { return offy; }
  void Zoom(double f)
    { Scale(f*scale); }
  void Pan(int h, int v)	// pan left/right and up/down
    { Center(offx+h*scale, offy+v*scale); }

  // drawing functions
  SFEXPORT void Vector(double x1, double y1, double x2, double y2, ArPose *);
  void Vector(double x1, double y1, double x2, double y2)
    { 
      double temp = x1;
      x1 = XRW_TO_PIX(x1,y1);
      y1 = YRW_TO_PIX(temp,y1);
      temp = x2;
      x2 = XRW_TO_PIX(x2,y2);
      y2 = YRW_TO_PIX(temp,y2);
      fl_line((int)x1,(int)y1,(int)x2,(int)y2); 
    }	
  SFEXPORT void Point(double x, double y, ArPose *);
  void Point(double x, double y)
    { 
      double temp = x;
      x = XRW_TO_PIX(x,y);
      y = YRW_TO_PIX(temp,y);
      fl_line((int)x,(int)y,(int)x,(int)y);   // draws a single point
    }

  SFEXPORT void Rectangle(double x1, double y1, double x2, double y2, ArPose *);
  void Rectangle(double x1, double y1, double x2, double y2)
    { 
      double temp = x1;
      int w, h;
      x1 = XRW_TO_PIX(x1,y1);
      y1 = YRW_TO_PIX(temp,y1);
      temp = x2;
      x2 = XRW_TO_PIX(x2,y2);
      y2 = YRW_TO_PIX(temp,y2);
      if (x2 < x1) { temp = x1; x1 = x2; x2 = temp; }
      if (y2 < y1) { temp = y1; y1 = y2; y2 = temp; }
      w = (int)(x2-x1);
      if (w <= 1) w = 2;	// a width or height of 1 is invisible
      h = (int)(y2-y1);
      if (h <= 1) h = 2;
      fl_rect((int)x1,(int)y1,w,h);
    }	

  void CRectangle(double x1, double y1, double x2, double y2, ArPose *p)
    {
      x2 *= 0.5;
      y2 *= 0.5;
      Rectangle(x1-x2, y1-y2, x1+x2, y1+y2, p);
    }	
  void CRectangle(double x1, double y1, double x2, double y2)
    { 
      x2 *= 0.5;
      y2 *= 0.5;
      Rectangle(x1-x2, y1-y2, x1+x2, y1+y2);
    }	

  SFEXPORT void Circle(double x, double y, double r, ArPose *p); // draws pose-relative circle of radius r
  void Circle(double x, double y, double r) // draws circle of radius r
    {
      double temp = x;
      x = XRW_TO_PIX(x,y);
      y = YRW_TO_PIX(temp,y);
      r *= iscale;
      x -= r/2;
      y -= r/2;
      fl_arc((int)x,(int)y, (int)r, (int)r, 0.0, 360.0);
    }

  SFEXPORT void Text(char *str, double x, double y, ArPose *p); // relative to pose
  void Text(char *str, double x, double y) // draws null-terminated text at location x,y
    {
      double temp = x;
      x = XRW_TO_PIX(x,y);
      y = YRW_TO_PIX(temp,y);
      int fnt = fl_font();
      int sz  = fl_size();
      fl_font(0, 11);
      fl_draw(str, (int)x, (int)y);
      fl_font(fnt, sz);
    }

  SFEXPORT void PenColor(int c); // sets pen color

  // How we draw every cycle
  void Process();

  // drawing params
  int  displayCycle;		// how often we draw
  bool displayLocal;		// local or global display
  bool displayRobot;		// keeps robot away from margins
  bool controlRobot;		// says if we use new C++ control fns

  // window size params, in rw units; should be private...
  double x_origin, y_origin;	// these are set on centering
  double x_size, y_size;	// these are set on centering

  // setting/clearing onscreen menu item
  void setOnscreen(bool val)
    {
      if (val)
	{
	  //	  onscreen_item->set();
	  displayRobot = true;
	}
      else
	{
	  //	  onscreen_item->clear();
	  displayRobot = false;
	}
    }

protected:
  void draw();          // drawing fn
  

private:
  // Geometry
  // An arbitrary 2D transform is applied to all drawing pts before the
  //   drawing fns are called.  This transform is given by offx, offy 
  //   (center of the window in RW coords), scale, and cs, sn (rotation).
  // For efficiency, x_origin, y_origin, and iscale are pre-computed to
  //   be used in the transform (see X/YRW_TO_PIX macro).

  ArPose *ourTrPose;		// transform pose, overrides rot and center
  double offx, offy;		// offset of center of window
  double scale, iscale;		// scale of the drawing, mm/pixel
  double cs, sn;		// sin and cos of rotation
  std::list<SfDrawable *> *dlist;	// what we want to draw

  // internal drawing buffers
  // TVector lets you draw relative to a coord system given by a pose;
  //   tcs and tsn hold sin and cos of rotation for efficiency

  double th, tcs, tsn;		// previous transformation of TVector
  int cycle;

  // save color 
  int prevcolor;

  // to check key and mouse events
  int handle(int);

  // onscreen menu item ptr
  Fl_Menu_Item *onscreen_item;

};


#endif // FLWIN_H
