//
// crawl.cpp
//
// Simple wall-following action 
//

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

#include "Saphira.h"
#define MAKE_LIBRARY
#include "export.h"

//
// This file defines four behavior actions:
//   SfCrawlAction(int side, int standoff, int deadband)
//   SfSwerveAction(int side, int near, int check)
//   SfArcAction(int side, int speed, int turn)
//   SfForwardAction(int speed)
//
// When invoked together in different combinations, these actions implement a
//   crawl action, either to the right or left side of the robot; or a wander
//   action.  See crawl.act for how to invoke them in a coordinated fashion.
//



// The Crawl action keeps the robot aligned with the wall, at a proper
//   distance.  It controls only the orientation of the robot.  The Arc
//   action will move the robot forward, and turn it in a short loop if
//   nothing prevents it.  In combination with Crawl, it will move the
//   robot forward along the wall, and curve around outside corners.
//   There's still a problem with inside corners, and obstacles straight
//   ahead.  To deal with these, the Swerve action checks ahead of the
//   robot, and turns *opposite* of the wall following behavior.  This
//   will get the robot around inside corners.
//
// SfCrawlAction(int side, int standoff, int deadband) is active when there is 
//   a wall structure on the side of the robot.  This action attempts
//   to maintain a distance standoff to the wall; deadband is a
//   tolerance for this distance.  
//
//   This action just controls the orientation of the robot.  It does
//   not control speed at all, so some other action must do this
//   (normally SfArcAction, or some other action like Forward).
//
//   There are two parts to controlling the orientation.  First, if
//   there is a valid wall on the side of the robot, then this wall
//   contributes to the orientation.  A wall is found by looking at 4
//   occupancy boxes on the side of the robot, starting from just
//   ahead of the centerpoint of the robot, to a little distance
//   behind the robot.  If one of the forward boxes find the wall, and
//   at least one other box does also, then there is a valid wall, and
//   the orientation is set to point the robot along the wall.
//
//   The second aspect concerns distance to the wall.  If either
//   of the first two boxes have a distance that is close to the wall
//   (less than the standoff), then the action attempts to turn the
//   robot away from the wall.  Conversely, if either of the two boxes
//   is too far from the wall, the action attempts to turn towards the
//   wall. 
//
//   The occupancy boxes use the cumulative readings of the sonars,
//   that is, ones that are very reliable, and kept for a longer
//   period of time.
//
//

//
// Crawl wall action
//

class SfCrawlAction : public ArAction, public SfArtifact
{
 public:
  SFEXPORT SfCrawlAction(int turn, int wall, int dead); // constructor
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired); // this defines the action
  static SfCrawlAction *invoke(int turn, int wall, int dead); // interface to Colbert
  SFEXPORT void draw(SfWin *w);	// drawing interface from SfArtifact

 private:
  ArActionDesired myDesired;
  int turnVal, wallVal, deadVal; // arguments to the action
  int f1Dist, f2Dist, b1Dist, b2Dist;	// distance to nearest sonar reading
  int sideClearance;            // distance we look on the side
  int sideNear, sideFar;        // near and far areas for side boxes
  int frontApproach;            // how close we are in the front
  int approachNear, approachFar; // whether we're too near or far on approaching a wall
  double approachGain;          // how much we try to turn based on distance
  double angleGain;             // how much we turn based on wall angle
  void drawit(int d, int x1, int x2, SfWin *w);
  bool CalcDY(double &dy);      // calculates average DY, given occ box distances
};

// 
// This constructor is a model for all actions.
// Chains to the ArAction class
//   turn is crawling direction (sfLEFT or sfRIGHT)
//   wall is standoff from wall (mm)
//   dead is standoff dead zone (mm)
//

SFEXPORT
SfCrawlAction::SfCrawlAction(int turn, int wall, int dead)
  : ArAction("Crawl")
{
  turnVal = turn;
  wallVal = wall;
  deadVal = dead;

  f1Dist = f2Dist = b1Dist = b2Dist = 5000;
  sideClearance = 1000;
  angleGain = 0.6;
  approachGain = 0.04;
}


//
// help function: calculates average dy over four occ boxes
// assumes f1Dist, etc have been set
//

bool
SfCrawlAction::CalcDY(double &dy)
{
  int y = -1;
  double dx = 0;
  int n = 0;
  if (f1Dist < 1000)
    y = f1Dist;
  else if (f2Dist < 1000)
    y = f2Dist;

  // calculate average dy
  // this is complicated by dropouts
  dy = 0;
  if (y > 0)			// ok, have a place to start
    {
      if (f1Dist < 1000)
	dx = 200;

      if (f2Dist < 1000)
	{
	  if (dx > 0)
	    {
	      n++;
	      dy += (double)(f2Dist - y) / dx;
	    }
	  y = f2Dist;
	  dx = 0;
	}
      dx += 200;
      
      if (b1Dist < 1000)
	{
	  n++;
	  dy += double(b1Dist - y) / dx;
	  y = b1Dist;
	  dx = 0;
	}
      dx += 200;

      if (b2Dist < 1000)
	{
	  n++;
	  dy += double(b2Dist - y) / dx;
	}

      if (n > 0)
	{
	  dy = dy / (double)n;
	  return true;
	}
    }
  
  return false;
}


//
// Body of action
// First get sensor readings on the side
// Calculate angle of the wall on the side, averaging the readings
// Check closest approach, using two front readings
//

#define STARTPOS 300

SFEXPORT ArActionDesired *
SfCrawlAction::fire(ArActionDesired currentDesired) // this defines the action
{
  // reset the actionDesired (must be done)
  myDesired.reset();

  int radius = (int)SfROBOT->getRobotRadius();
  if (turnVal == sfLEFT)
    {
      sideNear = 100;
      sideFar = sideClearance;
      approachGain = -ABS(approachGain);
      angleGain = -ABS(angleGain);
    }
  else
    {
      sideNear = -100;
      sideFar = -sideClearance;      
      approachGain = ABS(approachGain);
      angleGain = ABS(angleGain);
    }
  approachNear = radius + wallVal - deadVal / 2;
  approachFar  = radius + wallVal + deadVal / 2;

  double angle;
  frontApproach = 5000;		// not so close, by default

  // get sensor readings in four sequential areas on right side of robot
  SfSonarDevice *sd = Sf::sonar(); // get the device
  if (!sd) 
    f1Dist = f2Dist = b1Dist = b2Dist = 5000; // large value, no return
  else
    {
      ArPose rpose = SfROBOT->getPose();
      ArPose retPose;
      if (sd->getAccum()->getClosestBox(STARTPOS-200, sideNear, STARTPOS, sideFar, 
					rpose, 10000, &retPose) > 1000)
	f1Dist = 2000;
      else
	f1Dist = ABS(retPose.getY());
      if (sd->getAccum()->getClosestBox(STARTPOS-400, sideNear, STARTPOS-200, sideFar, 
					rpose, 10000, &retPose) > 1000)
	f2Dist = 2000;
      else
	f2Dist = ABS(retPose.getY());
      if (sd->getAccum()->getClosestBox(STARTPOS-600, sideNear, STARTPOS-400, sideFar, 
					rpose, 10000, &retPose) > 1000) 
	b1Dist = 2000;
      else
	b1Dist = ABS(retPose.getY());
      if (sd->getAccum()->getClosestBox(STARTPOS-800, sideNear, STARTPOS-600, sideFar, 
					rpose, 10000, &retPose) > 1000)
	b2Dist = 2000;
      else
	b2Dist = ABS(retPose.getY());
    }

  // calculate front approach, see how close we are
  frontApproach = MIN(f1Dist, f2Dist);

  //
  // calculate average DY over the four side boxes
  // a valid DY ranges from -1 (-45 deg) to +1 (45 deg)
  //

//  printf("%d %d %d %d\n", b2Dist, b1Dist, f2Dist, f1Dist);
  double dy;
  if (CalcDY(dy))
    {
      double heading;
      // ok, now determine what angle to place the robot at
      angle = RAD_TO_DEG * atan2(dy, 1.0);
      heading = (angleGain * angle) + SfROBOT->getTh();

      // now see if we need to turn based on how close we are to wall
      angle = 0.0;
      if (frontApproach < approachNear) // we're too close
	{
	  angle = (approachNear - frontApproach);
//	      sfMessage("%d", (int)angle);
	  angle = approachGain * angle;
	}
      if (frontApproach > approachFar && frontApproach < 1000) // we're too far
	{
	  angle = (approachFar - frontApproach);
	  angle = approachGain * angle;
	}
      if (angle < -10) angle = -10;
      if (angle > 10)  angle =  10;
//      sfMessage("%d %d", (int)heading, (int)angle);
      heading += angle;
      myDesired.setHeading(heading);
      return &myDesired;
    }

  return NULL;			// nope, not controlling it
}


// #define DRAWIT  // for turning off drawing, so we don't give too much away...

void 
SfCrawlAction::drawit(int d, int x1, int x2, SfWin *w)
{
#ifdef DRAWIT
  int dd = d;
  if (d < 1000)
    {
      w->PenColor(sfColorRed);
      if (sideFar < 0)
	dd = -d;
    }
  else
    {
      w->PenColor(sfColorGreen);
      dd = sideFar;
    }
  w->Rectangle(x1, sideNear, x2, dd, NULL);
#endif
}



SFEXPORT void
SfCrawlAction::draw(SfWin *w)
{
  if (isActive())
  {
	drawit(f1Dist, STARTPOS-200, STARTPOS, w);
    drawit(f2Dist, STARTPOS-400, STARTPOS-200, w);
    drawit(b1Dist, STARTPOS-600, STARTPOS-400, w);
    drawit(b2Dist, STARTPOS-800, STARTPOS-600, w);
    w->PenColor(0);
  }
}


//
// Swerve action
// Keeps the robot from running into things straight ahead
// Uses the OccBox functions to find a clear path at the robot's heading
// If no clear path, swerves to right or left until one is present
//
// Uses SfArtifact mixin to get drawing

class SfSwerveAction : public ArAction, public SfArtifact
{
 public:
  SFEXPORT SfSwerveAction(int turn, int avoid, int standoff); // 
  SFEXPORT ~SfSwerveAction() {};  // nothing doing
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired); // this defines the action
  static SfSwerveAction *invoke(int turn, int avoid, int standoff); // interface to Colbert
  SFEXPORT void draw(SfWin *w);	// drawing interface from SfArtifact

protected:			// arguments
  int turnDir;			// direction to turn, sfLEFT or sfRIGHT
  int avoidDist;		// close contact distance to robot
  int standDist;		// first notice distance to robot

 private:
  ArActionDesired myDesired;	// what the action wants to do
  int angleTurn;		// what we found for an angle
  int standOff;			// when we're near an obstacle
  int nearDist;			// full throttle on turning
  int frontDist;		// how close we actually are in front
};

// 
// This constructor is a model for all actions.
// Chains to the ArAction class
//

SFEXPORT
SfSwerveAction::SfSwerveAction(int turn, int avoid, int standoff)
  : ArAction("Swerve")
{
  turnDir = turn;
  avoidDist = avoid;
  standDist = standoff;
  standOff = standoff;
}


//
// Body of action
// First get sensor readings on the side
// Check closest approach, using two front readings
//

SFEXPORT ArActionDesired *
SfSwerveAction::fire(ArActionDesired d
)
{
  // reset the actionDesired (must be done)
  myDesired.reset();

  // set up parameters
  int radius = (int)SfROBOT->getRobotRadius();
  if (turnDir == sfLEFT)
    angleTurn = 5;		// how much we turn
  else
    angleTurn = -5;
  standOff = radius + standDist; // how far away on first contact
  nearDist = radius + avoidDist; // how close before full avoidance

  double x, y, strength;

  // get sensor readings in front of robot
  SfSonarDevice *sd = Sf::sonar(); // get the device
  if (!sd) 
    frontDist = 5000;		// large value, no return
  else
    {
      ArPose rpose = SfROBOT->getPose();
      frontDist = sd->getCurrent()->getClosestBox(100, -radius, 
						standOff + 200, radius, 
						rpose, 10000, NULL);
    }

  if (frontDist < standOff)	// we've found an obstacle in front
    {
      x = (double)(frontDist - nearDist);
      if (x < 0) x = 0;
      y = (double)(standOff - nearDist);
      // get stronger as we get closer
      strength = 1.0 - (x/y);
      myDesired.setHeading(SfROBOT->getTh() + angleTurn, strength);
      myDesired.setVel(0.0, strength);
    }
  return &myDesired;
}


// drawing function, indicate state of behavior graphically

SFEXPORT void
SfSwerveAction::draw(SfWin *w)
{
  if (!isActive())
    return;
  int radius = (int)SfROBOT->getRobotRadius();
  int dd = frontDist;
  if (dd < 0) 
    dd = 1000;
  if (dd < 1000)
    w->PenColor(sfColorRed);
  else
    {
      w->PenColor(sfColorGreen);
      dd = standOff + 200;
    }
  w->Rectangle(100, -radius, dd, radius, NULL);

  w->PenColor(0);
}




//
// Crawl forward movement action
// Keeps the robot moving forward, turning in an arc when nothing prevents it
// Arguments: speed to move, and angle to turn
// Turn angle argument is positive for CC turn (to the left), negative for C turn
//    10 is a quick arc, 2 is a long arc
//

class SfArcAction : public ArAction
{
 public:
  SFEXPORT SfArcAction(int speed, int turnang);
  SFEXPORT ~SfArcAction() {};  // nothing doing
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired); // this defines the action
  static SfArcAction *invoke(int speed, int turnang); // interface to Colbert

protected:			// arguments
  int speedVal;			// how fast we move
  int angleVal;			// how fast we turn
  ArActionDesired myDesired;	// what the action wants to do
};

// 
// This constructor is a model for all actions.
// Chains to the ArAction class
//

SFEXPORT
SfArcAction::SfArcAction(int speed, int turnang)
  : ArAction("Arc")
{
  speedVal = speed;
  angleVal = turnang;
}


//
// Body of action
// Just set the speed and turn from current heading
//

SFEXPORT ArActionDesired *
SfArcAction::fire(ArActionDesired d
)
{
  // reset the actionDesired (must be done)
  myDesired.reset();

  myDesired.setVel((double)speedVal);
  myDesired.setHeading((double)(SfROBOT->getTh() + angleVal));
  return &myDesired;
}


//
// Forward movement action
// Keeps the robot moving forward in a straight line
// Arguments: speed to move
//

class SfForwardAction : public ArAction
{
 public:
  SFEXPORT SfForwardAction(int speed);
  SFEXPORT ~SfForwardAction() {};  // nothing doing
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired); // this defines the action
  static SfForwardAction *invoke(int speed); // interface to Colbert

protected:			// arguments
  int speedVal;			// how fast we move
  ArActionDesired myDesired;	// what the action wants to do
};

// 
// This constructor is a model for all actions.
// Chains to the ArAction class
//

SFEXPORT
SfForwardAction::SfForwardAction(int speed)
  : ArAction("Forward")
{
  speedVal = speed;
}


//
// Body of action
// Just set the speed and turn from current heading
//

SFEXPORT ArActionDesired *
SfForwardAction::fire(ArActionDesired d
)
{
  // reset the actionDesired (must be done)
  myDesired.reset();

  myDesired.setVel((double)speedVal);
  return &myDesired;
}


//
// Interface to Colbert
// 
// This static function returns a behavioral action object,
//   with arguments that can be set from Colbert
//

SfCrawlAction *
SfCrawlAction::invoke(int turn, int wall, int dead)
{
  return new SfCrawlAction(turn, wall, dead);
}

SfSwerveAction *
SfSwerveAction::invoke(int turn, int avoid, int standoff)
{
  return new SfSwerveAction(turn, avoid, standoff);
}

SfArcAction *
SfArcAction::invoke(int speed, int turnang)
{
  return new SfArcAction(speed, turnang);
}

SfForwardAction *
SfForwardAction::invoke(int speed)
{
  return new SfForwardAction(speed);
}

SFEXPORT void			// define interface to Colbert here 
sfLoadInit (void)
{
  sfAddEvalAction("Crawl", (void *)SfCrawlAction::invoke, 3, sfINT, sfINT, sfINT);
  sfAddEvalAction("Swerve", (void *)SfSwerveAction::invoke, 3, sfINT, sfINT, sfINT);
  sfAddEvalAction("Arc", (void *)SfArcAction::invoke, 2, sfINT, sfINT);
  sfAddEvalAction("Forward", (void *)SfForwardAction::invoke, 1, sfINT);
}
