//
// movit.cpp
//
// Simple behavioral action 
//

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

#include "Saphira.h"
#include <math.h>
#define MAKE_LIBRARY
#include "export.h"

//
// Movit action
// Moves the robot forward a given distance
// Turns the robot to a given angle
//
// Parameters:
//  1) distance (in mm; INT)
//  2) absolute heading angle (in deg; INT)
//

class SfMovitAction : public ArAction
{
 public:
  SFEXPORT SfMovitAction(int distance, int heading); // constructor
  virtual ~SfMovitAction() {};	// nothing doing
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired);	// this defines the action
  SFEXPORT void reset() { gone = 0; ax = SfROBOT->getX(); ay = SfROBOT->getY(); }
  SFEXPORT void set(int distance, int heading) // let us set this on the fly
    { reset(); myDistance = distance; myHeading = heading; } 
  static SfMovitAction *invoke(int distance, int heading); // interface to Colbert

 protected:
  ArActionDesired myDesired;	// what the action wants to do
  int myDistance, myHeading;	// parameters to the action
  int gone;			// how far we've gone
  double ax, ay;		// old robot position
};

// 
// This constructor is a model for all actions.
// Chains to the basic ArAction class
//

SFEXPORT
SfMovitAction::SfMovitAction(int distance, int heading)
  : ArAction("Movit")
{
  myDistance = distance;
  myHeading = heading;
  reset();
}


//
// What the action does
//
// Returns and ArActionDesired pointer, containing what the action wants to do
//

SFEXPORT ArActionDesired *
SfMovitAction::fire(ArActionDesired d)
{

  // reset the actionDesired (must be done)
  myDesired.reset();

  // check the distance to be traveled
  double dx = ax - SfROBOT->getX();
  double dy = ay - SfROBOT->getY();
  ax = SfROBOT->getX();		// set new values
  ay = SfROBOT->getY();
  int ds = (int)sqrt(dx*dx + dy*dy);
  gone += (int)ds;
  sfMessage("Running Movit, gone %d", gone);

  if (gone >= myDistance)
    {
      sfMessage("Finished Movit");
      deactivate();		// turn off when done
      return NULL;
    }
  else
    {
      myDesired.setHeading(myHeading); // control the heading
      myDesired.setVel(200);	// moderate speed
    }
  return &myDesired;		// return the desired controls
}


// set the distance and heading, on the fly

void setit(int distance, int heading) 
{
  SfActTask *task;
  SfMovitAction *a;
  task = SfActRegister::getAct("Movit"); // this is the default name
  if (task != NULL) 
    {
      a = (SfMovitAction *)(task->action); // get the action object from the task shell
      a->set(distance, heading);
      sfMessage("New distance: %d  heading: %d", distance, heading);
    } 
}


//
// Interface to Colbert
// 
// This static function returns a behavioral action object,
//   with arguments that can be set from Colbert
//

SfMovitAction *
SfMovitAction::invoke(int distance, int heading)
{
  return new SfMovitAction(distance, heading);
}


SFEXPORT void			// define interface to Colbert here 
sfLoadInit ()
{
  sfAddEvalFn("setMovit", (void *)setit, sfVOID, 2, sfINT, sfINT);
  sfAddEvalAction("Movit", (void *)SfMovitAction::invoke, 2, sfINT, sfINT);
}


