/** Copyright 1999 by ActivMedia Robotics, LLC
 ** 
 ** ActivMedia Robotics hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial educational
 ** use only.  This software may not be distributed to others
 ** except by ActivMedia Robotics, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of ActivMedia Robotics.
 **
 ** ActivMedia Robotics does make any representations about the suitability
 ** of this software for any purpose.  It is provided "as is" without
 ** express or implied warranty.
 **
 **/

/*
    ACT Blob Packet structure:

    The packet begins with an index, which lists for each channel the
    first blob and number of blobs, such that

    0   1   2   3   4   5   6   7
    f n f n f n f n f n f n f n f n
    0 2 2 0 2 0 2 0 2 4 6 0 6 0 6 1

    indicates Channel 0 has 2 blobs, channel 4 has 4 blobs starting with
    blob 2, and channel 1 has 1 blob, blob 6. This information is
    redundant, but used for fast lookup.

    Then come the blobs, 10 bytes for each, as follows:
    
    [area]   4
    [x]      2
    [y]      2
    [left]   2
    [right]  2
    [top]    2
    [bottom] 2
			    
    so: the byte structure of a whole packet looks like (spaces added for
    clarity):  ffnn ffnn... aaaaxxyyllrrttbb aaaaxxyyllrrttbb...
    
    all bytes are unsigned, and in the packet are written as 1 + their
    value, to avoid null bytes.

    area is in 4 bytes, all others are 2 bytes; each byte is only 6 bits 
    (to avoid 0's...)
*/

#ifndef _SaphiraACTSClient
#define _SaphiraACTSClient
#include <iostream.h>
#include <string.h>
#include "acts_const.h"

#ifndef WIN32
#include "SocketClientComm.h"
#else
#include "WSocketClientComm.h"
#endif 

typedef struct blob_str {
  int area;
  int xcg,ycg;
  int left, right, top, bottom;
} blob;

/** Saphira integration structure.
    This class is responsible for handling the client-side integration of 
    Saphira and ACTS.  It listens to the socket, pings for information and
    receives data.  It parses this data and then stores it in the
    data structures provided (in the array of blobs).
*/

class SaphiraACTSClient {
  /// How many blobs for each channel?
  int blob_num[NUMCHAN];
  /// What index do the blobs start at in the blob stream?
  int blob_index[NUMCHAN];

  /// The socket communications system
  SocketClientComm * comm;

  /// Pointer to the returned data
  unsigned char * data;

public:
  /// Constructor
  SaphiraACTSClient();
  /// Destructor
  ~SaphiraACTSClient();

  /// Open the socket system
  bool openPort(char * service,char * hostname=NULL);
  /// Close it up
  bool closePort();

  /// Ask ACTS for a data packet
  bool requestPacket();
  /// Quit ACTS
  bool requestQuit();
  /// Check whether the header has been received yet
  bool receiveBlobInfo();

  /// Print out the blob information (debugging)
  bool viewBlob(int channel,int index);
  /// How many blobs are in this channel?
  int numBlob(int) const;

  /// Returns a blob structure
  bool getBlob(int channel, int index, blob * b);
};

#endif
