/** Copyright 1999 by ActivMedia Robotics, LLC
 ** 
 ** ActivMedia Robotics hereby grants to other individuals or organizations
 ** permission to use this software for non-commercial educational
 ** use only.  This software may not be distributed to others
 ** except by ActivMedia Robotics, under the conditions above.
 **
 ** Other than these cases, no part of this software may be used or
 ** distributed without written permission of ActivMedia Robotics.
 **
 ** ActivMedia Robotics does make any representations about the suitability
 ** of this software for any purpose.  It is provided "as is" without
 ** express or implied warranty.
 **
 **/

#include "SocketClientComm.h"

SocketClientComm::SocketClientComm(int size)
{
  // Install the SIGPIPE ignore handler
  struct sigaction sig;

  sig.sa_handler = SIG_IGN;
  sig.sa_flags = 0;
  sigemptyset(&sig.sa_mask);
  sigaction(SIGPIPE,&sig,NULL);

  maxsize=size;
  buf=new unsigned char[size];
  is_connected=false;
}

SocketClientComm::~SocketClientComm()
{
  closePort();
  delete [] buf;
}
//////////////////////////////////////////////////////////////////////////////
bool SocketClientComm::isConnected() const
{
  return is_connected;
}

//////////////////////////////////////////////////////////////////////////////
bool SocketClientComm::openPort(char * service, char *netaddress)
{
  struct in_addr * addr;
  struct sockaddr_in address;
  int port=-1;
  // Don't open the port again 
  if (is_connected) {
    cerr << "Open port request made while still connected!" << endl;
    return false;
  }
  cout << "service=" << service << endl;
  cout << "netaddress=" << netaddress << endl;

  port=a2Port(service,"tcp");
  addr=a2Addr(netaddress);
  if (NULL == addr) {
    cerr << "client: Invalid network address." << endl;
    return false;
  }
  cout << "port = " << htons(port) << endl;
  cout << "addr = " << inet_ntoa(*addr) << endl;

  // Set up the port descriptors
  memset((char *) &address, 0, sizeof(address));
  address.sin_family = AF_INET;
  address.sin_port = u_short(port);
  address.sin_addr.s_addr = addr->s_addr;

  cout << "Creating the socket" << endl;
  // Create the socket
  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("client: socket");
    return false;
  }
  cout << "Connecting the socket" << endl;
  // Connect the socket
  if (connect(sock, (struct sockaddr*) &address, sizeof(address)) < 0){
    perror("client: connect");
    cout << "errno = " << errno 
	 << " ECONNREFUSED = " << ECONNREFUSED 
	 << endl;
    return false;
  }
  is_connected=true;
  return is_connected;
}

//////////////////////////////////////////////////////////////////////////////
// Close the file descriptor
bool SocketClientComm::closePort()
{
  if (is_connected) {
    close(sock);
  }
  is_connected=false;
  return true;
}

//////////////////////////////////////////////////////////////////////////////
bool SocketClientComm::sendRequest(char req)
{
  if ((is_connected) && (1 == sockWrite(&req,1))) {
    return true;
  }
  return false;
}

bool SocketClientComm::receiveData(int buffsize)
{
  // Do a bit of range checking
  int size = buffsize;
  if (size > maxsize) {
    cout << "Buffsize " << buffsize 
	 << " too large.  Truncating to " << size 
	 << endl;
    size=maxsize;
  }
  if (is_connected) {
    if ((retval=sockRead(buf,buffsize)) > 0){
      //      cout << "Retval = " << retval << endl;
      return true;
    } else if (!retval) {
      cout << "Server down...\n";
      closePort();
    }
    //    cout << "Retval = " << retval << endl;
  }
  return false;
}

unsigned char * SocketClientComm::viewData() const
{
  return buf;
}

///////////////////////////////////////////////////////////////////////////////
int SocketClientComm::a2Port(char *service, char* proto)
{
  int port;
  long int lport;
  struct servent *serv;
  char *errpos;

  /* First try to read it from /etc/services */
  serv = getservbyname(service, proto);
  if (serv != NULL) {
    port = serv->s_port;
  } else { /* Not in services, maybe a number? */
    lport = strtol(service,&errpos,0);
    if ( (errpos[0] != 0) || (lport < 1) || (lport > 65535) )
      return -1; /* Invalid port address */
    port = htons(lport);
  }
  return port;
}

// Converts ascii text to in_addr struct.  NULL is returned if the address
// can not be found.
struct in_addr * SocketClientComm::a2Addr(char *address)
{
  struct hostent *host;
  static struct in_addr saddr;

  /* First try it as aaa.bbb.ccc.ddd. */
  saddr.s_addr = inet_addr(address);
  if (static_cast<unsigned int>(-1) != saddr.s_addr) {
    return &saddr;
  }
  host = gethostbyname(address);
  if (NULL != host) {
    return (struct in_addr *) *host->h_addr_list;
  }
  return NULL;
}

// This is just like the read() system call, accept that it will make
// sure that all your data goes through the socket.
int SocketClientComm::sockRead(unsigned char * buf, size_t count)
{
  size_t bytes_read = 0;
  int this_read;

  while (bytes_read < count) {
    do {
      this_read = read(sock, buf, count - bytes_read);
    } while ( (this_read < 0) && (errno == EINTR) );
    if (this_read < 0)
      return this_read;
    else if (this_read == 0)
      return bytes_read;
    bytes_read += this_read;
    buf += this_read;
  }
  return count;
}

/* This is just like the write() system call, accept that it will
   make sure that all data is transmitted. */
int SocketClientComm::sockWrite(const char * buf, size_t count)
{
  size_t bytes_sent = 0;
  int this_write;

  while (bytes_sent < count) {
    do {
      this_write = write(sock, buf, count - bytes_sent);
    } while ( (this_write < 0) && (errno == EINTR) );
    if (this_write <= 0)
      return this_write;
    bytes_sent += this_write;
    buf += this_write;
  }
  return count;
}

