//
// sfacts.cpp
//
// Saphira interface to the ACTS vision system
//

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

#include "Saphira.h"
#include "SaphiraACTSClient.h"
char * address="5001";
#define actsRUNNING 101

/* 
 * This file defines a saphira process that communicates to ACTS and
 * sets up a series of data structures for providing an interface to the
 * blob information from within Colbert.
 */

// This is the socket communications port to ACTS
SaphiraACTSClient * comm;

// This array stores the blob information coming back from the server
blob ACTSBlob[NUMCHAN][MAXBLOBS];

// Set this flag to connect/disconnect to the ACTS server
int sfActsConnectFlag = 0;	// when we want to connect or disconnect

//
// uTask definition, subclass of SfUTask for processing and SfArtifact 
//    for drawing
// 

class SfActsProcess : public SfUTask, public SfArtifact
{
public:
  SfActsProcess(char *name);
  void process();		// this is the workhorse, called every 100 ms
  void draw(SfWin *);		// let's draw something
};
    
SfActsProcess::SfActsProcess(char *name)
  : SfUTask("ACTS Process", 10)	// have to chain to the basic uTask object
{
  processState = sfINIT;
}

void
SfActsProcess::process()
{
  switch(processState)
    {
      // First thing, try to get connected
    case sfINIT:
    case sfRESUME:
      if (sfActsConnectFlag == 0) return;
      if (comm->openPort(address)) 
	{
	  comm->requestPacket();
	  sfMessage("Connected to ACTS process");
	  processState=actsRUNNING;
	}
      else
	{
	  sfMessage("*** ACTS process not running, please start it");
	  sfActsConnectFlag = 0;
	}
      break;
    case actsRUNNING:
      if (sfActsConnectFlag == 0)	// requested disconnect
	{
	  processState = sfINIT;
	  comm->closePort();
	  sfMessage("Disconnecting from ACTS process");
	}
      else if (comm->receiveBlobInfo())
	{
	  for (int channel=0;channel<1;channel++) // use NUMCHAN for all channels
	    {
	      for (int index=0;index<comm->numBlob(channel);index++) 
		{
		  if (index >= MAXBLOBS)
		    continue;
		  // set up the structure so we can access it later
		  comm->getBlob(channel, index,&ACTSBlob[channel][index]);
		  // print it out on the console
		  // comm->viewBlob(channel,index); 
		} 
	    }
	  comm->requestPacket();
	}
      break;
    case sfINTERRUPT:
    case sfSUSPEND:
      comm->closePort();
      break;
    }
}


// draw found blobs

int imageWidth = 160;
int imageHeight = 120;
float focalLength = 175.0; // in pixels; assume x and y pixel size the same
float objectWidth = 160.0; // in mm, width of ball object

void
SfActsProcess::draw(SfWin *w)
{
  if (comm->numBlob(0) > 0)	// check for blobs on chan 0
    {	
      // now calculate the angle, based on x center-of-gravity
      double ang = ACTSBlob[0][0].xcg; // x center-of-gravity
      
      ang = ang - imageWidth/2;	// from center of image, negative values on the right
      ang = RAD_TO_DEG * atan2(ang, focalLength); // now in degrees, from robot heading
      
      // and the distance
      double dist = ACTSBlob[0][0].right - ACTSBlob[0][0].left; // width of blob
      dist = focalLength * objectWidth / dist;			// range in mm

      // now compute x,y coords relative to robot; assumes camera at robot center,
      //    pointing straight ahead
      double x = dist * cos(DEG_TO_RAD * ang);
      double y = dist * sin(DEG_TO_RAD * ang);

      // draw relative to robot
      w->PenColor(sfColorRed);
      w->Circle(x,y,200,NULL);	// small circle at correct position
      w->PenColor(0);
    }
}

// instance of the process
SfActsProcess *myproc;

void Start(char *name)		// sets up an instance of the task
{
  myproc = new SfActsProcess(name);
}



int sfBlobValid(int chan,int index)
{
  if (index >= MAXBLOBS)
    return 0;
  if (comm->numBlob(chan) >= (index+1)) 
    return 1;
  else 
    return 0;
}

// here is the structure of a blob

//typedef struct blob_str {
//  int area;
//  int xcg,ycg;
//  int left, right, top, bottom;
//} blob;

int sfBlob;			// struct id for Colbert



// return the number of blobs on this channel

int
sfNumBlobs(int chan)
{
  return comm->numBlob(chan);
}

// return this blob

blob *
sfGetBlob(int chan, int bnum)
{
  return &ACTSBlob[chan][bnum];
}


SFEXPORT void
sfLoadInit(void)
{
  comm = new SaphiraACTSClient();
  Start("ACTS Process");

  // Add these functions
  sfAddEvalFn("sfBlobValid",(void *)sfBlobValid,sfINT,2,sfINT,sfINT);
  sfAddEvalFn("sfNumBlobs", (void *)sfNumBlobs, sfINT, 1, sfINT);
  sfAddEvalVar("sfActsConnectFlag", sfINT, &sfActsConnectFlag);

  // blob structure
  blob bl;
  sfBlob = sfAddEvalStruct("blob", sizeof(blob), (char *)&bl, 7, 
			       "area", &bl.area, sfINT,
			       "xcg", &bl.xcg, sfINT,
			       "ycg", &bl.ycg, sfINT,
			       "left", &bl.left, sfINT,
			       "right", &bl.right, sfINT,
			       "top", &bl.top, sfINT,
			       "bottom", &bl.bottom, sfINT);
  sfAddEvalFn("sfGetBlob", (void *)sfGetBlob, sfTypeRef(sfBlob), 2, sfINT, sfINT);

  sfAddEvalVar("focalLength", sfFLOAT, &focalLength);
  sfAddEvalVar("objectWidth", sfFLOAT, &objectWidth);
  sfAddEvalVar("imageWidth", sfINT, &imageWidth);
  sfAddEvalVar("imageHeight", sfINT, &imageHeight);
}


