//
// swerve.cpp
//
// Simple avoidance action 
//

//
// Copyright 2001 by Kurt Konolige
//
// The author hereby grants to SRI permission to use this software.
// The author also grants to SRI permission to distribute this software
// to schools for non-commercial educational use only.
//
// The author hereby grants to other individuals or organizations
// permission to use this software for non-commercial
// educational use only.  This software may not be distributed to others
// except by SRI, under the conditions above.
//
// Other than these cases, no part of this software may be used or
// distributed without written permission of the author.
//
// Neither the author nor SRI make any representations about the 
// suitability of this software for any purpose.  It is provided 
// "as is" without express or implied warranty.
//
// Kurt Konolige
// Senior Computer Scientist
// SRI International
// 333 Ravenswood Avenue
// Menlo Park, CA 94025
// E-mail:  konolige@ai.sri.com
//

#include "Saphira.h"
#define MAKE_LIBRARY
#include "export.h"

//
// This file defines the behavior action:
//   sfSwerveAction(int side, int near, int check)
//

//
// Swerve action
// Keeps the robot from running into things straight ahead
// Uses the OccBox functions to find a clear path at the robot's heading
// If no clear path, swerves to right or left until one is present
//
// Uses SfArtifact mixin to get drawing

class SfSwerveAction : public ArAction, public SfArtifact
{
 public:
  SFEXPORT SfSwerveAction(int turn, int avoid, int standoff); // 
  SFEXPORT ~SfSwerveAction() {};  // nothing doing
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired); // this defines the action
  static SfSwerveAction *invoke(int turn, int avoid, int standoff); // interface to Colbert
  SFEXPORT void draw(SfWin *w);	// drawing interface from SfArtifact

protected:			// arguments
  int turnDir;			// direction to turn, sfLEFT or sfRIGHT
  int avoidDist;		// close contact distance to robot
  int standDist;		// first notice distance to robot

 private:
  ArActionDesired myDesired;	// what the action wants to do
  int angleTurn;		// what we found for an angle
  int standOff;			// when we're near an obstacle
  int nearDist;			// full throttle on turning
  int frontDist;		// how close we actually are in front
};

// 
// This constructor is a model for all actions.
// Chains to the ArAction class
//

SFEXPORT
SfSwerveAction::SfSwerveAction(int turn, int avoid, int standoff)
  : ArAction("Swerve")
{
  turnDir = turn;
  avoidDist = avoid;
  standDist = standoff;
}


//
// Body of action
// First get sensor readings on the side
// Check closest approach, using two front readings
//

SFEXPORT ArActionDesired *
SfSwerveAction::fire(ArActionDesired d
)
{
  // reset the actionDesired (must be done)
  myDesired.reset();

  // set up parameters
  int radius = (int)SfROBOT->getRobotRadius();
  if (turnDir == sfLEFT)
    angleTurn = 5;		// how much we turn
  else
    angleTurn = -5;
  standOff = radius + standDist; // how far away on first contact
  nearDist = radius + avoidDist; // how close before full avoidance

  double x, y, strength;

  // get sensor readings in front of robot
  SfSonarDevice *sd = Sf::sonar(); // get the device
  if (!sd) 
    frontDist = 5000;		// large value, no return
  else
    {
      ArPose rpose = SfROBOT->getPose();
      frontDist = sd->getCurrent()->getClosestBox(100, -radius-100, 
						standOff + 200, radius+100, 
						rpose, 10000, NULL);
    }

  if (frontDist < standOff)	// we've found an obstacle in front
    {
      x = (double)(frontDist - nearDist);
      if (x < 0) x = 0;
      y = (double)(standOff - nearDist);
      // get stronger as we get closer
      strength = 1.0 - (x/y);
      myDesired.setHeading(SfROBOT->getTh() + angleTurn, strength);
      myDesired.setVel(0.0, strength);
      sfMessage("Strength: %f", (float)strength);
    }
  return &myDesired;
}


// drawing function, indicate state of behavior graphically

SFEXPORT void
SfSwerveAction::draw(SfWin *w)
{
  int radius = (int)SfROBOT->getRobotRadius();
  int dd = frontDist;
  if (dd < 1000)
    w->PenColor(sfColorRed);
  else
    {
      w->PenColor(sfColorGreen);
      dd = standOff + 200;
    }
  w->Rectangle(100, -radius-100, dd, radius+100, NULL);

  w->PenColor(0);
}


//
// SfForwardAction(int speed)
// Moves the robot forward at a constant speed
//

class SfForwardAction : public ArAction
{
 public:
  SFEXPORT SfForwardAction(int speed);
  SFEXPORT ~SfForwardAction() {};  // nothing doing
  SFEXPORT virtual ArActionDesired *fire(ArActionDesired currentDesired); // this defines the action
  static SfForwardAction *invoke(int speed); // interface to Colbert

protected:			// arguments
  int speedVal;			// how fast we move
  ArActionDesired myDesired;	// what the action wants to do
};

// 
// This constructor is a model for all actions.
// Chains to the ArAction class
//

SFEXPORT
SfForwardAction::SfForwardAction(int speed)
  : ArAction("Forward")
{
  speedVal = speed;
}


//
// Body of action
// Just set the speed
//

SFEXPORT ArActionDesired *
SfForwardAction::fire(ArActionDesired d
)
{
  // reset the actionDesired (must be done)
  myDesired.reset();

  myDesired.setVel((double)speedVal, 1.0);
  return &myDesired;
}



//
// Interface to Colbert
// 
// This static function returns a behavioral action object,
//   with arguments that can be set from Colbert
//

SfSwerveAction *
SfSwerveAction::invoke(int turn, int avoid, int standoff)
{
  return new SfSwerveAction(turn, avoid, standoff);
}

SfForwardAction *
SfForwardAction::invoke(int speed)
{
  return new SfForwardAction(speed);
}


SFEXPORT void			// define interface to Colbert here 
sfLoadInit (void)
{
  sfAddEvalAction("Swerve", (void *)SfSwerveAction::invoke, 3, sfINT, sfINT, sfINT);
  sfAddEvalAction("Forward", (void *)SfForwardAction::invoke, 1, sfINT);
}



